<?php

/**
 * shd.action
 *
 * @package SHD
 * @subpackage API
 * @author Maik Rhode <Maik.Rhode@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 */
class isys_api_model_shd_action implements isys_api_model_interface
{

    /**
     * Read Action
     *
     * @param array $p_params
     *     array (
     *         'map_id'       =>      'integer',
     *     )
     * @return array|boolean
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function read($p_params)
    {
        if (isset($p_params["action_id"]) == false) {
            throw new isys_exception_api('Parameter action_id missed', -32602);
        }
        return smview_cmdb_dao_action::instance(isys_application::instance()->database)->getAction($p_params);
    }

    /**
     * Create Map Entry
     *
     * @param array $p_params
     *     array (
     *         'title'       =>      'name der Map',
     *         'user_id'     =>      'BenutzerId',
     *     )
     * @return array (
     *         'message'       =>      'name der Map',
     *         'response'      =>      'BenutzerId',
     *         'current_date'  =>      'BenutzerId',
     *     )
     * @throws isys_exception_dao
     * @throws isys_exception_general
     */
    public function create($p_params)
    {
        $l_return_id = smview_cmdb_dao_action::instance(isys_application::instance()->database)->save(null, $p_params["action"]);
        return array(
            'message'      => 'action.create.success',
            'response'     => $l_return_id,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * Update Action
     *
     * @param array $p_params
     *     array (
     *         'action_id'      =>      'id der action',
     *         'action'         =>      'action key/value',
     *     )
     * @return array (
     *         'message'       =>      'name der Map',
     *         'response'      =>      'BenutzerId',
     *         'current_date'  =>      'BenutzerId',
     *     )
     * @throws isys_exception_dao
     * @throws isys_exception_general
     */
    public function update($p_params)
    {
        smview_cmdb_dao_action::instance(isys_application::instance()->database)->save($p_params["action_id"], $p_params["action"]);
        return array(
            'message'      => 'action.update.success',
            'response'     => true,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * Delete Map Entry
     *
     * @param array $p_params
     *     array (
     *         'map_id'      =>      'Id der Map',
     *     )
     * @return array (
     *         'message'       =>      'name der Map',
     * 'response'      =>      'BenutzerId',
     * 'current_date'  =>      'BenutzerId',
     * )
     * @throws isys_exception_api
     * @throws isys_exception_dao
     */
    public function delete($p_params)
    {
        if (isset($p_params["action_id"]) == false) {
            throw new isys_exception_api('Parameter action_id missed', -32602);
        }

        smview_cmdb_dao_action::instance(isys_application::instance()->database)->deleteAction($p_params["action_id"]);
        return array(
            'message'      => 'action.delete.success',
            'response'     => true,
            'current_date' => date('Y-m-d')
        );
    }
}
