<?php

/**
 * shd.apikey
 *
 * @package shd
 * @subpackage API
 * @author Maik Rhode <Maik.Rhode@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 */
class isys_api_model_shd_apikey
{
    
    /**
     * Path to key file.
     *
     * @var string
     */
    private $key_file;
    
    /**
     * Constructor.
     *
     * @global string $g_absdir
     */
    function __construct()
    {
        global $g_absdir;

        $this->key_file = $g_absdir . '/images/shd-smview.fdg';
    }

    /**
     * @return false
     */
    public function read()
    {
        return false;
    }

    /**
     * Enable key by adding API key to key file.
     *
     * @author Maik Rhode <maik.rhode@shd-online.de>
     */
    public function enableKey()
    {
        $this->toggleApikey(true);
    }
    
    /**
     * Disable key by removing API key from key file.
     *
     * @author Steve Groeber <steve.groeber@shd-online.de>
     */
    public function disableKey()
    {
        $this->toggleApikey(false);
    }
    
    /**
     * Toggle API key to be exported to key file or not.
     *
     * @param boolean $enabled
     */
    private function toggleApikey($enabled)
    {
        // load of the mandator data (key / title)
        $l_apikey_data = $this->getMandatorData();
        $apititle = $l_apikey_data["isys_mandator__title"];
        $apikey = $enabled ? $l_apikey_data["isys_mandator__apikey"] : '';
        
        // set apikey empty
        $this->writeKeyDataToFile($apititle, $apikey);
    }
    
    /**
     * Get mandator data from database.
     *
     * @return array
     */
    private function getMandatorData()
    {
        //load of the mandator data (key / title)
        $mandator__id = (int) isys_application::instance()->session->get_mandator_id();
        return isys_application::instance()->database_system->fetch_row_assoc(
            isys_application::instance()->database_system->query(
                'SELECT isys_mandator__apikey, isys_mandator__title FROM isys_mandator 
                  WHERE isys_mandator__id = "' . $mandator__id . '";'
            )
        );
    }
    
    /**
     * Write key data to key file.
     *
     * @param string $apiTitle
     * @param string $apiKey
     */
    private function writeKeyDataToFile($apiTitle, $apiKey)
    {
        $apiArray = array();
        $mandator__id = (int) isys_application::instance()->session->get_mandator_id();
        if (file_exists($this->key_file)) {
            // get current content
            $apikeyString = file_get_contents($this->key_file);
            $apiArray = json_decode(base64_decode($apikeyString), true);
        }
        
        // set mandator data
        $apiArray[$mandator__id]["title"] = $apiTitle;
        $apiArray[$mandator__id]["apikey"] = $apiKey;
        
        // save to file
        $ifp = fopen($this->key_file, "wb");
        fwrite($ifp, base64_encode(json_encode($apiArray)));
        fclose($ifp);
    }

    /**
     * Return if mandator is exported to key file or not.
     *
     * @author Maik Rhode <maik.rhode@shd-online.de>
     * @return boolean
     */
    public function isActive()
    {
        // no file found
        if (! file_exists($this->key_file)) {
            return false;
        }

        $apikeyString = file_get_contents($this->key_file);
        $apikey = json_decode(base64_decode($apikeyString), true);
        
        $l_apikey_data = $this->getMandatorData();
        $mandator__id = (int) isys_application::instance()->session->get_mandator_id();

        // check Apikey
        return $l_apikey_data["isys_mandator__apikey"] == $apikey[$mandator__id]["apikey"];
    }
}
