<?php

/**
 * i-doit APi
 *
 * @package i-doit
 * @subpackage API
 * @author Dennis Stücken <dstuecken@synetics.de>
 * @copyright synetics GmbH
 * @license http://www.i-doit.com/license
 */
class isys_api_model_shd_categories implements isys_api_model_interface
{

    /**
     * @param array $p_params
     * @return array
     */
    public function read($p_params)
    {
        return smview_cmdb_dao_category_property::instance(isys_application::instance()->database)->get_category();
    }

    /**
     * Update category.
     *
     * @param  array $p_params
     * @return array|boolean
     * @throws isys_exception_api
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function update($p_params)
    {
        $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao(isys_application::instance()->database));
        $message = "";

        try {
            if (!isset($p_params['obj_id']) && !is_numeric($p_params['obj_id'])) {
                return false;
            }

            $objectId = $p_params['obj_id'];
            foreach ($p_params["objects"] as $objekt) {
                $category_data = [];
                foreach ($objekt as $column_key => $value) {
                    //built a data array for category.create
                    $split = explode("-", $column_key);
                    if ($value == "-1") {
                        continue;
                    }

                    switch ($column_key) {
                        case "C__CATG__SMVIEW_CABLE-description_sort":
                            //reset Cable-Name to Objekt-title
                            $category_data["C__CATG__GLOBAL"]["title"] = $value;
                            unset($p_params["objects"][$column_key]);
                            break;

                        case "C__CATG__SMVIEW_CABLE-state_id":
                        case "C__CATG__SMVIEW_CABLE-group_id":
                        case "C__CATG__SMVIEW_CABLE-type_id":
                        case "C__CATG__SMVIEW_CABLE-is_section_cable":
                        case "C__CATG__SMVIEW_CABLE-connection_type_id":
                        case "C__CATG__SMVIEW_CABLE_WIRE-reserved":
                        case "C__CATG__SMVIEW_CABLE_WIRE-state_in":
                        case "C__CATG__SMVIEW_CABLE_WIRE-state_out":
                        case "C__CATG__SMVIEW_CABLE_WIRE-connection_type_id":
                        case "C__CATG__SMVIEW_CABLE_WIRE-user_state_id":
                        case "C__CATG__SMVIEW_CABLE_WIRE-section_id":
                        case "C__CATG__SMVIEW_CABLE_WIRE-route_id":
                        case "C__CATG__SMVIEW_CABLE_WIRE-virtual_route_id":
                        case "C__CATG__SMVIEW_CABLE_SECTION-user_state_id":
                        case "C__CATG__SMVIEW_CABLE_SECTION-route_id":
                        case "C__CATG__SMVIEW_CABLE_ROUTE-route_usage_id":
                        case "C__CATG__SMVIEW_CABLE_ROUTE-is_external":
                        case "C__CATG__SMVIEW_CABLE_ROUTE-is_rented":
                        case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-route_usage_id":
                        case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-is_external":
                            if ($value == "") {
                                //$value musst be deleted
                                $category_data[$split[0]][$split[1]] = 0;
                            } else {
                                $property_id = $this->getDialogId($column_key, $value);
                                $category_data[$split[0]][$split[1]] = $property_id;
                            }
                            break;

                        case "C__CATG__SMVIEW_CABLE-admin_unit":
                            if ($value == "") {
                                //$value musst be deleted
                                $category_data["C__CATG__SMVIEW_CABLE"]["admin_unit_id"] = "0";
                            } else {
                                $property_id = $this->getDialogId($column_key, $value);
                                $category_data["C__CATG__SMVIEW_CABLE"]["admin_unit_id"] = $property_id;
                            }
                            break;

                        default:
                            $category_data[$split[0]][$split[1]] = $value;
                            break;
                    }
                }

                foreach ($category_data as $category_key => $category_value) {
                    if ($category_key == "C__CATG__SMVIEW_CABLE") {
                        $category_value['obj_id'] = $objectId;

                        $p_cable_wire = [
                            'data' => $category_value
                        ];
                        $cable_api_model_cable = new cable_api_model_cable();
                        if (isset($category_value["id"])) {
                            $p_cable_wire = [
                                'id' => $category_value["id"],
                                'data' => $category_value
                            ];
                            $cable_api_model_cable->update($p_cable_wire);
                        } else {
                            $cable_api_model_cable->create($p_cable_wire);
                        }
                    } elseif ($category_key == "C__CATG__SMVIEW_CABLE_PIN") {
                        //replace out_, in_
                        $pin_data = [];
                        foreach ($category_value as $f_category_key => $f_category_value) {
                            $t_key =  str_replace("in_","", $f_category_key);
                            $t_key =  str_replace("scope","classification", $t_key);
                            $pin_data[$t_key] = $f_category_value;
                        }

                        //Update connection with wire_pin and connected_pin
                        if(isset($pin_data["cable_wire"])) {
                            $isys_cmdb_dao_smview_wire = isys_cmdb_dao_smview_cable_wire_pin::instance(isys_application::instance()->database);
                            $cable_api_model_cable = isys_cmdb_dao_smview_cable::instance(isys_application::instance()->database);
                            $temp_cable = $cable_api_model_cable->get_object("obj_id", $pin_data["cable"]);

                            $isys_cmdb_dao_category_g_smview_cable_wire = new isys_cmdb_dao_category_g_smview_cable_wire(isys_application::instance()->database);
                            $pin = $isys_cmdb_dao_category_g_smview_cable_wire->get_additional_wire_information_by_id($pin_data["cable_wire"]);

                            //check wire_pin is in use
                            if($pin["in_use"] !== "") {
                                $message .= " " . $objectId ." can't save the object. Wire is in use.";
                            }elseif($pin_data["cable_wire"] == "") {
                                //delete wire_pin connection to set wire free
                                $delete_wire_pin = $isys_cmdb_dao_smview_wire->get_object("connected_pin_id", $pin_data["id"]);
                                $isys_cmdb_dao_smview_wire->update_wire_pin($delete_wire_pin["id"], $delete_wire_pin["description"], null, null, null);
                            } elseif($temp_cable["location_in_id"] == $objectId){

                                // important: ID of left pin (incoming) is lower than ID of right pin
                                $wire_pin = $isys_cmdb_dao_smview_wire->get_object("wire_id", $pin_data["cable_wire"]);
                                if ($wire_pin['id'] > $wire_pin['sibling_id']) {
                                    $wire_pin = $isys_cmdb_dao_smview_wire->get_object("id", $wire_pin["sibling_id"]);
                                }
                                $isys_cmdb_dao_smview_wire->update_wire_pin($wire_pin["id"], $wire_pin["description"],null,null,$pin_data["id"]);
                            } elseif ($temp_cable["location_out_id"] == $objectId){

                                // important: ID of right pin (outgoing) is higher than ID of left pin
                                $wire_pin = $isys_cmdb_dao_smview_wire->get_object("wire_id", $pin_data["cable_wire"]);
                                if ($wire_pin['id'] < $wire_pin['sibling_id']) {
                                    $wire_pin = $isys_cmdb_dao_smview_wire->get_object("id", $wire_pin["sibling_id"]);
                                }
                                $isys_cmdb_dao_smview_wire->update_wire_pin($wire_pin["id"], $wire_pin["description"],null,null,$pin_data["id"]);
                            } else {
                                $message .= " " . $objectId ." can't save the object. Obj_id is not in location";
                            }
                        }

                        unset($pin_data["cable_wire"]);
                        unset($pin_data["cable"]);

                        //search for out_id and update description and scope

                        $cable_api_model_cable = new cable_api_model_pin();
                        $isys_cmdb_dao_smview_cable_pin = isys_cmdb_dao_smview_cable_pin::instance(isys_application::instance()->database);
                        foreach ($isys_cmdb_dao_smview_cable_pin->get_listing_by_obj_id($objectId) as $item) {
                            if ($item["obj_in"]["id"] == $pin_data["id"]) {
                                $sibling_id = $item["obj_out"]["id"];
                            }
                        }

                        foreach ($isys_cmdb_dao_smview_cable_pin->get_data($objectId) as $item) {
                            if ($item["id"] == $pin_data["id"]) {
                                $main_pin = $item;
                            }

                            if ($item["id"] == $sibling_id) {
                                $sibling_pin = $item;
                            }
                        }

                        if (isset($pin_data["id"])) {
                            //UPDATE
                            //more then id in array
                            if (count($pin_data) > 1) {
                                unset($pin_data["id"]);

                                $isys_cmdb_dao_smview_cable_pin->update_object($main_pin["id"], $pin_data);
                                $isys_cmdb_dao_smview_cable_pin->update_object($sibling_pin["id"], $pin_data);
                            }
                        } else {
                            //CREATE
                            $pin_data["obj_id"] = $objectId;
                            $pin_data["type"] = isys_cmdb_dao_smview_cable_pin::PIN_TYPE_INPUT;
                            $p_cable_wire = [
                                'data' => $pin_data
                            ];
                            $l_categoryDatas = $cable_api_model_cable->create($p_cable_wire);

                            $pin_data["obj_id"] = $objectId;
                            $pin_data["type"] = isys_cmdb_dao_smview_cable_pin::PIN_TYPE_OUTPUT;
                            $pin_data["internal_pin_id"] = $l_categoryDatas["id"];

                            $p_cable_wire = [
                                'data' => $pin_data
                            ];
                            $cable_api_model_cable->create($p_cable_wire);
                        }
                    } elseif ($category_key == "C__CATG__SMVIEW_CABLE_WIRE") {
                        $category_value['obj_id'] = $objectId;

                        $cable_api_model_wire = new cable_api_model_wire();
                        $cable_api_model_wire_pin = new cable_api_model_wire_pin();

                        $category_value_wire = [];
                        $category_value_pin_out = [];
                        $category_value_pin_in = [];

                        foreach ($category_value as $item_key => $value) {
                            switch ($item_key) {
                                case "state_out":
                                    $category_value_pin_out["status_id"] = $value;
                                    break;
                                case "description_outgoing":
                                    $category_value_pin_out["description"] = $value;
                                    break;
                                case "state_in":
                                    $category_value_pin_in["status_id"] = $value;
                                    break;
                                case "description_incoming":
                                    $category_value_pin_in["description"] = $value;
                                    break;
                                case "user_state":
                                    $category_value_wire[$item_key] = $value;
                                    break;
                                default:
                                    $category_value_wire[$item_key] = $value == "" ? "0" : $value;
                                    break;
                            }
                        }

                        $Connected_pin = $cable_api_model_wire_pin->read(["filter" => ["wire_id" => $category_value["id"]]]);
                        if ($Connected_pin[0]["id"] < $Connected_pin[1]["id"]) {
                            $p_cable_wire_pin_out = [
                                'id' => $Connected_pin[1]["id"],
                                'data' => $category_value_pin_out
                            ];

                            $p_cable_wire_pin_in = [
                                'id' => $Connected_pin[0]["id"],
                                'data' => $category_value_pin_in
                            ];
                        } else {
                            $p_cable_wire_pin_out = [
                                'id' => $Connected_pin[0]["id"],
                                'data' => $category_value_pin_out
                            ];

                            $p_cable_wire_pin_in = [
                                'id' => $Connected_pin[1]["id"],
                                'data' => $category_value_pin_in
                            ];
                        }

                        if (isset($category_value["id"])) {
                            $p_cable_wire = [
                                'id' => $category_value["id"],
                                'data' => $category_value_wire
                            ];
                            $cable_api_model_wire->update($p_cable_wire);
                            $cable_api_model_wire_pin->update($p_cable_wire_pin_out);
                            $cable_api_model_wire_pin->update($p_cable_wire_pin_in);
                        } else {
                            $p_cable_wire = [
                                'data' => $category_value_wire
                            ];
                            $cable_api_model_wire->create($p_cable_wire);
                        }
                    } elseif ($category_key == "C__CATG__SMVIEW_CABLE_PIN") {
                        //todo save the Wire Id
                        $dao = new isys_cmdb_dao_smview_cable_wire_pin(isys_application::instance()->database);
                        $dao->update_wire_pin(
                            $category_value["id"],
                            null,
                            null,
                            null,
                            $category_value["in_cable"]
                        );
                    } else {
                        foreach ($category_value as $f_key => $f_value) {
                            if ($f_key == "id") {
                                $t_entry = (int)$f_value;
                                $category_value[$f_key] = (int)$f_value;
                            } else {
                                $category_value[$f_key] = $this->getInfo($category_key, $f_key, $f_value);
                            }
                        }

                        if (isset($t_entry)) {
                            $p_param = [
                                'object'   => (int)$objectId,
                                'category' => $category_key,
                                'entry'    => $t_entry,
                                'data'     => $category_value
                            ];
                        } else {
                            $p_param = [
                                'object'   => (int)$objectId,
                                'category' => $category_key,
                                'data'     => $category_value
                            ];
                        }
                        $l_api_model_category->save($p_param);
                    }
                }
            }
        } catch (isys_exception_cmdb $e) {
            throw new isys_exception_api($e->getMessage());
        }

        if ($message == "") {
            return [
                'message'      => 'categories.update.success',
                'response'     => true,
                'current_date' => date('Y-m-d')
            ];
        } else {
            return [
                'message'      => $message,
                'response'     => false,
                'current_date' => date('Y-m-d')
            ];
        }
    }

    public function getInfo($category_const, $category_key, $value)
    {
        $isys_api_model_cmdb_category_info = new isys_api_model_cmdb_category_info(new isys_cmdb_dao(isys_application::instance()->database));
        $catType = stripos($category_const, 'C__CATS__') !== false ? C__CMDB__GET__CATS : C__CMDB__GET__CATG;
        $catType = stripos($category_const, 'C__CATG__CUSTOM') !== false ? C__CMDB__GET__CATG_CUSTOM : $catType;
        $info = $isys_api_model_cmdb_category_info->read([$catType => $category_const]);
        foreach ($info as $key => $item) {
            if ($key == $category_key) {
                if ($item["info"]["type"] == "dialog"|| $item["info"]["type"] == "dialog_plus") {
                    return $value;
                }
                switch ($item["data"]["type"]) {
                    case "int":
                        return (int) $value;
                    case "float":
                        return (float) $value;
                    case "double":
                        return (double) $value;
                    default:
                        return $value;
                }
            }
        }
        return $value;
    }

    public function delete($p_params)
    {
        $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao(isys_application::instance()->database));
        $return = [];
        if (!isset($p_params['obj_id']) && !is_numeric($p_params['obj_id'])) {
            return false;
        }

        foreach ($p_params["objects"] as $objekt) {
            $cateId = null;
            foreach ($objekt as $column_key => $value) {
                //built a data array for category.create
                $split = explode("-", $column_key);
                $category_key = $split[0];
                if ($split[1] == "id") {
                    $cateId = $value;
                }
            }

            if (isset($cateId)) {
                if (strpos($category_key, 'CATG') !== false) {
                    $p_param = [
                        'objID' => (int)$p_params['obj_id'],
                        'catgID' => $category_key,
                        'cateID' => $cateId
                    ];
                } else {
                    $p_param = [
                        'objID' => (int)$p_params['obj_id'],
                        'catsID' => $category_key,
                        'cateID' => $cateId
                    ];
                }

                switch ($category_key) {
                    case "C__CATG__SMVIEW_CABLE_WIRE":
                        $cable_api_model_wire = new cable_api_model_wire();
                        $return[] = $cable_api_model_wire->delete(["id" => $cateId]);
                        break;
                    case "C__CATG__SMVIEW_CABLE_PIN":
                        $isys_cmdb_dao_smview_cable_pin = isys_cmdb_dao_smview_cable_pin::instance(isys_application::instance()->database);

                        //search sibling pin
                        foreach ($isys_cmdb_dao_smview_cable_pin->get_listing_by_obj_id($p_params['obj_id']) as $item) {
                            if ($item["obj_in"]["id"] == $cateId) {
                                $sibling_id = $item["obj_out"]["id"];
                            }

                            if ($item["obj_out"]["id"] == $cateId) {
                                $sibling_id = $item["obj_in"]["id"];
                            }
                        }

                        $cable_api_model_pin = new cable_api_model_pin();
                        $return[] = $cable_api_model_pin->delete(["id" => $cateId]);
                        $return[] = $cable_api_model_pin->delete(["id" => $sibling_id]);
                        break;
                    default:
                        $return[] = $l_api_model_category->delete($p_param);
                        break;
                }
            }
        }
        return [
            'message'      => 'categories.delete.success',
            'response'     => $return,
            'current_date' => date('Y-m-d')
        ];
    }

    /**
     * Get ID of a dialog item.
     *
     * @param  string $property
     * @param  string $value
     * @return mixed|null
     * @throws isys_exception_database
     * @throws Exception
     */
    private function getDialogId($property, $value)
    {
        $language = isys_application::instance()->container->get('language');
        if ($property == "C__CATG__SMVIEW_CABLE-state_id") {
            $dao = new isys_cmdb_dao_smview_cable_user_state(isys_application::instance()->database);
            $all_states = $dao->get_objects();
            $states = [];
            foreach ($all_states as $s) {
                $name = $s["name"] . ': ' . $s["description"];
                $states[$name] = $s['id'];
            }
            return isset($states[$value]) ? $states[$value] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE-group_id") {
            $dao = new isys_cmdb_dao_smview_cable_group(isys_application::instance()->database);
            $group = $dao->get_object("description", $value);
            return isset($group) ? $group["id"] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE-admin_unit") {
            $dao = new isys_cmdb_dao_smview_cable_admin_unit(isys_application::instance()->database);
            $group = $dao->get_object("description", $value);
            return isset($group) ? $group["id"] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE-type_id") {
            $dao = new isys_cmdb_dao_smview_cable_type(isys_application::instance()->database);
            $group = $dao->get_object("description", $value);
            return isset($group) ? $group["id"] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE-is_section_cable") {
            switch ($value) {
                case $language->get("LC__UNIVERSAL__YES"):
                    return 1;
                case $language->get("LC__UNIVERSAL__NO"):
                    return 0;
                default:
                    return null;
            }
        } elseif ($property == "C__CATG__SMVIEW_CABLE-connection_type_id") {
            $dao = new isys_cmdb_dao_smview_cable_connection_type(isys_application::instance()->database);
            $group = $dao->get_object("description", $value);
            return isset($group) ? $group["id"] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE_WIRE-user_state_id") {
            $dao = new isys_cmdb_dao_smview_cable_user_state(isys_application::instance()->database);
            $group = $dao->get_object("name", $value);
            return isset($group) ? $group["id"] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE_WIRE-section_id") {
            $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
            $section = $dao->get_object_by_id($value)->get_row();
            return isset($section) ? $section["isys_obj__id"] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE_WIRE-route_id") {
            $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
            $route = $dao->get_object_by_id($value)->get_row();
            return isset($route) ? $route["isys_obj__id"] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE_WIRE-virtual_route_id") {
            $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
            $route = $dao->get_object_by_id($value)->get_row();
            return isset($route) ? $route["isys_obj__id"] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE_WIRE-state_out") {
            $states = isys_cmdb_dao_smview_cable_wire_pin::get_wire_states();
            return array_search($value, $states);
        } elseif ($property == "C__CATG__SMVIEW_CABLE_WIRE-state_in") {
            $states = isys_cmdb_dao_smview_cable_wire_pin::get_wire_states();
            return array_search($value, $states);
        } elseif ($property == "C__CATG__SMVIEW_CABLE_WIRE-reserved") {
            if ($value == $language->get("LC__UNIVERSAL__NO")) {
                return 0;
            } elseif ($value == $language->get("LC__UNIVERSAL__YES")) {
                return 1;
            }
        } elseif ($property == "C__CATG__SMVIEW_CABLE_WIRE-connection_type_id") {
            $dao = new isys_cmdb_dao_smview_cable_connection_type(isys_application::instance()->database);
            $group = $dao->get_object("name", $value);
            return isset($group) ? $group["id"] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE_WIRE-user_state_id") {
            $dao = new isys_cmdb_dao_smview_cable_user_state(isys_application::instance()->database);
            $group = $dao->get_object("description", $value);
            return isset($group) ? $group["id"] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE_SECTION-user_state_id") {
            $dao = new isys_cmdb_dao_smview_cable_user_state(isys_application::instance()->database);
            $all_states = $dao->get_objects();
            $states = [];
            foreach ($all_states as $s) {
                $name = $s["name"] . ': ' . $s["description"];
                $states[$name] = $s['id'];
            }
            return isset($states[$value]) ? $states[$value] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE_ROUTE-route_usage_id" || $property == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-route_usage_id") {
            $dao = new isys_cmdb_dao_smview_cable_route_usage(isys_application::instance()->database);
            if (strpos($value, '(') !== false) {
                $value = substr($value, 0, strpos($value, ' ('));
            }
            $group = $dao->get_object("name", $value);
            return isset($group) ? $group["id"] : null;
        } elseif ($property == "C__CATG__SMVIEW_CABLE_ROUTE-is_external" || $property == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-is_external") {
            if ($value == $language->get("LC__UNIVERSAL__NO")) {
                return 0;
            } elseif ($value == $language->get("LC__UNIVERSAL__YES")) {
                return 1;
            }
        } elseif ($property == "C__CATG__SMVIEW_CABLE_ROUTE-is_rented") {
            if ($value == $language->get("LC__UNIVERSAL__NO")) {
                return 0;
            } elseif ($value == $language->get("LC__UNIVERSAL__YES")) {
                return 1;
            }
        }
        return null;
    }
}
