<?php

use idoit\Component\Property\PropertyConfiguration;

/**
 * i-doit APi
 *
 * @package i-doit
 * @subpackage API
 * @author Dennis Stücken <dstuecken@synetics.de>
 * @copyright synetics GmbH
 * @license http://www.i-doit.com/license
 */

class isys_api_model_shd_category_info extends isys_api_model_cmdb_category implements isys_api_model_interface
{
    /**
     * Database component.
     *
     * @var isys_component_database
     */
    protected $m_dao;

    /**
     * Data formatting used in format methods
     *
     * @var array
     */
    protected $m_mapping = [];

    /**
     * Possible options and their parameters
     *
     * @var array
     */
    protected $m_options = [
        'read' => []
    ];

    /**
     * Constructor
     *
     * @param \isys_cmdb_dao $p_dao
     */
    public function __construct()
    {
        parent::__construct(new isys_cmdb_dao(isys_application::instance()->database));
    }

    /**
     * @param array $p_params
     * @return array
     */
    public function read($p_params)
    {
        $l_return = [];

        if ($this->isSpecialColumn($p_params["category"]) == true) {
            return $this->getSpecialColumn($p_params["category"]);
        }

        if (($l_cat = $this->prepare($p_params))) {
            if (!method_exists($l_cat, 'get_properties')) {
                throw new isys_exception_api('get_properties method does not exist for ' . get_class($l_cat), -32601);
            }

            $l_properties = [];
            $l_properties_tmp = $l_cat->get_properties();

            foreach ($l_properties_tmp as $l_propkey => $l_propdata) {
                $l_properties[$l_propkey] = [
                    'title'            => _L(@$l_propdata[C__PROPERTY__INFO]['title']),
                    C__PROPERTY__CHECK => [
                        C__PROPERTY__CHECK__MANDATORY => $l_propdata[C__PROPERTY__CHECK][C__PROPERTY__CHECK__MANDATORY]
                    ]
                ];

                $propertyParts = [C__PROPERTY__INFO, C__PROPERTY__DATA, C__PROPERTY__UI, C__PROPERTY__FORMAT];
                foreach ($propertyParts as $propertyPart) {
                    if (is_array($l_propdata[$propertyPart])) {
                        $l_properties[$l_propkey][$propertyPart] = $l_propdata[$propertyPart];
                    } elseif (is_object($l_propdata[$propertyPart]) && $l_propdata[$propertyPart] instanceof PropertyConfiguration) {
                        /** @var PropertyConfiguration $l_propdata [$propertyPart] */
                        $l_properties[$l_propkey][$propertyPart] = $l_propdata[$propertyPart]->toArray();
                    } elseif (\is_object($l_propdata[$propertyPart])) {
                        $l_properties[$l_propkey][$propertyPart] = (array)$l_propdata[$propertyPart];

                        // This is necessary since the keys will contain null-byte characters.
                        foreach ($l_properties[$l_propkey][$propertyPart] as $key => $value) {
                            if ($value === null) {
                                unset($l_properties[$l_propkey][$propertyPart][$key]);
                                continue;
                            }

                            if (is_array($value) && empty($value)) {
                                unset($l_properties[$l_propkey][$propertyPart][$key]);
                                continue;
                            }

                            $newKey = str_replace("\0*\0", '', $key);

                            if ($newKey === $key) {
                                continue;
                            }

                            $l_properties[$l_propkey][$propertyPart][$newKey] = $value;
                            unset($l_properties[$l_propkey][$propertyPart][$key]);
                        }
                    }
                }
            }
            return $l_properties;
        }
        return $l_return;
    }

    public function isSpecialColumn($catG)
    {
        return $catG == "C__CATG__SMVIEW_CABLE_PIN";
    }

    public function getSpecialColumn($catG)
    {
        $return = [];
        switch ($catG) {
            case "C__CATG__SMVIEW_CABLE_PIN":
                $return["in_description"]["title"] = "Eingang Beschreibung";
                $return["in_description"]["check"]["mandatory"] = false;
                $return["in_description"]["info"]["primary_field"] = false;
                $return["in_description"]["info"]["type"] = "text";
                $return["in_description"]["info"]["backward"] = false;
                $return["in_description"]["info"]["title"] = "LC__CMDB__CATG__SMVIEW_CABLE__DESCRIPTION";
                $return["in_description"]["info"]["description"] = "Description";

                $return["in_description"]["data"]["readonly"] = false;
                $return["in_description"]["data"]["type"] = "text";
                $return["in_description"]["data"]["index"] = false;
                $return["in_description"]["data"]["field"] = "isys_catg_smview_cable_pin_list__description";

                $return["in_description"]["ui"]["default"] = null;
                $return["in_description"]["ui"]["type"] = "text";
                $return["in_description"]["ui"]["params"] = ["p_nMaxLen" => 255];
                $return["in_description"]["ui"]["id"] = "C__CATG__SMVIEW_CABLE_DESCRIPTION";

                $return["in_scope"]["title"] = "Eingang Bereich";
                $return["in_scope"]["check"]["mandatory"] = false;
                $return["in_scope"]["info"]["primary_field"] = false;
                $return["in_scope"]["info"]["type"] = "text";
                $return["in_scope"]["info"]["backward"] = false;
                $return["in_scope"]["info"]["title"] = "LC__CMDB__CATG__SMVIEW_CABLE__DESCRIPTION";
                $return["in_scope"]["info"]["description"] = "Description";

                $return["in_scope"]["data"]["readonly"] = false;
                $return["in_scope"]["data"]["type"] = "text";
                $return["in_scope"]["data"]["index"] = false;
                $return["in_scope"]["data"]["field"] = "isys_catg_smview_cable_pin_list__description";

                $return["in_scope"]["ui"]["default"] = null;
                $return["in_scope"]["ui"]["type"] = "text";
                $return["in_scope"]["ui"]["params"] = ["p_nMaxLen" => 255];
                $return["in_scope"]["ui"]["id"] = "C__CATG__SMVIEW_CABLE_DESCRIPTION";

//                $return["out_description"]["title"] = "Ausgang Beschreibung";
//                $return["out_description"]["check"]["mandatory"] = false;
//                $return["out_description"]["info"]["primary_field"] = false;
//                $return["out_description"]["info"]["type"] = "text";
//                $return["out_description"]["info"]["backward"] = false;
//                $return["out_description"]["info"]["title"] = "LC__CMDB__CATG__SMVIEW_CABLE__DESCRIPTION";
//                $return["out_description"]["info"]["description"] = "Description";
//
//                $return["out_description"]["data"]["readonly"] = false;
//                $return["out_description"]["data"]["type"] = "text";
//                $return["out_description"]["data"]["index"] = false;
//                $return["out_description"]["data"]["field"] = "isys_catg_smview_cable_pin_list__description";
//
//                $return["out_description"]["ui"]["default"] = null;
//                $return["out_description"]["ui"]["type"] = "text";
//                $return["out_description"]["ui"]["params"] = ["p_nMaxLen" => 255];
//                $return["out_description"]["ui"]["id"] = "C__CATG__SMVIEW_CABLE_DESCRIPTION";
//
//                $return["out_scope"]["title"] = "Ausgang Bereich";
//                $return["out_scope"]["check"]["mandatory"] = false;
//                $return["out_scope"]["info"]["primary_field"] = false;
//                $return["out_scope"]["info"]["type"] = "text";
//                $return["out_scope"]["info"]["backward"] = false;
//                $return["out_scope"]["info"]["title"] = "LC__CMDB__CATG__SMVIEW_CABLE__PIN_OUT_SCOPE";
//                $return["out_scope"]["info"]["description"] = "Scope";
//
//                $return["out_scope"]["data"]["readonly"] = false;
//                $return["out_scope"]["data"]["type"] = "text";
//                $return["out_scope"]["data"]["index"] = false;
//                $return["out_scope"]["data"]["field"] = "isys_catg_smview_cable_pin_list__scope";
//
//                $return["out_scope"]["ui"]["default"] = null;
//                $return["out_scope"]["ui"]["type"] = "text";
//                $return["out_scope"]["ui"]["params"] = ["p_nMaxLen" => 255];
//                $return["out_scope"]["ui"]["id"] = "C__CATG__SMVIEW_CABLE_DESCRIPTION";

                break;
        }
        return $return;
    }
}
