<?php

/**
 * shd APi
 *
 * @package shd
 * @subpackage API
 * @author Maik Rhode <Maik.Rhode@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 */
class isys_api_model_shd_file implements isys_api_model_interface
{

    protected $smViewDir = "/upload/images/sm-view";

    /**
     * @param array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function read($p_params)
    {
        global $g_absdir;

        if (isset($p_params["path"]) == false) {
            throw new isys_exception_api('Parameter path missed', -32602);
        }

        $globalDir = $g_absdir . $this->smViewDir . $p_params["path"];

        //get name from path
        $a_name = explode("/", $p_params["path"]);
        $name = $a_name[count($a_name) - 1];

        $include_file = isset($p_params['include_file']) ? $p_params['include_file'] : null;
        return $this->getFileObject($globalDir, $name, $include_file);
    }

    /**
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function dir($p_params)
    {
        global $g_absdir;

        if (isset($p_params["path"]) == false) {
            throw new isys_exception_api('Parameter path missed', -32602);
        }

        $globalDir = $g_absdir . $this->smViewDir;
        if (isset($p_params["path"])) {
            $globalDir .= $p_params["path"];
        }
        return $this->getDirContents($globalDir);
    }

    /**
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function create($p_params)
    {
        if (! isset($p_params["path"])) {
            throw new isys_exception_api('Parameter path missed', -32602);
        }
        if (! isset($p_params["name"])) {
            throw new isys_exception_api('Parameter name missed', -32602);
        }
        if (! isset($p_params["type"])) {
            throw new isys_exception_api('Parameter name missed', -32602);
        }
        if (! isset($p_params["content"])) {
            $p_params["content"] = "";
        }

        $return = $p_params["type"] == "f" ? $this->upload($p_params) : $this->dirMk($p_params);

        $this->_handleMapScreenshot($p_params);

        return array(
            'message'      => 'file.create.success',
            'response'     => $return,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * @param  array $p_params
     * @return true
     */
    public function upload($p_params)
    {
        global $g_absdir;

        $path_upload = $g_absdir . $this->smViewDir;
        $path = $p_params["path"];
        $file = $p_params["name"];

        if (substr($path, 0, 1) == DIRECTORY_SEPARATOR) {
            $outputfile = $path_upload . $path;
        } else {
            $outputfile = $path_upload . DIRECTORY_SEPARATOR . $path;
        }

        //create Path
        mkdir($outputfile, 0777, true);

        //set DIRECTORY_SEPARATOR if is not Set
        if (substr($outputfile, strlen($outputfile) - 1, 1) != DIRECTORY_SEPARATOR) {
            $outputfile = $outputfile . DIRECTORY_SEPARATOR;
        }

        $ifp = fopen($outputfile . $file, "wb");
        fwrite($ifp, base64_decode($p_params["content"]));
        fclose($ifp);
        return true;
    }

    /**
     * @param  array $p_params
     * @return true
     */
    public function uploads($p_params)
    {
        global $g_absdir;

        $path_upload = $g_absdir . $this->smViewDir;
        foreach ($p_params["files"] as $file) {
            $path = $file["path"];
            $file_name = $file["name"];

            if (substr($path, 0, 1) == DIRECTORY_SEPARATOR) {
                $outputfile = $path_upload . $path;
            } else {
                $outputfile = $path_upload . DIRECTORY_SEPARATOR . $path;
            }

            //create Path
            mkdir($outputfile, 0777, true);

            //set DIRECTORY_SEPARATOR if is not Set
            if (substr($outputfile, strlen($outputfile) - 1, 1) != DIRECTORY_SEPARATOR) {
                $outputfile = $outputfile . DIRECTORY_SEPARATOR;
            }

            $ifp = fopen($outputfile . $file_name, "wb");
            fwrite($ifp, base64_decode($file["content"]));
            fclose($ifp);
        }
        return true;
    }

    /**
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function update($p_params)
    {
        if (! isset($p_params["path"])) {
            throw new isys_exception_api('Parameter path missed', -32602);
        }
        if (! isset($p_params["name"])) {
            throw new isys_exception_api('Parameter name missed', -32602);
        }
        if (! isset($p_params["type"])) {
            throw new isys_exception_api('Parameter name missed', -32602);
        }
        if (! isset($p_params["content"])) {
            $p_params["content"] = "";
        }

        $return = $p_params["type"] == "f" ? $this->upload($p_params) : false;

        $this->_handleMapScreenshot($p_params);

        return array(
            'message'      => 'file.update.success',
            'response'     => $return,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * Handle creation or update of a map screenshot.
     *
     * @param  array $p_params
     * @throws isys_exception_api
     */
    private function _handleMapScreenshot($p_params)
    {
        if ($p_params["path"] == '/maps' && $p_params["type"] == 'f') {
            // handle update of map screenshot
            try {
                smview_cmdb_dao_map::instance(isys_application::instance()->database)->createOrUpdateMapScreenshot($p_params["path"], $p_params["name"]);
            } catch (isys_exception_database $e) {
            } catch (isys_exception_filesystem $e) {
                throw new isys_exception_api('Could not provide map screenshot', -32602);
            }
        }
    }

    /**
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function move($p_params)
    {
        global $g_absdir;

        if (! isset($p_params["path_new"])) {
            throw new isys_exception_api('Parameter path_new missed', -32602);
        }
        if (! isset($p_params["path_old"])) {
            throw new isys_exception_api('Parameter name_old missed', -32602);
        }

        $path_upload = $g_absdir . $this->smViewDir;
        $path_new = $p_params["path_new"];
        $path_old = $p_params["path_old"];
        if (substr($path_old, 0, 1) == DIRECTORY_SEPARATOR) {
            $complete_path_old = $path_upload . $path_old;
        } else {
            $complete_path_old = $path_upload . DIRECTORY_SEPARATOR . $path_old;
        }

        if (substr($path_new, 0, 1) == DIRECTORY_SEPARATOR) {
            $complete_path_new = $path_upload . $path_new;
        } else {
            $complete_path_new = $path_upload . DIRECTORY_SEPARATOR . $path_new;
        }

        rename($complete_path_old, $complete_path_new);
        return array(
            'message'      => 'file.move.success',
            'response'     => true,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function delete($p_params)
    {
        global $g_absdir;

        if (isset($p_params["path"]) == false) {
            throw new isys_exception_api('Parameter path missed', -32602);
        }

        $path = $g_absdir. $this->smViewDir . $p_params["path"];
        $return = is_dir($path) ? $this->delTree($path) : unlink($path);
        return array(
            'message'      => 'file.delete.success',
            'response'     => $return,
            'current_date' => date('Y-m-d')
        );
    }

    private function delTree($dir)
    {
        $files = array_diff(scandir($dir), array('.','..'));
        foreach ($files as $file) {
            (is_dir("$dir/$file")) ? $this->delTree("$dir/$file") : unlink("$dir/$file");
        }
        return rmdir($dir);
    }

    /**
     * @param  array $p_params
     * @return boolean
     */
    private function dirMk($p_params)
    {
        global $g_absdir;

        $folderName = $p_params["name"];
        $path = $p_params["path"];
        $realpath = $g_absdir . $this->smViewDir;

        //add $path_upload
        if (substr($path, 0, 1) == DIRECTORY_SEPARATOR) {
            $path = $realpath . $path;
        } else {
            $path = $realpath . DIRECTORY_SEPARATOR . $path;
        }

        if (substr($folderName, 0, 1) == DIRECTORY_SEPARATOR) {
            $path = $path . $folderName;
        } else {
            $path = $path . DIRECTORY_SEPARATOR . $folderName;
        }
        return mkdir($path, 0777, true);
    }

    /**
     * @param  string $dir
     * @return array
     */
    private function getDirContents($dir)
    {
        $files = scandir($dir);
        foreach ($files as $value) {
            if ($value === "." || $value === "..") {
                continue;
            }

            $endString = substr($dir, strlen($dir) - strlen("images/sm-view/"), strlen("images/sm-view/"));
            if ($endString == "images/sm-view/" && $value == "maps") {
                continue;
            }

            $path = realpath($dir.DIRECTORY_SEPARATOR.$value);
            if (!is_dir($path)) {
                $file_results[] = $this->getFileObject($path, $value);
            } else {
                $path_results[] = $this->getFileObject($path, $value);
            }
        }

        //sort for case insensitive
        uasort($path_results, function ($a, $b) {
            return strnatcasecmp($a["name"], $b["name"]);
        });

        uasort($file_results, function ($a, $b) {
            return strnatcasecmp($a["name"], $b["name"]);
        });

        if ($path_results === null) {
            $path_results = [];
        }

        if ($file_results === null) {
            $file_results = [];
        }

        return array_merge($path_results, $file_results);
    }

    /**
     * @param  string $realPath
     * @param  string $name
     * @param  boolean|null $include_file
     * @return array
     */
    private function getFileObject($realPath, $name, $include_file = null)
    {
        $fileObject = [];
        $fileObject["name"] = $name;
        if (is_dir($realPath)) {
            $fileObject["type"] = "d";
            $fileObject["hash"] = "";
            $fileObject["createDate"] = filemtime($realPath);
            $fileObject["size"] = "";
        } else {
            $fileObject["type"] = "f";
            $fileObject["hash"] = hash_file('md5', $realPath);
            $fileObject["createDate"] = filemtime($realPath);
            $fileObject["size"] = filesize($realPath);
        }

        $path = substr($realPath, strrpos($realPath, $this->smViewDir) + strlen($this->smViewDir));
        $path = substr($path, 0, strrpos($path, $name) - 1);
        if ($path === false) {
            $path = "/";
        }

        $fileObject["path"] = $path;
        $fileObject["content"] = $include_file == true ? $this->_get_file($path . "/". $name) : "";
        return $fileObject;
    }

    /**
     * Get base64 encoded file.
     *
     * @param  string $filename
     * @return string
     */
    private function _get_file($filename)
    {
        global $g_absdir;

        $path =  $g_absdir . $this->smViewDir . $filename ;
        $content = file_get_contents($path);
        return base64_encode($content);
    }
}
