<?php

use idoit\Module\Api\Exception\ValidationException;

/**
 * SHD API.
 *
 * @package shd
 * @subpackage API
 * @author Maik Rhode <Maik.Rhode@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 */
class isys_api_model_shd_filter implements isys_api_model_interface
{

    /**
     * @param  array $p_params
     * @return array
     * @throws Exception
     */
    public function read($p_params)
    {
        global $g_comp_session;
        $userId = $g_comp_session->get_user_id();
        $p_params["user_id"] = $userId;

        $l_filter = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->get_filter($p_params);
        return $l_filter;
    }

    /**
     * @param  $p_params
     * @return array
     * @throws isys_exception_dao
     */
    public function create($p_params)
    {
        global $g_comp_session;
        $userId = $g_comp_session->get_user_id();

        $p_params["user_id"] = $userId;

        $columns = serialize($p_params["columns"]);
        $condition = serialize($p_params["condition"]);
        $category = serialize($p_params["category"]);

        $default = $p_params["default"] ?? 0;
        $context = $p_params["context"] ?? "";
        $context_value = $p_params["context_value"] ?? "";

        $l_return_id = smview_cmdb_dao_filter::instance(isys_application::instance()->database)
            ->createFilter(
                $p_params["title"],
                $p_params["obj_type_id"],
                $p_params["obj_id"],
                $p_params["display_rows"],
                $p_params["page"],
                $p_params["user_id"],
                0,
                1,
                $p_params["locked_column"],
                $columns,
                $condition,
                $category,
                $default,
                $context,
                $context_value
            );

        return array(
            'message'      => 'filter.create.success',
            'response'     => ['filter_id' =>$l_return_id],
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * @param $p_params
     * @return array
     */
    public function update($p_params)
    {
        global $g_comp_session;
        $userId = $g_comp_session->get_user_id();

        $p_params["user_id"] = $userId;
        $columns = isset($p_params["columns"]) == true ? serialize($p_params["columns"]) : serialize([]);
        $condition = isset($p_params["condition"]) == true ? serialize($p_params["condition"]) : serialize([]);
        $category = isset($p_params["category"]) == true ? serialize($p_params["category"]) : serialize([]);
        $default = $p_params["default"] ?? 0;

        $l_report_query = smview_cmdb_dao_filter::instance(isys_application::instance()->database)
            ->updateFilter(
                $p_params["filter_id"],
                $p_params["title"],
                $p_params["obj_type_id"],
                $p_params["obj_id"],
                $p_params["display_rows"],
                $p_params["page"],
                $p_params["user_id"],
                $p_params["display_relations"],
                $p_params["empty_values"],
                $p_params["locked_column"],
                $columns,
                $condition,
                $category,
                $default,
                $p_params["context"],
                $p_params["context_value"]
            );

        return array(
            'message'      => 'filter.update.success',
            'response'     => $l_report_query,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * @param  $p_params
     * @return array
     * @throws ValidationException
     * @throws isys_exception_api
     * @throws isys_exception_auth
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function duplicate($p_params)
    {
        if (isset($p_params["person_id"]) == false && isset($p_params["person_group_id"]) == false) {
            throw new isys_exception_api('Parameter person_id or person_group_id missed', -32602);
        }

        if (isset($p_params["person_id"]) == true && isset($p_params["person_group_id"]) == true) {
            throw new isys_exception_api("Parameter person_id and person_group_id could not set both", -32602);
        }

        $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao(isys_application::instance()->database));
        global $g_comp_session;
        $userId = $g_comp_session->get_user_id();

        $p_params["user_id"] = $userId;

        $l_filters = [];
        if (isset($p_params["filter_id"])) {
            $l_filters[] = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->get_filter($p_params);
        } else {
            $l_filters = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->get_all_filters($p_params);
        }

        foreach ($l_filters as $f_filter) {
            $columns = serialize($f_filter["shd_filter__columns"]);
            $condition = serialize($f_filter["shd_filter__condition"]);
            $category = serialize($f_filter["shd_filter__category"]);

            if (isset($p_params["person_id"]) == true) {
                $l_dao_user = isys_component_dao_user::instance(isys_application::instance()->database);
                $l_userarray = $l_dao_user->get_user($p_params["person_id"], null)
                    ->__to_array();

                if ($l_userarray == []) {
                    $l_return_id = false;
                } else {
                    $userId = $p_params["person_id"];

                    if($f_filter["shd_filter__default"] == "1"){
                        $this->check_default(["obj_type_id"=> $f_filter["shd_filter__obj_type_id"],"user_id"=> $userId, "default" => 1]);
                    }

                    $l_return_id = smview_cmdb_dao_filter::instance(isys_application::instance()->database)
                        ->createFilter(
                            $f_filter["shd_filter__title"],
                            $f_filter["shd_filter__obj_type_id"],
                            $f_filter["shd_filter__obj_id"],
                            $f_filter["shd_filter__display_rows"],
                            $f_filter["shd_filter__page"],
                            $userId,
                            0,
                            1,
                            $f_filter["shd_filter__locked_column"],
                            $columns,
                            $condition,
                            $category,
                            $f_filter["shd_filter__default"],
                            $f_filter["shd_filter__context"],
                            $f_filter["shd_filter__context_value"]
                        );
                }
            } elseif (isset($p_params["person_group_id"]) == true) {
                $p_param = [
                    'objID'    => $p_params["person_group_id"],
                    'category' => "C__CATS__PERSON_GROUP_MEMBERS"
                ];

                $l_categoryDatas = $l_api_model_category->read($p_param);
                foreach ($l_categoryDatas as $item) {
                    $userId = $item["connected_object"]["id"];
                    if ($f_filter["shd_filter__default"] == "1"){
                        $this->check_default(["obj_type_id"=> $f_filter["shd_filter__obj_type_id"],"user_id"=> $userId, "default" => 1]);
                    }
                    $l_return_id = smview_cmdb_dao_filter::instance(isys_application::instance()->database)
                        ->createFilter(
                            $f_filter["shd_filter__title"],
                            $f_filter["shd_filter__obj_type_id"],
                            $f_filter["shd_filter__obj_id"],
                            $f_filter["shd_filter__display_rows"],
                            $f_filter["shd_filter__page"],
                            $userId,
                            0,
                            1,
                            $f_filter["shd_filter__locked_column"],
                            $columns,
                            $condition,
                            $category,
                            $f_filter["shd_filter__default"],
                            $f_filter["shd_filter__context"],
                            $f_filter["shd_filter__context_value"]
                        );
                }
            } else {
                $l_return_id = false;
            }
        }
        return array(
            'message'      => 'filter.duplicate.success',
            'response'     => $l_return_id,
            'current_date' => date('Y-m-d')
        );
    }

    private function check_default($p_params){
        $l_filters = smview_cmdb_dao_filter::instance(isys_application::instance()->database)
            ->get_all_filters($p_params);

        foreach ($l_filters as $filter) {
            //Wenn es default ist soll es gelöscht werden.
            if($filter["shd_filter__default"] == "1"){
                $this->delete(["filter_id"=> $filter["shd_filter__id"]]);
            }
        }
    }

    /**
     * @param  $p_params
     * @return array
     * @throws isys_exception_dao
     */
    public function delete($p_params)
    {
        $l_return = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->deleteFilter($p_params["filter_id"]);
        return array(
            'message'      => 'filter.delete.success',
            'response'     => $l_return,
            'current_date' => date('Y-m-d')
        );
    }

    public function query($p_params)
    {
        $isys_api_model_shd_query = new isys_api_model_shd_query();
        return $isys_api_model_shd_query->read($p_params);
    }
}
