<?php

/**
 * shd API Insight
 *
 * @package shd
 * @subpackage API
 * @author Maik Rhode <maik.rhode@shd-online.de>
 */
class isys_api_model_shd_insight implements isys_api_model_interface
{

    /**
     * Insight DAO.
     *
     * @var smview_cmdb_dao_map|null
     */
    private $_insight_dao;

    public function __construct()
    {
        $this->_insight_dao = smview_cmdb_dao_insightprofil::instance(isys_application::instance()->database);
    }

    /**
     * Read Insight Entry
     * @author Maik Rhode <maik.rhode@shd-online.de>
     * $p_param structure:
     *     array (
     *         'insightid'       =>      integer
     *     )
     *
     * @return array of Insight.
     */
    public function read($p_params)
    {
        try {
            $p_params = array_change_key_case($p_params, CASE_LOWER);
            return $this->_insight_dao->getInsightProfil($p_params["insightid"]);
        } catch (Exception $ex) {
            var_dump($ex->getMessage());
        }
    }

    /**
     * Create Insight Entry
     *
     * @param array $p_params
     *
     * $p_param structure:
     *     array (
     *         'insightObject'       =>   array(
     *              'insightTitle'   =>     string,
     *              'toolTipEnabled' =>     bool,
     *              'ObjecttypeConfiguration' => array(
     *                  'objecttypeconst' => string
     *                  'style'           => string
     *                  'width'           => string
     *                  'linecount'           => string
     *     )
     * @return array
     * @throws isys_exception_dao
     */
    public function create($p_params)
    {
        $insightObject = $p_params["insightObject"];

        $l_Insight_id = $this->_insight_dao
            ->createInsightProfil(
                $insightObject["insightTitle"],
                $insightObject["toolTipEnabled"],
                $insightObject["layoutType"],
                $insightObject["legendIds"],
                $insightObject["descriptionEnabled"],
                $insightObject["expandIds"],
                $insightObject["lvl"]
            );

        foreach ($insightObject["objectTypeConfiguration"] as $objectTypeConfiguration) {
            $l_ObjectTypeConfiguration_id = $this->_insight_dao
                ->createObjectTypeConfiguration(
                    $objectTypeConfiguration["objecttypeconst"],
                    $objectTypeConfiguration["style"],
                    $objectTypeConfiguration["width"],
                    $objectTypeConfiguration["linecount"],
                    $l_Insight_id
                );

            foreach ($objectTypeConfiguration["LineProperties"] as $lineProperties) {
                $this->_insight_dao
                    ->createLineProperties(
                        $lineProperties["category"],
                        $lineProperties["property"],
                        $lineProperties["backgroundcolor"],
                        $lineProperties["fontcolor"],
                        $lineProperties["bold"],
                        $lineProperties["italic"],
                        $lineProperties["underline"],
                        $lineProperties["index"],
                        $l_ObjectTypeConfiguration_id
                    );
            }
        }

        return array(
            'message'      => 'insight.create.success',
            'response'     => $l_Insight_id,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * Update Insight Entry
     *
     * @param array $p_params
     *
     * $p_param structure:
     *     array (
     *         'insightObject'       =>   array(
     *              'insightTitle'   =>     string,
     *              'toolTipEnabled' =>     bool,
     *              'ObjecttypeConfiguration' => array(
     *                  'objecttypeconst' => string
     *                  'style'           => string
     *                  'width'           => string
     *                  'linecount'           => string
     *     )
     * @return array
     * @throws isys_exception_dao
     */
    public function update($p_params)
    {
        $insightObject = $p_params["insightObject"];

        $l_return_id = $this->_insight_dao
            ->updateInsightProfil(
                $insightObject["insightId"],
                $insightObject["insightTitle"],
                $insightObject["tooltipEnabled"],
                $insightObject["layoutType"],
                $insightObject["legendIds"],
                $insightObject["descriptionEnabled"],
                $insightObject["expandIds"],
                $insightObject["lvl"]
            );

        foreach ($insightObject["objectTypeConfiguration"] as $objectTypeConfiguration) {
            if (isset($objectTypeConfiguration["objecttypeconfigurationId"]) == true && is_numeric($objectTypeConfiguration["objecttypeconfigurationId"])) {
                $l_ObjectTypeConfiguration_id = $this->_insight_dao
                    ->updateObjectTypeConfiguration(
                        $objectTypeConfiguration["objecttypeconfigurationId"],
                        $objectTypeConfiguration["objecttypeconst"],
                        $objectTypeConfiguration["style"],
                        $objectTypeConfiguration["width"],
                        $objectTypeConfiguration["linecount"],
                        $insightObject["insightId"]
                    );
            } else {
                $l_ObjectTypeConfiguration_id = $this->_insight_dao
                    ->createObjectTypeConfiguration(
                        $objectTypeConfiguration["objecttypeconst"],
                        $objectTypeConfiguration["style"],
                        $objectTypeConfiguration["width"],
                        $objectTypeConfiguration["linecount"],
                        $insightObject["insightId"]
                    );
            }

            $l_objectTypeConfigurationIds [] = $l_ObjectTypeConfiguration_id;

            $l_LineProperties = [];
            foreach ($objectTypeConfiguration["LineProperties"] as $lineProperties) {
                if (isset($lineProperties["linepropertyId"]) == true && is_numeric($lineProperties["linepropertyId"])) {
                    $l_LineProperties[] = $this->_insight_dao
                        ->updateLineProperties(
                            $lineProperties["linepropertyId"],
                            $lineProperties["category"],
                            $lineProperties["property"],
                            $lineProperties["backgroundcolor"],
                            $lineProperties["fontcolor"],
                            $lineProperties["bold"],
                            $lineProperties["italic"],
                            $lineProperties["underline"],
                            $lineProperties["index"],
                            $l_ObjectTypeConfiguration_id
                        );
                } else {
                    $l_LineProperties[] = $this->_insight_dao
                        ->createLineProperties(
                            $lineProperties["category"],
                            $lineProperties["property"],
                            $lineProperties["backgroundcolor"],
                            $lineProperties["fontcolor"],
                            $lineProperties["bold"],
                            $lineProperties["italic"],
                            $lineProperties["underline"],
                            $lineProperties["index"],
                            $l_ObjectTypeConfiguration_id
                        );
                }
            }

            $this->_insight_dao
                ->deleteLinePropertiesByObjectTypeConfiguration($l_ObjectTypeConfiguration_id, $l_LineProperties);
        }

        $this->_insight_dao
            ->deleteObjectTypeConfigurationByInsight($insightObject["insightId"], $l_objectTypeConfigurationIds);
        return array(
            'message'      => 'insight.update.success',
            'response'     => $l_return_id,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * Delete Insight Entry
     *
     * @param array $p_params
     *
     * $p_param array (
     *         'insightId'       =>      'integer',
     *     )
     * @return array
     * @throws isys_exception_dao
     */
    public function delete($p_params)
    {
        $p_params = array_change_key_case($p_params, CASE_LOWER);
        $this->_insight_dao->deleteLineProperties($p_params["insightid"]);
        $this->_insight_dao->deleteObjectTypeConfiguration($p_params["insightid"]);
        $this->_insight_dao->deleteInsightProfil($p_params["insightid"]);
        return array(
            'message'      => 'insight.delete.success',
            'response'     => true,
            'current_date' => date('Y-m-d')
        );
    }
}
