<?php

/**
 * shd APi
 *
 * @package shd
 * @subpackage API
 * @author Maik Rhode <Maik.Rhode@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 */
class isys_api_model_shd_ldap implements isys_api_model_interface
{

    /**
     * @param array $p_params
     * @return array
     */
    public function readServer($p_params)
    {
        global $g_comp_database;

        $l_ldap = new isys_ldap_dao($g_comp_database);
        return $l_ldap->get_active_servers($p_params["server_id"])->get_row();
    }


    /**
     * @return array
     */
    public function readAllServer()
    {
        global $g_comp_database;

        $l_ldap = new isys_ldap_dao($g_comp_database);
        $return = [];

        $res = $l_ldap->get_active_servers();
        while ($l_data = $res->get_row()) {
            $return[] = $l_data;
        }
        return $return;
    }

    /**
     * @param  $p_params
     * @return boolean|int
     * @throws isys_exception_dao
     */
    public function createServer($p_params)
    {
        global $g_comp_database;

        $l_ldap = new isys_ldap_dao($g_comp_database);
        return $l_ldap->create_server(
            $p_params["directory_type"],
            $p_params["host"],
            $p_params["port"],
            $p_params["user_dn"],
            $p_params["pass"],
            $p_params["user_search"],
            $p_params["group_search"],
            $p_params["filter"],
            $p_params["active"]
        );
    }

    /**
     * @param  array $p_params
     * @return boolean
     * @throws isys_exception_dao
     */
    public function updateServer($p_params)
    {
        global $g_comp_database;

        $l_ldap = new isys_ldap_dao($g_comp_database);
        return $l_ldap->save_server(
            $p_params["directory_type"],
            $p_params["host"],
            $p_params["port"],
            $p_params["user_dn"],
            $p_params["pass"],
            $p_params["user_search"],
            $p_params["group_search"],
            $p_params["filter"],
            $p_params["active"],
            30,
            0,
            0,
            3,
            $p_params["server_id"],
            null
        );
    }

    /**
     * @param array $p_params
     */
    public function deleteServer($p_params)
    {
        global $g_comp_database;

        $l_ldap = new isys_ldap_dao($g_comp_database);
        $l_ldap->delete_server($p_params["server_id"]);
    }

    /**
     * @param array $p_params
     * @return array
     */
    public function readDirectory($p_params)
    {
        global $g_comp_database;

        $l_ldap = new isys_ldap_dao($g_comp_database);
        return $l_ldap->get_ldap_types($p_params["directory_id"])->get_row();
    }

    /**
     * @return array
     */
    public function readAllDirectory()
    {
        global $g_comp_database;

        $l_ldap = new isys_ldap_dao($g_comp_database);
        $return = [];

        $res = $l_ldap->get_ldap_types();
        while ($l_data = $res->get_row()) {
            $return[] = $l_data;
        }
        return $return;
    }

    /**
     * @param  array $p_params
     * @return boolean
     */
    public function updateDirectory($p_params)
    {
        global $g_comp_database;

        $l_ldap = new isys_ldap_dao($g_comp_database);
        return $l_ldap->save_ldap_directory($p_params["title"], $p_params["const"], $p_params["mapping"], $p_params["directory_id"]);
    }

    /**
     * Read data.
     *
     * @param  array $p_params Parameters (depends on data method)
     */
    public function read($p_params)
    {
        // TODO: Implement read() method.
    }
}
