<?php

/**
 * i-doit APi
 *
 * @package i-doit
 * @subpackage API
 * @author Dennis Stücken <dstuecken@synetics.de>
 * @copyright synetics GmbH
 * @license http://www.i-doit.com/license
 */
final class isys_api_model_shd_map implements isys_api_model_interface
{

    private $circle = 0;

    /**
     * The caching instance for livestatus objects.
     *
     * @var  isys_cache_memcache
     */
    private $m_cache_status;

    private $m_cache_events;

    private $m_monitored_host;

    /**
     * Map DAO.
     *
     * @var smview_cmdb_dao_map|null
     */
    private $_map_dao;

    function __construct()
    {
        try {
            $this->m_cache_status = new isys_cache_smview_memcached();
            $this->m_cache_status->ns('map_status');

            $this->m_cache_events = new isys_cache_smview_memcached();
            // this namespace is also used by:
            //      isys_api_model_shd_map_connection.class.php,
            //      isys_api_model_shd_map_object.class.php,
            //      isys_api_model_shd_maps.class.php
            //      isys_api_model_shd_map_lineproperty.class.php
            $this->m_cache_events->ns('map_events');

            $this->m_monitored_host = smview_cmdb_dao_status::instance(isys_application::instance()->database)->getMonitoredHosts();
        } catch (isys_exception_cache $ex) {
        }
        $this->_map_dao = smview_cmdb_dao_map::instance(isys_application::instance()->database);
    }

    /**
     * Read Map
     *
     * @author Maik Rhode <maik.rhode@shd-online.de>
     * @param array $p_params
     *     array (
     *         'map_id'       =>      'integer',
     *     )
     * @return array
     * @throws isys_exception_api
     */
    public function read($p_params)
    {
        if (isset($p_params["map_id"]) == false) {
            throw new isys_exception_api('Parameter map_id missed', -32602);
        }
        return $this->_map_dao->getMap($p_params["map_id"]);
    }

    /**
     * Create Map Entry
     *
     * @author Maik Rhode <maik.rhode@shd-online.de>
     * @param array $p_params
     *     array (
     *         'title'       =>      'name der Map',
     *         'user_id'     =>      'BenutzerId',
     *     )
     * @return array (
     *         'message'       =>      'name der Map',
     *         'response'      =>      'map_id',
     *         'current_date'  =>      'BenutzerId',
     *     )
     */
    public function create($p_params)
    {
        global $g_comp_session;

        $userID = isset($p_params["user_id"]) == true ? $p_params["user_id"] : $g_comp_session->get_user_id();
        $provide_screenshot = $p_params['provide_screenshot'] ?? null;
        $page_size_preview_mode = $p_params['page_size_preview_mode'] ?? null;
        $type = $p_params['type'] ?? null;

        $l_return_id = $this->_map_dao->createMap($p_params["title"], $userID, $provide_screenshot, $type, $page_size_preview_mode);
        $this->setEvent($l_return_id, "create");
        return $this->_getSuccessMessage(__FUNCTION__, $l_return_id);
    }

    /**
     * Update Map Entry
     *
     * @author Maik Rhode <maik.rhode@shd-online.de>
     * @param array $p_params
     *     array (
     *         'title'       =>      'name der Map',
     *         'user_id'     =>      'BenutzerId',
     *         'provide_screenshot' => 'Screenshot im i-doit bereitstellen',
     *         'page_size_preview_mode' => 'Seiten-Vorschau Modus',
     *     )
     * @return array (
     *         'message'       =>      'name der Map',
     *         'response'      =>      'BenutzerId',
     *         'current_date'  =>      'BenutzerId',
     *     )
     */
    public function update($p_params)
    {
        $this->_map_dao->updateMap($p_params);
        $this->setEvent($p_params["map_id"], "update");
        return $this->_getSuccessMessage(__FUNCTION__);
    }

    /**
     * Delete Map Entry
     *
     * @author Maik Rhode <maik.rhode@shd-online.de>
     * @param array $p_params
     *     array (
     *         'map_id'      =>      'Id der Map',
     *     )
     * @return array (
     *         'message'       =>      'name der Map',
     * 'response'      =>      'BenutzerId',
     * 'current_date'  =>      'BenutzerId',
     * )
     * @throws isys_exception_api
     */
    public function delete($p_params)
    {
        global $g_absdir;

        if (isset($p_params["map_id"]) == false) {
            throw new isys_exception_api('Parameter map_id missed', -32602);
        }

        $this->_map_dao->deleteMap($p_params["map_id"]);
        $this->_map_dao->deleteMapObjectByMapId($p_params["map_id"]);
        $this->_map_dao->deleteMapConnectionByMapId($p_params["map_id"]);

        //delete preview picture
        $path = $g_absdir . "/upload/images/sm-view/maps/" . $p_params["map_id"] . ".jpg";
        unlink($path);

        $this->setEvent($p_params["map_id"], "delete");
        return $this->_getSuccessMessage(__FUNCTION__);
    }

    public function duplicate($p_params)
    {
        if (isset($p_params["map_id"]) == false) {
            throw new isys_exception_api('Parameter map_id missed', -32602);
        }

        $l_map = $this->_map_dao->getMap($p_params["map_id"]);

        //ändern des map Titels
        if (isset($p_params["map_title"]) == false) {
            if (strpos($l_map["title"], "(") === false) {
                $map_title = $l_map["title"] . " (1)";
            } else {
                $l_count_number = substr($l_map["title"], strpos($l_map["title"], "(") + 1, strpos($l_map["title"], ")") - strpos($l_map["title"], "(") - 1);
                $l_count_number++;

                $map_title = substr($l_map["title"], 0, strpos($l_map["title"], "(")) . " (" . $l_count_number . ")";
            }
        } else {
            $map_title = $p_params["map_title"];
        }

        $provide_screenshot = $p_params['provide_screenshot'] ?? null;
        $page_size_preview_mode = $p_params['page_size_preview_mode'] ?? null;
        $type = $p_params['type'] ?? null;
        $l_duplicateMapId = $this->_map_dao->createMap($map_title, $l_map["isys_cats_person_list__id"], $provide_screenshot, $type, $page_size_preview_mode);

        $l_mapObjects = $this->_map_dao->getMapObjectByMap($p_params["map_id"]);
        foreach ($l_mapObjects as $mapObject) {
            $this->_map_dao->duplicateMapObject($mapObject, $l_duplicateMapId);
        }

        $l_mapConnections = $this->_map_dao->getMapConnectionByMap($p_params["map_id"]);
        foreach ($l_mapConnections as $mapConnection) {
            $this->_map_dao->duplicateMapConnection($mapConnection, $l_duplicateMapId);
        }

        $isys_api_model_shd_map_object = new isys_api_model_shd_map_object();
        $isys_api_model_shd_map_object->updateLayerIndex(["map_id" => $l_duplicateMapId]);

        $this->setEvent($p_params["map_id"], "create");
        return $this->_getSuccessMessage(__FUNCTION__, $l_duplicateMapId);
    }

    public function status($p_params)
    {
        if (isset($p_params["map_id"]) == false) {
            throw new isys_exception_api('Parameter map_id missed', -32602);
        }
        return $this->getMapStatus($p_params["map_id"], isset($p_params["force_status"]));
    }

    public function import($p_params)
    {
        try {
            set_time_limit(600);
            $map_string = realpath(".." . "/upload/images/sm-view/" . $p_params["map_path"]);

            //todo fixing the CSV-path

            $return = [];
            $count = 0;
            $files = scandir($map_string);
            foreach ($files as $fileName) {
                $count++;

                if ($fileName === "." || $fileName === "..") {
                    continue;
                }

                if ($fileName === "Maps.csv") {
                    $targetFile = $map_string . '//' . $fileName;

                    $Header_array = [];
                    $maps = [];
                    $data = [];
                    $fp = fopen($targetFile, 'rb');
                    while (!feof($fp)) {
                        if ($Header_array === []) {
                            $Header_array = fgetcsv($fp, 0, ";");
                        }

                        foreach (fgetcsv($fp, 0, ";") as $key => $value) {
                            $data[$Header_array[$key]] = utf8_encode($value);
                        }

                        $maps[$data["Alias"]] = $data;
                    }

                    $return["maps"] = $maps;
                } elseif (strpos($fileName, 'Connections') !== false) {
                    $map_name = str_replace(".csv", "", $fileName);
                    $map_name = str_replace("_Connections", "", $map_name);

                    $targetFile = $map_string . '//' . $fileName;

                    $Header_array = [];
                    $maps = [];
                    $data = [];
                    $fp = fopen($targetFile, 'rb');
                    while (!feof($fp)) {
                        if ($Header_array === []) {
                            $Header_array = fgetcsv($fp, 0, ";");
                        }

                        foreach (fgetcsv($fp, 0, ";") as $key => $value) {
                            $data[$Header_array[$key]] = utf8_encode($value);
                        }

                        $maps[$data["Alias"]] = $data;
                    }
                    $return["maps"][$map_name]["Connections"] = $maps;
                } else {
                    $map_name = str_replace(".csv", "", $fileName);
                    $targetFile = $map_string . '//' . $fileName;

                    $Header_array = [];
                    $maps = [];
                    $data = [];
                    $fp = fopen($targetFile, 'rb');
                    while (!feof($fp)) {
                        if ($Header_array === []) {
                            $Header_array = fgetcsv($fp, 0, ";");
                        }

                        foreach (fgetcsv($fp, 0, ";") as $key => $value) {
                            if ($Header_array[$key] === "imagePath") {
                                if (substr($value, 0, 1) === "/") {
                                    //delete first char
                                    $value = substr($value, 1, strlen($value) - 1);
                                    $data[$Header_array[$key]] = utf8_encode($value);
                                }

                                if (!isset($data[$Header_array[$key]])) {
                                    $data[$Header_array[$key]] = utf8_encode($value);
                                }
                            } elseif ($Header_array[$key] === "icon") {
                                $data["imagePath"] = utf8_encode($value);
                            } else {
                                $data[$Header_array[$key]] = utf8_encode($value);
                            }
                        }
                        $maps[$data["Alias"]] = $data;
                    }
                    $return["maps"][$map_name]["objects"] = $maps;
                }
            }
        } catch (Exception $exception) {
        }

        $this->truncateMaps();
        return $this->saveMap($return);
    }

    private function saveMap($map_array)
    {
        global $g_comp_session;

        $userID = $g_comp_session->get_user_id();
        foreach ($map_array["maps"] as $map_key => $map) {
            $map_id = $this->_map_dao->createMap($map["Name"], $userID);
            $map_array["maps"][$map_key]["map_id"] = $map_id;
        }

        foreach ($map_array["maps"] as $map_key => $map) {
            foreach ($map["objects"] as $map_object_key => $map_object) {
                $map_object_obj_id = "";
                $map_object_array = [
                    "shd_map__id"     => $map["map_id"],
                    "position"        => $map_object["xCoordinate"] . "," . $map_object["yCoordinate"],
                    "width"           => $map_object["width"],
                    "height"          => $map_object["height"],
                    "title"           => $map_object["name"],
                    "labelposition"   => $map_object["labelPosition"],
                    "layerindex"      => $map_object["layerIndex"],
                    "textvisible"     =>  ($map_object["isTextVisible"] == 'true') ? '1' : '0',
                    "iconvisible"     => ($map_object["isIconVisible"] == 'true') ? '1' : '0',
                    "realizertype"    => $map_object["realizerType"],
                    "shapetype"       => $map_object["shapeType"],
                    "fillcolor"       => $map_object["color1"],
                    "fillcolor2"      => $map_object["color2"],
                    "iscolorgradient" => "0",
                    "linecolor"       => $map_object["lineColor"],
                    "labellinecolor"  => $map_object["labelLineColor"],
                    "textcolor"       => $map_object["textColor"],
                    "font"            => $map_object["font"],
                    "textalignment"   => $map_object["textAlignment"],
                    "locked"          => $map_object["isLocked"],
                ];

                //todo mirror_namespace
                //                "mirror_namespace": "cwebbzlstdgssm1||cwebbzlstomsvs2|SNMP_OMS_NODE_130_OMS_10"
//                "mirror_namespace": "|||Applikationen-155173818",
//                "mirror_namespace": "SM-BOX5-TRIAL||bsm-server|IT-Servicemanagement [Geschaeftsprozesse-256847474]",
//                "mirror_namespace": "SM-BOX5-TRIAL||SRV-DD-Linux|",
//                "mirror_namespace": "staging-smbox5||Site1.host1|",
//                "mirror_namespace": "|||Business-Services-359754286",
//                "mirror_namespace": "|||Firewall-1893772845",
//                "mirror_namespace": "|||Stromversorgung-1909086843",
                $mirror_namespace = explode("|", $map_object["mirror_namespace"]);
                if (count($mirror_namespace) > 1 && !$mirror_namespace[0] == "") {
                    //monitoring
                    foreach ($this->m_monitored_host as $monitored_host) {
//                         "isys_monitoring_hosts__title" -> "Staging-box"
//                         "isys_catg_monitoring_list__host_name" -> "Site1.zu.Site2"
                        if ($monitored_host["isys_monitoring_hosts__title"] == $mirror_namespace[0]
                            && $monitored_host["isys_catg_monitoring_list__host_name"] == $mirror_namespace[2]) {
                            $map_object_obj_id = $monitored_host["isys_obj__id"];
                            if ($mirror_namespace[3] != "") {
                                $map_object_array["livestatus_services"][] = $mirror_namespace[3];
                            }
                        }
                    }
                }

                $map_object_array["isys_obj__id"] = !$map_object_obj_id == "" ? $map_object_obj_id : "";

                $map_id = "";
                $map_name = str_replace("map://", "", $map_object["link"]);
                foreach ($map_array["maps"] as $f_map_key => $f_map) {
                    if ($f_map_key == $map_name) {
                        $map_id = $f_map["map_id"];
                    }
                }
                $map_object_array["link"] = $map_id;

                if (!$map_object["imagePath"] == "") {
                    $map_object_array["imagepath"] = isys_helper_link::get_base() . "/upload/images/sm-view/" . $map_object["imagePath"];
                } else {
                    $map_object_array["imagepath"] = "";
                }

                $isys_api_model_shd_map_object = new isys_api_model_shd_map_object();
                $return = $isys_api_model_shd_map_object->create($map_object_array);

                $mapObject_id = $return["response"];
                $map_array["maps"][$map_key]["objects"][$map_object_key]["mapObject_id"] = $mapObject_id;
            }

            foreach ($map["Connections"] as $map_connection) {
                $source_id = "";
                $target_id = "";
                foreach ($map_array["maps"][$map_key]["objects"] as $map_object) {
                    if ($map_object["Alias"] == $map_connection["source"]) {
                        $source_id = $map_object["mapObject_id"];
                    }
                    if ($map_object["Alias"] == $map_connection["target"]) {
                        $target_id = $map_object["mapObject_id"];
                    }
                }

                $map_connection_array = [
                    "shd_map__id"                  => $map["map_id"],
                    "isys_obj__id"                 => "",
                    "title"                        => "",
                    "source_shd_map_mapobject__id" => $source_id,
                    "target_shd_map_mapobject__id" => $target_id,
                    "targetarrowtype"              => $map_connection["targetArrowType"],
                    "sourcearrowtype"              => $map_connection["sourceArrowType"],
                    "sourcepoint"                  => $map_connection["sourceX"] . "," . $map_connection["sourceY"],
                    "targetpoint"                  => $map_connection["targetX"] . "," . $map_connection["targetY"],
                    "bends"                        => $map_connection["bends"],
                    "linestyle"                    => $map_connection["lineStyle"],
                    "linewidth"                    => $map_connection["lineWidth"],
                    "linecolor"                    => $map_connection["lineColor"],
                    "textcolor"                    => $map_connection["textColor"],
                    "font"                         => $map_connection["font"],
                    "layerindex"                   => $map_connection["layerIndex"],
                    "realizertype"                 => $map_connection["realizerType"],
                    "connectiontype"               => $map_connection["connectionType"],
                    "textvisible"                  => $map_connection["isTextVisible"],
                    "locked"                       => false,
                ];
                $this->_map_dao->createMapConnection($map_connection_array);
            }
        }
        return $map_array;
    }

    public function truncateMaps()
    {
        $this->_map_dao->retrieve("TRUNCATE `shd_map`");
        $this->_map_dao->retrieve("TRUNCATE `shd_map_favoriten`");
        $this->_map_dao->retrieve("TRUNCATE `shd_map_homemap`");
        $this->_map_dao->retrieve("TRUNCATE `shd_map_lineproperty`");
        $this->_map_dao->retrieve("TRUNCATE `shd_map_mapconnection`");
        $this->_map_dao->retrieve("TRUNCATE `shd_map_mapobject`");
        $this->_map_dao->retrieve("TRUNCATE `shd_map_right`");
    }

    public function events()
    {
        if ($this->m_cache_events == null) {
            return;
        }
        return $this->m_cache_events->get("event");
    }

    private function setEvent($map_id, $eventart)
    {
        if ($this->m_cache_events == null) {
            return;
        }
        $eventArray = $this->m_cache_events->get("event");
        $date = new DateTime();

        //clear Objects after 60 sek
        foreach ($eventArray as $key => $f_event) {
            //the Element is updated so it's not throwed away after 60 sec
            if ($f_event["timestamp"] + 60 < $date->getTimestamp()) {
                unset($eventArray[$key]);
            }
        }

        foreach ($eventArray as $key => $f_event) {
            if ($f_event["map_id"] == $map_id) {
                $f_event["timestamp"] = $date->getTimestamp();

                $eventArray[$key] = $f_event;
                $this->m_cache_events->set("event", $eventArray, 60);
                return;
            }
        }

        $eventArray[] = ["map_id" => $map_id, "timestamp" => $date->getTimestamp(), "eventart" => $eventart];
        $this->m_cache_events->set("event", $eventArray, 60);
    }

    public function getMapStatus($map_id, $force_status = false)
    {
        $return = [];
        try {
            $map_objects = $this->_map_dao->getMapObjectAndMapLinkByMap($map_id);
            $map_connection = $this->_map_dao->getMapConnectionByMap($map_id);
        } catch (Exception $ex) {
            $map_objects = [];
            $map_connection = [];
        }

        $statusDao = smview_cmdb_dao_status::instance(isys_application::instance()->database);
        foreach ($map_objects as $f_mapObject) {
            if ($f_mapObject["isys_obj__id"] != "0") {
                // get live status
                $m_status = $statusDao->getStatus($f_mapObject["isys_obj__id"], true);
                if ($m_status !== []) {
                    if ($f_mapObject["livestatus_services"] !== "a:0:{}" && $f_mapObject["livestatus_services"] !== "") {
                        //check single Services for Status
                        $searched_services = unserialize($f_mapObject["livestatus_services"]);
                        $m_status = $statusDao->get_services_by_hostname($f_mapObject["isys_obj__id"], $searched_services);
                    }
                    $return["objects"][$f_mapObject["mapobject_id"]] = $m_status;
                } else {
                    // get BSM service status
                    $bsmServiceStatus = $statusDao->getBsmServiceStatus($f_mapObject["isys_obj__id"]);
                    if ($bsmServiceStatus) {
                        $return["objects"][$f_mapObject["mapobject_id"]] = $bsmServiceStatus;
                    }
                }
            }

            //get profile and services
            $l_map_lineproperties = $this->_map_dao->getAllMapObjectLineproperties($f_mapObject["mapobject_id"]);
            foreach ($l_map_lineproperties as $f_map_lineproperty) {
                if ($f_map_lineproperty["bsm_profil_id"] != null) {
                    $return["bsm_profil"][$f_map_lineproperty["bsm_profil_id"]] = $statusDao->getGreyStatus();
                } elseif ($f_map_lineproperty["services"] != null) {
                    $l_services = explode(";", $f_map_lineproperty["services"]);
                    foreach ($l_services as $l_service) {
                        $return["livestatus_service"][$f_mapObject["mapobject_id"] . ";"  . $l_service] = $statusDao->getGreyStatus();
                    }
                }
            }
        }

        foreach ($map_connection as $f_mapObject) {
            if ($f_mapObject["isys_obj__id"] != "0") {
                $m_status = $statusDao->getStatus($f_mapObject["isys_obj__id"]);

                if ($m_status !== []) {
                    $return["objects"][$f_mapObject["mapobject_id"]] = $m_status;
                }
            }
        }

        foreach ($map_objects as $f_mapObject) {
            if (is_numeric($f_mapObject["link"])) {
                $map_link_id = $f_mapObject["link"];
                if ($this->m_cache_status && $this->m_cache_status->exists($map_link_id)) {
                    $return["maps"][$map_link_id] = $this->m_cache_status->get($map_link_id);
                } else {
                    if ($this->circle > 3) {
                        return ;
                    }

                    $this->circle += 1;
                    $t_map = $this->getMapStatus($map_link_id);

                    $this->circle -= 1;
                    unset($t_map["objects"]);

                    $t_map = $t_map["status"];
                    $return["maps"][$map_link_id] = $t_map;
                }
            }
        }

        //Calc Status
        // -1 = Nothing -> Grey Status
        // 0 = Okay
        // 1 = Warning !
        // 2 = Unknown  !!
        // 3 = Critical !!!
        // 4 = down == 2 !!!
        // 5 = unreachable == 2 !!!
        $a_Status = null;
        foreach ($return["objects"] as $f_status) {
            $a_Status = $this->getHigherStatus($f_status, $a_Status);
        }

        foreach ($return["maps"] as $f_map) {
            $a_Status = $this->getHigherStatus($f_map, $a_Status);
        }

        if ($a_Status === null) {
            $a_Status = $statusDao->getGreyStatus();
        }

        $return["status"] = $a_Status;
        if ($this->m_cache_status) {
            //get the Status without new rendering
            if ($this->m_cache_status->exists($map_id) && ! $force_status) {
                return $this->m_cache_status->get($map_id);
            }
            $this->m_cache_status->set($map_id, $return, 60);
        }
        return $return;
    }

    private function getHigherStatus($new_status, $old_status)
    {
        switch ($new_status["status"]) {
            case 0:
            case 1:
            case -1:
                if ($old_status === null || $new_status["status"] > $old_status["status"]) {
                    $old_status = $new_status;
                }
                break;
            case 2:
                if ($old_status === null || $old_status["status"] != 2) {
                    $old_status = $new_status;
                }
                break;
            case 3:
                if ($old_status === null || $old_status["status"] < 2) {
                    $old_status = $new_status;
                }
                break;
        }
        return $old_status;
    }

    /**
     * Get a common success message.
     *
     * @param  string $methodName
     * @param  int|true $id
     * @return array
     */
    private function _getSuccessMessage($methodName, $id = true)
    {
        return array(
            'message'      => "map.$methodName.success",
            'response'     => $id,
            'current_date' => date('Y-m-d')
        );
    }
}
