<?php

/**
 * i-doit APi
 *
 * @package i-doit
 * @subpackage API
 * @author Dennis Stücken <dstuecken@synetics.de>
 * @copyright synetics GmbH
 * @license http://www.i-doit.com/license
 */

class isys_api_model_shd_map_connection implements isys_api_model_interface
{

    /**
     * The caching instance for livestatus objects.
     *
     * @var  isys_cache_memcache
     */
    protected $m_cache_events = null;

    function __construct()
    {
        try {
            $this->m_cache_events = new \isys_cache_smview_memcached();
            // this namespace is also used by:
            //      isys_api_model_shd_map.class.php,
            //      isys_api_model_shd_map_object.class.php,
            //      isys_api_model_shd_maps.class.php
            //      isys_api_model_shd_map_lineproperty.class.php
            $this->m_cache_events->ns('map_events');
        } catch (isys_exception $ex) {
        }
    }

    /**
     * @param array $p_params
     * @return array
     */
    public function read($p_params)
    {
        try {
            return smview_cmdb_dao_map::instance(isys_application::instance()->database)->getMapConnection($p_params["mapconnection_id"]);
        } catch (Exception $ex) {
            return array(
                'message'      => 'map_connection.read.error',
                'response'     => $ex->getMessage(),
                'current_date' => date('Y-m-d')
            );
        }
    }

    /**
     * @param  $p_params
     * @return array
     * @throws isys_exception_dao
     */
    public function create($p_params)
    {
        $l_return_id = smview_cmdb_dao_map::instance(isys_application::instance()->database)->createMapConnection($p_params);
        $this->setEvent($p_params["shd_map__id"], $l_return_id, "create");
        return array(
            'message'      => 'map_connection.create.success',
            'response'     => $l_return_id,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * @param $p_params
     * @return array
     */
    public function update($p_params)
    {
        smview_cmdb_dao_map::instance(isys_application::instance()->database)->updateMapConnection($p_params);
        $this->setEvent($p_params["shd_map__id"], $p_params["mapconnection_id"], "update");
        return array(
            'message'      => 'map_connection.update.success',
            'response'     => true,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * @param $p_params
     * @return array
     */
    public function delete($p_params)
    {
        $mapconnection = $this->read($p_params);
        smview_cmdb_dao_map::instance(isys_application::instance()->database)->deleteMapConnection($p_params["mapconnection_id"]);
        $this->setEvent($mapconnection["shd_map__id"], $p_params["mapconnection_id"], "delete");
        return array(
            'message'      => 'map_connection.delete.success',
            'response'     => true,
            'current_date' => date('Y-m-d')
        );
    }

    private function setEvent($map_id, $mapconnection_id, $eventart)
    {
        if ($this->m_cache_events == null) {
            return;
        }

        $eventArray = $this->m_cache_events->get("event");
        $date = new DateTime();

        //clear Objects after 60 sek
        foreach ($eventArray as $key => $f_event) {
            //the Element is updated so it's not throwed away after 60 sec
            if ($f_event["timestamp"] + 60 < $date->getTimestamp()) {
                unset($eventArray[$key]);
            }
        }

        foreach ($eventArray as $key => $f_event) {
            if ($f_event["mapconnection_id"] == $mapconnection_id) {
                $f_event["timestamp"] = $date->getTimestamp();
                $eventArray[$key] = $f_event;
                $this->m_cache_events->set("event", $eventArray, 60);
                return;
            }
        }

        $eventArray[] = ["map_id" => $map_id, "mapconnection_id" => $mapconnection_id, "timestamp" => $date->getTimestamp(), "eventart" => $eventart];
        $this->m_cache_events->set("event", $eventArray, 60);
    }
}
