<?php

/**
 * shd.api.map_object
 *
 * @package SHD
 * @subpackage API
 * @author Maik Rhode <Maik.Rhode@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 */
class isys_api_model_shd_map_lineproperty implements isys_api_model_interface
{

    /**
     * The caching instance for livestatus objects.
     *
     * @var  isys_cache_memcache
     */
    protected $m_cache_events = null;

    /**
     * Map DAO.
     *
     * @var smview_cmdb_dao_map|null
     */
    private $_map_dao = null;

    function __construct()
    {
        try {
            $this->m_cache_events = new \isys_cache_smview_memcached();
            // this namespace is also used by:
            //      isys_api_model_shd_map.class.php,
            //      isys_api_model_shd_map_object.class.php,
            //      isys_api_model_shd_maps.class.php
            //      isys_api_model_shd_map_connection.class.php
            $this->m_cache_events->ns('map_events');
        } catch (isys_exception $ex) {
        }
        $this->_map_dao = smview_cmdb_dao_map::instance(isys_application::instance()->database);
    }

    /**
     * @param array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function read($p_params)
    {
        if (isset($p_params["mapobject_id"]) == false) {
            throw new isys_exception_api('Parameter mapobject_id missed', -32602);
        }
        return $this->_map_dao->getMapObject($p_params["mapobject_id"]);
    }

    /**
     * @param array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function readByMap($p_params)
    {
        if (isset($p_params["map_id"]) == false) {
            throw new isys_exception_api('Parameter map_id missed', -32602);
        }
        return $this->_map_dao->getMapObjectByMap($p_params["map_id"]);
    }

    /**
     * @param $p_params
     * @return array
     */
    public function create($p_params)
    {
        $l_return_id = $this->_map_dao->createMapLineproperty($p_params);
        return array(
            'message'      => 'map_lineproperty.create.success',
            'response'     => $l_return_id,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * @param $p_params
     * @return array
     */
    public function update($p_params)
    {
        $this->_map_dao->updateMapObject($p_params);
        $this->setEvent($p_params["shd_map__id"], $p_params["mapobject_id"], "update");
        return array(
            'message'      => 'map_lineproperty.update.success',
            'response'     => true,
            'current_date' => date('Y-m-d')
        );
    }

    /**
     * @param $p_params
     * @return array
     */
    public function delete($p_params)
    {
        $this->_map_dao->deleteMapObject($p_params["maplineproperty_id"]);
        return array(
            'message'      => 'map_lineproperty.delete.success',
            'response'     => true,
            'current_date' => date('Y-m-d')
        );
    }

    private function setEvent($map_id, $mapobject_id, $eventart)
    {
        if ($this->m_cache_events == null) {
            return;
        }

        $eventArray = $this->m_cache_events->get("event");
        $date = new DateTime();

        //clear Objects after 60 sek
        foreach ($eventArray as $key => $f_event) {
            //the Element is updated so it's not throwed away after 60 sec
            if ($f_event["timestamp"] + 60 < $date->getTimestamp()) {
                unset($eventArray[$key]);
            }
        }

        foreach ($eventArray as $key => $f_event) {
            if ($f_event["mapobject_id"] == $mapobject_id) {
                $f_event["timestamp"] = $date->getTimestamp();
                $eventArray[$key] = $f_event;
                $this->m_cache_events->set("event", $eventArray, 60);
                return;
            }
        }

        $eventArray[] = ["map_id" => $map_id, "mapobject_id" => $mapobject_id, "timestamp" => $date->getTimestamp(), "eventart" => $eventart];
        $this->m_cache_events->set("event", $eventArray, 60);
    }
}
