<?php

/**
 * i-doit APi
 *
 * @package i-doit
 * @subpackage API
 * @author Dennis Stücken <dstuecken@synetics.de>
 * @copyright synetics GmbH
 * @license http://www.i-doit.com/license
 */
class isys_api_model_shd_maps implements isys_api_model_interface
{

    /**
     * The caching instance for livestatus objects.
     *
     * @var  isys_cache_memcache
     */
    protected $m_cache_events = null;

    function __construct()
    {
        try {
            $this->m_cache_events = new \isys_cache_smview_memcached();
            // this namespace is also used by:
            //      isys_api_model_shd_map.class.php,
            //      isys_api_model_shd_map_object.class.php,
            //      isys_api_model_shd_map_connection.class.php
            //      isys_api_model_shd_map_lineproperty.class.php
            $this->m_cache_events->ns('map_events');
        } catch (isys_exception $ex) {
        }
    }

    /**
     * @param array $p_params
     * @return array
     */
    public function read($p_params)
    {
        try {
            $return = [];
            if (isset($p_params["user_id"])) {
                $l_maps = smview_cmdb_dao_map::instance(isys_application::instance()->database)->getMaps($p_params["user_id"]);
            } else {
                $l_maps = smview_cmdb_dao_map::instance(isys_application::instance()->database)->getMaps();
            }

            $return["content"] = $l_maps;
            $return["count"] = count($l_maps);
            return $return;
        } catch (Exception $ex) {
            return array(
                'message'      => 'maps.read.error',
                'response'     => $ex->getMessage(),
                'current_date' => date('Y-m-d')
            );
        }
    }

    /**
     * @param $p_params
     * @return mixed
     * @throws isys_exception_api
     */
    public function status($p_params)
    {
        $isys_api_model_shd_map = new isys_api_model_shd_map();
        if (isset($p_params["user_id"])) {
            $l_maps = smview_cmdb_dao_map::instance(isys_application::instance()->database)->getMaps($p_params["user_id"]);
        } else {
            $l_maps = smview_cmdb_dao_map::instance(isys_application::instance()->database)->getMaps();
        }

        $maps = [];
        foreach ($l_maps as $f_map) {
            if (isset($p_params["force_status"])) {
                $t_map = $isys_api_model_shd_map->getMapStatus($f_map["map_id"], true);
            } else {
                $t_map = $isys_api_model_shd_map->getMapStatus($f_map["map_id"]);
            }

            //rework object for shd.maps.read
            $maps[$f_map["map_id"]] = $t_map["status"];
            $maps[$f_map["map_id"]]["name"] = $f_map["title"];
        }

        $return["maps"] = $maps;
        $return["count"] = count($maps);
        return $return;
    }

    /**
     * @param $p_params
     * @return mixed
     */
    public function events($p_params)
    {
        if ($this->m_cache_events == null) {
            return;
        }
        return $this->m_cache_events->get("event");
    }
}
