<?php

/**
 * SHD SM-VIEW Module Updater.
 *
 * @package    idoit
 * @subpackage smview
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_api_model_shd_module_updater implements isys_api_model_interface
{
    /**
     * Read available modules.
     *
     * @param array $p_params
     * @return array
     */
    public function read($p_params)
    {
        $updater = new idoit\Module\Smview\ModuleUpdater();
        $mods = $updater->getAvailableModules();
        
        $modules = [];
        $l_module_manager = isys_module_manager::instance();
        foreach ($mods as $module_identifier) {
            $module_id = $l_module_manager->is_installed($module_identifier);
            if ($module_id === false) {
                continue;
            }
            $mod_row = $l_module_manager->get_modules($module_id);
            $mod = $mod_row->get_row();
            
            $module = array(
                'id'           => $module_id,
                'identifier'   => $module_identifier,
                'title'        => $mod['isys_module__title'],
                'const'        => $mod['isys_module__const'],
                'date_install' => $mod['isys_module__date_install'],
            );
            if ($p_params['show_versions']) {
                $installed_version = $updater->getCurrentModuleVersion($module_identifier);
                $module['installed_version'] = $installed_version;
                
                $usePreview = ! empty($p_params['use_preview']);
                $available_version = $updater->getAvailableModuleVersion($module_identifier, $usePreview);
                $module['available_version'] = $available_version;
                
                $cmp = version_compare($installed_version, $available_version);
                $module['installable'] = $cmp < 0;
            }
            $modules[] = $module;
        }
        return $modules;
    }

    /**
     * Update specified modules.
     *
     * @param array $p_params
     * @return array
     */
    public function update($p_params)
    {
        if (empty($p_params['modules'])) {
            $message = "following parameters are missing: modules";
            throw new \isys_exception_validation($message, ['modules']);
        }
        
        $usePreview = ! empty($p_params['use_preview']);
        $modules = $p_params['modules'];
        $updater = new idoit\Module\Smview\ModuleUpdater();
        $updater->update($modules, $usePreview);

        return array(
            'message'      => "module_updater.update.success",
            'id'           => 0,
            'current_date' => date('Y-m-d'),
        );
    }
}
