<?php

/**
 * i-doit APi
 *
 * @package i-doit
 * @subpackage API
 * @author Dennis Stücken <dstuecken@synetics.de>
 * @copyright synetics GmbH
 * @license http://www.i-doit.com/license
 */
class isys_api_model_shd_object extends isys_api_model_cmdb implements isys_api_model_interface
{
    /**
     * Mandator database
     *
     * @var isys_component_database
     */
    protected $m_db;

    /**
     * Mandator database
     *
     * @var isys_cmdb_dao_relation
     */
    protected $m_dao;

    /**
     * the down lvl for the recursiv function
     *
     * @var isys_component_database
     */
    protected $m_down_lvl;

    /**
     * the up lvl for the recursiv function
     *
     * @var isys_component_database
     */
    protected $m_up_lvl;

    /**
     * the up lvl for the recursiv function
     *
     * @var isys_component_database
     */
    protected $m_columns;

    /**
     * the up lvl for the recursiv function
     *
     * @var isys_component_database
     */
    protected $m_return = [];

    /**
     * The caching instance for livestatus objects.
     *
     * @var  isys_cache_memcache
     */
    protected $m_request_id = null;

    /**
     * The caching instance for livestatus objects.
     *
     * @var  isys_cache_memcache
     */
    protected $m_is_canceled = false;

    /**
     * The caching instance for kill-Process
     *
     * @var  isys_cache_memcache
     */
    protected $m_cache_object = null;

    /**
     * Data formatting used in format methods
     *
     * @var array
     */
    protected $m_mapping = [
        'isys_obj__id' => 'id',
        'isys_obj__title' => 'title',
        'isys_cmdb_status__id' => 'cmdb_status',
        'isys_cmdb_status__title' => [
            '_L',
            'cmdb_status_title'
        ],
        'related' => 'related_object',
        'related_title' => 'related_title',
        'related_type' => 'related_type',
        'related_type_title' => [
            '_L',
            'related_type_title'
        ],
        'related_cmdb_status' => 'related_cmdb_status',
        'related_cmdb_status_title' => [
            '_L',
            'related_cmdb_status_title'
        ],
        'isys_catg_relation_list__isys_obj__id__master' => 'master',
        'isys_catg_relation_list__isys_obj__id__slave' => 'slave',
        'isys_obj_type__id' => 'type',
        'isys_obj_type__title' => [
            '_L',
            'type_title'
        ]
    ];

    function __construct()
    {
        parent::__construct();
        try {
            $this->m_cache_object = new isys_cache_smview_memcached();
            // this namespace is also use by isys_api_model_shd_cancel.class.php
            $this->m_cache_object->ns('object_cancel');
        } catch (isys_exception_cache $ex) {
        }
    }

    /**
     * @param array $p_params
     * @return array
     */
    public function read($p_params)
    {
        $l_id = (isset($p_params['objid'])) ? $p_params['objid'] : null;
        if (isset($p_params['columns']) == true) {
            foreach ($p_params['columns'] as $f_columnKey => $f_columns) {
                $this->m_columns[defined_or_default($f_columnKey)] = $f_columns;
            }
        }

        $this->m_db = new isys_cmdb_dao(isys_application::instance()->database);
        /* Start looking for objects */
        $this->m_dao = new isys_cmdb_dao_relation(isys_application::instance()->database);

        //get Self
        $this->_getSelfObject($l_id);
        return $this->m_return;
    }

    /**
     * @param $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function relation($p_params)
    {
        $start = microtime(true);
        $this->m_request_id = $p_params['request_id'];

        $this->m_db = new isys_cmdb_dao(isys_application::instance()->database);

        //Lower Keys to reduce errors
        $p_params = array_change_key_case($p_params, CASE_LOWER);

        /* Prepare filters */
        $l_id = (isset($p_params['objid'])) ? $p_params['objid'] : null;
        $l_relation_type = (isset($p_params['relation_type'])) ? $p_params['relation_type'] : null;
        $this->m_down_lvl = (isset($p_params['down_lvl'])) ? $p_params['down_lvl'] : 0;
        $this->m_up_lvl = (isset($p_params['up_lvl'])) ? $p_params['up_lvl'] : 0;

        if (isset($p_params['columns']) == true) {
            foreach ($p_params['columns'] as $f_columnKey => $f_columns) {
                $this->m_columns[defined_or_default($f_columnKey)] = $f_columns;
            }
        }

        /* Check if relation_type is a string constant */
        if (is_string($l_relation_type) && defined($l_relation_type)) {
            $l_relation_type = defined_or_default($l_relation_type);
        }

        if (!$l_id) {
            throw new isys_exception_api(
                'No object id given. Specify parameter "id" in order to filter objects by relation.'
            );
        }

        /* Start looking for objects */
        $this->m_dao = new isys_cmdb_dao_relation(isys_application::instance()->database);

        //get Self
        $this->_getSelfObject($l_id);

        //Up
        $this->_getUpObject($l_id, 0, $l_relation_type);

        //Down
        $this->_getDownObject($l_id, 0, $l_relation_type);

        if ($this->m_is_canceled == true) {
            return ['Request stop.'];
        }

        $t_return["objects"] = $this->m_return;
        $this->m_return = $t_return;
        $this->m_return["time"] = microtime(true) - $start;
        return $this->m_return;
    }

    /**
     * @param  array $p_params
     * @return array
     */
    public function relationfast($p_params)
    {
        $start = microtime(true);
        $smview_cmdb_dao_object = new smview_cmdb_dao_object();
        $return = $smview_cmdb_dao_object->relation_old($p_params);
        $return["time"] = microtime(true) - $start;
        return $return;
    }

    /**
     * @param  array $p_params
     * @return array
     */
    public function relationfast_new($p_params)
    {
        $start = microtime(true);
        $smview_cmdb_dao_object = new smview_cmdb_dao_object();
        $return = $smview_cmdb_dao_object->relationfast($p_params);
        $return["time"] = microtime(true) - $start;
        return $return;
    }

    public function relationsbyid($p_params)
    {
        $smview_cmdb_dao_object = new smview_cmdb_dao_object();
        return $smview_cmdb_dao_object->get_all_down_objects($p_params);
    }

    public function validate($p_params)
    {
        $l_message = '';
        $l_success = true;
        $l_data = "";

        $p_identifier = 'isys_cmdb_dao_category_g_global::title';
        if (isset($p_params["value"])) {
            $p_value = $p_params["value"];
        } else {
            throw new isys_exception_api('No value is given. In api call');
        }
        if (isset($p_params["obj_type_id"])) {
            if (defined($p_params["obj_type_id"])) {
                $p_obj_type_id = defined_or_default($p_params["obj_type_id"]);
            } else {
                $p_obj_type_id = $p_params["obj_type_id"];
            }
        } else {
            $p_obj_type_id = null;
        }
        $p_obj_id = $p_params["obj_id"] ?? null;
        $p_custom_category_id = 0;
        $p_category_entry_id = 0;

        // check object title
        $useStandardValidation = true;
        if (class_exists('idoit\Module\Cable\ObjectTitleValidator')) {
            $check_enabled = isys_settings::get(isys_module_smview_cable::OBJECT_TITLE_CHECK_ENABLED, false);
            if ($check_enabled) {
                $useStandardValidation = false;
                $propertiesToCheck = [
                    'title' => $p_value,
                ];
                if ($p_params["properties"]) {
                    $propertiesToCheck['purpose'] = $p_params["properties"]['purpose'] ?? null;
                    $propertiesToCheck['category'] = $p_params["properties"]['category'] ?? null;
                    $propertiesToCheck['cmdb_status'] = $p_params["properties"]['cmdb_status'] ?? null;
                    $propertiesToCheck['status'] = $p_params["properties"]['status'] ?? null;
                    $propertiesToCheck['location'] = $p_params["properties"]['location'] ?? null;
                }

                $result = \idoit\Module\Cable\ObjectTitleValidator::validate($p_obj_id, $p_obj_type_id, $propertiesToCheck);
                $l_success = $result['success'];
                $l_message = $result['success'] ? $l_message : $result['message'];
            }
        }

        list($l_dao_name, $l_prop) = explode('::', $p_identifier);
        if (class_exists($l_dao_name) && $useStandardValidation) {
            /* @var  $l_dao isys_cmdb_dao_category */
            $l_dao = isys_factory::get_instance($l_dao_name, isys_application::instance()->database);
            if (method_exists($l_dao, 'validate')) {
                if (method_exists($l_dao, 'set_catg_custom_id') && $p_custom_category_id > 0) {
                    $l_dao->set_catg_custom_id($p_custom_category_id);
                }

                // This might happen on the overview-page.
                if (substr($l_prop, 0, 2) === 'C_') {
                    $l_properties = $l_dao->get_properties();
                    foreach ($l_properties as $l_key => $l_property) {
                        if ($l_property[C__PROPERTY__UI][C__PROPERTY__UI__ID] == $l_prop) {
                            $l_prop = $l_key;
                            break;
                        }
                    }
                }

                $l_validated = $l_dao->set_object_id($p_obj_id)
                    ->set_list_id($p_category_entry_id)
                    ->set_object_type_id($p_obj_type_id)
                    ->validate([$l_prop => $p_value]);

                if (is_array($l_validated)) {
                    $l_message = $l_validated[$l_prop];
                    $l_success = false;
                    $l_data = $l_validated;
                }
            }
        }

        if ($l_message == null) {
            $l_message = "";
        }

        if ($l_data == null) {
            $l_data = "";
        }

        return [
            "message" => $l_message,
            "success" => $l_success,
            "data"    => $l_data
        ];
    }

    private function _getSelfObject($l_id)
    {
        $l_return = [];

        // Fetch more information about the related object:
        $l_related = $this->m_dao->get_object_by_id($l_id)->__to_array();

        $isys_api_model_shd_userright = new isys_api_model_shd_userright();
        $hasRight = $isys_api_model_shd_userright->hasUserObjTypeRight($l_related['isys_obj_type__id']);
        if ($hasRight == false) {
            $l_row['related_title'] = '*****';
            $l_row['related_type_title'] = '*****';
            $l_row['isys_obj_type__title'] = '*****';
        } else {
            $l_row['related_title'] = $l_related['isys_obj__title'];
            $l_row['related_type_title'] = _L($l_related['isys_obj_type__title']);
            $l_row['isys_obj_type__title'] = _L($l_related['isys_obj_type__title']);
        }

        $l_row['related_type'] = $l_related['isys_obj_type__id'];
        $l_row['related_cmdb_status_id'] = $l_related['isys_cmdb_status__id'];
        $l_row['related_cmdb_status_title'] = _L($l_related['isys_cmdb_status__title']);
        $l_row['related'] = $l_id;

        if ($this->m_columns[$l_row["related_type"]] != null) {
            //Abfrage für die Obj-Type-Id
            $l_row = $this->_getColumns($l_row, $l_row["related_type"]);
        } else {
            //Abfrage nach extra Columns für Global
            $l_row = $this->_getColumns($l_row, 0);
        }

        $l_value = [
            'data'     => $l_row,
            'children' => false
        ];

        $this->m_return[$l_id] = $l_value;
        return $l_return;
    }

    private function _getUpObject($l_id, $l_up_lvl, $l_relation_type)
    {
        if ($this->_killProcess() == true) {
            $this->m_is_canceled = true;
            return [];
        }

        if ($this->m_up_lvl <= $l_up_lvl) {
            return [];
        }

        $l_return = [];
        $l_data = $this->m_dao->get_related_objects($l_id, $l_relation_type);

        /* Iterate through result set*/
        while ($l_row = $l_data->get_row()) {
            //suche ob isys_obj__id schon vorhanden ist.
            if (array_key_exists($l_row['related'], $this->m_return) == true) {
                continue;
            }

            // Fetch more information about the related object:
            $l_related = $this->m_dao->get_object_by_id($l_row['related'])->__to_array();

            $isys_api_model_shd_userright = new isys_api_model_shd_userright();
            $hasRight = $isys_api_model_shd_userright->hasUserObjTypeRight($l_related['isys_obj_type__id']);
            if ($hasRight == false) {
                $l_row['related_title'] = '*****';
                $l_row['related_type_title'] = '*****';
                $l_row['isys_obj_type__title'] = '*****';
            } else {
                $l_row['related_title'] = $l_related['isys_obj__title'];
                $l_row['related_type_title'] = _L($l_related['isys_obj_type__title']);
                $l_row['isys_obj_type__title'] = _L($l_related['isys_obj_type__title']);
            }

            $l_row['related_type'] = $l_related['isys_obj_type__id'];
            $l_row['related_cmdb_status_id'] = $l_related['isys_cmdb_status__id'];
            $l_row['related_cmdb_status_title'] = _L($l_related['isys_cmdb_status__title']);
            $l_row['up_lvl'] = $l_up_lvl + 1;

            $l_key = $l_row['related'];
            if ($l_row["isys_catg_relation_list__isys_obj__id__slave"] == $l_id) {
                try {
                    if ($this->m_columns[$l_row["related_type"]] != null) {
                        //Abfrage für die Obj-Type-Id
                        $l_row = $this->_getColumns($l_row, $l_row["related_type"]);
                    } else {
                        //Abfrage nach extra Columns für Global
                        $l_row = $this->_getColumns($l_row, 0);
                    }

                    //andere Objekte nach oben suchen
                    $this->_getUpObject($l_row["isys_catg_relation_list__isys_obj__id__master"], $l_up_lvl + 1, $l_relation_type);
                    $l_value = [
                        'data'     => $l_row,
                        'children' => false
                    ];

                    $this->m_return[$l_key] = $l_value;
                } catch (Exception $ex) {
                    dump($ex->getMessage());
                }
            }
        }
        return $l_return;
    }

    private function _getDownObject($l_id, $l_down_lvl, $l_relation_type)
    {
        if ($this->_killProcess() == true) {
            $this->m_is_canceled = true;
            return [];
        }

        if ($this->m_down_lvl <= $l_down_lvl) {
            return [];
        }

        $l_return = [];
        $l_data = $this->m_dao->get_related_objects($l_id, $l_relation_type);

        /* Iterate through result set*/
        while ($l_row = $l_data->get_row()) {
            //suche ob related_id schon vorhanden ist.
            if (array_key_exists($l_row['related'], $this->m_return) == true) {
                continue;
            }

            // Fetch more information about the related object:
            $l_related = $this->m_dao->get_object_by_id($l_row['related'])->__to_array();

            $isys_api_model_shd_userright = new isys_api_model_shd_userright();
            $hasRight = $isys_api_model_shd_userright->hasUserObjTypeRight($l_related['isys_obj_type__id']);
            if ($hasRight == false) {
                $l_row['related_title'] = '*****';
                $l_row['related_type_title'] = '*****';
                $l_row['isys_obj_type__title'] = '*****';
            } else {
                $l_row['related_title'] = $l_related['isys_obj__title'];
                $l_row['related_type_title'] = _L($l_related['isys_obj_type__title']);
                $l_row['isys_obj_type__title'] = _L($l_related['isys_obj_type__title']);
            }

            $l_row['related_type'] = $l_related['isys_obj_type__id'];
            $l_row['related_cmdb_status_id'] = $l_related['isys_cmdb_status__id'];
            $l_row['related_cmdb_status_title'] = _L($l_related['isys_cmdb_status__title']);
            $l_row['down_lvl'] = $l_down_lvl + 1;

            $l_key = $l_row['related'];
            if ($l_row["isys_catg_relation_list__isys_obj__id__master"] == $l_id) {
                try {
                    if ($this->m_columns[$l_row["related_type"]] != null) {
                        //Abfrage für die Obj-Type-Id
                        $l_row = $this->_getColumns($l_row, $l_row["related_type"]);
                    } else {
                        //Abfrage nach extra Columns für Global
                        $l_row = $this->_getColumns($l_row, 0);
                    }

                    //andere Objekte nach unten suchen
                    $this->_getDownObject($l_row["isys_catg_relation_list__isys_obj__id__slave"], $l_down_lvl + 1, $l_relation_type);
                    $l_value = [
                        'data'     => $l_row,
                        'children' => false
                    ];

                    $this->m_return[$l_key] = $l_value;
                } catch (Exception $ex) {
                    dump($ex->getMessage());
                }
            }
        }
        return $l_return;
    }

    private function _getColumns($l_row, $obj_type)
    {
        $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao(isys_application::instance()->database));
        foreach ($this->m_columns[$obj_type] as $m_column) {
            try {
                //bsp. $m_column['id'] = C__CATG__CPU-title
                $l_split = explode("-", $m_column['id']);
                $p_param = [
                    'objID'    => $l_row['related'],
                    'category' => $l_split[0]
                ];

                $l_categoryData = $l_api_model_category->read($p_param);
                if ($l_categoryData != null) {
                    $isys_api_model_shd_userright = new isys_api_model_shd_userright();
                    $hasRight = $isys_api_model_shd_userright->hasUserCategoryRight($l_split[0]);

                    if ($hasRight == true) {
                        if ($l_split[1] == "title") {
                            $l_row["category"][$m_column['id']] = $l_categoryData[0]["title"];
                        } elseif (strpos($l_split[0], "CUSTOM_FIELDS") !== false) {
                            if (strpos($l_split[1], "text") !== false) {
                                $l_row["category"][$m_column['id']] = $l_categoryData[0][$l_split[1]];
                            } else {
                                $l_row["category"][$m_column['id']] = $l_categoryData[0][$l_split[1]]["title"];
                            }
                        } elseif ($l_split[1] == "changed") {
                            $l_row["category"][$m_column['id']] = $l_categoryData[0]["changed"];
                        } elseif ($l_split[1] == "changed_by") {
                            $l_row["category"][$m_column['id']] = $l_categoryData[0]["changed_by"];
                        } elseif ($l_split[1] == "id") {
                            $l_row["category"][$m_column['id']] = $l_categoryData[0]["id"];
                        } else {
                            $l_row["category"][$m_column['id']] = $l_categoryData[0][$l_split[1]]["title"];
                        }
                    } else {
                        $l_row["category"][$m_column['id']] = '*****';
                    }
                }
            } catch (Exception $ex) {
            }
        }
        return $l_row;
    }

    public function tree($p_params)
    {
        $return = [];

        $smview_cmdb_dao_location = new smview_cmdb_dao_location(isys_application::instance()->database);
        $t_objects = $smview_cmdb_dao_location->get_relation_list();

        $p_params['filter']['container'] = "1";

        $t_object_types = smview_cmdb_dao_object_type::instance(isys_application::instance()->database)->get_object_types_by_params($p_params);
        foreach ($t_objects as $t_object) {
            $add = false;
            foreach ($t_object_types as $t_object_type) {
                if ($t_object_type["isys_obj_type__id"] == $t_object["slave_type.isys_obj_type__id"]) {
                    $add = true;
                }
            }

            if ($add == true) {
                $return[] = [
                    "id"           => $t_object["slave.isys_obj__id"],
                    "title"        => $t_object["slave.isys_obj__title"],
                    "parent_id"    => $t_object["master.isys_obj__id"],
                    "obj_type__id" => $t_object["slave_type.isys_obj_type__id"]];
            }
        }
        return $return;
    }

    public function duplicate($p_params)
    {
        global $g_comp_session, $g_comp_database;

        $l_return = [
            'success' => true,
            'data'    => [],
            'message' => null,
        ];

        try {
            $g_comp_session->write_close();

            // Start logging:
            $l_log = isys_factory_log::get_instance('duplicate');
            $l_log->set_verbose_level(isys_log::C__NONE);

            // Retrieve objects:
            $l_objects = [];
            if (isset($p_params['id']) && $p_params['id'] > 0) {
                $l_objects = $p_params['id'];
            }

            /**
             * @var $l_cmdb_dao isys_cmdb_dao_object_type
             */
            $l_cmdb_dao = isys_cmdb_dao_object_type::instance($g_comp_database);

            // Iterate though objects:
            $l_object_type = null;
            foreach ($l_objects as $l_object_id) {
                // Determine object type identifier:
                if (!isset($l_object_type)) {
                    $l_object_type = $l_cmdb_dao->get_objTypeID($l_object_id);
                }
            }

            // Retrieve categories:
            $l_categories = [];
            // Global categories:
            if (isset($p_params['category'])) {
                $l_catg = $p_params['category'];
                $l_categories[C__CMDB__CATEGORY__TYPE_GLOBAL] = $l_catg;
            }
            // Specific categories:
            if (isset($p_params['export_specific_catg'])) {
                $l_cats = [];
                $l_cats_dataset = $l_cmdb_dao->get_specific_category(
                    $l_object_type,
                    defined_or_default('C__RECORD_STATUS__NORMAL'),
                    null,
                    true
                )
                    ->__as_array();
                foreach ($l_cats_dataset as $l_cats_data) {
                    $l_cats[] = $l_cats_data['isysgui_cats__id'];
                }
                $l_categories[C__CMDB__CATEGORY__TYPE_SPECIFIC] = $l_cats;
            }

            // Custom categories:
            if (isset($p_params['custom_category'])) {
                $l_catc = $p_params['custom_category'];
                $l_categories[C__CMDB__CATEGORY__TYPE_CUSTOM] = $l_catc;
            }

            // Export data...
            $l_export = new isys_export_cmdb_object(
                'isys_export_type_xml',
                $g_comp_database
            );

            $l_parser = $l_export->export(
                $l_objects,
                $l_categories,
                defined_or_default('C__RECORD_STATUS__NORMAL'),
                true
            )->parse();

            $l_data = isys_glob_utf8_encode($l_parser->get_export());

            unset($l_export, $l_parser);

            // ...and import it:
            $l_import = new isys_import_handler_cmdb($l_log, $g_comp_database);
            $l_import->set_option('update-object-changed', false)
                ->set_mode(isys_import_handler_cmdb::C__APPEND)
                ->set_multivalue_categories_mode(isys_import_handler_cmdb::C__APPEND);

            if (isset($p_params['update_globals'])) {
                $l_import->set_update_globals();
            }
            $l_import->load_xml_data($l_data);

            if ($l_import->parse() === false) {
                return false;
            }

            $l_import->prepare();

            // Set title inside import method
            foreach ($l_objects as $l_object_id) {
                $l_title = null;
                if (count($l_objects) > 1) {
                    // There are more than one objects to be duplicated, so the need their names:
                    if (isset($p_params['object_title']) && $p_params['object_title'] != '') {
                        $l_title = $p_params['object_title'];
                    } else {
                        $l_title = $l_cmdb_dao->get_obj_name_by_id_as_string($l_object_id);
                    }
                } elseif (isset($p_params['object_title'])) {
                    // Only one object:
                    assert('isset($p_params["object_title"]) && is_string($p_params["object_title"])');
                    $l_title = $p_params['object_title'];
                }

                // Set title:
                if ($l_title !== null) {
                    $l_import->set_replaced_title($l_title, $l_object_id);
                }
            }

            if ($l_import->import() === false) {
                return false;
            }

            $l_object_ids = $l_import->get_object_ids();
            unset($l_import);

            $m_imported_objects = [];

            $l_auth_dao = new isys_auth_dao($g_comp_database);
            foreach ($l_objects as $l_object_id) {
                // Skip objects that could not be duplicated:
                if (!isset($l_object_ids[$l_object_id])) {
                    continue;
                }

                // Call custom duplication methods:
                if ($l_cmdb_dao->has_cat($l_object_type, [
                        'C__CATS__PERSON_GROUP',
                        'C__CATS__PERSON'
                    ]
                )) {
                    $l_auth_dao->duplicate($l_object_id, $l_object_ids[$l_object_id]);
                }

                // Handle options:
                $this->_handleOptions($l_object_ids[$l_object_id], $p_params['duplicate_options']);
                $m_imported_objects[] = $l_object_ids[$l_object_id];
            }

            $l_return['response']['imported'] = array_unique($m_imported_objects);
            unset($l_cmdb_dao);
        } catch (Exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }
        return $l_return;
    }

    private function _killProcess()
    {
        $processe = $this->m_cache_object->get("process_cancel");
        foreach ($processe as $process) {
            if ($this->m_request_id === $process) {
                echo '{"jsonrpc":"2.0","result":["Request stop."],"id":1}';
                exit;
            }
        }
        return false;
    }

    /**
     * Handles options given by the duplicate dialog.
     *
     * @global  isys_component_database $g_comp_database Database component
     *
     * @param   integer                 $p_object_id
     */
    private function _handleOptions($p_object_id, $duplicate_options)
    {
        global $g_comp_database;

        switch ($duplicate_options) {
            case 'virtualize':
                $l_dao = new isys_cmdb_dao_category_g_virtual_machine($g_comp_database);
                $l_dao->set_vm_status($p_object_id, C__VM__GUEST);
                break;

            case 'devirtualize':
                $l_dao = new isys_cmdb_dao_category_g_virtual_machine($g_comp_database);
                $l_dao->set_vm_status($p_object_id, C__VM__NO);
                break;
        }
    }

    public function check_delete($p_params)
    {
        $l_condition = '';
        $l_relation_dao = new isys_cmdb_dao_category_g_relation(isys_application::instance()->database);
        if (isset($p_params["obj_id"])) {
            $objectId = $p_params["obj_id"];
        }

        $l_relation_res = $l_relation_dao->get_data(null, $objectId, $l_condition);
        while ($relationData = $l_relation_res->get_row()) {
            $l_return[] = ["title" => $relationData["isys_obj__title"] ,
                           "obj_id" => $relationData["isys_catg_relation_list__isys_obj__id__master"],
                           "type_id" => $relationData["isys_obj__isys_obj_type__id"]];
        }

        $cable_catg = new isys_cmdb_dao_category_g_smview_cable(isys_application::instance()->database);
        $relationData = $cable_catg->find_usages($p_params['obj_id']);
        foreach ($relationData as $item) {
            if (is_array($item) == true) {
                $l_return[] = ["title" => $item[0]["title"] ,
                               "obj_id" => $item[0]["obj_id"],
                               "type_id" => $item[0]["type_id"],
                ];
            }
        }

        $cable_catg = new isys_cmdb_dao_category_g_smview_cable_pin(isys_application::instance()->database);
        $relationData = $cable_catg->find_usages($p_params['obj_id']);
        foreach ($relationData as $item) {
            if (is_array($item) == true) {
                $l_return[] = ["title" => $item[0]["title"] ,
                               "obj_id" => $item[0]["obj_id"],
                               "type_id" => $item[0]["type_id"],];
            }
        }

        $cable_catg = new isys_cmdb_dao_category_g_smview_cable_section(isys_application::instance()->database);
        $relationData = $cable_catg->find_usages($p_params['obj_id']);
        foreach ($relationData as $item) {
            if (is_array($item) == true) {
                $l_return[] = ["title" => $item[0]["title"] ,
                               "obj_id" => $item[0]["obj_id"] ,
                               "type_id" => $item[0]["type_id"],];
            }
        }

        $cable_catg = new isys_cmdb_dao_category_g_smview_cable_wire(isys_application::instance()->database);
        $relationData = $cable_catg->find_usages($p_params['obj_id']);
        foreach ($relationData as $item) {
            if (is_array($item) == true) {
                $l_return[] = ["title" => $item[0]["title"] ,
                               "obj_id" => $item[0]["obj_id"],
                               "type_id" => $item[0]["type_id"],];
            }
        }
        return ["data" => $l_return] ;
    }
}
