<?php

/**
 * i-doit APi
 *
 */
class isys_api_model_shd_object_type_groups implements isys_api_model_interface
{

    protected $m_mapping = [
        'isys_obj_type_group__id'          => 'id',
        'isys_obj_type_group__title'       => ['_L', 'title'],
        'isys_obj_type_group__description' => 'description',
        'isys_obj_type_group__const'       => 'const',
        'isys_obj_type_group__status'      => 'status',
        'isys_obj_type_group__sort'        => 'sort',
    ];

    /**
     * @param array $p_params
     *
     * @return array|mixed|string
     */
    public function read($p_params)
    {
        try {
            $l_dao = new isys_cmdb_dao_object_type(isys_application::instance()->database);

            if (!array_key_exists('filter', $p_params) || !is_array($p_params['filter'])) {
                $p_params['filter'] = [];
            }

            // Force limit to record status 'normal':
            $p_params['filter']['status'] = C__RECORD_STATUS__NORMAL;

            // Raw mode:
            $l_raw      = (bool) $p_params['raw'];
            $l_order_by = null;
            $l_sort     = null;
            $l_limit    = null;

            // Order by:
            if (isset($p_params['order_by']) && !empty($p_params['order_by'])) {
                $l_order_by = $p_params['order_by'];
            }

            // Sort:
            if (isset($p_params['sort']) && !empty($p_params['sort'])) {
                $l_sort = $p_params['sort'];
            }

            // Limitation:
            if (isset($p_params['limit']) && !empty($p_params['limit'])) {
                $l_limit = $p_params['limit'];
            }

            // Data retrieval:
            $l_data = $l_dao->get_object_type_groups_by_properties(
                $p_params['filter'],
                $l_order_by,
                $l_sort,
                $l_limit
            );

            $l_return = [];

            // Data formatting:
            while ($l_row = $l_data->get_row()) {
                $l_return[] = ($l_raw ? $l_row : $this->format_by_mapping($this->m_mapping, $l_row));
            }

            // Order by translated titles:
            uasort($l_return, function ($a, $b) {
                return ((int)$a["sort"] > (int)$b["sort"]);
            });
            $t_array = [];
            $i = 0;
            foreach ($l_return as $item) {
                $t_array[$i] = $item;
                $i++;
            }
            $l_return = $t_array;
            return $l_return;
        } catch (Exception $ex) {
            dump($ex->getMessage());
            return "";
        }
    }

    /**
     * Formats data array by mapping and encodes data to UTF-8.
     *
     * @param   array $p_mapping The mapping itself
     * @param   array $p_row     Data array
     *
     * @return  array  Formatted data array
     */
    protected function format_by_mapping(array $p_mapping, $p_row)
    {
        $l_return = [];
        try {
            foreach ($p_mapping as $l_key => $l_map) {
                if (isset($p_row[$l_key]) || is_array($l_map)) {
                    if (is_array($l_map)) {
                        if (is_callable($l_map[0])) {
                            if (strpos($l_map[0], 'str') === 0 || $l_map[0] === '_L') {
                                $l_return[$l_map[1]] = @call_user_func($l_map[0], @$p_row[$l_key]);
                            } else {
                                $l_return[$l_map[1]] = @call_user_func($l_map[0], @$p_row[$l_key], $p_row);
                            }
                        }
                    } elseif (is_scalar($l_map)) {
                        $l_return[$l_map] = $p_row[$l_key];
                    }
                }
            }
        } catch (Exception $e) {
        }
        return $l_return;
    }
}
