<?php

/**
 * i-doit APi
 *
 * @package i-doit
 * @subpackage API
 * @author Dennis Stücken <dstuecken@synetics.de>
 * @copyright synetics GmbH
 * @license http://www.i-doit.com/license
 */
class isys_api_model_shd_object_types implements isys_api_model_interface
{

    /**
     * @return array
     */
    public function connectable()
    {
        return smview_cmdb_dao_object_type::instance(isys_application::instance()->database)->get_connectableobjects();
    }

    /**
     * @return array
     */
    public function mountable()
    {
        return smview_cmdb_dao_object_type::instance(isys_application::instance()->database)->get_insertionobject();
    }

    public function read($p_params)
    {
        try {
            assert('is_array($p_params)');
            if (array_key_exists('filter', $p_params)) {
                assert('is_array($p_params["filter"])');
            } else {
                $p_params['filter'] = [];
            }

            // Force limit to record status 'normal':
            $p_params['filter']['status'] = C__RECORD_STATUS__NORMAL;

            // Raw mode:
            $l_raw = false;
            if (array_key_exists('raw', $p_params)) {
                $l_raw = filter_var($p_params['raw'], FILTER_VALIDATE_BOOLEAN);
            }

            // Order by:
            $l_order_by = array_key_exists('order_by', $p_params) ? $p_params['order_by'] : null;

            // Sort:
            $l_sort = array_key_exists('sort', $p_params) ? $p_params['sort'] : null;

            // Limitation:
            $l_limit = array_key_exists('limit', $p_params) ? $p_params['limit'] : null;

            // Show object count
            $l_count = isset($p_params['countobjects']) && $p_params['countobjects'] ? true : false;

            // Data retrieval:
            $_dao = new isys_cmdb_dao(isys_application::instance()->database, null);
            $l_data = $_dao->get_object_types_by_properties(
                $p_params['filter'],
                $l_order_by,
                $l_sort,
                $l_limit,
                $l_count
            );

            // Data formatting:
            $l_return = [];
            $isys_api_model_shd_userright = new isys_api_model_shd_userright();
            while ($l_row = $l_data->get_row()) {
                if ($l_row['isys_obj_type__id'] != C__OBJTYPE__LOCATION_GENERIC && $l_row['isys_obj_type__id'] != C__OBJTYPE__MIGRATION_OBJECT) {
                    $hasObjTypeRight = $isys_api_model_shd_userright->hasUserObjTypeRight($l_row['isys_obj_type__id']) == true;
                    $l_new = $l_row;
                    if (! $l_raw) {
                        $l_new["isys_obj_type__title"] = _L($l_new["isys_obj_type__title"]);
                        $l_new["isys_obj_type_group__title"] = _L($l_new["isys_obj_type_group__title"]);
                        $l_new["isys_obj_type__obj_img_name"] = isys_helper_link::get_base() . "images/objecttypes/" . @$l_new['isys_obj_type__obj_img_name'];

                        if ($l_count) {
                            $l_objectIds = smview_cmdb_dao_object_type::instance(isys_application::instance()->database)->get_object_id_by_type($l_row['isys_obj_type__id']);
                            $l_objectCount = 0;
                            if ($hasObjTypeRight) {
                                $l_objectCount = $l_objectIds ? count($l_objectIds) : 0;
                            } else {
                                foreach ($l_objectIds as $f_objectId) {
                                    if ($isys_api_model_shd_userright->hasUserObjRight($f_objectId["objects"]) == true) {
                                        $l_objectCount++;
                                    }
                                }
                            }
                            $l_new['objectcount'] = $l_objectCount;
                        }
                    }

                    if ($hasObjTypeRight || $l_objectCount != 0) {
                        $l_return[] = $l_new;
                    }
                }
            }

            usort($l_return, function ($a, $b) {
                return strcasecmp(strval($a['isys_obj_type__title']), strval($b['isys_obj_type__title']));
            });
            return $l_return;
        } catch (Exception $ex) {
            dump($ex->getMessage());
            return "";
        }
    }

    public function tree($p_params)
    {
        $p_params['filter']['container'] = "1";
        return smview_cmdb_dao_object_type::instance(isys_application::instance()->database)->get_object_types_by_params($p_params);
    }

    public function relation()
    {
        $l_dao_relation = isys_cmdb_dao_category_g_relation::instance(isys_application::instance()->database);
        $object_types_relation = $l_dao_relation->get_relation_type(null, null, true);
        foreach ($object_types_relation as $key => $item) {
            foreach ($item as $item_key => $value) {
                $object_types_relation[$key][$item_key] = _L($value);
            }
        }
        return $object_types_relation;
    }
}
