<?php

/**
 * SHD Objects API.
 *
 * @package    i-doit
 * @subpackage smview
 * @author     Maik Rhode <Maik.Rhode@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_api_model_shd_objects extends isys_api_model_cmdb implements isys_api_model_interface
{
    /**
     * Mandator database
     *
     * @var isys_component_database
     */
    protected $m_dao;

    function __construct()
    {
        parent::__construct();

        global $g_comp_database;

        try {
            $this->m_dao = new isys_cmdb_dao($g_comp_database);
        } catch (isys_exception $ex) {
        }
    }

    /**
     * Obsolete method.
     *
     * @param  array $p_params
     * @return null
     */
    public function read($p_params)
    {
//        if (isset($p_params['columns']) == true) {
//            foreach ($p_params['columns'] as $f_columnKey => $f_columns) {
//                $this->m_columns[defined_or_default($f_columnKey)] = $f_columns;
//            }
//        }
//
//        $this->m_db = new isys_cmdb_dao(isys_application::instance()->database);
//        /* Start looking for objects */
//        $this->m_dao = new isys_cmdb_dao_relation(isys_application::instance()->database);
//        return $this->m_return;
        return null;
    }

    /**
     * Creates an object.
     *
     * @param  array $p_params Parameters (depends on data method).
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_dao
     */
    public function create($p_params)
    {
        $l_return = [];
        $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao(isys_application::instance()->database));
        $category_data = [];
        try {
            // Object-Type.
            $p_params['obj_type_id'] = is_numeric($p_params['obj_type_id']) ? $p_params['obj_type_id'] : (defined($p_params['obj_type_id']) ? defined_or_default($p_params['obj_type_id']) : null);
            foreach ($p_params["objects"] as $objekt) {
                // properties for extended validation
                $propertiesToCheck = [
                    'title'       => null,
                    'purpose'     => null,
                    'category'    => null,
                    'cmdb_status' => null,
                    'status'      => null,
                    'location'    => null,
                ];

                $temp_id = null;
                foreach ($objekt as $column_key => $value) {
                    switch ($column_key) {
                        case "C__CATG__GLOBAL-cmdb_status":
                            $cmdbStatus = $this->_getGlobalProperty('cmdb_status', $value);
                            $objekt[$column_key] = $cmdbStatus ? $cmdbStatus['id'] : defined_or_default('C__CMDB_STATUS__IN_OPERATION', 6);
                            $propertiesToCheck['cmdb_status'] = $cmdbStatus ? $cmdbStatus['id'] : null;
                            break;

                        case "C__CATG__GLOBAL-purpose":
                            $purpose = $this->_getGlobalProperty('purpose', $value);
                            $objekt[$column_key] = $purpose ? $purpose['id'] : null;
                            $propertiesToCheck['purpose'] = $purpose ? $purpose['title'] : null;
                            break;

                        case "C__CATG__GLOBAL-category":
                            $category = $this->_getGlobalProperty('category', $value);
                            $objekt[$column_key] = $category ? $category['id'] : null;
                            $propertiesToCheck['category'] = $category ? $category['title'] : null;
                            break;

                        case "C__CATG__GLOBAL-id":
                            $temp_id = $objekt["C__CATG__GLOBAL-id"];
                            break;

                        case "C__CATG__SMVIEW_CABLE-description_sort":
                            $objekt['C__CATG__GLOBAL-title'] = $objekt["C__CATG__SMVIEW_CABLE-description_sort"];
                            unset($objekt["C__CATG__SMVIEW_CABLE-description_sort"]);
                            continue 2;
                    }

                    //built a data array for category.create
                    $split = explode("-", $column_key);
                    if ($split[0] == "C__CATG__GLOBAL") {
                        // set known properties for extended validation
                        if (array_key_exists($split[1], $propertiesToCheck)) {
                            $propertiesToCheck[$split[1]] = $propertiesToCheck[$split[1]] ?? $value;
                        }
                        continue;
                    }

                    if ($value == "-1") {
                        continue;
                    }

                    //get Dialog value
                    $value = $this->_getDialogId($split[0], $split[1], $value);

                    //check value for Int Value
                    $category_data[$split[0]][$split[1]] = $this->_checkValue($split[0], $split[1], $value);
                }

                $objekt['C__CATG__GLOBAL-status'] = $objekt['C__CATG__GLOBAL-status'] ?? defined_or_default('C__RECORD_STATUS__NORMAL', 2);
                $objekt['C__CATG__GLOBAL-cmdb_status'] = $objekt['C__CATG__GLOBAL-cmdb_status'] ?? defined_or_default('C__CMDB_STATUS__IN_OPERATION', 6);

                // validate object
                $validate_Object = [];
                foreach ($objekt as $l_key => $l_value) {
                    $split = explode("-", $l_key);
                    if ($split[0] == "C__CATG__GLOBAL") {
                        $validate_Object[$split[1]] = $l_value;
                    }
                }

                $global = isys_cmdb_dao_category_g_global::instance(isys_application::instance()->database);
                $l_message = $global->validate($validate_Object);
                if ($l_message !== true) {
                    return [
                        'message'      => 'objects.create.failure',
                        'response'     => $l_message,
                        'current_date' => date('Y-m-d')
                    ];
                }

                // check object title if cable add-on is available
                if (class_exists('idoit\Module\Cable\ObjectTitleValidator')) {
                    $validator = new \idoit\Module\Cable\ObjectTitleValidator();
                    $result = $validator->validate(null, $p_params['obj_type_id'], $propertiesToCheck);
                    if (! $result['success']) {
                        return [
                            'message'      => 'objects.create.failure',
                            'response'     => $result['message'],
                            'current_date' => date('Y-m-d')
                        ];
                    }
                }

                $objekt_id = $this->m_dao->insert_new_obj(
                    $p_params['obj_type_id'],
                    false,
                    $objekt['C__CATG__GLOBAL-title'],
                    null,
                    $objekt['C__CATG__GLOBAL-status'],
                    null,
                    null,
                    false,
                    null,
                    null,
                    null,
                    null,
                    $objekt['C__CATG__GLOBAL-category'],
                    $objekt['C__CATG__GLOBAL-purpose'],
                    $objekt['C__CATG__GLOBAL-cmdb_status'],
                    $objekt['C__CATG__GLOBAL-description']
                );

                if ($temp_id === null) {
                    $l_return[] = [$objekt_id => $objekt_id];
                } else {
                    $l_return[] = [$temp_id => $objekt_id];
                }

                foreach ($category_data as $category_key => $category_value) {
                    if ($category_key["C__CATG__SMVIEW_CABLE_ROUTE"] && isset($category_value["redundant_routes"])) {
                        $redundant_routes = explode(',', $category_value["redundant_routes"]);
                        $dao = new isys_cmdb_dao_category_g_smview_cable_route(isys_application::instance()->database);
                        $dao->update_redundant_routes($objekt_id, $redundant_routes);
                    } elseif ($category_key["C__CATG__SMVIEW_CABLE_ROUTE"] && isset($category_value["associated_route"])) {
                        $dao = new isys_cmdb_dao_category_g_smview_cable_route(isys_application::instance()->database);
                        $dao->update_associated_route($objekt_id, $category_value["associated_route"]);
                    } else {
                        $p_param = [
                            'object'   => (int)$objekt_id,
                            'category' => $category_key,
                            'data'     => $category_value
                        ];
                        $l_api_model_category->save($p_param);
                    }
                }
            }
        } catch (isys_exception_cmdb | Exception $e) {
            throw new isys_exception_api($e->getMessage());
        }

        return [
            'message'      => 'objects.create.success',
            'response'     => $l_return,
            'current_date' => date('Y-m-d')
        ];
    }

    /**
     * Get a global multi-value property.
     *
     * @param  string $propertyKey
     * @param  string $value
     * @return array|null
     * @throws Exception
     */
    private function _getGlobalProperty($propertyKey, $value)
    {
        $propertyId = null;
        if (is_numeric($value)) {
            $propertyId = $value;
        } elseif (defined($value)) {
            $propertyId = defined_or_default($value);
        }

        $tableName = null;
        switch ($propertyKey) {
            case 'cmdb_status':
                $tableName = 'isys_cmdb_status';
                break;
            case 'purpose':
                $tableName = 'isys_purpose';
                break;
            case 'category':
                $tableName = 'isys_catg_global_category';
                break;
        }

        if (empty($tableName)) {
            return null;
        }

        $dao = new isys_cmdb_dao(isys_application::instance()->database);
        $lang = isys_application::instance()->container->get('language');
        $sql =  "SELECT {$tableName}__id as id, {$tableName}__title as title FROM $tableName WHERE {$tableName}__status = " . C__RECORD_STATUS__NORMAL;
        $res = $dao->retrieve($sql);
        $globalProperty = null;
        while ($row = $res->get_row()) {
            if (($propertyId && $row['id'] == $propertyId) || (! $propertyId && $lang->get($row["title"]) == $value)) {
                $globalProperty = $row;
                break;
            }
        }
        return $globalProperty;
    }

    /**
     * Updates an object.
     *
     * @param  array $p_params Parameters (depends on data method).
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_auth
     * @throws isys_exception_database
     * @throws Exception
     */
    public function update($p_params)
    {
        $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao(isys_application::instance()->database));
        try {
            foreach ($p_params["objects"] as $objekt) {
                $category_data = [];

                foreach ($objekt as $column_key => $value) {
                    switch ($column_key) {
                        case "C__CATG__GLOBAL-cmdb_status":
                            $objekt["C__CATG__GLOBAL-cmdb_status"] = is_numeric($objekt["C__CATG__GLOBAL-cmdb_status"]) ? $objekt["C__CATG__GLOBAL-cmdb_status"] : (defined($objekt["C__CATG__GLOBAL-cmdb_status"]) ? defined_or_default($objekt["C__CATG__GLOBAL-cmdb_status"]) : null);
                            break;

                        case "C__CATG__GLOBAL-id":
                            $objekt_id = is_numeric($objekt["C__CATG__GLOBAL-id"]) ? $objekt["C__CATG__GLOBAL-id"] : null;
                            break;

                        case "C__CATG__SMVIEW_CABLE-description_sort":
                            //reset Cable-Name to Objekt-title
                            $column_key = 'C__CATG__GLOBAL-title';
                            unset($objekt["C__CATG__SMVIEW_CABLE-description_sort"]);
                            break;
                        case "C__CATG__SMVIEW_CABLE_ROUTE-is_external":
                        case "C__CATG__SMVIEW_CABLE_ROUTE-is_rented":
                        case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-is_external":
                            $value = $value == "" ? null : $value;
                            break;
                    }
                    //built a data array for category.create
                    $split = explode("-", $column_key);

                    //get Dialog value
                    $value = $this->_getDialogId($split[0], $split[1], $value);

                    //check value for Int Value
                    $category_data[$split[0]][$split[1]] = $this->_checkValue($split[0], $split[1], $value);
                }

                // properties for extended validation
                $propertiesToCheck = [
                    'title'       => null,
                    'purpose'     => null,
                    'category'    => null,
                    'cmdb_status' => null,
                    'status'      => null,
                    'location'    => null,
                ];

                // get all global values and check, if extended properties have changed
                foreach ($category_data as $category_key => $category_value) {
                    if ($category_key == "C__CATG__GLOBAL") {
                        foreach ($category_value as $k => $v) {
                            switch ($k) {
                                case "cmdb_status":
                                    $cmdbStatus = $this->_getGlobalProperty('cmdb_status', $v);
                                    $propertiesToCheck['cmdb_status'] = $cmdbStatus ? $cmdbStatus['id'] : null;
                                    break;
                                case "purpose":
                                    $purpose = $this->_getGlobalProperty('purpose', $v);
                                    $propertiesToCheck['purpose'] = $purpose ? $purpose['title'] : null;
                                    break;
                                case "category":
                                    $category = $this->_getGlobalProperty('category', $v);
                                    $propertiesToCheck['category'] = $category ? $category['title'] : null;
                                    break;
                                default:
                                    if (array_key_exists($k, $propertiesToCheck)) {
                                        $propertiesToCheck[$k] = $propertiesToCheck[$k] ?? $v;
                                    }
                            }
                        }
                    }
                    if ($category_key == "C__CATG__LOCATION") {
                        if (isset($category_value["parent"])){
                            $propertiesToCheck['location'] = $propertiesToCheck['location'] ?? $category_value["parent"];
                        }
                    }
                }

                foreach ($category_data as $category_key => $category_value) {
                    switch ($category_key) {
                        case "C__CATG__GLOBAL":
                            // check object title first if cable add-on is available
                            if (class_exists('idoit\Module\Cable\ObjectTitleValidator')) {
                                $validator = new \idoit\Module\Cable\ObjectTitleValidator();
                                $result = $validator->validate($objekt_id, null, $propertiesToCheck);
                                if (! $result['success']) {
                                    throw new isys_exception_api($result['message']);
                                }
                            }

                            $isys_cmdb_dao_category_g_global = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
                            if (isset($category_value["title"])){
                                $isys_cmdb_dao_category_g_global->save_title($objekt_id, $category_value["title"]);
                            }

                            if (isset($category_value["description"])){
                                $isys_cmdb_dao_category_g_global->save_description($objekt_id, $category_value["description"]);
                            }

                            if (isset($category_value["purpose"])){
                                $isys_api_model_cmdb_dialog = new isys_api_model_cmdb_dialog(new isys_cmdb_dao(isys_application::instance()->database));
                                $array = ["category" => "C__CATG__GLOBAL", "property" => "purpose"];
                                $r_dialog = $isys_api_model_cmdb_dialog->read($array);
                                foreach ($r_dialog as $item) {
                                    if ($item["title"] == $category_value["purpose"]) {
                                        $isys_cmdb_dao_category_g_global->save_purpose($objekt_id, $item["id"]);
                                    }
                                }

                                if ($category_value["purpose"] == "") {
                                    $isys_cmdb_dao_category_g_global->save_purpose($objekt_id, null);
                                }
                            }

                            if (isset($category_value["category"])){
                                $isys_api_model_cmdb_dialog = new isys_api_model_cmdb_dialog(new isys_cmdb_dao(isys_application::instance()->database));
                                $array = ["category" => "C__CATG__GLOBAL", "property" => "category"];
                                $r_dialog = $isys_api_model_cmdb_dialog->read($array);
                                foreach ($r_dialog as $item) {
                                    if ($item["title"] == $category_value["category"]) {
                                        $isys_cmdb_dao_category_g_global->save_category($objekt_id, $item["id"]);
                                    }
                                }

                                if($category_value["category"] == "") {
                                    $isys_cmdb_dao_category_g_global->save_category($objekt_id, null);
                                }
                            }

                            if (isset($category_value["cmdb_status"])){
                                $isys_api_model_cmdb_dialog = new isys_api_model_cmdb_dialog(new isys_cmdb_dao(isys_application::instance()->database));
                                $array = ["category" => "C__CATG__GLOBAL", "property" => "cmdb_status"];
                                $r_dialog = $isys_api_model_cmdb_dialog->read($array);

                                foreach ($r_dialog as $item) {
                                    if ($item["title"] == $category_value["cmdb_status"]) {
                                        $l_sql = "UPDATE isys_obj
                                            SET isys_obj__isys_cmdb_status__id = " . $item["id"] . "
                                            WHERE isys_obj__id = " . $objekt_id . ";";
                                        $isys_cmdb_dao_category_g_global->retrieve($l_sql);
                                    }
                                }
                                if ($category_value["cmdb_status"] == "") {
                                    $l_sql = "UPDATE isys_catg_global_list
                                            SET isys_catg_global_list__status = null
                                            WHERE isys_catg_global_list__isys_obj__id = " . $objekt_id . ";";
                                    $isys_cmdb_dao_category_g_global->retrieve($l_sql);
                                }
                            }

                            if (isset($category_value["type"])){
                                $l_sql = "UPDATE isys_obj
                                            SET isys_obj__isys_obj_type__id = " . $category_value["type"] . "
                                            WHERE isys_obj__id = " . $objekt_id . ";";
                                $isys_cmdb_dao_category_g_global->retrieve($l_sql);
                            }
                            break;
                        case "C__CATG__SMVIEW_CABLE_ROUTE":
                            if (isset($category_value["redundant_routes"])) {
                                $redundant_routes = explode(',', $category_value["redundant_routes"]);
                                $dao = new isys_cmdb_dao_category_g_smview_cable_route(isys_application::instance()->database);
                                $dao->update_redundant_routes($objekt_id, $redundant_routes);
                                break;
                            }
                            if (array_key_exists('associated_route', $category_value)) {
                                $dao = new isys_cmdb_dao_category_g_smview_cable_route(isys_application::instance()->database);
                                $dao->update_associated_route($objekt_id, $category_value["associated_route"]);
                                break;
                            }
                            // fall through
                        default:
                            $p_param = [
                                'object'   => (int)$objekt_id,
                                'category' => $category_key,
                                'data'     => $category_value
                            ];
                            $l_api_model_category->save($p_param);
                            break;
                    }
                }
            }
        } catch (isys_exception_cmdb $e) {
            throw new isys_exception_api($e->getMessage());
        }

        return [
            'message'      => 'objects.update.success',
            'response'     => true,
            'current_date' => date('Y-m-d')
        ];
    }

    /**
     * Checks an object.
     *
     * @param  array $p_params Parameters (depends on data method).
     * @return array
     * @throws isys_exception_api
     */
    public function check($p_params)
    {
        $category_data = [];
        try {
            // Object-Type.
            $p_params['obj_type_id'] = is_numeric($p_params['obj_type_id']) ? $p_params['obj_type_id'] : (defined($p_params['obj_type_id']) ? defined_or_default($p_params['obj_type_id']) : null);
            foreach ($p_params["objects"] as $objekt) {
                foreach ($objekt as $column_key => $value) {
                    switch ($column_key) {
                        case "C__CATG__GLOBAL-cmdb_status":
                            $objekt["C__CATG__GLOBAL-cmdb_status"] = is_numeric($objekt["C__CATG__GLOBAL-cmdb_status"]) ? $objekt["C__CATG__GLOBAL-cmdb_status"] : (defined($objekt["C__CATG__GLOBAL-cmdb_status"]) ? defined_or_default($objekt["C__CATG__GLOBAL-cmdb_status"]) : null);
                            break;
                    }
                    //built a data array for category.create
                    $split = explode("-", $column_key);
                    if ($split[0] == "C__CATG__GLOBAL") {
                        continue;
                    }

                    if ($value == "-1") {
                        continue;
                    }
                    $category_data[$split[0]][$split[1]] = $value;
                }

                $objekt['C__CATG__GLOBAL-status'] = isset($objekt['C__CATG__GLOBAL-status']) ? $objekt['C__CATG__GLOBAL-status'] : "2";

                //todo suche default wert
                $objekt['C__CATG__GLOBAL-cmdb_status'] = isset($objekt['C__CATG__GLOBAL-cmdb_status']) ? $objekt['C__CATG__GLOBAL-cmdb_status'] : "";

                //todo validate
                $global = isys_cmdb_dao_category_g_global::instance(isys_application::instance()->database);

                $validate_Object = [];
                foreach ($objekt as $l_key => $l_value) {
                    $split = explode("-", $l_key);
                    if ($split[0] == "C__CATG__GLOBAL") {
                        $validate_Object[$split[1]] = $l_value;
                    }
                }

                $l_message = $global->validate($validate_Object);
                if ($l_message !== true) {
                    //Wrong validate and return message
                    $return["global"] = $l_message;
                }

                $properties = smview_cmdb_dao_properties::instance(isys_application::instance()->database)->getProperties(false);
                foreach ($category_data as $category_key => $category_value) {
                    $isys_cmdb_dao_category = null;
                    foreach ($properties as $property) {
                        if ($property["const"] == $category_key) {
                            //find cmdb_dao
                            $isys_cmdb_dao_category = new $property["class"](isys_application::instance()->database);
                            break;
                        }
                    }

                    if ($isys_cmdb_dao_category != null) {
                        $l_message = $isys_cmdb_dao_category->validate($category_value);
                        if ($l_message !== true) {
                            $return[$category_key] = $l_message;
                        }
                    }
                }
            }
        } catch (isys_exception_cmdb $e) {
            throw new isys_exception_api($e->getMessage());
        }

        if ($return == []) {
            return [
                'message'      => 'objects.check.success',
                'response'     => true,
                'current_date' => date('Y-m-d')
            ];
        } else {
            return [
                'message'      => 'objects.check.failure',
                'response'     => $return,
                'current_date' => date('Y-m-d')
            ];
        }
    }

    private function _checkValue($catID, $property, $value){
        $isys_api_model_cmdb_category_info = new isys_api_model_cmdb_category_info(new isys_cmdb_dao(isys_application::instance()->database));
        $catType = stripos($catID, 'C__CATS__') !== false ? C__CMDB__GET__CATS : C__CMDB__GET__CATG;
        $catType = stripos($catID, 'C__CATG__CUSTOM') !== false ? C__CMDB__GET__CATG_CUSTOM : $catType;
        $info = $isys_api_model_cmdb_category_info->read([$catType => $catID]);
        $propertyType = $info[$property]["info"]["type"];

        if ($propertyType == "int") {
            return (int) $value;
        } elseif ($propertyType == "object_browser") {
            if ($value == "-1") {
                return null;
            } else {
                return (int) $value;
            }
        } elseif ($propertyType == "float") {
            return (float) $value;
        } elseif ($property == "id") {
            return (int) $value;
        } elseif ($value == "-1") {
            return "";
        } else {
            return $value;
        }
    }

    private function _getDialogId($catgID, $property, $value){
        if (class_exists("isys_cmdb_dao_smview_cable_group") == true) {
            if ($catgID == "C__CATG__SMVIEW_CABLE" && $property == "state_id") {
                $dao = new isys_cmdb_dao_smview_cable_user_state(isys_application::instance()->database);
                $group = $dao->get_data();
                foreach ($group as $item) {
                    $name = $item["name"] . ': ' . $item["description"];
                    if ($name == $value) {
                        return $item["id"];
                    }
                }

                if($value == ""){
                    return "NULL";
                }
            } elseif ($catgID == "C__CATG__SMVIEW_CABLE" && $property == "group_id") {
                $dao = new isys_cmdb_dao_smview_cable_group(isys_application::instance()->database);
                $ret = $this->_getDialogIdByData($dao->get_data(), $value);
                if ($ret !== null) {
                    return $ret;
                }
            } elseif ($catgID == "C__CATG__SMVIEW_CABLE" && $property  == "admin_unit") {
                $dao = new isys_cmdb_dao_smview_cable_admin_unit(isys_application::instance()->database);
                $ret = $this->_getDialogIdByData($dao->get_data(), $value);
                if ($ret !== null) {
                    return $ret;
                }
            } elseif ($catgID == "C__CATG__SMVIEW_CABLE" && $property == "type_id") {
                $dao = new isys_cmdb_dao_smview_cable_type(isys_application::instance()->database);
                $ret = $this->_getDialogIdByData($dao->get_data(), $value);
                if ($ret !== null) {
                    return $ret;
                }
            } elseif ($catgID == "C__CATG__SMVIEW_CABLE" && $property == "is_section_cable") {
                $language = isys_application::instance()->container->get('language');
                switch ($value) {
                    case $language->get("LC__UNIVERSAL__YES"):
                        return 1;
                    case $language->get("LC__UNIVERSAL__NO"):
                        return 0;
                    default:
                        return null;
                }
            } elseif ($catgID == "C__CATG__SMVIEW_CABLE" && $property == "connection_type_id") {
                $dao = new isys_cmdb_dao_smview_cable_connection_type(isys_application::instance()->database);
                $ret = $this->_getDialogIdByData($dao->get_data(), $value);
                if ($ret !== null) {
                    return $ret;
                }
            } elseif ($catgID == "C__CATG__SMVIEW_CABLE_WIRE" && $property == "user_state_id") {
                $dao = new isys_cmdb_dao_smview_cable_user_state(isys_application::instance()->database);
                $states = $dao->get_raw_user_state();
                foreach ($states as $state) {
                    $l_dialog[] = ["id" => $state["id"], "title" => $state["name"], "const" => $state["description"]];
                }
            } elseif ($catgID == "C__CATG__SMVIEW_CABLE_WIRE" && $property == "state_out") {
                $states = isys_cmdb_dao_smview_cable_wire_pin::get_wire_states();
                $ret = $this->_getDialogIdByData($states, $value);
                if ($ret !== null) {
                    return $ret;
                }
            } elseif ($catgID == "C__CATG__SMVIEW_CABLE_WIRE" && $property == "state_in") {
                $states = isys_cmdb_dao_smview_cable_wire_pin::get_wire_states();
                $ret = $this->_getDialogIdByData($states, $value);
                if ($ret !== null) {
                    return $ret;
                }
            } elseif ($catgID == "C__CATG__SMVIEW_CABLE_WIRE" && $property == "connection_type_id") {
                if ($value == ""){
                    return "NULL";
                }
            } elseif ($catgID == "C__CATG__SMVIEW_CABLE_SECTION" && $property == "user_state_id") {
                $dao = new isys_cmdb_dao_smview_cable_user_state(isys_application::instance()->database);
                $group = $dao->get_data();

                foreach ($group as $item) {
                    $name = $item["name"] . ': ' . $item["description"];
                    if ($name == $value) {
                        return $item["id"];
                    }
                }

                if($value == ""){
                    return "NULL";
                }
            } elseif (($catgID == "C__CATG__SMVIEW_CABLE_ROUTE" || $catgID == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE") && $property == "route_usage_id") {
                $dao = new isys_cmdb_dao_smview_cable_route_usage(isys_application::instance()->database);
                if (strpos($value, '(') !== false) {
                    $value = substr($value, 0, strpos($value, ' ('));
                }
                $ret = $this->_getDialogIdByData($dao->get_data(), $value);
                if ($ret !== null) {
                    return $ret;
                }
            }
        }
        return $value;
    }

    private function _getDialogIdByData($data, $value)
    {
        foreach ($data as $item) {
            if ($item["description"] == $value) {
                return $item["id"];
            }
        }
        return $value == "" ? "NULL" : null;
    }
}
