<?php

/**
 * SHD Properties API.
 *
 * @package    i-doit
 * @subpackage smview
 * @author     Maik Rhode <maik.rhode@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_api_model_shd_properties implements isys_api_model_interface
{

    /**
     * The caching instance for monitoring rows.
     *
     * @var  isys_cache_memcached
     */
    private $m_cache = null;

    /**
     * Constructor.
     */
    public function __construct()
    {
        try {
            $this->m_cache = new isys_cache_smview_memcached();
            $this->m_cache->ns('smview.properties');
        } catch (isys_exception_cache $ex) {
            $this->m_cache = null;
        }
    }

    /**
     * Read properties.
     *
     * @param  array $p_params
     * @return array
     */
    public function read($p_params)
    {
        try {
            $cacheKey = isset($p_params["withInfo"]) ? 'properties_info' : 'properties';
            $objectType = $p_params["objType"] ?? null;
            $withoutDialog = isset($p_params["withoutDialog"]) && $p_params["withoutDialog"] == '1';
            $cacheKey = $withoutDialog ? 'properties_without_dialog' : $cacheKey;
            $cacheKey .= $objectType ? '_' . $objectType : '';
            $status = $p_params["status"] ?? C__RECORD_STATUS__NORMAL;
            if (isset($this->m_cache)) {
                $properties = $this->m_cache->get($cacheKey);
                if (! empty($properties)) {
                    return $properties;
                }
            }

            if ($withoutDialog) {
                $properties = smview_cmdb_dao_properties::instance(isys_application::instance()->database)->getProperties(false, false, $objectType, $status);
            } else {
                if (isset($p_params["withInfo"])) {
                    $properties = smview_cmdb_dao_properties::instance(isys_application::instance()->database)->getProperties(false, true, $objectType, $status);
                } else {
                    $properties = smview_cmdb_dao_properties::instance(isys_application::instance()->database)->getProperties(true, false, $objectType, $status);
                }
            }

            if (isset($this->m_cache)) {
                $this->m_cache->set($cacheKey, $properties, 120);
            }
            return $properties;
        } catch (Exception $ex) {
            return [];
        }
    }

    /**
     * Read properties by ID.
     *
     * @deprecated
     * @param $p_params
     * @return array|string
     */
    public function readById($p_params)
    {
        try {
            return smview_cmdb_dao_properties::instance(isys_application::instance()->database)->getPropertiesById($p_params["const"], $p_params["property"]);
        } catch (Exception $ex) {
            var_dump($ex->getMessage());
        }
    }
}
