<?php

use idoit\Module\Api\Exception\ValidationException;

/**
 * i-doit APi
 *
 * @package i-doit
 * @subpackage API
 * @author Dennis Stücken <dstuecken@synetics.de>
 * @copyright synetics GmbH
 * @license http://www.i-doit.com/license
 */

class isys_api_model_shd_rack implements isys_api_model_interface
{
    /**
     * Database component.
     *
     * @var isys_cmdb_dao
     */
    protected $m_cmdb_dao;

    protected $rack_id;

    /**
     * @param  array $p_params
     * @return array
     * @throws ValidationException
     * @throws isys_exception_api
     * @throws isys_exception_auth
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    public function read($p_params)
    {
        if (!isset($p_params['objId'])) {
            throw new isys_exception_general("objId not set.");
        }

        $this->rack_id = $p_params['objId'];
        $isys_api_model_cmdb_object = new isys_api_model_cmdb_object($this->m_cmdb_dao);
        $isys_api_model_cmdb_category = new isys_api_model_cmdb_category($this->m_cmdb_dao);

        $l_rack = $isys_api_model_cmdb_object->read(['id' => $this->rack_id]);
        if ($l_rack['objecttype'] != defined_or_default('C__OBJTYPE__ENCLOSURE')) {
            throw new isys_exception_general("not an rack.");
        }

        $t_enclosure = $isys_api_model_cmdb_category->read(['objID' => $this->rack_id, 'category' => 'C__CATS__ENCLOSURE'])[0];
        if (isset($t_enclosure["slot_sorting"]["id"])) {
            $t_enclosure["slot_sorting"] = $t_enclosure["slot_sorting"]["id"];
        }

        $l_rack["enclosure"] = $t_enclosure;
        $l_rack["formfactor"] = $isys_api_model_cmdb_category->read(['objID' => $this->rack_id, 'category' => 'C__CATG__FORMFACTOR'])[0];
        $l_rack = $this->get_half_he_property($this->rack_id, $l_rack);
        $l_rack["devices"] = $this->getDevices();
        return $l_rack;
    }

    /**
     * @param  array $p_params
     * @return array
     * @throws isys_exception_database
     */
    public function assign_object($p_params)
    {
        return $this->assign_object_to_rack($p_params);
    }

    /**
     * @param array $p_params
     * @return array
     */
    public function detach_object($p_params)
    {
        return $this->detach_object_from_rack($p_params);
    }

    /**
     * @param  array $p_params
     * @return array
     * @throws isys_exception_database
     */
    public function check_is_something_in_rack($p_params)
    {
        $height = $p_params["height"] ?? 1;
        $insertion = $p_params["insertion"] ?? null;
        $excludedObjId = $p_params["exclude_obj_id"] ?? null;
        $l_res = isys_cmdb_dao_location::instance($this->m_cmdb_dao->get_database_component())
            ->get_location($p_params["obj_id"], null, C__RECORD_STATUS__NORMAL, null, false);

        $l_rack = isys_cmdb_dao_category_s_enclosure::instance($this->m_cmdb_dao->get_database_component())
            ->get_data(null, $p_params["obj_id"])
            ->get_row();

        $l_formfactor = isys_cmdb_dao_category_g_formfactor::instance($this->m_cmdb_dao->get_database_component())
            ->get_data(null, $p_params["obj_id"])
            ->get_row();

        if (isset($l_formfactor["isys_catg_formfactor_list__rackunits"])) {
            $rackunits = $l_formfactor["isys_catg_formfactor_list__rackunits"];
        } else {
            return [false];
        }

        $pos = (int) $p_params["pos"];
        $startPos = $l_rack["isys_cats_enclosure_list__slot_sorting"] == "desc" ? $pos - $height + 1 : $pos;
        $rangeToBeAvailable = array_fill($startPos, $height, 1);
        while ($l_row = $l_res->get_row()) {
            $itemPos = (int) $l_row['isys_catg_location_list__pos'];
            if ($itemPos == 0) {
                continue;
            }

            if ($l_row['isys_obj__id'] == $excludedObjId) {
                continue;
            }

            if (isset($insertion) && $l_row['isys_catg_location_list__insertion'] != $insertion) {
                continue;
            }

            $itemHeight = (int) $l_row['isys_catg_formfactor_list__rackunits'];
            $rangeInUse = array_fill($itemPos, $itemHeight, 1);
            if ($l_rack["isys_cats_enclosure_list__slot_sorting"] == "desc") {
                // calculate descending item position
                $itemPos = $rackunits - $itemPos + 1;
                $rangeInUse = array_fill($itemPos - $itemHeight + 1, $itemHeight, 1);
            }

            $intersectingPositions = array_intersect_key($rangeToBeAvailable, $rangeInUse);
            if (count($intersectingPositions) > 0) {
                // there are some intersecting positions so this element can not be placed here
                return [true];
            }
        }
        return [false];
    }

    /**
     * @param  array $p_params
     * @return array
     * @throws isys_exception_database
     */
    public function get_rack($p_params)
    {
        $rack_array = [];
        $l_res = isys_cmdb_dao_location::instance($this->m_cmdb_dao->get_database_component())
            ->get_location($p_params["obj_id"], null, C__RECORD_STATUS__NORMAL, null, false);

        $l_rack = isys_cmdb_dao_category_s_enclosure::instance($this->m_cmdb_dao->get_database_component())
            ->get_data(null, $p_params["obj_id"])
            ->get_row();

        $l_formfactor = isys_cmdb_dao_category_g_formfactor::instance($this->m_cmdb_dao->get_database_component())
            ->get_data(null, $p_params["obj_id"])
            ->get_row();

        if (isset($l_formfactor["isys_catg_formfactor_list__rackunits"])) {
            $rackunits = $l_formfactor["isys_catg_formfactor_list__rackunits"];
            // sorting DESC or ASC
            if($l_rack["isys_cats_enclosure_list__slot_sorting"] == "asc") {
                for ($i = 1; $i <= $rackunits; $i++) {
                    $rack_array[$i] = "";
                }
            } else {
                for ($i = $rackunits; $i >= 1; $i--) {
                    $rack_array[$i] = "";
                }
            }
        } else {
            return [false];
        }

        if ($l_rack["isys_cats_enclosure_list__slot_sorting"] == "desc") {
            while ($l_row = $l_res->get_row()) {
                if ($l_row["isys_catg_location_list__pos"] != null ) {
                    //              47      -               33                       + 1
                    $l_new_pos = $rackunits - $l_row["isys_catg_location_list__pos"] + 1;
                    if ($l_row["isys_catg_formfactor_list__rackunits"] > 1) {
                        for ($f_rackunit = 0; $f_rackunit < $l_row["isys_catg_formfactor_list__rackunits"]; $f_rackunit++) {
                            $rack_array[$l_new_pos - $f_rackunit] = $l_row["isys_obj__id"];
                        }
                    } else {
                        $rack_array[$l_new_pos] = $l_row["isys_obj__id"];
                    }
                }
            }
        } else {
            while ($l_row = $l_res->get_row()) {
                if ($l_row["isys_catg_location_list__pos"] != null ) {
                    $l_new_pos = $l_row["isys_catg_location_list__pos"];
                    if ($l_row["isys_catg_formfactor_list__rackunits"] > 1) {
                        for ($f_rackunit = 0; $f_rackunit < $l_row["isys_catg_formfactor_list__rackunits"]; $f_rackunit++) {
                            $rack_array[$l_new_pos + $f_rackunit] = $l_row["isys_obj__id"];
                        }
                    } else {
                        $rack_array[$l_new_pos] = $l_row["isys_obj__id"];
                    }
                }
            }
        }
        return $rack_array;
    }

    protected function getDevices()
    {
        $lang = isys_application::instance()->container->get('language');
        $globalDir = 'upload/files';

        $l_cmdb_res = isys_cmdb_dao_status::instance($this->m_cmdb_dao->get_database_component())->get_cmdb_status();
        $l_objects  = $l_cmdb_status = [];
        while ($l_row = $l_cmdb_res->get_row()) {
            $l_cmdb_status[$l_row['isys_cmdb_status__id']] = [
                'color' => '#' . $l_row['isys_cmdb_status__color'],
                'title' => isys_glob_htmlentities($lang->get($l_row['isys_cmdb_status__title']))
            ];
        }

        $l_res = isys_cmdb_dao_location::instance($this->m_cmdb_dao->get_database_component())
            ->get_location($this->rack_id, null, C__RECORD_STATUS__NORMAL, null, false, ' AND isys_obj_type__show_in_rack = 1');

        // We prepare all objects in one array, so we can just assign them later on.
        while ($l_row = $l_res->get_row()) {
            $l_obj_formfactor = ['isys_catg_formfactor_type__title' => isys_tenantsettings::get('gui.empty_value', '-')];
            if ($l_row['isys_catg_formfactor_list__isys_catg_formfactor_type__id'] > 0) {
                $l_obj_formfactor = isys_factory_cmdb_dialog_dao::get_instance('isys_catg_formfactor_type', $this->m_cmdb_dao->get_database_component())
                    ->get_data($l_row['isys_catg_formfactor_list__isys_catg_formfactor_type__id']);
            }

            $l_rack_units = $l_row['isys_catg_formfactor_list__rackunits'] ?: 1;
            $l_object = [
                'id'                    => $l_row['isys_obj__id'],
                'title'                 => $l_row['isys_obj__title'],
                'type'                  => $lang->get($l_row['isys_obj_type__title']),
                'type_id'               => $lang->get($l_row['isys_obj_type__id']),
                'formfactor'            => $l_obj_formfactor['isys_catg_formfactor_type__title'],
                'icon'                  => $l_row['isys_obj_type__icon'],
                'color'                 => '#' . $l_row['isys_obj_type__color'],
                'cmdb_color'            => $l_cmdb_status[$l_row['isys_obj__isys_cmdb_status__id']]['color'],
                'cmdb_status'           => $l_cmdb_status[$l_row['isys_obj__isys_cmdb_status__id']]['title'],
                'cmdb_status_id'        => $l_row['isys_obj__isys_cmdb_status__id'],
                'height'                => $l_rack_units,
                'option'                => $l_row['isys_catg_location_list__option'],
                'insertion'             => $l_row['isys_catg_location_list__insertion'],
                'image'                 => '',
                'image_id'              => '',
                'image_checksumme'      => '',
                'image_back'            => '',
                'image_id_back'         => '',
                'image_checksumme_back' => '',
                'pos'                   => $l_row['isys_catg_location_list__pos']
            ];

            $images = $this->_get_dimages($l_row["isys_obj__id"]);
            foreach ($images as $image) {
                $file_title = $image["isys_file_physical__filename"];
                $dimage = $globalDir . DS . $file_title;

                $isBackImage = strpos($file_title, '_back_') !== false || strpos($file_title, '_rear_') !== false;
                if ($isBackImage) {
                    $l_object['image_back'] = $dimage;
                    $l_object['image_id_back'] = $image["isys_cats_file_list__isys_obj__id"];
                    $l_object['image_checksumme_back'] = $image['isys_file_physical__md5'];
                } else {
                    $l_object['image'] = $dimage;
                    $l_object['image_id'] = $image["isys_cats_file_list__isys_obj__id"];
                    $l_object['image_checksumme'] = $image['isys_file_physical__md5'];
                }
            }

            $l_object = $this->get_half_he_property($l_row['isys_obj__id'], $l_object);
            $l_objects[$l_row['isys_obj__id']] = $l_object;
        }
        return $l_objects;
    }

    /**
     * Method for assigning an object to a rack (position and insertion).
     *
     * @param   array $p_params
     * @return  array
     * @throws  isys_exception_database
     * @author  Maik Rhode <maik.rhode@shd-online.de>
     */
    protected function assign_object_to_rack($p_params)
    {
        //if rack_id neue zuweisung
        isys_cmdb_dao_location::instance($this->m_cmdb_dao->get_database_component())
            ->get_location_by_object_id($p_params['device_obj_id'])->get_row();

        $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao($this->m_cmdb_dao->get_database_component()));
        $p_param = [
            'object'   => intval($p_params['device_obj_id']),
            'category' => 'C__CATG__LOCATION',
            'data'     => [ 'parent' => intval($p_params['parent_obj_id'])]
        ];

        $l_api_model_category->save($p_param);
        isys_cmdb_dao_location::instance($this->m_cmdb_dao->get_database_component())
            ->update_position($p_params['device_obj_id'], $p_params['rack_option'], (int)$p_params['insertion'], $p_params['pos']);

        return [
            'message'      => 'rack.assign_object.success',
            'response'     => true,
            'current_date' => date('Y-m-d')
        ];
    }

    /**
     * Method for detaching an objects location.
     *
     * @param   array $p_params
     * @return  array
     * @author  Maik Rhode <maik.rhode@shd-online.de>
     */
    protected function detach_object_from_rack($p_params)
    {
        isys_cmdb_dao_location::instance($this->m_cmdb_dao->get_database_component())
            ->update_position($p_params["device_obj_id"]);

        return [
            'message'      => 'rack.detach_object.success',
            'response'     => true,
            'current_date' => date('Y-m-d')
        ];
    }

    public function get_half_he_property($objectId, $array)
    {
        $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao(isys_application::instance()->database));
        $p_param = [
            'objID'    => $objectId,
            'category' => "C__CATG__SMVIEW_PATCH_FORMFACTOR"
        ];

        $l_categoryData = $l_api_model_category->read($p_param);
        foreach ($l_categoryData as $l_item) {
            foreach ($l_item as $l_property) {
                if (is_array($l_property) == true && $l_property["id"] == 1) {
                    $array["half_he"] = true;
                }
            }
        }

        if (!isset($array["half_he"])) {
            $array["half_he"] = false;
        }
        return $array;
    }

    public function __construct()
    {
        global $g_comp_database;

        $this->m_cmdb_dao = new isys_cmdb_dao($g_comp_database);
    }

    /**
     * Get all images containing "dimage" within file name.
     *
     * @param  int $obj_id
     * @return array
     */
    private function _get_dimages($obj_id)
    {
        try {
            $l_sql = "SELECT * FROM isys_catg_file_list
			LEFT JOIN isys_connection ON isys_connection__id = isys_catg_file_list__isys_connection__id
			LEFT JOIN isys_cats_file_list ON isys_cats_file_list__isys_obj__id = isys_connection__isys_obj__id
			LEFT JOIN isys_file_version ON isys_file_version__id = isys_cats_file_list__isys_file_version__id
            LEFT JOIN isys_file_physical ON isys_file_physical__id = isys_file_version__isys_file_physical__id
            WHERE isys_catg_file_list.isys_catg_file_list__status = " . C__RECORD_STATUS__NORMAL .
                " AND isys_catg_file_list.isys_catg_file_list__isys_obj__id = " . $obj_id;
            $res = $this->m_cmdb_dao->retrieve($l_sql);
            $images = [];
            while ($row = $res->get_row()) {
                if (strpos($row['isys_file_physical__filename'], 'dimage') !== false) {
                    $images[] = $row;
                }
            }
            return $images;
        } catch (Exception $ex) {
            return [];
        }
    }
}
