<?php

/**
 * shd API Insights
 *
 * @package shd
 * @subpackage API
 * @author Maik Rhode <maik.rhode@shd-online.de>
 */
class isys_api_model_shd_userright implements isys_api_model_interface
{

    /**
     * Database component.
     *
     * @var isys_component_database
     */
    protected $m_user_path;

    /**
     * CMDB module.
     *
     * @var array
     */
    private $cmdbModul;

    public function __construct()
    {
        global $g_comp_session;

        $this->m_user_path = isys_api_model_shd_userright::get_path($g_comp_session->get_user_id());
        $moduleId = defined_or_default('C__MODULE__CMDB');
        $this->cmdbModul = $this->m_user_path["modules"][$moduleId];
    }

    /**
     * Read Insights Entry
     *
     * @param array $p_params
     * @return array
     */
    public function read($p_params)
    {
        global $g_comp_session;

        //get User_groups
        $l_usergroup_res = isys_cmdb_dao_list_cats_person_assigned_groups::instance(isys_application::instance()->database)->get_result(null, $g_comp_session->get_user_id());
        $l_usergroup = [];
        while ($l_row = $l_usergroup_res->get_row()) {
            $l_usergroup[$l_row["isys_person_2_group__id"]] = $l_row;
        }

        $l_return = isys_api_model_shd_userright::get_path($g_comp_session->get_user_id());
        $l_return["user_group"] = $l_usergroup;
        return $l_return;
    }

    /**
     * Read Insights Entry
     *
     * @param array $p_params
     * @return array
     */
    public function readByCategorie($p_params)
    {
        global $g_comp_session;

        $l_rights = isys_api_model_shd_userright::get_path($g_comp_session->get_user_id());
        $return = $l_rights["modules"]["2"]["paths"]["obj_in_type"][strtolower($p_params["obj_type"])];
        if (isset($l_rights["modules"]["2"]["group_paths"]["obj_in_type"][strtolower($p_params["obj_type"])]) == true) {
            $return = $return + $l_rights["modules"]["2"]["group_paths"]["obj_in_type"][strtolower($p_params["obj_type"])];
        }
        return $return;
    }

    /**
     * Read Insights Entry
     *
     * @param integer $obj_type_id
     * @return boolean
     */
    public function hasUserRight($p_obj_id, $obj_type_id)
    {
        $hasRight = $this->hasUserObjRight($p_obj_id);
        if (! $hasRight) {
            $hasRight = $this->hasUserObjTypeRight($obj_type_id);
        }
        return $hasRight;
    }

    /**
     * Read Insights Entry
     *
     * @param integer $obj_type_id
     * @return boolean
     */
    public function hasUserObjTypeRight($obj_type_id)
    {
        $hasRight = false;
        $cmdb_modul = $this->cmdbModul;
        foreach ($cmdb_modul["paths"]["obj_in_type"] as $objTypeKey => $objType) {
            $key = defined_or_default(strtoupper($objTypeKey));
            if ($key == $obj_type_id) {
                $hasRight = true;
            }

            //user has all rights
            if ($objTypeKey == "*") {
                $hasRight = true;
            }
        }

        //all rights from group
        foreach ($cmdb_modul["group_paths"]["obj_in_type"] as $objTypeKey => $objType) {
            $key = defined_or_default(strtoupper($objTypeKey));
            if ($key == $obj_type_id) {
                $hasRight = true;
            }

            //user has all rights
            if ($objTypeKey == "*") {
                $hasRight = true;
            }
        }
        return $hasRight;
    }

    /**
     * Read Insights Entry
     *
     * @param integer $obj_type_id
     * @return boolean
     */
    public function hasUserObjTypeWrite($obj_type_id)
    {
        $hasRight = false;
        $cmdb_modul = $this->cmdbModul;
        foreach ($cmdb_modul["paths"]["obj_in_type"] as $objTypeKey => $objType) {
            $key = defined_or_default(strtoupper($objTypeKey));
            $id = defined_or_default(strtoupper($obj_type_id));
            if ($key == $id) {
                foreach ($objType as $item) {
                    if ($item == 2 || $item == 32) {
                        return true;
                    }
                }
            }

            //user has all rights
            if ($objTypeKey == "*") {
                return true;
            }
        }

        //all rights from group
        foreach ($cmdb_modul["group_paths"]["obj_in_type"] as $objTypeKey => $objType) {
            foreach ($objType as $item) {
                if ($item == 2 || $item == 32) {
                    return true;
                }
            }

            //user has all rights
            if ($objTypeKey == "*") {
                return true;
            }
        }
        return $hasRight;
    }

    /**
     * Read Insights Entry
     *
     * @param  $modul
     * @param  $category
     * @param  $p_right
     * @return boolean
     */
    public function hasUserModulCategoryRight($modul, $category, $p_right)
    {
        $hasRight = false;
        $cmdb_modul = $this->m_user_path["modules"][$modul];
        foreach ($cmdb_modul["paths"] as $objTypeKey => $array_ids) {
            if (strtolower($objTypeKey) == strtolower($category)) {
                foreach ($array_ids as $ids) {
                    foreach ($ids as $f_rights) {
                        if ($f_rights >= $p_right) {
                            return true;
                        }
                    }
                }
            }
        }

        //all rights from group
        foreach ($cmdb_modul["group_paths"] as $objTypeKey => $array_ids) {
            if (strtolower($objTypeKey) == strtolower($category)) {
                foreach ($array_ids as $ids) {
                    foreach ($ids as $f_rights) {
                        if ($f_rights >= $p_right) {
                            return true;
                        }
                    }
                }
            }
        }
        return $hasRight;
    }

    /**
     * Read Insights Entry
     *
     * @param integer $p_obj_id
     * @return boolean
     */
    public function hasUserObjRight($p_obj_id)
    {
        $hasRight = false;
        $cmdb_modul = $this->cmdbModul;
        foreach ($cmdb_modul["paths"]["obj_id"] as $objIdKey => $f_objId) {
            if ($objIdKey == $p_obj_id) {
                $hasRight = true;
            }

            //user has all rights
            if ($objIdKey == "*") {
                $hasRight = true;
            }
        }

        //all rights from group
        foreach ($cmdb_modul["group_paths"]["obj_id"] as $objIdKey => $f_objId) {
            if ($objIdKey == $p_obj_id) {
                $hasRight = true;
            }

            //user has all rights
            if ($objIdKey == "*") {
                $hasRight = true;
            }
        }
        return $hasRight;
    }

    /**
     * Read Insights Entry
     *
     * @param  int $p_category_id
     * @return boolean
     */
    public function hasUserCategoryRight($p_category_id)
    {
        $hasRight = false;
        $cmdb_modul = $this->cmdbModul;
        if ($cmdb_modul["paths"]) {
            foreach ($cmdb_modul["paths"]["category"] as $objIdKey => $f_objId) {
                if (strtolower($objIdKey) == strtolower($p_category_id)) {
                    $hasRight = true;
                }

                //user has all rights
                if ($objIdKey == "*") {
                    $hasRight = true;
                }
            }
        }

        //all rights from group
        foreach ($cmdb_modul["group_paths"]["category"] as $objIdKey => $f_objId) {
            //make str "c__objtype__server" upper and check ConstID
            if (strtolower($objIdKey) == strtolower($p_category_id)) {
                $hasRight = true;
            }

            //user has all rights
            if ($objIdKey == "*") {
                $hasRight = true;
            }
        }
        return $hasRight;
    }

    private static function get_path($p_obj_id)
    {
        $l_paths = [];
        $l_dao = isys_auth_dao::instance(isys_application::instance()->database);
        $l_res = $l_dao->get_paths($p_obj_id);
        if ($l_res->count() > 0) {
            while ($l_row = $l_res->get_row()) {
                // This needs to be done, for the reference to work.
                $moduleId = $l_row['isys_auth__isys_module__id'];
                if ($l_paths[$moduleId] === null) {
                    $l_paths[$moduleId] = self::_get_module_data($moduleId);
                }

                // Add the user specific paths.
                $l_dao->build_path($l_paths[$moduleId]['paths'], $l_row);
            }
        }

        // Check, if the given obj-id is a person, so we can load the inherited rights.
        $l_obj_type = isys_cmdb_dao::instance(isys_application::instance()->database)->get_objTypeID($p_obj_id);
        if ($l_obj_type != defined_or_default('C__OBJTYPE__PERSON_GROUP')) {
            $l_res = $l_dao->get_group_paths_by_person($p_obj_id);
            if ($l_res->count() > 0) {
                while ($l_row = $l_res->get_row()) {
                    // This needs to be done, for the reference to work.
                    $moduleId = $l_row['isys_auth__isys_module__id'];
                    if ($l_paths[$moduleId] === null) {
                        $l_paths[$moduleId] = self::_get_module_data($moduleId);
                    }

                    // Add the user specific paths.
                    $l_dao->build_path($l_paths[$moduleId]['group_paths'], $l_row);
                }
            }
        }

        // Retrieve the rights and make sure, the titles are UTF8.
        $l_rights = isys_auth::get_rights();
        return [
            'method'      => 'obj-id',
            'auth_rights' => $l_rights,
            'modules'     => $l_paths
        ];
    }

    private static function _get_module_data($moduleId)
    {
        $l_module = isys_module_manager::instance()->get_modules($moduleId)->get_row();

        $language = isys_application::instance()->container->get('language');
        $l_module['isys_module__title'] = $language->get($l_module['isys_module__title']);

        // Prepare the module specific methods.
        $l_methods = [];
        $l_auth_instance = isys_module_manager::instance()->get_module_auth($moduleId);
        if ($l_auth_instance) {
            $l_methods = $l_auth_instance->get_auth_methods();
        }

        return [
            'paths'       => [],
            'group_paths' => [],
            'info'        => [
                'data'    => $l_module,
                'methods' => $l_methods
            ]
        ];
    }
}
