const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const { resolve } = require('path');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');

module.exports = {
  entry: {
    updater:[
      './src/updater/index.tsx',
      './../sass/updater.scss'
    ]
  },
  output: {
    filename: '[name]/[name].[contenthash].js',
    path: resolve(__dirname, './../dist/'),
    publicPath: 'src/classes/modules/smview/assets/dist/',
  },
  resolve: {
    cacheWithContext: false,
    extensions: ['.ts', '.tsx', '.js', '.json'],
    symlinks: false,
  },
  module: {
    rules: [
      {
        test: /\.tsx?$/,
        include: resolve(__dirname, 'src'),
        use: 'ts-loader',
        exclude: /node_modules/,
      },
      // sass to css file
      {
        test: /\.(scss)$/,
        include: resolve(__dirname, './../sass'),
        use: [MiniCssExtractPlugin.loader, 'css-loader', 'sass-loader'],
        sideEffects: true
      }
    ]
  },
  optimization: {
    minimize: false,
    // separate 3rd-party libs because they won't change so often
    moduleIds: 'deterministic',
    chunkIds: 'named',
    splitChunks: {
      chunks: 'async',
      maxAsyncRequests: 6,
      maxInitialRequests: 4,
      cacheGroups: {
        vendors: {
          test: /[\\/]node_modules[\\/]/,
          name: 'vendors',
          chunks: 'all',
          filename: 'vendors.[contenthash].js'
        }
      }
    }
  },
  plugins: [
    new CleanWebpackPlugin(),
    new HtmlWebpackPlugin({
      path: resolve(__dirname, './../dist/'),
      inject: false,
      chunks: ['updater'],
      filename: resolve(__dirname, './../dist/updater/index.html'),
      templateContent: ({htmlWebpackPlugin}) => `${htmlWebpackPlugin.tags.headTags} ${htmlWebpackPlugin.tags.bodyTags}`
    }),
    new MiniCssExtractPlugin({
      filename: '[name]/style.css'
    }),
  ]
};
