<?php

/**
 * i-doit
 * Auth: Class for Notifications module authorization rules.
 *
 * @package     i-doit
 * @subpackage  auth
 * @author      Van Quyen Hoang <qhoang@i-doit.com>
 * @copyright   synetics GmbH
 * @license     http://www.i-doit.com/license
 */
class isys_auth_smview extends isys_auth implements isys_auth_interface
{
    /**
     * Container for singleton instance.
     *
     * @var  isys_auth_smview
     */
    private static $m_instance = null;

    /**
     * Retrieve singleton instance of authorization class.
     *
     * @return  isys_auth_smview
     */
    public static function instance()
    {
        global $g_comp_database;

        // If the DAO has not been loaded yet, we initialize it now.
        if (self::$m_dao === null) {
            self::$m_dao = new isys_auth_dao($g_comp_database);
        }

        if (self::$m_instance === null) {
            self::$m_instance = new self;
        }
        return self::$m_instance;
    }

    /**
     * Method for returning the available auth-methods. This will be used for the GUI.
     *
     * @return array
     * @throws Exception
     */
    public function get_auth_methods()
    {
        $lang = isys_application::instance()->container->get('language');
        return [
            'smview__maps' => [
                'title'  => $lang->get('LC__MODULE__SMVIEW__MAPS'),
                'type'   => 'boolean',
                'rights' => array(isys_auth::VIEW, isys_auth::EDIT, isys_auth::CREATE, isys_auth::DELETE )
            ],
            'smview__insight' => [
                'title'  => $lang->get('LC__MODULE__SMVIEW__INSIGHT'),
                'type'   => 'boolean',
                'rights' => array(isys_auth::VIEW, isys_auth::EDIT, isys_auth::CREATE, isys_auth::DELETE )
            ],
            'smview__bsm' => [
                'title'  => $lang->get('LC__MODULE__SMVIEW__BSM'),
                'type'   => 'boolean',
                'rights' => array(isys_auth::VIEW, isys_auth::EDIT, isys_auth::CREATE, isys_auth::DELETE )
            ],
            'smview__patch' => [
                'title'  => $lang->get('LC__MODULE__SMVIEW__PATCH'),
                'type'   => 'boolean',
                'rights' => array(isys_auth::VIEW, isys_auth::EDIT, isys_auth::CREATE, isys_auth::DELETE )
            ],
            'smview__cable' => [
                'title'  => $lang->get('LC__MODULE__SMVIEW__CABLE'),
                'type'   => 'boolean',
                'rights' => array(isys_auth::VIEW, isys_auth::EDIT, isys_auth::CREATE, isys_auth::DELETE )
            ],
            'smview__tv' => [
                'title'  => $lang->get('LC__MODULE__SMVIEW__TV'),
                'type'   => 'boolean',
                'rights' => array(isys_auth::VIEW, isys_auth::EDIT, isys_auth::CREATE, isys_auth::DELETE )
            ],
        ];
    }

    /**
     * Get ID of related module.
     *
     * @return  int
     */
    public function get_module_id()
    {
        return defined_or_default('C__MODULE__SMVIEW');
    }

    /**
     * Get title of related module.
     *
     * @return  string
     */
    public function get_module_title()
    {
        return "LC__MODULE__SMVIEW";
    }
}
