<?php
/**
 * Class smview_cmdb_dao_action
 */

class smview_cmdb_dao_category extends isys_api_model_cmdb_category
{

    /**
     * Read category data.
     * Parameter example:
     * [
     *    'objID'  => 132,
     *    'catgID' => 10
     * ]
     *
     * Or:
     * [
     *    'objID'  => 132,
     *    'catsID' => 20
     * ]
     *
     * Or:
     * [
     *    'objID'    => 132,
     *    'category' => "C__CATG__CPU"
     * ]
     *
     * @param array $p_params
     *
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function read_all($p_params)
    {
        // Init
        $l_return = [];
        $i = 0;

        // Process data
        $l_cat = $this->prepare($p_params);
        if (! $l_cat) {
            return $l_return;
        }

        if (!method_exists($l_cat, 'get_data')) {
            throw new isys_exception_api('get_data method does not exist for ' . get_class($l_cat), -32601);
        }

        // Status-Handling
        if (isset($p_params['status'])) {
            $l_status = is_numeric($p_params['status']) ? $p_params['status'] : (defined($p_params['status']) ? defined_or_default($p_params['status']) : null);
        } else {
            $l_status = C__RECORD_STATUS__NORMAL;
        }

        if (isset($p_params["condition"])) {
            $l_conditions = $p_params["condition"];
        } else {
            $l_conditions = null;
        }

        $l_catentries = $l_cat->get_data_as_array(null, null, $l_conditions, null, $l_status);

        // Count category result
        if (! count($l_catentries)) {
            return $l_return;
        }

        if (!isset($p_params['raw']) || !$p_params['raw']) {
            $l_properties = $l_cat->get_properties();
            $l_content_field = $this->get_content_field();

            // Format category result
            foreach ($l_catentries as $l_row) {
                // Set object and category entry id
                $l_return[$i]['id'] = $l_row[$this->get_category_source_table('__id')];
                $l_return[$i]['objID'] = $l_row[$this->get_category_source_table('__isys_obj__id')];

                // Property-Handling
                foreach ($l_properties as $l_key => $l_propdata) {
                    if (! is_string($l_key)) {
                        continue;
                    }

                    if (isset($l_propdata[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][0]) &&
                        isset($l_propdata[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1])) {
                        if (is_object($l_propdata[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][0])) {
                            $l_method = $l_propdata[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1];
                            $l_return[$i][$l_key] = $l_propdata[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][0]->$l_method(
                                $l_row[$l_propdata[C__PROPERTY__DATA][$l_content_field]],
                                $l_row
                            );
                        } // Call helper object to retrieve more information
                        elseif (class_exists($l_propdata[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][0])) {
                            $l_helper = new $l_propdata[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][0]($l_row, $this->m_dao->get_database_component(),
                                $l_propdata[C__PROPERTY__DATA], $l_propdata[C__PROPERTY__FORMAT], $l_propdata[C__PROPERTY__UI]);

                            // Set the Unit constant for the convert-helper
                            if ($l_propdata[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] == 'convert') {
                                if (method_exists($l_helper, 'set_unit_const')) {
                                    $l_row_unit = $l_properties[$l_propdata[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__UNIT]][C__PROPERTY__DATA][$l_content_field];
                                    $l_helper->set_unit_const($l_row[$l_row_unit]);
                                }
                            }

                            // Call callback
                            $l_return[$i][$l_key] = call_user_func([
                                $l_helper,
                                $l_propdata[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1]
                            ], $l_row[$l_propdata[C__PROPERTY__DATA][$l_content_field]]);

                            // Set data id
                            $p_params['data']['id'] = $this->get_category_id();

                            // Retrieve data from isys_export_data
                            if ($l_return[$i][$l_key] instanceof isys_export_data) {
                                $l_export_data = $l_return[$i][$l_key];

                                /** @var isys_export_data $l_export_data */
                                $l_return[$i][$l_key] = $l_export_data->get_data();
                            }
                        }
                    } else {
                        $l_return[$i][$l_key] = $l_row[$l_propdata[C__PROPERTY__DATA][$l_content_field]];
                        if ($l_propdata[C__PROPERTY__DATA][C__PROPERTY__DATA__ENCRYPT]) {
                            // @see  API-28
                            $l_return[$i][$l_key] = isys_helper_crypt::decrypt($l_return[$i][$l_key]);
                        }
                    }
                    unset($l_helper_class, $l_helper);
                }
                $i++;
            }
        } else {
            $l_return = $l_catentries;
        }
        return $l_return;
    }
}
