<?php
/**
 * Class smview_cmdb_dao_action
 */

class smview_cmdb_dao_category_property extends isys_cmdb_dao_category_property
{

    /**
     * Method for retrieving properties.
     *
     * @param  mixed $p_property_id May be a array or an integer.
     * @param  mixed $p_catg_id May be a array or an integer.
     * @param  mixed $p_cats_id May be a array or an integer.
     * @param  int|null $p_provides
     * @param  string $p_condition
     * @param  boolean $p_dynamic_properties
     * @param  int|null $p_catg_custom_id
     *
     * @return isys_component_dao_result
     * @throws isys_exception_database
     */
    public function retrieve_properties_shd($p_property_id = null, $p_catg_id = null, $p_cats_id = null, $p_provides = null, $p_condition = "", $p_dynamic_properties = false, $p_catg_custom_id = null)
    {
        $l_condition = " WHERE TRUE ";
        if ($p_property_id !== null) {
            $value = is_array($p_property_id) ? $this->prepare_in_condition($p_property_id) : $this->convert_sql_id($p_property_id);
            $l_condition .= "AND isys_property_2_cat__id = $value ";
        }

        if ($p_catg_id !== null) {
            $value = is_array($p_catg_id) ? $this->prepare_in_condition($p_catg_id) : $this->convert_sql_id($p_catg_id);
            $l_condition .= "AND isys_property_2_cat__isysgui_catg__id = $value ";
        }

        if ($p_cats_id !== null) {
            $value = is_array($p_cats_id) ? $this->prepare_in_condition($p_cats_id) : $this->convert_sql_id($p_cats_id);
            $l_condition .= "AND isys_property_2_cat__isysgui_cats__id = $value ";
        }

        if ($p_catg_custom_id !== null) {
            $value = is_array($p_catg_custom_id) ? $this->prepare_in_condition($p_catg_custom_id) : $this->convert_sql_id($p_catg_custom_id);
            $l_condition .= "AND isys_property_2_cat__isysgui_catg_custom__id = $value ";
        }

        if ($p_provides !== null && $p_provides > 0) {
            $l_condition .= "AND isys_property_2_cat__prop_provides & " . $this->convert_sql_int($p_provides) . " ";
        }

        if (!$p_dynamic_properties) {
            $l_condition .= "AND isys_property_2_cat__prop_type = " . C__PROPERTY_TYPE__STATIC . " ";
        }
        $l_category_join = " LEFT JOIN isysgui_catg ON isysgui_catg__id = isys_property_2_cat__isysgui_catg__id " .
            " LEFT JOIN isysgui_cats ON isysgui_cats__id = isys_property_2_cat__isysgui_cats__id " .
            " LEFT JOIN isysgui_catg_custom ON isysgui_catg_custom__id = isys_property_2_cat__isysgui_catg_custom__id ";

        // We rename the fields for easier usage.
        $l_sql = "SELECT isys_property_2_cat__id AS 'id', " .
            "isys_property_2_cat__isysgui_catg__id AS 'catg', " .
            "isys_property_2_cat__isysgui_cats__id AS 'cats', " .
            "isys_property_2_cat__isysgui_catg_custom__id AS 'catg_custom', " .
            "isys_property_2_cat__cat_const AS 'const', " .
            "isysgui_catg.isysgui_catg__title AS 'catg_title', " .
            "isysgui_cats.isysgui_cats__title AS 'cats_title', " .
            "isysgui_catg_custom.isysgui_catg_custom__title AS 'catg_custom_title', " .
            "isys_property_2_cat__prop_type AS 'type', " .
            "isys_property_2_cat__prop_title AS 'title', " . "isys_property_2_cat__prop_key AS 'key', " .
            "(CASE WHEN isys_property_2_cat__isysgui_catg__id IS NOT NULL THEN isysgui_catg__list_multi_value " .
            "WHEN isys_property_2_cat__isysgui_cats__id IS NOT NULL THEN isysgui_cats__list_multi_value " .
            "WHEN isys_property_2_cat__isysgui_catg_custom__id IS NOT NULL THEN isysgui_catg_custom__list_multi_value END) AS 'multi', " .
            "(CASE WHEN isys_property_2_cat__isysgui_catg__id IS NOT NULL THEN isysgui_catg__class_name " .
            "WHEN isys_property_2_cat__isysgui_cats__id IS NOT NULL THEN isysgui_cats__class_name " .
            "WHEN isys_property_2_cat__isysgui_catg_custom__id IS NOT NULL THEN isysgui_catg_custom__class_name END) AS 'class', " .
            "(CASE WHEN isys_property_2_cat__isysgui_catg__id IS NOT NULL THEN isysgui_catg__source_table " .
            "WHEN isys_property_2_cat__isysgui_cats__id IS NOT NULL THEN isysgui_cats__source_table " .
            "WHEN isys_property_2_cat__isysgui_catg_custom__id IS NOT NULL THEN 'isys_catg_custom_fields_list' END) AS 'table', " .
            "isys_property_2_cat__prop_provides AS provides ".
            "FROM isys_property_2_cat " .
            $l_category_join . $l_condition;

        return $this->retrieve($l_sql . $p_condition . ";");
    }

    public function get_category()
    {
        $language = isys_application::instance()->container->get('language');
        try {
            $l_sql = "SELECT DISTINCT 
                isys_property_2_cat__cat_const AS 'const',
                isysgui_catg.isysgui_catg__title AS 'catg_title', 
                isysgui_cats.isysgui_cats__title AS 'cats_title', 
                isysgui_catg_custom.isysgui_catg_custom__title AS 'catg_custom_title'
                FROM isys_property_2_cat 
                LEFT JOIN isysgui_catg ON isysgui_catg__id = isys_property_2_cat__isysgui_catg__id 
                LEFT JOIN isysgui_cats ON isysgui_cats__id = isys_property_2_cat__isysgui_cats__id 
                LEFT JOIN isysgui_catg_custom ON isysgui_catg_custom__id = isys_property_2_cat__isysgui_catg_custom__id;";
            $l_res = $this->retrieve($l_sql);
            $categories =[];
            while ($l_data = $l_res->get_row()) {
                if ($l_data["catg_title"] != null) {
                    $l_data['cat_title'] = $language->get($l_data["catg_title"]);
                } elseif ($l_data["cats_title"] != null) {
                    $l_data['cat_title'] = $language->get($l_data["cats_title"]);
                } elseif ($l_data["catg_custom_title"] != null) {
                    $l_data['cat_title'] = $language->get($l_data["catg_custom_title"]);
                } else {
                    $l_data['cat_title'] = "";
                }

                unset($l_data["catg_title"]);
                unset($l_data["cats_title"]);
                unset($l_data["catg_custom_title"]);
                $categories[] = $l_data;
            }
            return $categories;
        } catch (Exception $ex) {
            return [];
        }
    }
}
