<?php

use idoit\Component\Helper\Memory;
use idoit\Module\Report\Validate\Query;

/**
 * i-doit
 *
 * Template Module Dao.
 *
 * @package     i-doit
 * @subpackage  Modules
 * @author      Dennis Stücken <dstuecken@i-doit.org>
 * @version     0.9
 * @copyright   synetics GmbH
 * @license     http://www.i-doit.com/license
 */
class smview_cmdb_dao_filter extends isys_module_dao
{
    protected static $m_instance = null;

    private $m_hidden_columns = [
        "isys_cats_person_list__user_pass" => true,
        "isys_mandator__db_host"           => true,
        "isys_mandator__db_user"           => true,
        "isys_mandator__db_pass"           => true,
        "isys_ldap__password"              => true,
        "isys_logbook__changes"            => true
    ];

    /**
     * @param  $col
     * @return boolean
     */
    public function isHiddenColumn($col)
    {
        return isset($this->m_hidden_columns[$col]);
    }

    /**
     * @param  $p_title
     * @param  $obj_type_id
     * @param  $obj_id
     * @param  $display_rows
     * @param  $page
     * @param  $user_id
     * @param  $display_relations
     * @param  $empty_values
     * @param  $locked_column
     * @param  $columns
     * @param  $condition
     * @param  $category
     * @param  $default
     * @param  $context
     * @param  $context_value
     * @return boolean|int
     * @throws isys_exception_dao
     */
    public function createFilter(
        $p_title,
        $obj_type_id,
        $obj_id,
        $display_rows,
        $page,
        $user_id,
        $display_relations,
        $empty_values,
        $locked_column,
        $columns,
        $condition,
        $category,
        $default,
        $context,
        $context_value
    ) {
        $l_update = "INSERT INTO shd_filter SET
			shd_filter__title = " . $this->convert_sql_text($p_title) . ",
			shd_filter__obj_type_id = " . $this->convert_sql_id($obj_type_id) . ",
			shd_filter__obj_id = " . $this->convert_sql_id($obj_id) . ",
			shd_filter__display_rows = " . $this->convert_sql_text($display_rows) . ",
			shd_filter__page = " . $this->convert_sql_text($page) . ",
			shd_filter__user_id = " . $this->convert_sql_text($user_id) . ",
			shd_filter__default = " . $this->convert_sql_text($default) . ",
			shd_filter__display_relations = " . $this->convert_sql_text($display_relations) . ",
			shd_filter__empty_values = " . $this->convert_sql_text($empty_values) . ",
			shd_filter__locked_column = " . $this->convert_sql_text($locked_column) . ",
			shd_filter__columns = " . $this->convert_sql_text($columns) . ",
			shd_filter__category = " . $this->convert_sql_text($category) . ",
			shd_filter__context = " . $this->convert_sql_text($context) . ",
			shd_filter__context_value = " . $this->convert_sql_text($context_value) . ",
			shd_filter__condition = " . $this->convert_sql_text($condition). ";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    public function updateFilter(
        $p_filter_id,
        $p_title,
        $obj_type_id,
        $obj_id,
        $display_rows,
        $page,
        $user_id,
        $display_relations,
        $empty_values,
        $locked_column,
        $columns,
        $condition,
        $category,
        $default,
        $context,
        $context_value
    ) {
        $l_update = "UPDATE shd_filter SET
			shd_filter__title = " . $this->convert_sql_text($p_title) . ",
			shd_filter__obj_type_id = " . $this->convert_sql_id($obj_type_id) . ",
			shd_filter__obj_id = " . $this->convert_sql_id($obj_id) . ",
			shd_filter__display_rows = " . $this->convert_sql_text($display_rows) . ",
			shd_filter__page = " . $this->convert_sql_text($page) . ",
			shd_filter__user_id = " . $this->convert_sql_text($user_id) . ",
			shd_filter__default = " . $this->convert_sql_text($default) . ",
			shd_filter__display_relations = " . $this->convert_sql_text($display_relations) . ",
			shd_filter__empty_values = " . $this->convert_sql_text($empty_values) . ",
			shd_filter__locked_column = " . $this->convert_sql_text($locked_column) . ",
			shd_filter__columns = " . $this->convert_sql_text($columns) . ",
			shd_filter__condition = " . $this->convert_sql_text($condition). ",
			shd_filter__category = " . $this->convert_sql_text($category). ",
			shd_filter__context = " . $this->convert_sql_text($context). ",
			shd_filter__context_value = " . $this->convert_sql_text($context_value). "
			WHERE shd_filter__id = ". $this->convert_sql_text($p_filter_id) .";";

        if ($this->update($l_update) && $this->apply_update()) {
            return true;
        }
        return false;
    }

    /**
     * Deletes an filter.
     *
     * @param  int $filter_id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteFilter($filter_id)
    {
        $this->begin_update();

        /* Delete old dataitems / contact references and main record */
        $l_q   = "DELETE FROM shd_filter WHERE shd_filter__id = " . $this->convert_sql_id($filter_id);
        $l_res = $this->update($l_q);
        if ($l_res) {
            return $this->apply_update();
        }
        return false;
    }

    /**
     * Get data method. Unused.
     *
     * @return  null
     */
    public function get_data()
    {
        return null;
    }

    /**
     * Retrieve a single report.
     *
     * @global isys_component_session $g_comp_session
     * @param  array $p_params
     * @return array
     * @throws Exception
     */
    public function get_filter($p_params)
    {
        global $g_comp_database;

        if (isset($p_params['filter_id']) == true) {
            $l_row = $this->retrieve("SELECT * FROM shd_filter WHERE shd_filter__id = " . (int) $p_params['filter_id'] . " AND shd_filter__user_id = " . $p_params['user_id'])->get_row();

            if ($l_row === null) {
                throw new isys_exception_api("Parameter filter_id is set but not exist", -32602);
            }
        } elseif (isset($p_params['obj_type_id']) == true) {
            $l_row = $this->retrieve("SELECT * FROM shd_filter WHERE shd_filter__obj_type_id = " . (int) $p_params['obj_type_id'] .
                " AND shd_filter__default = 1 AND shd_filter__user_id = " . $p_params['user_id'])->get_row();

            if ($l_row === null) {
                throw new isys_exception_api("Parameter obj_type_id is set but not exist", -32602);
            }
        } else {
            throw new isys_exception_api('Parameter filter_id or obj_type_id missed', -32602);
        }

        $l_return = $l_row;
        $l_return["shd_filter__columns"] = unserialize($l_row["shd_filter__columns"]);
        $l_return["shd_filter__condition"] = unserialize($l_row["shd_filter__condition"]);
        $l_return["shd_filter__category"] = unserialize($l_row["shd_filter__category"]);
        $l_return["shd_filter__display_relations"] = "0";
        $l_return["shd_filter__empty_values"] = "1";

        $l_return["columns"] = unserialize($l_row["shd_filter__columns"]);
        $l_return["condition"] = unserialize($l_row["shd_filter__condition"]);
        $l_return["obj_type_id"] = $l_row["shd_filter__obj_type_id"];
        $l_return["count_objects"] = $p_params['count_objects'];

        //count
        if (isset($p_params['count_objects']) == true && $p_params['count_objects'] == true) {
            $l_query_dao = new smview_cmdb_dao_queryfast($g_comp_database);
            try {
                if ($l_return["shd_filter__context"] == "CMDBObjectDetailView") {
                    //todo calc with query.category
                } elseif ($l_return["shd_filter__obj_type_id"] != null) {
                    $l_return["obj_type_id"] = $l_return["shd_filter__obj_type_id"];
                    $l_return["count"] = $l_query_dao->read($l_return);
                } else {
                    $l_return["obj_id"] = $l_return["shd_filter__obj_id"];
                    $l_return["count"] = $l_query_dao->relation($l_return);
                }
            } catch (Exception $exception) {
                $l_return["count"] = 0;
            }

            unset($l_return["columns"]);
            unset($l_return["condition"]);
            unset($l_return["obj_type_id"]);
            unset($l_return["obj_id"]);
            unset($l_return["count_objects"]);
        }

        if (empty($l_return)) {
            return false;
        }
        return $l_return;
    }

    /**
     * Retrieve a single report.
     *
     * @global isys_component_session $g_comp_session
     * @param  array $p_params
     * @return array
     * @throws Exception
     */
    public function get_all_filters($p_params)
    {
        global $g_comp_database;

        $filters = array();
        if (isset($p_params['obj_type_id']) == true && (isset($p_params['default']) == true && ($p_params['default'] == "1" || $p_params['default'] == true ))) {
            $l_res = $this->retrieve("SELECT * FROM shd_filter WHERE shd_filter__obj_type_id = " .
                $this->convert_sql_text($p_params['obj_type_id']) . " AND shd_filter__default = 1 AND shd_filter__user_id = " . $p_params['user_id']);

        } elseif (isset($p_params['obj_type_id']) == true) {
            $l_res = $this->retrieve("SELECT * FROM shd_filter WHERE shd_filter__obj_type_id = " .
                $this->convert_sql_text($p_params['obj_type_id']) . " AND shd_filter__default = 0 AND shd_filter__user_id = " . $p_params['user_id']);

        } elseif (isset($p_params['default']) == true && ($p_params['default'] == "1" || $p_params['default'] == true )) {
            $l_res = $this->retrieve("SELECT * FROM shd_filter WHERE shd_filter__default = 1 AND shd_filter__user_id = " . $p_params['user_id']);
        } elseif (isset($p_params['context']) == true) {
            $l_res = $this->retrieve("SELECT * FROM shd_filter WHERE shd_filter__context = " .
                $this->convert_sql_text($p_params['context']) . " AND shd_filter__user_id = " . $p_params['user_id']);
        } else {
            $l_res = $this->retrieve("SELECT * FROM shd_filter WHERE shd_filter__default = 0 AND shd_filter__context = '' AND shd_filter__user_id = " . $p_params['user_id']);
        }

        while ($l_data = $l_res->get_row()) {
            $l_return = $l_data;
            $l_return["shd_filter__columns"] = unserialize($l_data["shd_filter__columns"]);
            $l_return["shd_filter__condition"] = unserialize($l_data["shd_filter__condition"]);
            $l_return["shd_filter__category"] = unserialize($l_data["shd_filter__category"]);
            $l_return["shd_filter__display_relations"] = "0";
            $l_return["shd_filter__empty_values"] = "1";

            //count
            if (isset($p_params['count_objects']) == true) {
                $l_start_time        = microtime(true);
                $l_query_dao = new smview_cmdb_dao_queryfast($g_comp_database);

                $l_return["columns"] = unserialize($l_data["shd_filter__columns"]);
                $l_return["condition"] = unserialize($l_data["shd_filter__condition"]);
                $l_return["count_objects"] = $p_params['count_objects'];

                try {
                    if ($l_data["shd_filter__obj_type_id"] != null) {
                        $l_return["obj_type_id"] = $l_data["shd_filter__obj_type_id"];
                        $l_return["count"] = $l_query_dao->read($l_return);
                    } else {
                        $l_return["obj_id"] = $l_data["shd_filter__obj_id"];
                        $l_return["count"] = $l_query_dao->relation($l_return);
                    }
                } catch (Exception $exception) {
                    $l_return["count"] = 0;
                }

                unset($l_return["columns"]);
                unset($l_return["condition"]);
                unset($l_return["obj_type_id"]);
                unset($l_return["obj_id"]);
                unset($l_return["count_objects"]);
                $l_return["time"] = (microtime(true) - $l_start_time);
            }
            $filters[] = $l_return;
        }
        return $filters;
    }

    /**
     * @param  string  $p_query
     * @return array
     * @throws Exception
     */
    public function query($p_query)
    {
        $p_query = $this->check_sql($p_query);
        $l_listing = $l_groups = [];
        $p_query = trim($p_query);
        if (!empty($p_query)) {
            if ($this->validate_query($p_query)) {
                $l_db             = isys_application::instance()->database;
                $l_result         = $l_db->query($p_query);
                $l_num            = $l_db->num_rows($l_result);
                $l_listing["num"] = $l_num;

                if ($l_num > 0) {
                    $l_memory      = Memory::instance();
                    $l_first_row   = true;
                    $translatedKey = $l_callbacks = [];

                    while ($l_row = $l_db->fetch_row_assoc($l_result)) {
                        $l_origin_key = null;
                        $l_memory->outOfMemoryBreak();

                        if ($l_first_row) {
                            $l_first_row = false;
                            foreach ($l_row as $l_key => $l_row_value) {
                                $l_origin_key = $l_key;
                                if (strpos($l_key, 'isys_cmdb_dao_category') === 0) {
                                    $l_key_arr = explode('::', $l_key);
                                    $l_key     = array_pop($l_key_arr);


                                    // Add Callback only if the callback class exists
                                    // This increases performance by reducing class_exists calls for each row
                                    // .. also adding the category dao instance to the callback instead of retrieving it in each row
                                    if (class_exists($l_key_arr[0])) {
                                        $l_callbacks[$l_origin_key] = [
                                            call_user_func(
                                                [
                                                    $l_key_arr[0],
                                                    'instance'
                                                ],
                                                isys_application::instance()->database
                                            ),
                                            $l_key_arr[1],
                                            $l_key_arr[2]
                                        ];
                                    }
                                }

                                if ($l_cut_to = strpos($l_key, '###')) {
                                    $l_key = substr($l_key, 0, $l_cut_to);
                                }

                                if (strpos($l_key, '#')) {
                                    $l_title_arr = explode('#', $l_key);
                                    $l_title_key = implode(
                                        ' -> ',
                                        array_reverse(
                                            array_map(
                                                function ($val) {
                                                    return _L($val);
                                                },
                                                $l_title_arr
                                            )
                                        )
                                    );
                                } else {
                                    $l_title_key = $l_key;
                                }

                                // Increase performance by caching translation in first row, to have it then available in all remaining rows
                                if ($l_title_key) {
                                    $translatedKey[$l_origin_key] = $l_title_key;
                                }
                            }
                        }

                        $l_row["__obj_id__"] = $l_row["__id__"] = $l_row["isys_obj__id"] = isys_glob_which_isset(
                            $l_row["isys_obj__id"],
                            $l_row["__obj_id__"],
                            $l_row["__id__"]
                        );

                        // Fixing translations.
                        $l_fixed_row = [];
                        foreach ($l_row as $l_key => $l_value) {
                            $l_title_key = $translatedKey[$l_key];
                            if (!$l_title_key) {
                                continue;
                            }

                            if (isset($l_callbacks[$l_key])) {
                                if ($l_value) {
                                    $l_callback = $l_callbacks[$l_key];
                                    $l_callback_row = $l_row;
                                    if (isset($l_callback[2])) {
                                        $l_callback_row[$l_callback[2]] = $l_value;
                                    }
                                    $l_value = call_user_func(
                                        [
                                            $l_callback[0],
                                            $l_callback[1]
                                        ],
                                        $l_callback_row
                                    );
                                } else {
                                    $l_value = '';
                                }
                            }

                            //Translation
                            $l_value = _L($l_value);

                            //escape Html
                            if (strpos($l_value, "<")!==false) {
                                $l_value = strip_tags($l_value);
                                $l_value = preg_replace("/[\n\r]/", "", $l_value);
                            }

                            $l_fixed_row[$l_title_key] = html_entity_decode($l_value, ENT_QUOTES, $GLOBALS['g_config']['html-encoding']);

                            // This replaces all plain text links to clickable links. See ID-2604
                            if (strpos($l_fixed_row[$l_key], '://') !== false || strpos($l_fixed_row[$l_key], 'www') !== false) {
                                $l_fixed_row[$l_key] = isys_helper_textformat::link_urls_in_string($l_fixed_row[$l_key], "'");
                            }

                            // This replaces all plain text emails to clickable links. See ID-2604
                            if (strpos($l_fixed_row[$l_key], '@') !== false) {
                                $l_fixed_row[$l_key] = isys_helper_textformat::link_mailtos_in_string($l_fixed_row[$l_key], "'");
                            }
                        }
                        $l_listing["content"][] = $l_fixed_row;
                    }

                    // Free memory result
                    $l_db->free_result($p_query);

                    // Get column names.
                    if (count($l_groups) > 0) {
                        $l_listing["grouped"] = true;
                        $l_tmp                = $l_listing["content"][$l_groups[0]][0];
                    } else {
                        if (isset($l_listing["content"][0])) {
                            $l_tmp = $l_listing["content"][0];
                        } else {
                            $l_tmp = false;
                        }
                    }

                    if (is_array($l_tmp)) {
                        $l_tmp     = array_keys($l_tmp);
                        $l_columns = [];
                        foreach ($l_tmp as $l_value) {
                            if (!isset($this->m_hidden_columns[$l_value]) && !preg_match("/^__[\w]+__$/i", $l_value)) {
                                $l_columns[] = array("const" => $l_value, "title" => _L($l_value));
                            }
                        }
                    } else {
                        $l_columns = null;
                    }
                    $l_listing["headers"] = $l_columns;
                } else {
                    $l_listing["content"] = array();
                    $l_listing["headers"] = array();
                }
            }
        }
        return $l_listing;
    }

    public function query_sql($p_sql)
    {
        $p_sql = $this->check_sql($p_sql);
        $l_res = $this->retrieve($p_sql);
        $return = [];
        while ($l_data = $l_res->get_row()) {
            $return[] = $l_data;
        }
        return $return;
    }

    public function check_sql($p_sql)
    {
        //wrong naming in Database
        return str_replace("isys_catg_cmk_list_2_isys_obj__isys_obj__id", "isys_obj__id", $p_sql);
    }

    /**
     * Method for validating that there are no updates, drops, truncates, ... inside the query.
     *
     * @param   string $p_query
     *
     * @return  boolean
     * @throws  Exception
     */
    public function validate_query($p_query)
    {
        return Query::validate($p_query);
    }
}
