<?php
/**
 * i-doit
 *
 * DAO: abstraction layer for CMDB global categories.
 *
 * @package        i-doit
 * @subpackage     CMDB_Categories
 * @author         Leonard Fischer <lfischer@i-doit.org>
 * @version        Van Quyen Hoang <qhoang@i-doit.org> 19.05.2014
 * @copyright      synetics GmbH
 * @license        http://www.i-doit.com/license
 */
class smview_cmdb_dao_filter_sql extends smview_cmdb_dao_abstract_sql
{

    /**
     * Method for creating the dynamic SQL (Originally from the report editor).
     *
     * @param   array $p_filter               filter array
     *
     * @return  string|array
     * @author  Leonard Fischer <lfischer@i-doit.org>
     * @author  Van Quyen Hoang <qhoang@i-doit.org>
     */
    public function create_property_query_for_filter($p_filter, $onlyCount = false, $onlyIds = false)
    {
        try {
            $p_conditions = null;

            $l_select_columns = [];
            $l_sorting_default = ($p_filter['sorting_default'] ?? null);
            $l_sorting_direction = ($p_filter['sorting_direction'] ?? null);

            // Here we go through the selected categories.
            if (isset($p_filter['shd_filter__columns'])) {
                $l_select_columns        = isys_format_json::decode($p_filter['shd_filter__columns']);
                $this->m_query_as_report = false;
            }

            if (isset($p_filter['shd_filter__empty_values'])) {
                $this->m_empty_values = $p_filter['shd_filter__empty_values'];
            }

            // Workaroud for dozens of checks.
            if (is_array($l_select_columns) !== true) {
                $l_select_columns = [];
            }

            foreach ($l_select_columns as $l_select_value) {
                $l_property_ids[] = $l_select_value["id"];
            }

            // We don't want duplicates.
            $l_property_ids = array_unique($l_property_ids);

            // Prepare the property-rows.
            if ($l_property_ids === null) {
                return [];
            }

            if (!$this->m_prepared_data_for_query_construction) {
                $this->prepare_necessary_tasks($l_property_ids);
            }

            // Prepare the select-part of the query.
            $l_selects = $this->create_property_query_select($l_property_ids, [], false, false, true);

            $status = C__RECORD_STATUS__NORMAL;
            if (isset($p_filter['shd_filter__status']) == true) {
                $status = $p_filter['shd_filter__status'];
            }

            // Prepare the joins for our query.
            $this->setMultivalueStatusFilter([$status]);
            $l_joins = $this->create_property_query_join($l_property_ids);

            // Prepare all the conditions for the query.
            $querycondition = array();

            $l_precondition = '';
            $conditionCount = 1;
            $querycondition[] = '(';

            if (!isset($p_filter['shd_filter__display_relations']) || !$p_filter['shd_filter__display_relations']) {
                $querycondition[$conditionCount][0]['category'] = 'C__CATG__GLOBAL';
                $querycondition[$conditionCount][0]['property'] = 'C__CATG__GLOBAL-type';
                $querycondition[$conditionCount][0]['comparison'] = '!=';
                $querycondition[$conditionCount][0]['value'] = C__OBJTYPE__RELATION;
                $conditionCount++;

                $querycondition[$conditionCount] = 'AND';
                $conditionCount++;

                $querycondition[$conditionCount][0]['category'] = 'C__CATG__GLOBAL';
                $querycondition[$conditionCount][0]['property'] = 'C__CATG__GLOBAL-type';
                $querycondition[$conditionCount][0]['comparison'] = '!=';
                $querycondition[$conditionCount][0]['value'] = C__OBJTYPE__PARALLEL_RELATION;
                $conditionCount++;

                $querycondition[$conditionCount] = 'AND';
                $conditionCount++;

                if (isset($p_filter['shd_filter__obj_type_id']) == true) {
                    $querycondition[$conditionCount][0]['category'] = 'C__CATG__GLOBAL';
                    $querycondition[$conditionCount][0]['property'] = 'C__CATG__GLOBAL-type';
                    $querycondition[$conditionCount][0]['comparison'] = '=';
                    $querycondition[$conditionCount][0]['value'] = $p_filter['shd_filter__obj_type_id'];
                    $conditionCount++;

                    $querycondition[$conditionCount] = 'AND';
                    $conditionCount++;
                }

                //Abfrage zum Status nur alle "Normalen"
                $querycondition[$conditionCount][0]['category'] = 'C__CATG__GLOBAL';
                $querycondition[$conditionCount][0]['property'] = 'C__CATG__GLOBAL-status';
                $querycondition[$conditionCount][0]['comparison'] = '=';
                $querycondition[$conditionCount][0]['value'] = $status;
                $conditionCount++;

                $querycondition[$conditionCount] = ')';
                $conditionCount++;
            } else {
                //display_relations = 1;
                if (isset($p_filter['shd_filter__obj_type_id']) == true) {
                    $querycondition[$conditionCount][0]['category'] = 'C__CATG__GLOBAL';
                    $querycondition[$conditionCount][0]['property'] = 'C__CATG__GLOBAL-type';
                    $querycondition[$conditionCount][0]['comparison'] = '=';
                    $querycondition[$conditionCount][0]['value'] = $p_filter['shd_filter__obj_type_id'];
                    $conditionCount++;

                    $querycondition[$conditionCount] = 'AND';
                    $conditionCount++;
                }
                //Abfrage zum Status nur alle "Normalen"
                $querycondition[$conditionCount][0]['category'] = 'C__CATG__GLOBAL';
                $querycondition[$conditionCount][0]['property'] = 'C__CATG__GLOBAL-status';
                $querycondition[$conditionCount][0]['comparison'] = '=';
                if (isset($p_filter['shd_filter__status']) == true) {
                    $querycondition[$conditionCount][0]['value'] = $p_filter['shd_filter__status'];
                } else {
                    $querycondition[$conditionCount][0]['value'] = C__RECORD_STATUS__NORMAL;
                }
                $conditionCount++;

                $querycondition[$conditionCount] = ')';
                $conditionCount++;
            }

            if (isset($p_filter['shd_filter__condition']) == true) {
                foreach ($p_filter['shd_filter__condition'] as $condition) {
                    if (strpos($condition["property"], "C__SHD") === 0) {
                        continue;
                    }

                    if ($l_precondition == $condition["property"]) {
                        //or
                        $querycondition[$conditionCount] = 'OR';
                        $conditionCount++;

                        $querycondition[$conditionCount][0]['category'] = $condition["category"];
                        $querycondition[$conditionCount][0]['property'] = $condition["property"];
                        $querycondition[$conditionCount][0]['comparison'] = $condition["comparison"];
                        $querycondition[$conditionCount][0]['value'] = $condition["value"];
                        $conditionCount++;
                    } else {
                        //neusetzen
                        if ($l_precondition != "") {
                            $querycondition[$conditionCount] = ')';
                            $conditionCount++;
                        }

                        $l_precondition = $condition["property"];

                        $querycondition[$conditionCount] = 'AND';
                        $conditionCount++;

                        $querycondition[$conditionCount] = '(';
                        $conditionCount++;

                        $querycondition[$conditionCount][0]['category'] = $condition["category"];
                        $querycondition[$conditionCount][0]['property'] = $condition["property"];
                        $querycondition[$conditionCount][0]['comparison'] = $condition["comparison"];
                        $querycondition[$conditionCount][0]['value'] = $condition["value"];
                        $conditionCount++;
                    }
                }
            }

            if ($p_filter['shd_filter__condition'] != null) {
                //Wenn es Bedingungen gibt dann muss die Klammer geschlossen werden.
                $querycondition[$conditionCount] = ')';
            }

            $l_conditions = $this->create_property_query_condition($querycondition);

            if (!isset($p_filter['shd_filter__display_relations']) || !$p_filter['shd_filter__display_relations']) {
                unset($p_filter['shd_filter__condition'][0], $p_filter['shd_filter__condition'][1]);
            }

            // First the normal joins
            if (is_array($l_conditions['joins']) && count($l_conditions['joins']) > 0) {
                foreach ($l_conditions['joins'] as $l_arr_joins) {
                    $l_joins = array_merge($l_joins, $l_arr_joins);
                }
            }

            // Secondly the sub joins
            if (count($this->m_sub_joins) > 0) {
                foreach ($this->m_sub_joins as $l_arr_joins) {
                    $l_joins = array_merge($l_joins, $l_arr_joins);
                }
            }

            if ($onlyCount === true) {
                // Returning the SQL-query and the other dara (title, description, ...).
                $l_sql = "SELECT Count(*) FROM isys_obj AS obj_main \n" . implode(
                    " \n",
                    $l_joins
                ) . " \n\n" . "WHERE TRUE \n" . $p_conditions . " \n" . ((isset($l_conditions['conditions'])) ? rtrim($l_conditions['conditions'], 'AND OR') : '') . "";

                return $l_sql . ';';
            } elseif ($onlyIds === true) {
                // Returning the SQL-query and the other dara (title, description, ...).
                $l_sql = "SELECT DISTINCT \n" .  $l_selects[0] . " \n\n" . "FROM isys_obj AS obj_main \n" . implode(
                    " \n",
                    $l_joins
                ) . " \n\n" . "WHERE TRUE \n" . $p_conditions . " \n" . ((isset($l_conditions['conditions'])) ? rtrim($l_conditions['conditions'], 'AND OR') : '') . "";

                $l_sql .= $this->set_report_query_sorting($l_selects, $l_sorting_default, $l_sorting_direction);
                return $l_sql . ';';
            } else {
                // Returning the SQL-query and the other dara (title, description, ...).
                $l_sql = "SELECT DISTINCT \n" . implode(", \n", $l_selects) . " \n\n" . "FROM isys_obj AS obj_main \n" . implode(
                    " \n",
                    $l_joins
                ) . " \n\n" . "WHERE TRUE \n" . $p_conditions . " \n" . ((isset($l_conditions['conditions'])) ? rtrim($l_conditions['conditions'], 'AND OR') : '') . "";

                $l_sql .= $this->set_report_query_sorting($l_selects, $l_sorting_default, $l_sorting_direction);

                if (!$p_filter['shd_filter__display_rows'] === null) {
                    $l_sql .= ' LIMIT ' . (int)$p_filter['shd_filter__page'] * (int)$p_filter['shd_filter__display_rows'] . ', ' . $p_filter['shd_filter__display_rows'];
                }
                return $l_sql . ';';
            }
        } catch (Exception $ex) {
            print_r($ex->getMessage());
            return false;
        }
    }

    /**
     * Method for building the property-select query.
     *
     * @param   array   $p_property_ids              Array which holds the property-id's (from "isys_property_2_cat").
     * @param   array   $p_selects                   You may enter some SELECT-statements here (see $this->create_property_query_for_lists()).
     * @param   boolean $p_leave_field_identifiers   Set to true to keep the original field-names instead of "LANGUACE_CONSTANT###123".
     * @param   boolean $p_use_property_ids_as_title Set to true to retrieve the property-IDs as field-names.
     * @param   boolean $p_group_by_object
     *
     * @return  array
     * @author  Leonard Fischer <lfischer@i-doit.org>
     * @author  Van Quyen Hoang <qhoang@i-doit.org>
     */
    private function create_property_query_select(array $p_property_ids, $p_selects = [], $p_leave_field_identifiers = false, $p_use_property_ids_as_title = false, $p_group_by_object = false)
    {
        // We need to know if all the necessary stuff has been done!
        if (!$this->m_prepared_data_for_query_construction) {
            $this->prepare_necessary_tasks($p_property_ids);
        }

        // Now we prepare the SELECT's.
        $l_selects = array_merge(
            $p_selects,
            [
                "obj_main.isys_obj__id AS '__id__'"
            ]
        );
        $l_alias_field = 'isys_obj__title';

        // And add the selected ones from the report-builder.
        foreach ($p_property_ids as $l_select) {
            if (!isset($this->m_property_rows[$l_select])) {
                continue;
            }

            $l_table_main_id     = $this->m_property_rows[$l_select]["table"] . "__id";
            $l_field_name        = '';
            $l_special_field     = '';
            $l_special_selection = false;
            $l_db_field          = $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
            $l_prop_key          = $this->m_property_rows[$l_select]['key'];

            if ($p_use_property_ids_as_title) {
                $l_field_name = ' AS \'' . $l_select . '\'';
            }

            $l_table = current(explode('__', $l_db_field));

            // We may have a selected property, which has no real table-fields (the dynamic properties for example).
            if ($this->m_property_rows[$l_select]['type'] == C__PROPERTY_TYPE__DYNAMIC) {
                $l_callback_class = get_class($this->m_property_rows[$l_select]['data'][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][0]);
                if (strpos($l_callback_class, 'isys_cmdb_dao_category_') === 0) {
                    if ($l_db_field == 'isys_obj__id') {
                        $l_field = 'obj_main.isys_obj__id ';
                    } else {
                        if ($l_table != 'isys_obj') {
                            $l_field = 'j' . $this->retrieve_alias($l_table) . '.' . $l_db_field;
                        } else {
                            $l_field = 'obj_main.' . $l_db_field;
                        }
                    }

                    $l_selects[$l_select] = $l_field . ' AS \'' . $l_callback_class . '::' . $this->m_property_rows[$l_select]['data'][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] . '::' . $l_db_field . '::' . $this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TITLE] . '\'';
                }
                continue;
            } elseif ($this->m_property_rows[$l_select]['type'] == C__PROPERTY_TYPE__DYNAMIC) {
                if (isset($l_db_field) && $l_table !== 'isys_obj') {
                    $l_field_alias = '';
                    if (isset($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS])) {
                        $l_field_alias = ' AS ' . $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS];
                    }

                    $l_selects[$l_select] = 'j' . $this->retrieve_alias(
                        'isys_obj',
                        $l_table
                    ) . '.' . $l_db_field . $l_field_alias;
                }
                continue;
            }

            if ($l_table == 'isys_catg_custom_fields_list') {
                $l_field_alias  = $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS];
                $l_field_type   = $this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE];
                $l_alias        = $this->retrieve_alias($l_field_alias . '#isys_catg_custom_fields_list');
                $l_output_alias = "'" . $l_select . "'" ;

                switch ($l_field_type) {
                    case C__PROPERTY__INFO__TYPE__OBJECT_BROWSER:
                    case C__PROPERTY__INFO__TYPE__N2M:
                        $l_alias_sec = $this->retrieve_alias('isys_catg_custom_fields_list', $l_field_alias . '#isys_obj');
                        $l_field     = 'j' . $l_alias_sec . '.isys_obj__title AS ' . $l_output_alias;
                        break;
                    case C__PROPERTY__INFO__TYPE__DIALOG:
                    case C__PROPERTY__INFO__TYPE__DIALOG_PLUS:
                    case C__PROPERTY__INFO__TYPE__MULTISELECT:
                        $l_alias_sec = $this->retrieve_alias('isys_catg_custom_fields_list', $l_field_alias . '#isys_dialog_plus_custom');
                        $l_field     = 'j' . $l_alias_sec . '.isys_dialog_plus_custom__title AS ' . $l_output_alias;
                        break;
                    default:
                        $l_field = 'j' . $l_alias . '.' . $l_db_field . ' AS ' . $l_output_alias;
                        break;
                }

                $l_selects[$l_select] = $l_field;
                continue;
            }

            // First we check for some special selected fields.
            if (isset($this->m_special_selects[$l_db_field])) {
                // Check if its a primary field
                if ($p_group_by_object && $this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__PRIMARY]) {
                    $l_sp_select          = $this->m_special_selects[$l_db_field];
                    $l_sp_select          = '' . substr($l_sp_select, 0, strpos($l_sp_select, 'AS')) . ' ' . substr(
                        $l_sp_select,
                        strpos($l_sp_select, 'AS'),
                        strlen($l_sp_select)
                    );
                    $l_selects[$l_select] = $l_sp_select;
                    continue;
                } elseif (is_array($this->m_special_selects[$l_db_field])) {
                    $l_special_selection = true;
                    $l_table             = $this->m_special_selects[$l_db_field][0];
                    $l_special_field     = (strpos(
                        $l_db_field,
                        $l_table
                    )) ? $l_db_field : $this->m_special_selects[$l_db_field][2];

                    if ($l_table == 'isys_obj' && !$this->m_query_as_report) {
                        $l_special_field                                                                               = 'isys_obj__title';
                        $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] = $this->m_special_selects[$l_db_field][0];
                        $l_table                                                                                       = $this->m_special_selects[$l_db_field][1];
                    } else {
                        $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] = $this->m_special_selects[$l_db_field][1];
                    }
                } else {
                    $l_selects[$l_select] = $this->m_special_selects[$l_db_field];
                    continue;
                }
            }

            // We might have a dialog- or object-browser field and want to handle it properly.
            $l_ui_type = $this->retrieve_ui_type((array) $this->m_property_rows[$l_select]['data']);

            if (($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] !== null && substr(
                $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                0,
                5
            ) == 'isys_') || $this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_strPopupType'] == 'browser_location'
            ) {
                $l_alias = $this->retrieve_alias($l_table, $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0]);

                // We have to join 'job' on references to isys_connection.
                if ($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] == 'isys_connection') {
                    $l_selects[$l_select] = "job" . $l_alias . "." . $l_alias_field . " AS '" . $l_select . "'";
                    if ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == C__CMDB__CATG__CONTACT__CONNECTED_OBJECT) {
                        $l_selects[$l_select. "_id"] =  "job" . $l_alias . ".isys_obj__id  AS '" . $l_select . "___id'";
                    } elseif ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == C__CATG__OPERATING_SYSTEM_OBJ_APPLICATION) {
                        $l_selects[$l_select. "_id"] =  "job" . $l_alias . ".isys_obj__id  AS '" . $l_select . "___id'";
                    }
                } elseif ($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] == 'isys_contact') {
                    $l_alias              = $this->retrieve_alias('isys_obj', 'isys_contact_2_isys_obj');
                    $l_selects[$l_select] = "job" . $l_alias . "." . $l_alias_field . " AS '" . $l_select . "'";
                } elseif ($this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_strPopupType'] == 'browser_location') {
                    $l_alias              = $this->retrieve_alias($l_table, 'isys_obj');
                    $l_selects[$l_select] = "j" . $l_alias . "." . $l_alias_field . " AS '" . $l_select . "'";
                } else {
                    if ($l_special_selection && $l_special_field != '') {
                        if ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == C__CMDB__CATG__ITSERVICE__CONNECTED_OBJECT) {
                            //add title to Servicezuweisung
                            $l_selects[$l_select] = "j" . $l_alias . "." . $l_special_field . " AS '" . $l_select . "'";
                        } elseif ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == C__CATS__APPLICATION_OBJ_APPLICATION) {
                            //add title to Installiert auf SHD
                            $l_alias_name = $this->retrieve_alias($this->m_property_rows[$l_select]["table"], 'isys_obj');
                            $l_selects[$l_select] = "j" . $l_alias_name . ".isys_obj__title  AS '" . $l_select . "'";
                            $l_selects[$l_select . "_id"] = "j" . $l_alias . "." . $l_special_field . " AS '" . $l_select . "___id'";
                        } elseif ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == C__CATG__CONNECTOR__CONNECTION_TYPE) {
                            $l_selects[$l_select] = "j" . $l_alias . "." . $l_special_field . " AS '" . $l_select . "'";
                            $l_selects[$l_select . "_id"] = "j" . $l_alias . ".isys_connection_type__id AS '" . $l_select . "___id'";
                        } elseif ($this->m_property_rows[$l_select]["const"] == "C__CATS__PERSON_ASSIGNED_GROUPS") {
                            $l_selects[$l_select] = "j" . $l_alias . ".isys_cats_person_group_list__title  AS '" . $l_select . "'";
                            $l_selects[$l_select . "_id"] = "j" . $l_alias . "." . $l_special_field . " AS '" . $l_select . "___id'";
                        } elseif ($this->m_property_rows[$l_select]["const"] == "C__CATS__PERSON_GROUP_MEMBERS") {
                            $l_alias_name = $this->retrieve_alias("isys_cats_person_list", 'isys_obj');

                            $l_selects[$l_select] = "j" . $l_alias_name . ".isys_obj__title  AS '" . $l_select . "'";
                            $l_selects[$l_select . "_id"] = "j" . $l_alias . "." . $l_special_field . " AS '" . $l_select . "___id'";
                        } elseif ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == C__CATG__GUEST_SYSTEM_CONNECTED_OBJECT) {
                            $l_alias_name = $this->retrieve_alias($this->m_property_rows[$l_select]["table"], 'isys_obj');

                            $l_selects[$l_select] = "j" . $l_alias_name . ".isys_obj__title  AS '" . $l_select . "'";
                            $l_selects[$l_select . "_id"] = "j" . $l_alias . "." . $l_special_field . " AS '" . $l_select . "___id'";
                        } else {
                            $l_selects[$l_select] = "j" . $l_alias . "." . $l_special_field . " AS '" . $l_select . "'";
                        }
                    } elseif ($this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__OBJECT_BROWSER) {
                        $l_object_field = $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . "__isys_obj__id";
                        if ($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] == 'isys_obj') {
                            $l_object_field = 'isys_obj__id';
                        }

                        if ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == C__CATG__LIC_ASSIGN__LICENSE) {
                            //add title to Servicezuweisung
                            $l_aliasname = $this->retrieve_alias('isys_cats_lic_list', 'isys_obj');
                            $l_selects[$l_select] = "j" . $l_aliasname  . ".isys_obj__title  AS '" . $l_select . "'";

                            $l_selects[$l_select . "_id"] = "j" . $l_alias . "." . $l_object_field . " AS '" . $l_select . "___id'";
                        } else {
                            $l_selects[$l_select] = "j" . $l_alias . "." . $l_object_field . " AS '" . $l_select . "'";
                        }
                    } else {
                        if (isset($this->m_aliases[$this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0]])) {
                            $l_selects[$l_select] = $this->m_aliases[$this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0]] . "."
                                . $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] .  "__title";
                        } else {
                            if ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == C__CATG__CONNECTOR__CONNECTION_TYPE) {
                                $l_selects[$l_select] = "j" . $l_alias . "." .
                                    ($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][2] ?:
                                        $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . "__title")
                                    . " AS '" . $l_select  . "'" ;

                                $l_selects[$l_select . "_id"] = "j" . $l_alias . ".isys_connection_type__id AS '" . $l_select . "___id'";
                            } else {
                                $l_selects[$l_select] = "j" . $l_alias . "." .
                                    ($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][2] ?:
                                        $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . "__title")
                                    . " AS '" . $l_select  . "'" ;
                            }

                            if ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == "C__CATG__CPU_FREQUENCY_UNIT") {
                                $l_selects[$l_select . "_id"] = "j" . $l_alias . ".isys_frequency_unit__id AS '" . $l_select . "___id'";
                            } elseif ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == "C__CATG__CPU_MANUFACTURER") {
                                $l_selects[$l_select . "_id"] = "j" . $l_alias . ".isys_catg_cpu_manufacturer__id AS '" . $l_select . "___id'";
                            } elseif ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == "C__CATG__CPU_TYPE") {
                                $l_selects[$l_select . "_id"] = "j" . $l_alias . ".isys_catg_cpu_type__id AS '" . $l_select . "___id'";
                            } elseif ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == "C__CATG__MEMORY_MANUFACTURER") {
                                $l_selects[$l_select . "_id"] = "j" . $l_alias . ".isys_memory_manufacturer__id AS '" . $l_select . "___id'";
                            } elseif ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == "C__CATG__MEMORY_TITLE_ID") {
                                $l_selects[$l_select . "_id"] = "j" . $l_alias . ".isys_memory_title__id AS '" . $l_select . "___id'";
                            } elseif ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == "C__CATG__MEMORY_TYPE") {
                                $l_selects[$l_select . "_id"] = "j" . $l_alias . ".isys_memory_type__id AS '" . $l_select . "___id'";
                            } elseif ($this->m_property_rows[$l_select]["data"]["ui"]["id"] == "C__CATG__MEMORY_UNIT") {
                                $l_selects[$l_select . "_id"] = "j" . $l_alias . ".isys_memory_unit__id AS '" . $l_select . "___id'";
                            }
                        }
                    }

                    if (isset($this->m_text_fields[$this->m_property_rows[$l_select]['const']])) {
                        if (isset($this->m_text_fields[$this->m_property_rows[$l_select]['const']][$this->m_property_rows[$l_select]['key']])) {
                            $l_selects[$l_select] = substr(
                                $l_selects[$l_select],
                                0,
                                (strpos($l_selects[$l_select], '.') + 1)
                            ) . $this->m_text_fields[$this->m_property_rows[$l_select]['const']][$this->m_property_rows[$l_select]['key']] .
                                (!$p_leave_field_identifiers ? " AS '" . $l_select . "'" : $l_field_name);
                        }
                    }

                    // Check if its a primary field
                    if ($p_group_by_object && $this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__PRIMARY]) {
                        $l_sp_select = $l_selects[$l_select];
                        $l_sp_select = ''.substr($l_sp_select, 0, strpos($l_sp_select, 'AS')).' '.substr($l_sp_select, strpos($l_sp_select, 'AS'), strlen($l_sp_select));
                        $l_selects[$l_select] = $l_sp_select;
                    }
                }
                continue;
            } elseif ($this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__OBJECT_BROWSER) {
                if ($l_db_field == $l_table . '__isys_obj__id') {
                    if (isset($this->m_aliases['isys_connection#' . $l_table])) {
                        $l_alias = 'j' . $this->m_aliases[$l_table . '#isys_connection'];
                        $l_selects[$l_select] = $l_alias . "." . $l_db_field . (!$p_leave_field_identifiers ? " AS '" . $l_select . "'" : $l_field_name);
                    }

                    if (isset($this->m_aliases['isys_obj#' . $l_table])) {
                        $l_alias = 'j' . $this->m_aliases[$l_table . '#isys_obj'];
                        $l_selects[$l_select] = $l_alias . "." . $l_db_field . (!$p_leave_field_identifiers ? " AS '" . $l_select . "'" : $l_field_name);
                    }
                    continue;
                } elseif (strpos($l_db_field, 'isys_cable_connection__id') !== false) {
                    if (isset($this->m_aliases['isys_obj#isys_cable_connection'])) {
                        $l_alias              = 'j' . $this->m_aliases['isys_obj#isys_cable_connection'];
                        $l_selects[$l_select] = $l_alias . "." . $l_alias_field . (!$p_leave_field_identifiers ? " AS '" . $l_select . "'" : $l_field_name);
                    }
                    continue;
                } elseif (strpos($l_db_field, 'isys_catg_connector_list__id') !== false) {
                    if ($l_table != 'isys_catg_connector_list') {
                        if (isset($this->m_aliases[$l_table . '#isys_catg_connector_list#isys_obj'])) {
                            $l_alias              = 'j' . $this->m_aliases[$l_table . '#isys_catg_connector_list#isys_obj'];
                            $l_selects[$l_select] = $l_alias . "." . $l_alias_field . (!$p_leave_field_identifiers ? " AS '" . $l_select . "'" : $l_field_name);
                            continue;
                        }
                    } else {
                        if (isset($this->m_aliases['isys_catg_connector_list#isys_obj'])) {
                            $l_alias              = 'j' . $this->m_aliases['isys_catg_connector_list#isys_obj'];
                            $l_selects[$l_select] = $l_alias . "." . $l_alias_field . (!$p_leave_field_identifiers ? " AS '" . $l_select . "'" : $l_field_name);
                            continue;
                        }
                    }
                }
            }

            if ($l_select != "434") {
                if (substr($this->retrieve_alias($l_table), 0, 1) == "j") {
                    $l_alias = $this->retrieve_alias($l_table);
                } else {
                    $l_alias = 'j' . $this->retrieve_alias($l_table);
                }
            } else {
                if ($l_alias == "obj_main") {
                    $l_alias = $this->retrieve_alias($l_table);
                }

                if (is_numeric(substr($l_alias, 0, 1)) == true) {
                    $l_alias = 'j' . $l_alias;
                }
            }

            // Then we check for special table-names inside the select.
            if (isset($this->m_aliases[$l_table])) {
                $l_alias = $this->m_aliases[$l_table];
            } elseif (isset($this->m_aliases[$l_table . '#main_obj'])) {
                $l_alias = 'j' . $this->m_aliases[$l_table . '#main_obj'];
            } elseif ($l_table == 'isys_logbook') {
                $l_alias = 'j' . $this->retrieve_alias('isys_catg_logb_list', 'isys_logbook');
            }

            // If we got a "yes/no" dialog-field we want to display the result as such.
            if ($l_ui_type == C__PROPERTY__UI__TYPE__DIALOG && $this->m_property_rows[$l_select]['data'][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] == 'get_yes_or_no') {
                $l_selects[$l_select] = "(CASE " . $l_alias . "." . $l_db_field . " " . "WHEN 0 THEN 'LC__UNIVERSAL__NO' WHEN 1 THEN 'LC__UNIVERSAL__YES' END)" . (!$p_leave_field_identifiers ? " AS '" . $l_select . "'" : ($p_use_property_ids_as_title ? $l_field_name : $l_db_field));
            } elseif (!empty($this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData']) && !in_array(
                $this->m_property_rows[$l_select]['data'][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1],
                self::$m_ignored_format_callbacks
            )
            ) {
                if (is_object($this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData']) && get_class(
                    $this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData']
                ) == 'isys_callback') {
                     $this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData'] =
                         $this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData']->execute();
                }

                if (is_array($this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData'])) {
                    if (count($this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData']) > 0) {
                        $l_selects_dialog = "(CASE " . $l_alias . "." . $l_db_field . " ";
                        foreach ($this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData'] as $l_key => $l_value) {
                            if (is_array($l_value)) {
                                if (isset($l_value['value'])) {
                                    $l_selects_dialog .= "WHEN " . $this->convert_sql_text($l_key) . " THEN " . $this->convert_sql_text($l_value['value']) . " ";
                                } elseif (isset($l_value['val'])) {
                                    $l_selects_dialog .= "WHEN " . $this->convert_sql_text($l_key) . " THEN " . $this->convert_sql_text($l_value['val']) . " ";
                                } else {
                                    // Cannot extract value from array
                                    $l_selects_dialog .= "WHEN " . $this->convert_sql_text($l_key) . " THEN " . isys_tenantsettings::get('gui.empty_value', '-') . " ";
                                }
                            } else {
                                $l_selects_dialog .= "WHEN " . $this->convert_sql_text($l_key) . " THEN " . $this->convert_sql_text($l_value) . " ";
                            }
                        }
                        $l_selects_dialog .= "END) AS '" . $l_select . "'";
                    } else {
                        // No values found
                        $l_selects_dialog = " '" . isys_tenantsettings::get('gui.empty_value', '-') . "' ";
                    }
                    $l_selects[$l_select] = $l_selects_dialog;
                }
            } else {
                if ($l_table == 'isys_logbook') {
                    $l_alias = 'j' . $this->retrieve_alias('isys_catg_logb_list', 'isys_logbook');
                }

                $l_selection = $l_alias . "." . $l_db_field . " AS '" . $l_select . "'";
                if (isset($this->m_property_wrap_functions[$l_prop_key])) {
                    $l_selection = $this->wrap_mysql_function($l_prop_key, $l_selection);
                }
                $l_selects[$l_select] = $l_selection;

                //Check if field is in Table to put __id at the end of the table
                if (strpos($this->m_property_rows[$l_select]["data"]["data"]["field"], $this->m_property_rows[$l_select]["table"]) !== false) {
                    $l_selects[$l_select . "_id"] = $l_alias . "." . $l_table_main_id . " AS '" . $l_select . "___id'";
                }
            }
        }

        // Clearing out all duplicate selects.
        return array_unique($l_selects);
    }

    /**
     * @var array
     */
    private $m_property_wrap_functions = [
        'longitude' => 'Y',
        'latitude'  => 'X'
    ];

    /**
     * Helper Method which wraps mysql functions around the selected db field
     *
     * @param $p_key
     * @param $p_field
     *
     * @return string
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    private function wrap_mysql_function($p_key, $p_field)
    {
        return $this->m_property_wrap_functions[$p_key] . '(' . $p_field . ')';
    }

    /**
     * Helper method which builds the inner condition
     *
     * @param $p_inner_condition
     * @param $p_data
     * @param $p_condition_field
     * @param $p_condition
     * @param $p_alias
     * @param $p_unit_field
     * @param $p_unit_id
     *
     * @return string
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    protected function build_inner_condition($p_inner_condition, $p_data, $p_condition_field, $p_condition, $p_alias, $p_unit_field, $p_unit_id)
    {
        if (isset($p_data[C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES]) && $p_data[C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__DIALOG || $p_data[C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__DIALOG_PLUS) {
            if ((strpos($p_data[C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], 'catg') !== false || strpos(
                        $p_data[C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                        'cats'
                    ) !== false) && strpos(
                    $p_data[C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                    '_list'
                ) !== false && strpos($p_condition_field, '__id_') === false
            ) {
                $p_condition_field = str_replace('__id', '__title', $p_condition_field);
            }
        } elseif ($p_data[C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__TEXT && strpos(
                $p_condition_field,
                '__id'
            ) > 0
        ) {
            $p_condition_field = str_replace('__id', '__title', $p_condition_field);
        }

        if (isset($p_data[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK]) && $p_unit_id !== null) {
            if ($p_data[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] == 'convert') {
                $l_method = $p_data[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][2][0];
                // convert value
                $p_condition['value'] = isys_convert::$l_method($p_condition['value'], $p_unit_id);
            }
        }

        if ($p_condition['comparison'] == 'LIKE %...%') {
            $p_condition['comparison'] = 'LIKE';
            $p_condition['value']      = '%' . $p_condition['value'] . '%';
        }

        if ($p_condition['comparison'] == 'NOT LIKE %...%') {
            $p_condition['comparison'] = 'NOT LIKE';
            $p_condition['value']      = '%' . $p_condition['value'] . '%';
        }

        $p_condition['value'] = str_replace(
            [
                '[',
                ']',
                '"'
            ],
            '',
            $p_condition['value']
        );
        if (strpos($p_condition['value'], ',') !== false) {
            if ($p_condition['comparison'] == '=') {
                $p_condition['comparison'] = 'IN';
            } else {
                $p_condition['comparison'] = 'NOT IN';
            }
            $p_inner_condition .= ' ' . $p_condition_field . ' ' . $p_condition['comparison'] . ' (' . $p_condition['value'] . ') ' . (isset($p_condition['operator']) ? $p_condition['operator'] : '');
        } else {
            if ($p_condition['comparison'] == 'under_location') {
                $p_inner_condition .= ' (' . $p_condition['location_lft'] . ' > ' . '(SELECT isys_catg_location_list__lft FROM isys_catg_location_list WHERE isys_catg_location_list__isys_obj__id = ' . $this->convert_sql_id(
                        $p_condition['value']
                    ) . ') AND ' . $p_condition['location_rgt'] . ' < ' . '(SELECT isys_catg_location_list__rgt FROM isys_catg_location_list WHERE isys_catg_location_list__isys_obj__id = ' . $this->convert_sql_id(
                        $p_condition['value']
                    ) . ') ';
            } else {
                $p_inner_condition .= ' (' . $p_condition_field . ' ' . $p_condition['comparison'] . ' ' .
                    ((isset($p_condition['value']) && ($p_condition['comparison'] !== 'IS NULL' && $p_condition['comparison'] !== 'IS NOT NULL')) ? $this->convert_sql_text($p_condition['value']) : '') . ' ';

                if (isset($p_condition['value']) && ($p_condition['value'] === '-1' ||
                        (is_numeric($p_condition['value']) && $p_data[C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] === C__PROPERTY__INFO__TYPE__OBJECT_BROWSER))
                ) {
                    if ($p_condition['comparison'] !== 'IS NULL' && $p_condition['comparison'] !== 'IS NOT NULL') {
                        if ((int) $p_condition['value'] === 0 || $p_condition['value'] == '-1') {
                            if ($p_condition['comparison'] == '=') {
                                $l_comparison_addition = ' IS ';
                                $l_log_operator        = ' OR ';
                            } else {
                                $l_comparison_addition = ' IS NOT ';
                                $l_log_operator        = ' AND ';
                            }
                            $p_inner_condition .= $l_log_operator . $p_condition_field . ' ' . $l_comparison_addition . ' NULL ';
                        }
                    }
                } else {
                    switch ($p_condition['comparison']) {
                        case 'IS NULL':
                            $p_inner_condition .= ' OR ' . $p_condition_field . ' = \'\' ';
                            break;
                        case 'IS NOT NULL':
                            $p_inner_condition .= ' AND ' . $p_condition_field . ' != \'\' ';
                            break;
                        default:
                            break;
                    }
                }
            }
            $p_inner_condition .= ') ' . (isset($p_condition['operator']) ? $p_condition['operator'] : '');
        }

        if ($p_unit_field !== null) {
            $p_inner_condition .= ' AND ' . $p_alias . '.' . $p_unit_field . ' = ' . $this->convert_sql_text($p_unit_id) . ' ';
        }
        return $p_inner_condition;
    }
}
