<?php
/**
 * i-doit - Documentation and CMDB solution for IT environments
 *
 * This file is part of the i-doit framework. Modify at your own risk.
 *
 * Please visit http://www.i-doit.com/license for a full copyright and license information.
 *
 * @version     1.8.3
 * @package     i-doit
 * @author      synetics GmbH
 * @copyright   synetics GmbH
 * @url         http://www.i-doit.com
 * @license     http://www.i-doit.com/license
 */

/**
 * i-doit
 *
 * Template Module Dao.
 *
 * @package     i-doit
 * @subpackage  Modules
 * @author      Dennis Stücken <dstuecken@i-doit.org>
 * @version     0.9
 * @copyright   synetics GmbH
 * @license     http://www.i-doit.com/license
 */

class smview_cmdb_dao_insightprofil extends isys_module_dao
{

    protected static $m_instance = null;

    /**
     * Retrieve the Insight
     *
     * @param  int $p_insight_id
     * @return array
     * @throws Exception
     */
    public function getInsightProfil($p_insight_id)
    {
        $l_row = $this->retrieve("SELECT shd_insight.* FROM shd_insight 
              WHERE shd_insight.shd_insight__id = " . $this->convert_sql_text($p_insight_id))->get_row();

        if (empty($l_row)) {
            return [];
        }

        $l_row["insightId"]          = $l_row['shd_insight__id'];
        $l_row["insightTitle"]       = $l_row['shd_insight__title'];
        $l_row["tooltipEnabled"]     = $l_row['shd_insight__tooltipEnabled'];
        $l_row["layoutType"]         = $l_row['shd_insight__layoutType'];
        $l_row["legendIds"]          = $l_row['shd_insight__legendIds'];
        $l_row["descriptionEnabled"] = $l_row['shd_insight__descriptionEnabled'];
        $l_row["lvl"]                = $l_row['shd_insight__lvl'];
        $l_row["expand_ids"]         = $l_row['shd_insight__expand_ids'];

        unset($l_row["shd_insight__id"]);
        unset($l_row["shd_insight__title"]);
        unset($l_row["shd_insight__tooltipEnabled"]);
        unset($l_row["shd_insight__layoutType"]);
        unset($l_row["shd_insight__legendIds"]);
        unset($l_row["shd_insight__descriptionEnabled"]);
        unset($l_row["shd_insight__lvl"]);
        unset($l_row["shd_insight__expand_ids"]);

        $l_row["ObjectTypeConfiguration"] = $this->getObjectTypeConfiguration($p_insight_id);
        return $l_row;
    }

    /**
     * Retrieve the InsightProfil.
     *
     * @return array
     * @throws Exception
     */
    public function getAllInsightProfil()
    {
        $l_res = $this->retrieve("SELECT shd_insight.* FROM shd_insight");
        while ($l_data = $l_res->get_row()) {
            $l_data["insightId"]          = $l_data['shd_insight__id'];
            $l_data["insightTitle"]       = $l_data['shd_insight__title'];
            $l_data["tooltipEnabled"]     = $l_data['shd_insight__tooltipEnabled'];
            $l_data["layoutType"]         = $l_data['shd_insight__layoutType'];
            $l_data["legendIds"]          = $l_data['shd_insight__legendIds'];
            $l_data["descriptionEnabled"] = $l_data['shd_insight__descriptionEnabled'];
            $l_data["lvl"]                = $l_data['shd_insight__lvl'];
            $l_data["expand_ids"]         = $l_data['shd_insight__expand_ids'];

            unset($l_data["shd_insight__id"]);
            unset($l_data["shd_insight__title"]);
            unset($l_data["shd_insight__tooltipEnabled"]);
            unset($l_data["shd_insight__layoutType"]);
            unset($l_data["shd_insight__legendIds"]);
            unset($l_data["shd_insight__descriptionEnabled"]);
            unset($l_data["shd_insight__lvl"]);
            unset($l_data["shd_insight__expand_ids"]);

            $insightProfil[] = $l_data;
        }
        return $insightProfil;
    }

    /**
     * Method for creating a InsightProfil.
     *
     * @param  string $p_title
     * @param  string $p_tooltipEnabled
     * @param  string $p_layoutType
     * @param  string $p_legendIds
     * @param  string $p_descriptionEnabled
     * @return false|int boolean false or integer
     * @throws isys_exception_dao
     */
    public function createInsightProfil($p_title, $p_tooltipEnabled, $p_layoutType, $p_legendIds, $p_descriptionEnabled, $p_expand_ids, $p_lvl)
    {
        $l_update = "INSERT INTO shd_insight SET
			shd_insight__title = " . $this->convert_sql_text($p_title) . ",
			shd_insight__layoutType = " . $this->convert_sql_text($p_layoutType) . ",
			shd_insight__legendIds = " . $this->convert_sql_text($p_legendIds) . ",
			shd_insight__descriptionEnabled = " . $this->convert_sql_text($p_descriptionEnabled) . ",
			shd_insight__expand_ids = " . $this->convert_sql_text($p_expand_ids) . ",
			shd_insight__lvl = " . $this->convert_sql_text($p_lvl) . ",
			shd_insight__tooltipEnabled = " . $this->convert_sql_text($p_tooltipEnabled) . ";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    /**
     * Method for updating a InsightProfil.
     *
     * @param  string $p_insight_id
     * @param  string $p_title
     * @param  string $p_tooltipEnabled
     * @param  string $p_layoutType
     * @param  string $p_legendIds
     * @param  string $p_descriptionEnabled
     * @return false|string boolean false or integer
     * @throws isys_exception_dao
     */
    public function updateInsightProfil($p_insight_id, $p_title, $p_tooltipEnabled, $p_layoutType, $p_legendIds, $p_descriptionEnabled, $p_expand_ids, $p_lvl)
    {
        $l_update = "UPDATE shd_insight SET
			shd_insight__title = " . $this->convert_sql_text($p_title) . ",
			shd_insight__layoutType = " . $this->convert_sql_text($p_layoutType) . ",
			shd_insight__tooltipEnabled = " . $this->convert_sql_text($p_tooltipEnabled) . ",
			shd_insight__legendIds = " . $this->convert_sql_text($p_legendIds) . ",
			shd_insight__descriptionEnabled = " . $this->convert_sql_text($p_descriptionEnabled) . ",
			shd_insight__expand_ids = " . $this->convert_sql_text($p_expand_ids) . ",
			shd_insight__lvl = " . $this->convert_sql_text($p_lvl) . "
            WHERE shd_insight__id = ". $this->convert_sql_text($p_insight_id) .";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $p_insight_id;
        }
        return false;
    }

    /**
     * Delete a InsightProfil.
     *
     * @param  int $insight_id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteInsightProfil($insight_id)
    {
        /* Delete old dataitems / contact references and main record */
        $l_q   = "DELETE shd_insight FROM shd_insight WHERE shd_insight__id = " . $this->convert_sql_id($insight_id);
        return $this->_applySqlUpdate($l_q);
    }

    /**
     * Retrieve the Map.
     *
     * @param  int $insightprofil_id
     * @return array
     * @throws Exception
     */
    public function getObjectTypeConfiguration($insightprofil_id)
    {
        $l_res = $this->retrieve("SELECT shd_insight_objecttypeconfiguration.* FROM shd_insight_objecttypeconfiguration 
                                  WHERE shd_insight_objecttypeconfiguration__shd_insight__id = " . $this->convert_sql_text($insightprofil_id));

        while ($l_data = $l_res->get_row()) {
            $l_data["objecttypeconfigurationId"]            = $l_data["shd_insight_objecttypeconfiguration__id"];
            $l_data["objecttypeconst"]  = $l_data['shd_insight_objecttypeconfiguration__objecttypeconst'];
            $l_data["style"]         = $l_data['shd_insight_objecttypeconfiguration__style'];
            $l_data["width"]         = $l_data['shd_insight_objecttypeconfiguration__width'];
            $l_data["linecount"]  = $l_data['shd_insight_objecttypeconfiguration__linecount'];

            unset($l_data["shd_insight_objecttypeconfiguration__id"]);
            unset($l_data["shd_insight_objecttypeconfiguration__objecttypeconst"]);
            unset($l_data["shd_insight_objecttypeconfiguration__style"]);
            unset($l_data["shd_insight_objecttypeconfiguration__width"]);
            unset($l_data["shd_insight_objecttypeconfiguration__linecount"]);
            unset($l_data["shd_insight_objecttypeconfiguration__shd_insight__id"]);

            $l_data["LineProperties"] = $this->getLineProperties($l_data["objecttypeconfigurationId"]);

            $objectTypeConfiguration[] = $l_data;
        }
        return $objectTypeConfiguration;
    }

    /**
     * Method for creating a InsightProfil.
     *
     * @param  string $p_objecttypeconst
     * @param  string $p_style
     * @param  string $p_width
     * @param  string $p_linecount
     * @param  int $p_insightId
     * @return false|int boolean false or integer
     * @throws isys_exception_dao
     */
    public function createObjectTypeConfiguration($p_objecttypeconst, $p_style, $p_width, $p_linecount, $p_insightId)
    {
        $l_update = "INSERT INTO shd_insight_objecttypeconfiguration SET
            shd_insight_objecttypeconfiguration__objecttypeconst = " . $this->convert_sql_text($p_objecttypeconst) . ",
			shd_insight_objecttypeconfiguration__style = " . $this->convert_sql_text($p_style) . ",
			shd_insight_objecttypeconfiguration__width = " . $this->convert_sql_text($p_width) . ",
			shd_insight_objecttypeconfiguration__linecount = " . $this->convert_sql_text($p_linecount) . ",
			shd_insight_objecttypeconfiguration__shd_insight__id = " . $this->convert_sql_text($p_insightId) . ";";


        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    /**
     * Method for updating a InsightProfil.
     *
     * @param  $objecttypeconfigurationId
     * @param  $p_objecttypeconst
     * @param  $p_style
     * @param  $p_width
     * @param  $p_linecount
     * @param  $p_insightId
     * @return int|false boolean false or integer
     * @throws isys_exception_dao
     */
    public function updateObjectTypeConfiguration($objecttypeconfigurationId, $p_objecttypeconst, $p_style, $p_width, $p_linecount, $p_insightId)
    {
        $l_update = "UPDATE shd_insight_objecttypeconfiguration SET
			shd_insight_objecttypeconfiguration__objecttypeconst = " . $this->convert_sql_text($p_objecttypeconst) . ",
			shd_insight_objecttypeconfiguration__style = " . $this->convert_sql_text($p_style) . ",
			shd_insight_objecttypeconfiguration__width = " . $this->convert_sql_text($p_width) . ",
			shd_insight_objecttypeconfiguration__linecount = " . $this->convert_sql_text($p_linecount) . ",
			shd_insight_objecttypeconfiguration__shd_insight__id = " . $this->convert_sql_text($p_insightId) . "
            WHERE shd_insight_objecttypeconfiguration__id = ". $this->convert_sql_text($objecttypeconfigurationId) .";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $objecttypeconfigurationId;
        }
        return false;
    }

    /**
     * Delete a InsightProfil.
     *
     * @param  int $p_insightId
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteObjectTypeConfiguration($p_insightId)
    {
        /* Delete old dataitems / contact references and main record */
        $l_q   = "DELETE shd_insight_objecttypeconfiguration FROM shd_insight_objecttypeconfiguration 
                WHERE shd_insight_objecttypeconfiguration__shd_insight__id = " . $this->convert_sql_id($p_insightId);
        return $this->_applySqlUpdate($l_q);
    }

    /**
     * @param  int $p_insightId
     * @param  array $p_objectTypeConfigurationIds
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteObjectTypeConfigurationByInsight($p_insightId, $p_objectTypeConfigurationIds)
    {
        /* Delete old dataitems / contact references and main record */
        $l_q   = "DELETE objecttypeconfiguration
                FROM shd_insight_objecttypeconfiguration AS objecttypeconfiguration
                WHERE objecttypeconfiguration.shd_insight_objecttypeconfiguration__shd_insight__id = " . $p_insightId;

        foreach ($p_objectTypeConfigurationIds as $f_objectTypeConfigurationId) {
            $l_q .= " AND shd_insight_objecttypeconfiguration__id != " . $this->convert_sql_id($f_objectTypeConfigurationId);
        }
        return $this->_applySqlUpdate($l_q);
    }

    /**
     * @param  $insightprofil_objectTypeConfiguration_id
     * @return array
     * @throws isys_exception_database
     */
    public function getLineProperties($insightprofil_objectTypeConfiguration_id)
    {
        $l_res = $this->retrieve("SELECT shd_insight_lineproperty.* FROM shd_insight_lineproperty
                                WHERE shd_insight_lineproperty__shd_insight_objecttypeconfiguration_id = " . $this->convert_sql_text($insightprofil_objectTypeConfiguration_id));

        $lineProperties = [];
        while ($l_data = $l_res->get_row()) {
            $l_data["index"]            = $l_data["shd_insight_lineproperty__index"];
            $l_data["linepropertyId"]   = $l_data['shd_insight_lineproperty__id'];
            $l_data["category"]         = $l_data['shd_insight_lineproperty__category'];
            $l_data["property"]         = $l_data['shd_insight_lineproperty__property'];
            $l_data["backgroundcolor"]  = $l_data['shd_insight_lineproperty__backgroundcolor'];
            $l_data["fontcolor"]        = $l_data['shd_insight_lineproperty__fontcolor'];
            $l_data["bold"]             = $l_data['shd_insight_lineproperty__bold'];
            $l_data["italic"]           = $l_data['shd_insight_lineproperty__italic'];
            $l_data["underlined"]       = $l_data['shd_insight_lineproperty__underlined'];

            unset($l_data["shd_insight_lineproperty__index"]);
            unset($l_data["shd_insight_lineproperty__id"]);
            unset($l_data["shd_insight_lineproperty__category"]);
            unset($l_data["shd_insight_lineproperty__property"]);
            unset($l_data["shd_insight_lineproperty__backgroundcolor"]);
            unset($l_data["shd_insight_lineproperty__fontcolor"]);
            unset($l_data["shd_insight_lineproperty__bold"]);
            unset($l_data["shd_insight_lineproperty__italic"]);
            unset($l_data["shd_insight_lineproperty__underlined"]);
            unset($l_data["shd_insight_lineproperty__shd_insight_objecttypeconfiguration_id"]);

            $lineProperties[] = $l_data;
        }
        return $lineProperties;
    }

    /**
     * Method for creating a new report.
     *
     * @param  $p_category
     * @param  $p_property
     * @param  $p_backgroundcolor
     * @param  $p_fontcolor
     * @param  $p_bold
     * @param  $p_italic
     * @param  $p_underline
     * @param  $p_index
     * @param  $p_objecttypeconfigurationId
     * @return boolean|int
     * @throws isys_exception_dao
     */
    public function createLineProperties($p_category, $p_property, $p_backgroundcolor, $p_fontcolor, $p_bold, $p_italic, $p_underline, $p_index, $p_objecttypeconfigurationId)
    {
        $l_update = "INSERT INTO shd_insight_lineproperty SET
			shd_insight_lineproperty__category = " . $this->convert_sql_text($p_category) . ",
			shd_insight_lineproperty__property = " . $this->convert_sql_text($p_property) . ",
			shd_insight_lineproperty__backgroundcolor = " . $this->convert_sql_text($p_backgroundcolor) . ",
			shd_insight_lineproperty__fontcolor = " . $this->convert_sql_text($p_fontcolor) . ",
			shd_insight_lineproperty__bold = " . $this->convert_sql_text($p_bold) . ",
			shd_insight_lineproperty__italic = " . $this->convert_sql_text($p_italic) . ",
			shd_insight_lineproperty__underlined = " . $this->convert_sql_text($p_underline) . ",
			shd_insight_lineproperty__index = " . $this->convert_sql_text($p_index) . ",
			shd_insight_lineproperty__shd_insight_objecttypeconfiguration_id = " . $this->convert_sql_text($p_objecttypeconfigurationId) . ";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    /**
     * @param  $linepropertyId
     * @param  $p_category
     * @param  $p_property
     * @param  $p_backgroundcolor
     * @param  $p_fontcolor
     * @param  $p_bold
     * @param  $p_italic
     * @param  $p_underline
     * @param  $p_index
     * @param  $p_objecttypeconfigurationId
     * @return boolean
     * @throws isys_exception_dao
     */
    public function updateLineProperties($linepropertyId, $p_category, $p_property, $p_backgroundcolor, $p_fontcolor, $p_bold, $p_italic, $p_underline, $p_index, $p_objecttypeconfigurationId)
    {
        $l_update = "UPDATE shd_insight_lineproperty SET
			shd_insight_lineproperty__category = " . $this->convert_sql_text($p_category) . ",
			shd_insight_lineproperty__property = " . $this->convert_sql_text($p_property) . ",
			shd_insight_lineproperty__backgroundcolor = " . $this->convert_sql_text($p_backgroundcolor) . ",
			shd_insight_lineproperty__fontcolor = " . $this->convert_sql_text($p_fontcolor) . ",
			shd_insight_lineproperty__bold = " . $this->convert_sql_text($p_bold) . ",
			shd_insight_lineproperty__italic = " . $this->convert_sql_text($p_italic) . ",
			shd_insight_lineproperty__underlined = " . $this->convert_sql_text($p_underline) . ",
			shd_insight_lineproperty__index = " . $this->convert_sql_text($p_index) . "
            WHERE shd_insight_lineproperty__id = ". $this->convert_sql_text($linepropertyId) .";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $linepropertyId;
        }
        return false;
    }

    /**
     * Deletes a LineProperty.
     *
     * @param  int $p_insightId
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteLineProperties($p_insightId)
    {
        /* Delete old dataitems / contact references and main record */
        $l_q   = "Delete lineproperty FROM shd_insight_lineproperty AS lineproperty
                INNER JOIN shd_insight_objecttypeconfiguration AS objtype
                  ON objtype.shd_insight_objecttypeconfiguration__id = lineproperty.shd_insight_lineproperty__shd_insight_objecttypeconfiguration_id
                WHERE objtype.shd_insight_objecttypeconfiguration__shd_insight__id = " . $this->convert_sql_id($p_insightId);
        return $this->_applySqlUpdate($l_q);
    }

    /**
     * Deletes an filter.
     *
     * @param  int $objectTypeConfiguration_id
     * @param  array $p_linepropertyIds
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteLinePropertiesByObjectTypeConfiguration($objectTypeConfiguration_id, $p_linepropertyIds)
    {
        /* Delete old dataitems / contact references and main record */
        $l_q   = "DELETE lineproperty FROM shd_insight_lineproperty AS lineproperty
                INNER JOIN shd_insight_objecttypeconfiguration AS objtype
                  ON objtype.shd_insight_objecttypeconfiguration__id = lineproperty.shd_insight_lineproperty__shd_insight_objecttypeconfiguration_id
                WHERE lineproperty.shd_insight_lineproperty__shd_insight_objecttypeconfiguration_id = " . $objectTypeConfiguration_id;

        foreach ($p_linepropertyIds as $f_linepropertyId) {
            $l_q .= " AND shd_insight_lineproperty__id != " . $this->convert_sql_id($f_linepropertyId);
        }
        return $this->_applySqlUpdate($l_q);
    }

    /**
     * Apply a SQL update.
     *
     * @param  string $l_sql
     * @return boolean
     * @throws isys_exception_dao
     */
    private function _applySqlUpdate($l_sql)
    {
        $this->begin_update();
        $l_res = $this->update($l_sql);
        if ($l_res) {
            return $this->apply_update();
        }
        return false;
    }

    /**
     * Get data method. Unused.
     *
     * @return null
     */
    public function get_data()
    {
        return null;
    }
}
