<?php

/**
 * i-doit
 *
 * License Dao.
 *
 * @package    i-doit
 * @subpackage smview client
 * @author     Maik Rhode <maik.rhode@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class smview_cmdb_dao_lic extends isys_module_dao
{
    protected static $m_instance = null;

    const MODULE_INSIGHT = "INSIGHT";
    const MODULE_MAPS = "MAPS";
    const MODULE_BSM = "BSM";
    const MODULE_CABLE = "CABLE";
    const MODULE_PATCH = "PATCH";
    const MODULE_TV = "TV";

    /**
     * Object counts.
     *
     * @var string[]
     */
    private $counts = [
        "1000",
        "2000",
        "10000",
        "20000",
        "40000",
        "100000",
        "200000",
        "500000",
    ];

    /**
     * Check if license key is valid.
     *
     * @param  string $license_key
     * @return boolean
     */
    public function checkLicenseKey($license_key)
    {
        if (file_exists(__DIR__ . "/../../init_index.php")) {
            include __DIR__ . "/../../init_index.php";
        }

        $licenceRequest = "";
        $lr_checksum = 0;
        $offset = strlen($license_key) > 20 ? 25 : 17;
        for ($i = 0; $i < strlen($license_key); $i++) {
            if ($license_key[$i] == "-") {
                continue;
            }

            $char = hexdec($license_key[$i]) - 4;
            if ($i < $offset) {
                $lr_checksum += $char;
            }
            $licenceRequest .= $char;
        }

        //check checksum
        if (strlen($license_key) > 20) {
            $lk_checksum = substr($licenceRequest, 20, 4);
            $lk_checksum = (int) $lk_checksum;
        } else {
            $lk_checksum = substr($licenceRequest, 14, 2);
        }

        if ($lk_checksum != $lr_checksum) {
            return false;
        }

        $expireDate = substr($licenceRequest, 0, 4);
        $publishDate = DateTime::createFromFormat('my', $expireDate);
        $datetime = new Datetime();
        if ($datetime > $publishDate) {
            return false;
        }

        //todo: check count

        //check secret
        $secret = substr($licenceRequest, 4, 6);
        return ! ($secret != lic);
    }

    /**
     * Check if a module is licensed.
     *
     * @param  string $p_module
     * @return array|boolean
     */
    public function checkLicenseByModule($p_module)
    {
        $licenseData = $this->_getLicenseData();
        if (empty($licenseData)) {
            return [];
        }

        //check module
        $module_id = $licenseData['module_id'];
        $module = decbin($module_id);
        $a_licence = [];
        for ($i = 0; $i < strlen($module); $i++) {
            if ($module[$i] == "1") {
                $count_index = (int) $licenseData['count_id'];
                $count = ! empty($licenseData['count_id']) ? $this->counts[$count_index] : $licenseData['count'];
                $a_licence[$this->_getModuleByIndex($i)] = [
                    "count"  => $count,
                    "expire" => $licenseData['expire_date'],
                ];
            }
        }

        foreach ($a_licence as $module_name => $module) {
            if ($module_name == $p_module) {
                return true;
            }
        }
        return false;
    }

    /**
     * Get license status.
     *
     * @return array
     */
    public function getLicStatus()
    {
        $licenseData = $this->_getLicenseData();
        if (empty($licenseData)) {
            return [];
        }

        $expireDate = $licenseData['expire_date'];

        //1218 to 12-2018
        $formatExpireDate = $expireDate[0] . $expireDate[1] . "-20" . $expireDate[2] . $expireDate[3];

        if (file_exists(__DIR__ . "/../../init_index.php")) {
            include __DIR__ . "/../../init_index.php";
            if ($licenseData['secret'] != lic) {
                return [];
            }
        }

        //check module
        $module_id = $licenseData['module_id'];
        $module = decbin($module_id);
        $a_licence = [];
        $index = 0;

        $count_index = (int) $licenseData['count_id'];
        $count = ! empty($licenseData['count_id']) ? $this->counts[$count_index] : $licenseData['count'];
        for ($i = strlen($module) - 1; $i >= 0; $i--) {
            if ($module[$i] == "1") {
                $a_licence[$this->_getModuleByIndex($index)] = [
                    "count"  => $count,
                    "expire" => $formatExpireDate
                ];
            }
            $index++;
        }
        return $a_licence;
    }

    /**
     * Generate license key by license request.
     *
     * @param  string $licenseRequest
     * @param  string $month
     * @param  string $year
     * @param  array $modules
     * @return string
     */
    public function generateLicense($licenseRequest, $month, $year, $modules)
    {
        $month = (int) $month;
        if ($month < 1 || $month > 12) {
            return 'month must be between 1 and 12';
        }

        $year = (int) $year;
        if ($year < 2022 || $year > 2050) {
            return 'year must be between 2022 and 2050';
        }

        //4 Ablaufdatum
        //6 Geheimnis
        //2 Modul nummer
        //2 Anzahl

        //2 Prüfsumme
        // (XX-XXXXXX-XX-XX )
        // Request
        //2 Modulnummer
        //6 Geheimnis
        //2 Anzahl
        //2 Prüfsumme

        $licenseRequest = trim($licenseRequest);
        $licenseRequest = str_replace('-', '', $licenseRequest);

        $isOldLicenseRequest = strlen($licenseRequest) <= 12;

        $moduleNumber = "";
        $secret = "";
        $objectCount = "";
        $checksumRequest = "";

        $index = 0;
        $checksum = 0;
        foreach (str_split($licenseRequest) as $aChar) {
            $number = hexdec($aChar);
            $number -= 5;
            if ($isOldLicenseRequest) {
                if ($index < 2) {
                    $moduleNumber = sprintf("%s%d", $moduleNumber, $number);
                } else if ($index < 8) {
                    $secret = sprintf("%s%d", $secret, $number);
                } else if ($index < 10) {
                    $objectCount = sprintf("%s%d", $objectCount, $number);
                } else if ($index < 12) {
                    $checksumRequest = sprintf("%s%d", $checksumRequest, $number);
                }
                if ($index < 10) {
                    $checksum += $number;
                }
            } else {
                if ($index < 2) {
                    $moduleNumber = sprintf("%s%d", $moduleNumber, $number);
                } else if ($index < 8) {
                    $secret = sprintf("%s%d", $secret, $number);
                } else if ($index < 16) {
                    $objectCount = sprintf("%s%d", $objectCount, $number);
                } else if ($index < 20) {
                    $checksumRequest = sprintf("%s%d", $checksumRequest, $number);
                }
                if ($index < 16) {
                    $checksum += $number;
                }
            }
            $index++;
        }

        // trim leading zeros
        $checksumRequest = (int) $checksumRequest;
        $licenseKey = "no valid key";
        $moduleCode = $this->_getSelectedModuleCode($modules);
        if ($checksum == $checksumRequest) {
            // Format MMYY
            $date = sprintf("%s%s", $month, substr($year, 0, 2));
            $moduleCode = sprintf("%02d", $moduleCode);

            $tempLicenseKey = $date . $secret . $moduleCode . $objectCount;
            $hexLicenseKey = "";

            $index2 = 0;
            $checksum2 = 0;
            foreach (str_split($tempLicenseKey) as $tempChar) {
                $number = (int) $tempChar;
                $checksum2 += $number;
                $number += 4;
                $hexNumber = dechex($number);
                if ($index2 > 0 && $index2 % 4 == 0) {
                    $hexLicenseKey = $hexLicenseKey . "-";
                }
                $hexLicenseKey = $hexLicenseKey . $hexNumber;
                $index2++;
            }

            $checksum2Text = strval($checksum2);
            $checksum2Final = "";
            foreach (str_split($checksum2Text) as $checksum2Char) {
                $number = (int) $checksum2Char;
                $number += 4;
                $hexNumber = dechex($number);
                $checksum2Final = $checksum2Final . $hexNumber;
            }

            if ($isOldLicenseRequest) {
                $licenseKey = $hexLicenseKey . $checksum2Final;
            } else {
                // pad checksum to be 4 characters long
                $checksum2Final = str_pad($checksum2Final, 4, '4', STR_PAD_LEFT);
                $licenseKey = $hexLicenseKey . "-" . $checksum2Final;
            }
            $licenseKey = strtoupper( $licenseKey);
        }
        return $licenseKey;
    }

    /**
     * Get selected module code.
     *
     * @param  array $modules
     * @return int
     */
    private function _getSelectedModuleCode($modules) {
        $code = 0;
        foreach ($modules as $module) {
            switch ($module) {
                case self::MODULE_INSIGHT:
                    $code += 1;
                    break;
                case self::MODULE_MAPS:
                    $code += 2;
                    break;
                case self::MODULE_BSM:
                    $code += 4;
                    break;
                case self::MODULE_CABLE:
                    $code += 8;
                    break;
                case self::MODULE_PATCH:
                    $code += 16;
                    break;
                case self::MODULE_TV:
                    $code += 32;
                    break;
            }
        }
        return $code;
    }

    /**
     * Get current license data.
     *
     * @return array
     */
    private function _getLicenseData()
    {
        // OLD LICENSE KEY:
        // (XXXX-XXXX-XXXX)

        // NEW LICENSE KEY:
        // (XXXX-XXXX-XXXX-XXXX-XXXX)

        if (file_exists(__DIR__ . "/../../lic.lic")) {
            $licenceKey = file_get_contents(__DIR__ . "/../../lic.lic");
        } else {
            //error
            return [];
        }

        $licenceKey = trim($licenceKey);
        $licenceRequest = "";
        for ($i = 0; $i < strlen($licenceKey); $i++) {
            if ($licenceKey[$i] == "-") {
                continue;
            }
            $licenceRequest .= hexdec($licenceKey[$i]) - 4;
        }

        $data = [
            'expire_date' => substr($licenceRequest, 0, 4),
            'secret'      => substr($licenceRequest, 4, 6),
            'module_id'   => substr($licenceRequest, 10, 2),
        ];
        if (strlen($licenceKey) < 20) {
            $data['count_id'] = substr($licenceRequest, 12, 2);
        } else {
            $count = substr($licenceRequest, 12, 8);
            $data['count'] = (int) $count;
        }
        return $data;
    }

    /**
     * Get module by index.
     *
     * @param  int $module_index
     * @return string
     */
    private function _getModuleByIndex($module_index)
    {
        $modules = [
            self::MODULE_INSIGHT,
            self::MODULE_MAPS,
            self::MODULE_BSM,
            self::MODULE_CABLE,
            self::MODULE_PATCH,
            self::MODULE_TV,
        ];
        return $modules[$module_index];
    }

    /**
     * get_data always retrieves the data of the main table of this module
     */
    public function get_data()
    {
        // TODO: Implement get_data() method.
    }
}
