<?php
/**
 * i-doit
 *
 * Template Module Dao.
 *
 * @package     i-doit
 * @subpackage  Modules
 * @author      Dennis Stücken <dstuecken@i-doit.org>
 * @version     0.9
 * @copyright   synetics GmbH
 * @license     http://www.i-doit.com/license
 */

class smview_cmdb_dao_map extends isys_module_dao
{
    protected static $m_instance = null;

    protected $circle = 0;

    /**
     * Retrieve the Map.
     *
     * @param   int                $map_id
     * @return  array
     * @throws  Exception
     */
    public function getMap($map_id)
    {
        $l_row = $this->retrieve("SELECT map.* FROM shd_map AS map
              WHERE map.shd_map__id = " . $this->convert_sql_text($map_id))->get_row();
        if (empty($l_row)) {
            return [];
        }

        $l_return = [];
        foreach ($l_row as $f_key => $f_item) {
            if ($f_key == "shd_map__id") {
                $l_return["map_id"] = $f_item;
            } else {
                $l_return[str_replace("shd_map__", "", $f_key)] = $f_item;
            }
        }

        $date = new DateTime();
        if ($this->circle >= 2) {
            return $l_return;
        }
        $this->circle += 1;

        $l_return["map_objects"] = $this->getMapObjectByMap($l_row["shd_map__id"]);
        $l_return["map_connection"] = $this->getMapConnectionByMap($l_row["shd_map__id"]);
        $l_return["map_link"] = $this->getMapLinkByMap($l_row["shd_map__id"]);
        $l_return["timestamp"] = $date->getTimestamp();
        $l_return["layerindex"] = $this->formatLayerindex($l_return["layerindex"], $l_return["map_objects"]);

        $this->circle -= 1;
        return $l_return;
    }

    /**
     * Retrieve the Map.
     *
     * @param  string|null $user_id
     * @return array
     * @throws isys_exception_database
     */
    public function getMaps($user_id = null)
    {
        if ($user_id == null) {
            $l_res = $this->retrieve("SELECT map.* FROM shd_map AS map");
        } else {
            $l_res = $this->retrieve("SELECT map.* FROM shd_map AS map
              WHERE shd_map__isys_cats_person_list__id = ". $this->convert_sql_text($user_id) .";");
        }

        while ($l_data = $l_res->get_row()) {
            $l_return = [];
            foreach ($l_data as $f_key => $f_item) {
                if ($f_key == "shd_map__id") {
                    $l_return["map_id"] = $f_item;
                } else {
                    $l_return[str_replace("shd_map__", "", $f_key)] = $f_item;
                }
            }

            $l_return["map_link"] = $this->getMapLinkByMap($l_data["shd_map__id"]);
            $l_return["map_right"] = $this->getMapRight($l_data["shd_map__id"]);
            $maps[] = $l_return;
        }
        return $maps;
    }

    /**
     * Method for creating a new report.
     *
     * @param  string $p_title
     * @param  string $p_user_id
     * @param  boolean $provide_screenshot
     * @param  int $page_size_preview_mode
     * @param  string $type
     * @return false|int  boolean false or integer
     * @throws isys_exception_dao
     */
    public function createMap($p_title, $p_user_id, $provide_screenshot = false, $type = null, $page_size_preview_mode = null)
    {
        $l_update = "INSERT INTO shd_map SET
			shd_map__title = " . $this->convert_sql_text($p_title) . ",
			shd_map__isys_cats_person_list__id = " . $this->convert_sql_text($p_user_id);

        if (isset($provide_screenshot)) {
            $l_update .= " shd_map__provide_screenshot = " . $this->convert_sql_int($provide_screenshot);
        }
        if (isset($type)) {
            $l_update .= " shd_map__type = " . $this->convert_sql_text($type);
        }
        if (isset($page_size_preview_mode)) {
            $l_update .= " shd_map__page_size_preview_mode = " . $this->convert_sql_text($page_size_preview_mode);
        }
        $l_update .= ";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    public function updateMap($p_params)
    {
        $l_update = "UPDATE shd_map SET ";
        if (isset($p_params["title"]) == true) {
            $l_update .= " shd_map__title = " . $this->convert_sql_text($p_params["title"]) . ", " ;

            //check reference in connection and rename in Map
            $this->updateObjectNameInMap($p_params["title"], $p_params["map_id"]);
        }
        if (isset($p_params["user_id"]) == true) {
            $l_update .= " shd_map__isys_cats_person_list__id = " . $this->convert_sql_text($p_params["user_id"]) . ", " ;
        }
        if (isset($p_params["layerindex"]) == true) {
            $l_update .= " shd_map__layerindex = " . $this->convert_sql_text($p_params["layerindex"]) . ", " ;
        }
        if (isset($p_params["provide_screenshot"])) {
            $l_update .= " shd_map__provide_screenshot = " . $this->convert_sql_int($p_params["provide_screenshot"]) . ", " ;
        }
        if (isset($p_params["type"])) {
            $l_update .= " shd_map__type = " . $this->convert_sql_text($p_params["type"]) . ", " ;
        }
        if (isset($p_params["page_size_preview_mode"])) {
            $l_update .= " shd_map__page_size_preview_mode = " . $this->convert_sql_int($p_params["page_size_preview_mode"]) . ", " ;
        }

        $l_update = substr($l_update, 0, -2);
        $l_update .= " WHERE shd_map__id = ". $this->convert_sql_text($p_params["map_id"]) .";";
        if ($this->update($l_update) && $this->apply_update()) {
            return true;
        }
        return false;
    }

    /**
     * Deletes an Map
     *
     * @param int $map_id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteMap($map_id)
    {
        $l_q   = "DELETE FROM shd_map WHERE shd_map__id = " . $this->convert_sql_id($map_id);
        return $this->_deleteByQuery($l_q);
    }

    /**
     * Create or update a file object to a corresponding map screenshot.
     *
     * @param  string $path
     * @param  string $fileName
     * @throws isys_exception_database
     * @throws isys_exception_filesystem
     */
    public function createOrUpdateMapScreenshot($path, $fileName)
    {
        global $g_absdir, $g_comp_session;

        // get unique filename
        $mapPrefix = 'smview_map_';
        $mapId = str_replace(['.jpg', '.JPG', '.png', '.PNG'], '', $fileName);
        $unique_filename = $mapPrefix . $mapId;
        $map = $this->getMap($mapId);
        if (empty($map['provide_screenshot'])) {
            return;
        }

        $smViewDir = "/upload/images/sm-view";
        $path_upload = $g_absdir . $smViewDir;
        $path = trim($path, DIRECTORY_SEPARATOR);
        $filePath = $path_upload . DIRECTORY_SEPARATOR . $path . DIRECTORY_SEPARATOR . $fileName;

        // copy map screenshot to i-doit file structure
        $idoitFileDir = isys_application::instance()->getOrCreateUploadFileDir($unique_filename);
        $idoitFilePath = $idoitFileDir . $unique_filename;
        copy($filePath, $idoitFilePath);

        // try to get existing screenshot by its name
        $fileDao = new isys_cmdb_dao_category_s_file(isys_application::instance()->database);
        $sql = "SELECT * FROM isys_file_physical physical
			INNER JOIN isys_file_version version ON version.isys_file_version__isys_file_physical__id = physical.isys_file_physical__id
            WHERE isys_file_physical__filename = " . $fileDao->convert_sql_text($unique_filename);
        $row = $fileDao->retrieve($sql)->get_row();
        if (empty($row)) {
            // create new object
            $obj_id = $fileDao->create_object($map['title'], defined_or_default('C__OBJTYPE__FILE'));

            // create database entries
            $md5_hash = md5_file($idoitFilePath);
            $l_physical_id = $fileDao->create_physical_file($unique_filename, $fileName, $md5_hash, $g_comp_session->get_user_id());
        } else {
            // get existing object
            $obj_id = $row['isys_obj__id'];
            $l_physical_id = $row['isys_file_physical__id'];
        }

        //if ($l_physical_id > 0) {
        if (empty($row) && $l_physical_id > 0) {
            // add new version (only for a new object)
            $l_version_id = $fileDao->create_version($obj_id, $l_physical_id, $map['title'], null);

            // update connectors
            $l_cats_id = $fileDao->create_connector('isys_cats_file_list', $obj_id);
            $fileDao->update_cats_file_list($l_cats_id, $l_version_id, null, '');
        }
    }

    /**
     * Retrieve the Map.
     *
     * @param   int      $mapobject_id
     * @return  array
     * @throws  Exception
     */
    public function getMapObject($mapobject_id)
    {
        $lang = isys_application::instance()->container->get('language');
        $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao(isys_application::instance()->database));
        $l_row = $this->retrieve("SELECT mapobject.* FROM shd_map_mapobject AS mapobject
              WHERE mapobject.shd_map_mapobject__id = " . $this->convert_sql_text($mapobject_id))->get_row();
        if (empty($l_row)) {
            return [];
        }

        $l_return = [];
        foreach ($l_row as $f_key => $f_item) {
            if ($f_key == "shd_map_mapobject__id") {
                $l_return["mapobject_id"] = $f_item;
            } else {
                $l_return[str_replace("shd_map_mapobject__", "", $f_key)] = $f_item;
            }
        }

        if ($l_return["is_title_custom"] != "1") {
            if (is_numeric($l_return["isys_obj__id"]) && $l_return["isys_obj__id"] > 0) {
                $category = ! empty($l_return["cat_const"]) ? $l_return["cat_const"] : "C__CATG__GLOBAL";
                if (! defined_or_default($category)) {
                    $category = "C__CATG__GLOBAL";
                }
                $property = ! empty($l_return["property_key"]) ? $l_return["property_key"] : "title";
                $p_param = [
                    C__CMDB__GET__OBJECT => $l_return["isys_obj__id"],
                    'category'           => $category,
                ];

                $l_categoryData = $l_api_model_category->read($p_param);
                if ($l_categoryData === []) {
                    // object deleted but reference still in i-doit
                    $l_return["title"] = $lang->get("LC__MODULE__SMVIEW__MAP_OBJECT_DELETED");
                } else {
                    $item = array_pop($l_categoryData);
                    if (! in_array($property, array_keys($item))) {
                        $l_return["title"] = $lang->get("LC__MODULE__SMVIEW__MAP_OBJECT_PROPERTY_NOT_FOUND");
                    } elseif ($item[$property] == '') {
                        $l_return["title"] = $lang->get("LC__MODULE__SMVIEW__MAP_OBJECT_PROPERTY_EMPTY");
                    } else {
                        $prop = $item[$property];
                        if (is_array($prop) && ! empty($prop['title'])) {
                            $prop = empty($prop['ref_title']) ? $prop['title'] : $prop['ref_title'];
                        }
                        $l_return["title"] = $prop;
                    }
                }
            } elseif (is_numeric($l_return["link"]) && $l_return["link"] > 0) {
                $map = $this->getMap($l_return["link"]);
                $l_return["title"] = $map["title"];
            }
        }

        //from the Database comes the complettpath and we have to cut it for the File.read
        if (strpos($l_return["imagepath"], isys_helper_link::get_base() . "upload/images/sm-view") !== false) {
            $l_return["imagepath"] = str_replace(isys_helper_link::get_base() . "upload/images/sm-view", "", $l_return["imagepath"]);
            $l_return["imagepath"] = str_replace("%20", " ", $l_return["imagepath"]);
        }

        if ($l_return["is_imagepath_custom"] == "0") {
            if (is_numeric($l_return["isys_obj__id"]) && $l_return["isys_obj__id"] > 0) {
                $p_param = [
                    'objID' => $l_return["isys_obj__id"],
                    'category' => "C__CATG__GLOBAL"
                ];

                $l_categoryDatas = $l_api_model_category->read($p_param);
                if ($l_categoryDatas === []) {
                    $l_return["imagepath"] = "";
                } else {
                    foreach ($l_categoryDatas as $item) {
                        $obj_type_id = $item["type"]["id"];
                    }

                    $_dao = new isys_cmdb_dao(isys_application::instance()->database, null);

                    // Data retrieval:
                    $l_data = $_dao->get_object_types_by_properties(["id" => $obj_type_id]);
                    while ($l_row = $l_data->get_row()) {
                        $l_return["imagepath"] = isys_helper_link::get_base() . "images/objecttypes/" . $l_row["isys_obj_type__obj_img_name"];
                    }
                }
            } elseif (is_numeric($l_return["link"]) && $l_return["link"] > 0) {
                $l_return["imagepath"] = "network";
            }
        }

        if ($l_return["livestatus_services"] != "") {
            $l_return["livestatus_services"] = unserialize($l_return["livestatus_services"]);
        }

        if (is_numeric($l_return["isys_obj__id"]) && $l_return["isys_obj__id"] > 0) {
            $l_livestatus =  smview_cmdb_dao_status::instance($this->get_database_component())->get_livestatus_info($l_return["isys_obj__id"]);
            if ($l_livestatus["isys_monitoring_hosts__title"] === null) {
                $l_return["livestatus_instance_name"] = "";
            } else {
                $l_return["livestatus_instance_name"] = $l_livestatus["isys_monitoring_hosts__title"];
            }

            if ($l_livestatus["isys_catg_monitoring_list__host_name_selection"] == 0) {
                $l_return["livestatus_hostname"] = $l_livestatus["isys_catg_monitoring_list__host_name"];
            } elseif ($l_livestatus["isys_catg_monitoring_list__host_name_selection"] == 1) {
            //            Hostname (FQDN) ("") 1
                $l_api_model_category = new isys_cmdb_dao_category_g_ip(isys_application::instance()->database);
                $data = $l_api_model_category->get_data_as_array(null, $l_livestatus["isys_obj__id"], " AND isys_catg_ip_list__primary = 1", null, null);
                // isys_catg_ip_list__hostname   isys_catg_ip_list__domain
                $l_return["livestatus_hostname"] = $data[0]["isys_catg_ip_list__hostname"] . "." . $data[0]["isys_catg_ip_list__domain"];
            } elseif ($l_livestatus["isys_catg_monitoring_list__host_name_selection"] == 2) {
            //            Hostname ("")  2
                $l_api_model_category = new isys_cmdb_dao_category_g_ip(isys_application::instance()->database);
                $data = $l_api_model_category->get_data_as_array(null, $l_livestatus["isys_obj__id"], " AND isys_catg_ip_list__primary = 1", null, null);
                // isys_catg_ip_list__hostname   isys_catg_ip_list__domain
                $l_return["livestatus_hostname"] = $data[0]["isys_catg_ip_list__hostname"];
            } elseif ($l_livestatus["isys_catg_monitoring_list__host_name_selection"] == 3) {
                $l_return["livestatus_hostname"] = $l_livestatus["isys_obj__title"];
            } else {
                $l_return["livestatus_hostname"] = "";
            }

            $fileDao = new isys_cmdb_dao_category_g_file(isys_application::instance()->database);
            $res = $fileDao->get_data(null, $l_return["isys_obj__id"], null, null, C__RECORD_STATUS__NORMAL);
            $files = [];
            while ($row = $res->get_row()) {
                $fileTitle = $row['isys_file_version__title'];
                if (! empty($row['isys_cats_file_list__isys_obj__id'])) {
                    $fileObj = $fileDao->get_object($row['isys_cats_file_list__isys_obj__id'])->get_row();
                    $fileTitle = $fileObj['isys_obj__title'];
                }
                $files[] = [
                    'file_id'    => $row['isys_cats_file_list__isys_obj__id'],
                    'file_title' => $fileTitle,
                    'file_link'  => $row['isys_catg_file_list__link'],
                ];
            }
            $l_return["files"] = $files;

            // todo: activate this feature if TWI agrees
//            if (! empty($l_return['shd_insight__id'])) {
//                $l_return = $this->_mergeWithInsightStyling($l_return);
//            }
        }

        // get type of this map object
        $l_return["type"] = $this->_getMapObjectType($l_return);
        return $l_return;
    }

    /**
     * Merge map object styling with insight styles.
     *
     * @param  array $l_return
     * @return array
     * @throws Exception
     */
    private function _mergeWithInsightStyling($l_return)
    {
        // get object type of CI object
        $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $obj = $dao->get_data(null, $l_return["isys_obj__id"])->get_row();
        $objTypeConst = $obj['isys_obj_type__const'];

        // get insight profile
        $dao = smview_cmdb_dao_insightprofil::instance(isys_application::instance()->database);
        $profile = $dao->getInsightProfil($l_return['shd_insight__id']);

        $cat = $l_return['cat_const'];
        $catProp = $l_return['property_key'];
        if (empty($cat) || empty($catProp)) {
            return $l_return;
        }

        // merge with insight styling
        foreach ($profile["ObjectTypeConfiguration"] as $config) {
            // get object type config of current obj_id
            if ($config["objecttypeconst"] != $objTypeConst && $config["objecttypeconst"] != 'C__OBJTYPE__ALL') {
                continue;
            }

            foreach ($config["LineProperties"] as $prop) {
                if ($prop["category"] != $cat || $prop["property"] != $catProp) {
                    continue;
                }

                $l_return['fillcolor'] = $prop["backgroundcolor"];
                $l_return['textcolor'] = $prop["fontcolor"];

                $font = explode(';' , $l_return['font']);
                if (count($font) != 3) {
                    continue;
                }

                $style = 0;
                if ($prop["bold"] == '1') {
                    $style = 1;
                }
                if ($prop["italic"] == '1') {
                    $style = 2;
                }
                if ($prop["bold"] == '1' && $prop["italic"] == '1') {
                    $style = 3;
                }
                $font[1] = $style;
                $l_return['font'] = implode(';', $font);
            }
        }
        return $l_return;
    }

    /**
     * Get the type of a map object.
     *
     * @param  array $l_return
     * @return string
     */
    private function _getMapObjectType($l_return)
    {
        // get type of this map object
        switch ($l_return['realizertype']) {
            case '0':
                // image node
                $type = 'image_node';
                break;
            case '1':
            case '2':
                // text or shape node
                if (isset($l_return['title']) && $l_return['title'] != '') {
                    $type = 'text_node';
                } else {
                    switch ($l_return['shapetype']) {
                        case '0': $type = 'shape_node_rect'; break; // Rechteck
                        case '1': $type = 'shape_node_round_rect'; break; // Rechteck, runde Ecken
                        case '2': $type = 'shape_node_ellipse'; break; // Kreis
                        case '3': $type = 'shape_node_rhomboid'; break; // Parallelogramm
                        case '4': $type = 'shape_node_banner'; break; // Banner
                        case '5': $type = 'shape_node_triangle'; break; // Dreieck
                        case '7': $type = 'shape_node_stop_sign'; break; // Stoppschild
                        case '8': $type = 'shape_node_rhomb'; break; // Raute
                        case '9': $type = 'shape_node_cone_top'; break; // Trichter oben
                        case '10': $type = 'shape_node_cone_bottom'; break; // Trichter unten
                        default: $type = 'shape_node';
                    }
                }
                break;
            default:
                $type = 'unknown';
        }
        return $type;
    }

    /**
     * Retrieve the Map.
     *
     * @param   int                $map_id
     * @return  array
     * @throws  Exception
     */
    public function getMapObjectByMap($map_id)
    {
        $l_query = "SELECT mapobject.* FROM shd_map_mapobject AS mapobject
              WHERE mapobject.shd_map_mapobject__shd_map__id = " . $this->convert_sql_text($map_id);
        $l_res = $this->retrieve($l_query);
        while ($l_data = $l_res->get_row()) {
            $mapsObject[] = $this->getMapObject($l_data["shd_map_mapobject__id"]);
        }
        return $mapsObject;
    }

    /**
     * Deletes an Map
     *
     * @param  string $map_name
     * @param  int $map_id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function updateObjectNameInMap($map_name, $map_id)
    {
        $this->begin_update();

        /* Delete old dataitems / contact references and main record */
        $l_q   = "UPDATE `shd_map_mapobject` SET `shd_map_mapobject__title` = '" .$map_name .
            "' WHERE shd_map_mapobject.shd_map_mapobject__is_title_custom = 0 AND `shd_map_mapobject`.`shd_map_mapobject__link` = " . $this->convert_sql_id($map_id);
        $l_res = $this->update($l_q);
        if ($l_res) {
            return $this->apply_update();
        }
        return false;
    }

    public function getMapObjectAndMapLinkByMap($map_id)
    {
        $l_query = "SELECT mapobject.* FROM shd_map_mapobject AS mapobject
              WHERE mapobject.shd_map_mapobject__shd_map__id = " . $this->convert_sql_text($map_id) .
            " AND (mapobject.shd_map_mapobject__isys_obj__id != 0 Or mapobject.shd_map_mapobject__link != '')";

        $l_res = $this->retrieve($l_query);
        while ($l_data = $l_res->get_row()) {
            $l_newRow = [];
            foreach ($l_data as $f_key => $f_item) {
                if ($f_key == "shd_map_mapobject__id") {
                    $l_newRow["mapobject_id"] = $f_item;
                } else {
                    $l_newRow[str_replace("shd_map_mapobject__", "", $f_key)] = $f_item;
                }
            }
            $mapsObject[] = $l_newRow;
        }
        return $mapsObject;
    }

    /**
     * Retrieve the Map.
     *
     * @param   int                $map_id
     * @return  array
     * @throws  Exception
     */
    public function getMapLinkByMap($map_id)
    {
        $l_res = $this->retrieve("SELECT mapobject.shd_map_mapobject__link FROM shd_map_mapobject AS mapobject
              WHERE mapobject.shd_map_mapobject__shd_map__id = " . $this->convert_sql_text($map_id) . " AND mapobject.shd_map_mapobject__link != ''");

        while ($l_data = $l_res->get_row()) {
            $mapsObject[$l_data["shd_map_mapobject__link"]] = $l_data["shd_map_mapobject__link"];
        }
        return $mapsObject;
    }

    /**
     * @param  array $p_params
     * @return boolean|int
     * @throws isys_exception_dao
     */
    public function createMapObject($p_params)
    {
        if ($p_params["livestatus_services"] != "") {
            $livestatus_services = serialize($p_params["livestatus_services"]);
        } else {
            $livestatus_services = "";
        }

        $insightId = ! empty($p_params["shd_insight__id"]) ? $this->convert_sql_id($p_params["shd_insight__id"]) : 'NULL';
        $l_update = "INSERT INTO shd_map_mapobject SET
			shd_map_mapobject__shd_map__id = " . $this->convert_sql_text($p_params["shd_map__id"]) . ",
			shd_map_mapobject__isys_obj__id = " . $this->convert_sql_text($p_params["isys_obj__id"]) . ",
			shd_map_mapobject__livestatus_profile = " . $this->convert_sql_text($p_params["livestatus_profile"]) . ",
			shd_map_mapobject__livestatus_services = " . $this->convert_sql_text($livestatus_services) . ",
			shd_map_mapobject__position = " . $this->convert_sql_text($p_params["position"]) . ",
			shd_map_mapobject__width = " . $this->convert_sql_text($p_params["width"]) . ",
			
			shd_map_mapobject__height = " . $this->convert_sql_text($p_params["height"]) . ",
			shd_map_mapobject__title = " . $this->convert_sql_text($p_params["title"]) . ",
			shd_map_mapobject__is_title_custom = " . $this->convert_sql_text($p_params["is_title_custom"]) . ",
			shd_map_mapobject__labelposition = " . $this->convert_sql_text($p_params["labelposition"]) . ",
			shd_map_mapobject__imagepath = " . $this->convert_sql_text($p_params["imagepath"]) . ",
			shd_map_mapobject__is_imagepath_custom = " . $this->convert_sql_text($p_params["is_imagepath_custom"]) . ",
			shd_map_mapobject__layerindex = " . $this->convert_sql_text($p_params["layerindex"]) . ",
			
			shd_map_mapobject__textvisible = " . $this->convert_sql_text($p_params["textvisible"]) . ",
			shd_map_mapobject__iconvisible = " . $this->convert_sql_text($p_params["iconvisible"]) . ",
			shd_map_mapobject__link = " . $this->convert_sql_text($p_params["link"]) . ",
			shd_map_mapobject__realizertype = " . $this->convert_sql_text($p_params["realizertype"]) . ",
			shd_map_mapobject__shapetype = " . $this->convert_sql_text($p_params["shapetype"]) . ",
			
			shd_map_mapobject__livestatus_color = " . $this->convert_hex_to_signed_integer($p_params["livestatus_color"]) . ",
			shd_map_mapobject__fillcolor = " . $this->convert_hex_to_signed_integer($p_params["fillcolor"]) . ",
			shd_map_mapobject__fillcolor2 = " . $this->convert_hex_to_signed_integer($p_params["fillcolor2"]) . ",
			shd_map_mapobject__iscolorgradient = " . $this->convert_sql_text($p_params["iscolorgradient"]) . ",
			shd_map_mapobject__linecolor = " . $this->convert_hex_to_signed_integer($p_params["linecolor"]) . ",
			shd_map_mapobject__labellinecolor = " . $this->convert_hex_to_signed_integer($p_params["labellinecolor"]) . ",
			
			shd_map_mapobject__cat_const = " . $this->convert_sql_text($p_params["cat_const"]) . ",
			shd_map_mapobject__property_key = " . $this->convert_sql_text($p_params["property_key"]) . ",
			shd_map_mapobject__shd_insight__id = " . $insightId . ",
			
			shd_map_mapobject__layout = " . $this->convert_sql_text($p_params["layout"]) . ",
			shd_map_mapobject__layout_configuration = " . $this->convert_sql_text($p_params["layout_configuration"]) . ",
			shd_map_mapobject__cable_position = " . $this->convert_sql_text($p_params["cable_position"]) . ",
			shd_map_mapobject__cable_orientation = " . $this->convert_sql_text($p_params["cable_orientation"]) . ",
			
			shd_map_mapobject__textcolor = " . $this->convert_hex_to_signed_integer($p_params["textcolor"]) . ",
			shd_map_mapobject__font = " . $this->convert_sql_text($p_params["font"]) . ",
			shd_map_mapobject__textalignment = " . $this->convert_sql_text($p_params["textalignment"]) . ",
			shd_map_mapobject__locked = " . $this->convert_sql_text($p_params["locked"]) . ";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    /**
     * Convert hex-value to signed integer.
     *
     * @param  string $color
     * @return string
     */
    private function convert_hex_to_signed_integer($color)
    {
        $value = null;
        if (! isset($color)) {
            return $this->convert_sql_text($value);
        }

        $value = $color;
        $color = trim($color, '#');
        if (! ctype_xdigit($color)) {
            return $this->convert_sql_text($value);
        }

        if (strlen($color) == 3) {
            // fill up
            $full = '';
            $array = str_split($color);
            foreach ($array as $c) {
                $full .= $c . $c;
            }
            $color = $full;
        }
        if (strlen($color) != 6) {
            // whats now? return original parameter
            return $this->convert_sql_text($value);
        }
        // convert
        $c = 'FF' . $color;
        $value = reset(unpack("l", pack("l", hexdec($c))));
        return $this->convert_sql_text($value);
    }

    /**
     * Method for creating a new report.
     *
     * @param  array $p_params
     * @param  string $map_id
     * @return false|int  boolean false or integer
     * @throws isys_exception_dao
     */
    public function duplicateMapObject($p_params, $map_id)
    {
        $insightId = ! empty($p_params["shd_insight__id"]) ? $this->convert_sql_id($p_params["shd_insight__id"]) : 'NULL';
        $l_update = "INSERT INTO shd_map_mapobject SET
			shd_map_mapobject__shd_map__id = " . $map_id . ",
			shd_map_mapobject__isys_obj__id = " . $this->convert_sql_text($p_params["isys_obj__id"]) . ",
			shd_map_mapobject__position = " . $this->convert_sql_text($p_params["position"]) . ",
			shd_map_mapobject__width = " . $this->convert_sql_text($p_params["width"]) . ",
			
			shd_map_mapobject__height = " . $this->convert_sql_text($p_params["height"]) . ",
			shd_map_mapobject__title = " . $this->convert_sql_text($p_params["title"]) . ",
			shd_map_mapobject__is_title_custom = " . $this->convert_sql_text($p_params["is_title_custom"]) . ",
			shd_map_mapobject__labelposition = " . $this->convert_sql_text($p_params["labelposition"]) . ",
			shd_map_mapobject__imagepath = " . $this->convert_sql_text($p_params["imagepath"]) . ",
			shd_map_mapobject__is_imagepath_custom = " . $this->convert_sql_text($p_params["is_imagepath_custom"]) . ",
			shd_map_mapobject__layerindex = " . $this->convert_sql_text($p_params["layerindex"]) . ",
			
			shd_map_mapobject__textvisible = " . $this->convert_sql_text($p_params["textvisible"]) . ",
			shd_map_mapobject__iconvisible = " . $this->convert_sql_text($p_params["iconvisible"]) . ",
			shd_map_mapobject__link = " . $this->convert_sql_text($p_params["link"]) . ",
			shd_map_mapobject__realizertype = " . $this->convert_sql_text($p_params["realizertype"]) . ",
			shd_map_mapobject__shapetype = " . $this->convert_sql_text($p_params["shapetype"]) . ",
			
			shd_map_mapobject__cat_const = " . $this->convert_sql_text($p_params["cat_const"]) . ",
			shd_map_mapobject__property_key = " . $this->convert_sql_text($p_params["property_key"]) . ",
			shd_map_mapobject__shd_insight__id = " . $insightId . ",
			
			shd_map_mapobject__layout = " . $this->convert_sql_text($p_params["layout"]) . ",
			shd_map_mapobject__layout_configuration = " . $this->convert_sql_text($p_params["layout_configuration"]) . ",
			shd_map_mapobject__cable_position = " . $this->convert_sql_text($p_params["cable_position"]) . ",
			shd_map_mapobject__cable_orientation = " . $this->convert_sql_text($p_params["cable_orientation"]) . ",
			
			shd_map_mapobject__livestatus_color = " . $this->convert_hex_to_signed_integer($p_params["livestatus_color"]) . ",
			shd_map_mapobject__fillcolor = " . $this->convert_hex_to_signed_integer($p_params["fillcolor"]) . ",
			shd_map_mapobject__fillcolor2 = " . $this->convert_hex_to_signed_integer($p_params["fillcolor2"]) . ",
			shd_map_mapobject__iscolorgradient = " . $this->convert_sql_text($p_params["iscolorgradient"]) . ",
			shd_map_mapobject__linecolor = " . $this->convert_hex_to_signed_integer($p_params["linecolor"]) . ",
			shd_map_mapobject__labellinecolor = " . $this->convert_hex_to_signed_integer($p_params["labellinecolor"]) . ",
			
			shd_map_mapobject__textcolor = " . $this->convert_hex_to_signed_integer($p_params["textcolor"]) . ",
			shd_map_mapobject__font = " . $this->convert_sql_text($p_params["font"]) . ",
			shd_map_mapobject__textalignment = " . $this->convert_sql_text($p_params["textalignment"]) . ",
			shd_map_mapobject__locked = " . $this->convert_sql_text($p_params["locked"]) . ";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    public function updateMapObject($p_params)
    {
        $textProperties = [
            "shd_map__id",
            "isys_obj__id",
            "position",
            "livestatus_profile",
            "width",
            "height",
            "title",
            "labelposition",
            "imagepath",
            "layerindex",
            "link",
            "realizertype",
            "shapetype",
            "cat_const",
            "property_key",
            "shd_insight__id",
            "layout",
            "layout_configuration",
            "cable_position",
            "cable_orientation",
            "font",
            "textalignment",
            "locked",
        ];

        $hexProperties = [
            "fillcolor",
            "fillcolor2",
            "linecolor",
            "labellinecolor",
            "livestatus_color",
            "textcolor",
        ];

        $listProperties = [
            "livestatus_services",
        ];

        $boolProperties = [
            "is_title_custom",
            "is_imagepath_custom",
            "textvisible",
            "iconvisible",
            "iscolorgradient",
        ];

        $l_update = $this->_getUpdateSql($p_params, 'shd_map_mapobject', $textProperties, $hexProperties, $listProperties, $boolProperties);
        if (isset($p_params["shd_insight__id"])) {
            $insightId = ! empty($p_params["shd_insight__id"]) ? $this->convert_sql_id($p_params["shd_insight__id"]) : 'NULL';
            $l_update .= ", shd_insight__id = $insightId";
        }
        $l_update .= " WHERE shd_map_mapobject__id = ". $this->convert_sql_text($p_params["mapobject_id"]) .";";
        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    private function _getUpdateSql($p_params, $tableName, $textProperties, $hexProperties, $listProperties = [], $boolProperties = [])
    {
        $allProperties = array_merge($textProperties, $hexProperties, $listProperties, $boolProperties);
        $l_update = "UPDATE $tableName SET ";
        foreach ($allProperties as $prop) {
            if (isset($p_params[$prop])) {
                $key = $tableName . '__' . $prop;
                $value = $this->convert_sql_text($p_params[$prop]);
                if (in_array($prop, $hexProperties)) {
                    $value = $this->convert_hex_to_signed_integer($p_params[$prop]);
                }
                if (in_array($prop, $boolProperties)) {
                    $value = $this->convert_sql_boolean($p_params[$prop]);
                }
                if (in_array($prop, $listProperties)) {
                    $value = $this->convert_sql_text(serialize($p_params[$prop]));
                }
                $l_update .= " $key = $value, " ;
            }
        }

        // remove last comma
        $l_update = substr($l_update, 0, -2);
        return $l_update;
    }

    /**
     * Deletes an filter.
     *
     * @param int $mapobject_id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteMapObject($mapobject_id)
    {
        $l_q   = "DELETE FROM shd_map_mapobject WHERE shd_map_mapobject__id = " . $this->convert_sql_id($mapobject_id);
        return $this->_deleteByQuery($l_q);
    }

    /**
     * Deletes an filter.
     *
     * @param int $map_id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteMapObjectByMapId($map_id)
    {
        $l_q   = "DELETE FROM shd_map_mapobject WHERE shd_map_mapobject__shd_map__id = " . $this->convert_sql_id($map_id);
        return $this->_deleteByQuery($l_q);
    }

    /**
     * Delete an object by query.
     *
     * @param  string $sql
     * @return boolean
     * @throws isys_exception_dao
     */
    private function _deleteByQuery($sql)
    {
        $this->begin_update();
        return $this->update($sql) && $this->apply_update();
    }

    /**
     * Retrieve the Map.
     *
     * @param   int  $mapconnection_id
     * @return  array
     * @throws  Exception
     */
    public function getMapConnection($mapconnection_id)
    {
        $l_row = $this->retrieve("SELECT mapconnection.* FROM shd_map_mapconnection AS mapconnection
              WHERE mapconnection.shd_map_mapconnection__id = " . $this->convert_sql_text($mapconnection_id))->get_row();

        if (empty($l_row)) {
            return [];
        }

        $l_return = [];
        foreach ($l_row as $f_key => $f_item) {
            if ($f_key == "shd_map_mapconnection__id") {
                $l_return["mapconnection_id"] = $f_item;
            } else {
                $l_return[str_replace("shd_map_mapconnection__", "", $f_key)] = $f_item;
            }
        }
        return $l_return;
    }

    /**
     * Retrieve the Map.
     *
     * @param   int                $map_id
     * @return  array
     * @throws  Exception
     */
    public function getMapConnectionByMap($map_id)
    {
        $l_res = $this->retrieve("SELECT mapconnection.* FROM shd_map_mapconnection AS mapconnection
              WHERE mapconnection.shd_map_mapconnection__shd_map__id = " . $this->convert_sql_text($map_id));

        while ($l_data = $l_res->get_row()) {
            $l_return = [];
            foreach ($l_data as $f_key => $f_item) {
                if ($f_key == "shd_map_mapconnection__id") {
                    $l_return["mapconnection_id"] = $f_item;
                } else {
                    $l_return[str_replace("shd_map_mapconnection__", "", $f_key)] = $f_item;
                }
            }
            $mapConnection[] = $l_return;
        }
        return $mapConnection;
    }

    /**
     * Method for creating a new map connection.
     *
     * @param  array $p_params
     * @return int|false
     * @throws isys_exception_dao
     */
    public function createMapConnection($p_params)
    {
        $sourcearrowtype = isset($p_params["sourcearrowtype"]) ? $p_params["sourcearrowtype"] : "1";
        $targetarrowtype = isset($p_params["targetarrowtype"]) ? $p_params["targetarrowtype"] : "2";
        $l_update = "INSERT INTO shd_map_mapconnection SET
			shd_map_mapconnection__shd_map__id = " . $this->convert_sql_text($p_params["shd_map__id"]) . ",
			shd_map_mapconnection__isys_obj__id = " . $this->convert_sql_text($p_params["isys_obj__id"]) . ",
			shd_map_mapconnection__title = " . $this->convert_sql_text($p_params["title"]) . ",
			shd_map_mapconnection__source_shd_map_mapobject__id = " . $this->convert_sql_text($p_params["source_shd_map_mapobject__id"]) . ",
			
			shd_map_mapconnection__target_shd_map_mapobject__id = " . $this->convert_sql_text($p_params["target_shd_map_mapobject__id"]) . ",
			shd_map_mapconnection__targetarrowtype = " . $this->convert_sql_text($targetarrowtype) . ",
			shd_map_mapconnection__sourcearrowtype = " . $this->convert_sql_text($sourcearrowtype) . ",
			shd_map_mapconnection__sourcepoint = " . $this->convert_sql_text($p_params["sourcepoint"]) . ",
			shd_map_mapconnection__targetpoint = " . $this->convert_sql_text($p_params["targetpoint"]) . ",
			shd_map_mapconnection__bends = " . $this->convert_sql_text($p_params["bends"]) . ",
			
			shd_map_mapconnection__linestyle = " . $this->convert_sql_text($p_params["linestyle"]) . ",
			shd_map_mapconnection__linewidth = " . $this->convert_sql_text($p_params["linewidth"]) . ",
			shd_map_mapconnection__linecolor = " . $this->convert_hex_to_signed_integer($p_params["linecolor"]) . ",
			shd_map_mapconnection__textcolor = " . $this->convert_hex_to_signed_integer($p_params["textcolor"]) . ",
			shd_map_mapconnection__font = " . $this->convert_sql_text($p_params["font"]) . ",
			
			shd_map_mapconnection__layerindex = " . $this->convert_sql_text($p_params["layerindex"]) . ",
			shd_map_mapconnection__realizertype = " . $this->convert_sql_text($p_params["realizertype"]) . ",
			shd_map_mapconnection__connectiontype = " . $this->convert_sql_text($p_params["connectiontype"]) . ",
			shd_map_mapconnection__textvisible = " . $this->convert_sql_text($p_params["textvisible"]) . ",
			shd_map_mapconnection__locked = " . $this->convert_sql_text($p_params["locked"]) . ";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    /**
     * Method for creating a new report.
     *
     * @param  array $p_params
     * @param  int $p_map_id
     * @return false|int  boolean false or integer
     * @throws isys_exception_dao
     */
    public function duplicateMapConnection($p_params, $p_map_id)
    {
        $l_update = "INSERT INTO shd_map_mapconnection SET
			shd_map_mapconnection__shd_map__id = " . $p_map_id . ",
			shd_map_mapconnection__isys_obj__id = " . $this->convert_sql_text($p_params["isys_obj__id"]) . ",
			shd_map_mapconnection__title = " . $this->convert_sql_text($p_params["title"]) . ",
			shd_map_mapconnection__source_shd_map_mapobject__id = " . $this->convert_sql_text($p_params["source_shd_map_mapobject__id"]) . ",
			
			shd_map_mapconnection__target_shd_map_mapobject__id = " . $this->convert_sql_text($p_params["target_shd_map_mapobject__id"]) . ",
			shd_map_mapconnection__targetarrowtype = " . $this->convert_sql_text($p_params["targetarrowtype"]) . ",
			shd_map_mapconnection__sourcearrowtype = " . $this->convert_sql_text($p_params["sourcearrowtype"]) . ",
			shd_map_mapconnection__sourcepoint = " . $this->convert_sql_text($p_params["sourcepoint"]) . ",
			shd_map_mapconnection__targetpoint = " . $this->convert_sql_text($p_params["targetpoint"]) . ",
			shd_map_mapconnection__bends = " . $this->convert_sql_text($p_params["bends"]) . ",
			
			shd_map_mapconnection__linestyle = " . $this->convert_sql_text($p_params["linestyle"]) . ",
			shd_map_mapconnection__linewidth = " . $this->convert_sql_text($p_params["linewidth"]) . ",
			shd_map_mapconnection__linecolor = " . $this->convert_hex_to_signed_integer($p_params["linecolor"]) . ",
			shd_map_mapconnection__textcolor = " . $this->convert_hex_to_signed_integer($p_params["textcolor"]) . ",
			shd_map_mapconnection__font = " . $this->convert_sql_text($p_params["font"]) . ",
			
			shd_map_mapconnection__layerindex = " . $this->convert_sql_text($p_params["layerindex"]) . ",
			shd_map_mapconnection__realizertype = " . $this->convert_sql_text($p_params["realizertype"]) . ",
			shd_map_mapconnection__connectiontype = " . $this->convert_sql_text($p_params["connectiontype"]) . ",
			shd_map_mapconnection__textvisible = " . $this->convert_sql_text($p_params["textvisible"]) . ",
			shd_map_mapconnection__locked = " . $this->convert_sql_text($p_params["locked"]) . ";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    /**
     * @param  array $p_params
     * @return boolean|int
     * @throws isys_exception_dao
     */
    public function updateMapConnection($p_params)
    {
        $textProperties = [
            "shd_map__id",
            "isys_obj__id",
            "title",
            "source_shd_map_mapobject__id",
            "target_shd_map_mapobject__id",
            "targetarrowtype",
            "sourcearrowtype",
            "sourcepoint",
            "targetpoint",
            "bends",
            "linestyle",
            "linewidth",
            "font",
            "layerindex",
            "realizertype",
            "connectiontype",
            "textvisible",
            "locked",
        ];

        $hexProperties = [
            "linecolor",
            "textcolor",
        ];

        $l_update = $this->_getUpdateSql($p_params, 'shd_map_mapconnection', $textProperties, $hexProperties);
        $l_update .= " WHERE shd_map_mapconnection__id = ". $this->convert_sql_text($p_params["mapconnection_id"]) .";";
        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    /**
     * Deletes an Mapconnection
     *
     * @param int $mapconnection_id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteMapConnection($mapconnection_id)
    {
        $l_q   = "DELETE FROM shd_map_mapconnection WHERE shd_map_mapconnection__id = " . $this->convert_sql_id($mapconnection_id);
        return $this->_deleteByQuery($l_q);
    }

    /**
     * Deletes an Mapconnection by Map
     *
     * @param int $map_id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteMapConnectionByMapId($map_id)
    {
        $l_q   = "DELETE FROM shd_map_mapconnection WHERE shd_map_mapconnection__shd_map__id = " . $this->convert_sql_id($map_id);
        return $this->_deleteByQuery($l_q);
    }

    /**
     * Retrieve the homemap.
     *
     * @param   int                $user_id
     * @return  array
     * @throws  Exception
     */
    public function getHome($user_id)
    {
        $l_row = $this->retrieve("SELECT map.* FROM shd_map_homemap AS maphome
              JOIN shd_map as map ON maphome.shd_map_homemap__shd_map__id = map.shd_map__id
              WHERE maphome.shd_map_homemap__isys_cats_person_list__id = " . $this->convert_sql_text($user_id))
            ->get_row();

        if (empty($l_row)) {
            return [];
        }

        $l_return = [];
        foreach ($l_row as $f_key => $f_item) {
            if ($f_key == "shd_map__id") {
                $l_return["map_id"] = $f_item;
            } else {
                $l_return[str_replace("shd_map__", "", $f_key)] = $f_item;
            }
        }
        return $l_return;
    }

    /**
     * Retrieve the homemap.
     *
     * @param   int                $user_id
     * @return  boolean
     * @throws  Exception
     */
    public function addHome($user_id, $map_id)
    {
        $l_sql = 'INSERT INTO shd_map_homemap (shd_map_homemap__shd_map__id, shd_map_homemap__isys_cats_person_list__id) VALUES ('
            . $this->convert_sql_text($map_id) .', ' . $this->convert_sql_text($user_id) . ')';

        if ($this->update($l_sql)) {
            $this->m_strLogbookSQL = $l_sql;
            return $this->apply_update();
        }
    }

    /**
     * Retrieve the homemap.
     *
     * @param   int                $user_id
     * @return  boolean
     * @throws  Exception
     */
    public function deleteHome($user_id)
    {
        $l_sql = "DELETE FROM shd_map_homemap WHERE shd_map_homemap.shd_map_homemap__isys_cats_person_list__id = " . $this->convert_sql_text($user_id);
        return $this->_deleteByQuery($l_sql);
    }

    /**
     *
     * @param   integer                $user_id
     *
     * @return  array
     * @throws  Exception
     */
    public function getFavorites($user_id)
    {
        $l_res = $this->retrieve("SELECT mapfavoriten.* FROM shd_map_favoriten AS mapfavoriten
              WHERE mapfavoriten.shd_map_favoriten__isys_cats_person_list__id = " . $this->convert_sql_text($user_id));

        while ($l_data = $l_res->get_row()) {
            $l_return = [];
            foreach ($l_data as $f_key => $f_item) {
                if ($f_key == "shd_map_favoriten__id") {
                    $l_return["map_id"] = $f_item;
                } else {
                    $l_return[str_replace("shd_map_favoriten__", "", $f_key)] = $f_item;
                }
            }

            //count
            if (isset($p_params['count_objects']) == true) {
                $smview_cmdb_dao_filter_sql = new smview_cmdb_dao_filter_sql(isys_application::instance()->database);
                $query_filter = $smview_cmdb_dao_filter_sql->create_property_query_for_filter($l_return, true);

                $return = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->query($query_filter);
                $l_return["count"] = $return["content"][0]["Count(*)"];
            }
            $favoriten[] = $l_return;
        }
        return $favoriten;
    }

    /**
     * Method for creating a new report.
     *
     * @param string $p_user_id
     * @param string $p_parent_id
     * @param string $p_title
     * @return false|int  boolean false or integer
     * @throws isys_exception_dao
     */
    public function createMapFavorites($p_user_id, $p_parent_id, $p_title)
    {
        $l_update = "INSERT INTO shd_map_favoriten SET
			shd_map_favoriten__title = " . $this->convert_sql_text($p_title) . ",
			shd_map_favoriten__parent_id = " . $this->convert_sql_text($p_parent_id) . ",
			shd_map_favoriten__isys_cats_person_list__id = " . $this->convert_sql_text($p_user_id) . ";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    public function updateMapFavorites($map_favoriten_id, $p_user_id, $p_parent_id, $p_title)
    {
        $l_update = "UPDATE shd_map_favoriten SET
			shd_map_favoriten__title = " . $this->convert_sql_text($p_title) . ",
			shd_map_favoriten__parent_id = " . $this->convert_sql_text($p_parent_id) . ",
			shd_map_favoriten__isys_cats_person_list__id = " . $this->convert_sql_text($p_user_id)  . "         
            WHERE shd_map_favoriten__id = ". $this->convert_sql_text($map_favoriten_id) .";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    /**
     * @param int $map_favoriten_id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteMapFavorites($map_favoriten_id)
    {
        $l_q   = "DELETE FROM shd_map_favoriten WHERE shd_map_favoriten__id = " . $this->convert_sql_id($map_favoriten_id);
        return $this->_deleteByQuery($l_q);
    }

    /**
     * @param   int  $mapright_id
     * @return  array
     * @throws  Exception
     */
    public function getMapRight($mapright_id)
    {
        $l_row = $this->retrieve("SELECT mapright.* FROM shd_map_right AS mapright
              WHERE mapright.shd_map_right__id = " . $this->convert_sql_text($mapright_id))->get_row();
        if (empty($l_row)) {
            return [];
        }

        $l_return = [];
        foreach ($l_row as $f_key => $f_item) {
            if ($f_key == "shd_map_right__id") {
                $l_return["mapright_id"] = $f_item;
            } else {
                $l_return[str_replace("shd_map_right__", "", $f_key)] = $f_item;
            }
        }
        return $l_return;
    }

    /**
     * @param   int  $map_id
     * @return  array
     * @throws  Exception
     */
    public function getAllMapRightByMapId($map_id)
    {
        $sql = "SELECT mapright.* FROM shd_map_right AS mapright
              WHERE mapright.shd_map_right__map_id = " . $this->convert_sql_text($map_id);
        return $this->_getMapRightByQuery($sql);
    }

    /**
     *
     * @return  array
     * @throws  Exception
     */
    public function getAllMapRight()
    {
        $sql = "SELECT mapright.* FROM shd_map_right AS mapright";
        return $this->_getMapRightByQuery($sql);
    }

    /**
     * Get map right by query.
     *
     * @param  string $sql
     * @return array
     * @throws isys_exception_database
     */
    private function _getMapRightByQuery($sql)
    {
        $l_res = $this->retrieve($sql);
        $l_mapRight = [];
        while ($l_data = $l_res->get_row()) {
            foreach ($l_data as $f_key => $f_item) {
                if ($f_key == "shd_map_right__id") {
                    $l_return["mapright_id"] = $f_item;
                } else {
                    $l_return[str_replace("shd_map_right__", "", $f_key)] = $f_item;
                }
            }
            $l_mapRight[] = $l_return;
        }
        return $l_mapRight;
    }

    /**
     * Method for creating a new report.
     *
     * @param array $p_params
     * @return false|int  boolean false or integer
     * @throws isys_exception_dao
     */
    public function createMapRight($p_params)
    {
        $l_update = "INSERT INTO shd_map_right SET
			shd_map_right__map_id = " . $this->convert_sql_text($p_params["map_id"]) . ",
			shd_map_right__user_id = " . $this->convert_sql_text($p_params["user_id"]) . ",
			shd_map_right__user_group_id = " . $this->convert_sql_text($p_params["user_group_id"]) . ",
			shd_map_right__right = " . $this->convert_sql_text($p_params["right"]) . ";";

        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    /**
     * @param  array $p_params
     * @return boolean|int
     * @throws isys_exception_dao
     */
    public function updateMapRight($p_params)
    {
        $textProperties = [
            "user_id",
            "user_group_id",
            "right",
        ];

        $l_update = $this->_getUpdateSql($p_params, 'shd_map_right', $textProperties, []);
        $l_update .= " WHERE shd_map_right__id = ". $this->convert_sql_text($p_params["mapright_id"]) .";";
        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    /**
     * Deletes an Mapconnection
     *
     * @param  int $mapright_id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteMapRight($mapright_id)
    {
        $l_q   = "DELETE FROM shd_map_right WHERE shd_map_right__id = " . $this->convert_sql_id($mapright_id);
        return $this->_deleteByQuery($l_q);
    }

    /**
     *
     * @return  array
     * @throws  Exception
     */
    public function getAllMapObjectLineproperties($p_mapObjectId)
    {
        $l_res = $this->retrieve("SELECT maplineproperty.* FROM shd_map_lineproperty AS maplineproperty
                          WHERE maplineproperty.shd_map_lineproperty__shd_map_mapobject_id = " . $this->convert_sql_id($p_mapObjectId));

        $l_mapRight = [];
        while ($l_data = $l_res->get_row()) {
            foreach ($l_data as $f_key => $f_item) {
                if ($f_key == "shd_map_lineproperty__id") {
                    $l_return["maplineproperty_id"] = $f_item;
                } elseif ($f_key == "shd_map_lineproperty__shd_map_mapobject_id") {
                    $l_return["mapobject_id"] = $f_item;
                } else {
                    $l_return[str_replace("shd_map_lineproperty__", "", $f_key)] = $f_item;
                }
            }
            $l_mapRight[$l_return["index"]] = $l_return;
        }
        return $l_mapRight;
    }

    /**
     * @param  $p_params
     * @return boolean|int
     * @throws isys_exception_dao
     */
    public function createMapLineproperty($p_params)
    {
        $l_update = "INSERT INTO shd_map_lineproperty SET
			shd_map_lineproperty__shd_map_mapobject_id = " . $this->convert_sql_text($p_params["mapobject_id"]) . ",
			shd_map_lineproperty__title = " . $this->convert_sql_text($p_params["title"]) . ",
			shd_map_lineproperty__backgroundcolor = " . $this->convert_hex_to_signed_integer($p_params["backgroundcolor"]) . ",
			shd_map_lineproperty__fontcolor = " . $this->convert_hex_to_signed_integer($p_params["fontcolor"]) . ",
			
			shd_map_lineproperty__bold = " . $this->convert_sql_text($p_params["bold"]) . ",
			shd_map_lineproperty__italic = " . $this->convert_sql_text($p_params["italic"]) . ",
			shd_map_lineproperty__underlined = " . $this->convert_sql_text($p_params["underlined"]) . ",
			shd_map_lineproperty__index = " . $this->convert_sql_text($p_params["index"]) . ", ";
            
        if (isset($p_params["bsm_profil_id"]) == true) {
            $l_update .= "shd_map_lineproperty__bsm_profil_id = " . $this->convert_sql_text($p_params["bsm_profil_id"]) . ", ";
        }

        if (isset($p_params["services"]) == true) {
            $l_update .= "shd_map_lineproperty__services = " . $this->convert_sql_text($p_params["services"]) . ", ";
        }

        $l_update = substr($l_update, 0, -2);
        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    public function updateMapLineproperty($p_params)
    {
        $textProperties = [
            "title",
            "bold",
            "italic",
            "underlined",
            "index",
            "bsm_profil_id",
            "services",
        ];

        $hexProperties = [
            "backgroundcolor",
            "fontcolor",
        ];

        $l_update = $this->_getUpdateSql($p_params, 'shd_map_lineproperty', $textProperties, $hexProperties);
        $l_update .= " WHERE shd_map_lineproperty__id = ". $this->convert_sql_text($p_params["maplineproperty_id"]) .";";
        if ($this->update($l_update) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }
        return false;
    }

    /**
     * @param $layerindex
     * @param $p_mapObjects
     * @return bool|string
     */
    private function formatLayerindex($layerindex, &$p_mapObjects)
    {
        $o_layerIndex = json_decode($layerindex);
        $s_return = "";
        foreach ($o_layerIndex as $f_key => $f_layerIndex) {
            foreach ($p_mapObjects as $f_mapObjectKey => $f_mapObject) {
                if ($f_mapObject["mapobject_id"] == $f_layerIndex) {
                    $p_mapObjects["$f_mapObjectKey"]["layerindex"] = $f_key;
                }
            }
            $s_return .= $f_layerIndex . ";";
        }

        $s_return = substr($s_return, 0, -1);
        if ($s_return === false) {
            $s_return = "600";
        }
        return $s_return;
    }

    /**
     * Get data method. Unused.
     *
     * @return  null
     */
    public function get_data()
    {
        return null;
    }
}
