<?php
/**
 * i-doit - Documentation and CMDB solution for IT environments
 *
 * This file is part of the i-doit framework. Modify at your own risk.
 *
 * Please visit http://www.i-doit.com/license for a full copyright and license information.
 *
 * @version     1.8.3
 * @package     i-doit
 * @author      synetics GmbH
 * @copyright   synetics GmbH
 * @url         http://www.i-doit.com
 * @license     http://www.i-doit.com/license
 */
use idoit\Module\Cmdb\Model\Ci\Table\Config;

/**
 * i-doit
 *
 * Objecttype DAO
 *
 * @package    i-doit
 * @subpackage CMDB_Low-Level_API
 * @author     Dennis Stuecken <dstuecken@synetics.de>
 * @version    1.4
 * @copyright  synetics GmbH
 * @license    http://www.i-doit.com/license
 */
class smview_cmdb_dao_object extends isys_cmdb_dao_object
{

    /**
     * The caching instance for relation objects
     *
     * @var array
     */
    protected $m_cache_entities = null;

    function __construct()
    {
        try {
            $smview_cmdb_dao_location = new smview_cmdb_dao_location(isys_application::instance()->database);
            $this->m_cache_entities = $smview_cmdb_dao_location->get_relation_list();
        } catch (isys_exception $ex) {
        }
    }

    /**
     * @param $p_params
     * @return array
     */
    public function relation_old($p_params)
    {
        $start = microtime(true);
        $p_params['down_lvl'];

        $this->m_return_entities[$p_params['objId']] = $this->get_self_Object_old($p_params['objId']);
        $this->getRelationsByid_old($p_params['objId'], true, 1);
        $this->getRelationsByid_old($p_params['objId'], false, 1);

        foreach ($this->m_cache_return as &$item) {
            if (isset($item["up_lvl"]) && ($item["up_lvl"] + 1 <= $p_params['up_lvl'])) {
                if (!isset($this->m_return_entities[$item["isys_catg_relation_list__isys_obj__id__slave"]])) {
                    $this->getRelationsByid_old($item["isys_catg_relation_list__isys_obj__id__slave"], true, $item["up_lvl"] + 1);
                }
            }

            if (isset($item["down_lvl"]) && ($item["down_lvl"] + 1 <= $p_params['down_lvl'])) {
                if (!isset($this->m_return_entities[$item["isys_catg_relation_list__isys_obj__id__master"]])) {
                    $this->getRelationsByid_old($item["isys_catg_relation_list__isys_obj__id__master"], false, $item["down_lvl"] + 1);
                }
            }
            $t_item = ["data" => $item, "children" => false];
            $this->m_return_entities[$item["related"]] = $t_item;
        }
        return [
            "objects" => $this->m_return_entities,
            "time"    => microtime(true) - $start,
        ];
    }

    /**
     * @param $p_params
     * @return array
     */
    public function relationfast($p_params)
    {
        $start = microtime(true);
        $p_params['down_lvl'];

        $this->getRelationsByid($p_params['objId'], true, 1);
        $this->getRelationsByid($p_params['objId'], false, 1);

        foreach ($this->m_cache_return as &$item) {
            if (isset($item["up_lvl"]) && ($item["up_lvl"] + 1 <= $p_params['up_lvl'])) {
                if (!isset($this->m_return_entities[$item["slave.isys_obj__id"]])) {
                    $this->getRelationsByid($item["slave.isys_obj__id"], false, $item["up_lvl"] + 1);
                }
            }

            if (isset($item["down_lvl"]) && ($item["down_lvl"] + 1 <= $p_params['down_lvl'])) {
                if (!isset($this->m_return_entities[$item["slave.isys_obj__id"]])) {
                    $this->getRelationsByid($item["slave.isys_obj__id"], true, $item["down_lvl"] + 1);
                }
            }
            $this->m_return_entities[$item["relation.isys_obj__id"]] = $item;
        }
        return [
            "objects" => $this->m_return_entities,
            "time"    => microtime(true) - $start,
        ];
    }

    /**
     * @param $p_params
     * @return array
     */
    public function get_all_down_objects($p_params)
    {
        $start = microtime(true);
        $this->getRelationsByid($p_params['objid'], true, 1);

        foreach ($this->m_cache_return as &$item) {
            if (!isset($this->m_return_entities[$item["relation.isys_obj__id"]])) {
                $this->getRelationsByid($item["slave.isys_obj__id"], true, $item["up_lvl"] + 1);
            }

            foreach ($p_params["not_pursue_relation"] as $f_relation) {
                if (defined_or_default($f_relation) == $item["relation_type.isys_relation_type__id"]) {
                    continue 2;
                }
            }
            $this->m_return_entities[$item["relation.isys_obj__id"]] = $item;
        }
        return [
            "objects" => $this->m_return_entities,
            "time"    => microtime(true) - $start,
        ];
    }

    private function get_self_Object_old($p_objid)
    {
        foreach ($this->m_cache_entities as $f_object) {
            if ($f_object["master.isys_obj__id"] == $p_objid) {
                $f_object = $this->formatObject_slave($f_object);

                unset($f_object["isys_obj__id"]);
                unset($f_object["isys_obj__title"]);
                unset($f_object["isys_obj_type__id"]);
                unset($f_object["isys_obj_type__title"]);
                unset($f_object["isys_catg_relation_list__isys_obj__id__master"]);
                unset($f_object["isys_catg_relation_list__isys_obj__id__slave"]);
                unset($f_object["isys_catg_relation_list__isys_obj__id"]);

                $f_object["category"] = ["C__CATG__GLOBAL-title" => $f_object["related_title"] , "C__CATG__GLOBAL-id" => $f_object["related"]];
                $f_object = ["data" => $f_object, "children" => false];
                return $f_object;
            }
        }
    }

    private function getRelationsByid_old($p_objid, $p_isMaster, $p_lvl)
    {
        //List of all Relations
        foreach ($this->m_cache_entities as $f_object) {
            if ($p_isMaster == true) {
                if ($f_object["master.isys_obj__id"] == $p_objid) {
                    $f_object = $this->formatObject_master($f_object);
                    $f_object["up_lvl"] = $p_lvl;
                    $this->m_cache_return[$f_object["isys_catg_relation_list__isys_obj__id__slave"]] = $f_object;
                }
            } else {
                if ($f_object["slave.isys_obj__id"] == $p_objid) {
                    $f_object = $this->formatObject_slave($f_object);
                    $f_object["down_lvl"] = $p_lvl;
                    $this->m_cache_return[$f_object["isys_catg_relation_list__isys_obj__id__master"]] = $f_object;
                }
            }
        }
    }

    private function getRelationsByid($p_objid, $p_isMaster, $p_lvl)
    {
        //List of all Relations
        foreach ($this->m_cache_entities as $f_object) {
            if ($p_isMaster == true) {
                if ($f_object["master.isys_obj__id"] == $p_objid) {
                    $f_object = $this->formatObject($f_object);
                    $f_object["up_lvl"] = $p_lvl;
                    $this->m_cache_return[$f_object["relation.isys_obj__id"]] = $f_object;
                }
            } else {
                if ($f_object["slave.isys_obj__id"] == $p_objid) {
                    $f_object = $this->formatObject($f_object);
                    $f_object["down_lvl"] = $p_lvl;
                    $this->m_cache_return[$f_object["relation.isys_obj__id"]] = $f_object;
                }
            }
        }
    }

    private function formatObject_master($f_object)
    {
        $language = isys_application::instance()->container->get('language');
        $format_Object = $this->getBaseformatObject($f_object);
        $format_Object["related"] = $f_object["slave.isys_obj__id"];
        $format_Object["related_title"] = $f_object["slave.isys_obj__title"];
        $format_Object["related_type_title"] = $language->get($f_object["slave_type.isys_obj_type__title"]);
        $format_Object["related_type"] = $f_object["slave_type.isys_obj_type__id"];
        $format_Object["related_cmdb_status_id"] = "3"; //$f_object["master.isys_obj__id"];
        $format_Object["related_cmdb_status_title"] = "in operation"; //$f_object["master.isys_obj__id"];

        $format_Object["category"] = [
            "C__CATG__GLOBAL-title" => $f_object["master.isys_obj__title"] ,
            "C__CATG__GLOBAL-id" => $f_object["master.isys_obj__id"],
            "C__CATG__GLOBAL-status" => $f_object["master.isys_obj__status"],
            "C__CATG__GLOBAL-created" => $f_object["master.isys_obj__created"],
            "C__CATG__GLOBAL-created_by" => $f_object["master.isys_obj__created_by"],
            "C__CATG__GLOBAL-changed" => $f_object["master.isys_obj__updated"],
            "C__CATG__GLOBAL-changed_by" => $f_object["master.isys_obj__updated_by"],
            "C__CATG__GLOBAL-sysid" => $f_object["master.isys_obj__sysid"],
            "C__CATG__GLOBAL-cmdb_status" => $f_object["master.isys_obj__isys_cmdb_status__id"],
            "C__CATG__GLOBAL-type" => $language->get($f_object["master_type.isys_obj_type__title"]),
            "C__CATG__GLOBAL-description" => $f_object["master.isys_obj__description"],
        ];
        return $format_Object;
    }

    private function formatObject_slave($f_object)
    {
        $language = isys_application::instance()->container->get('language');
        $format_Object = $this->getBaseformatObject($f_object);
        $format_Object["related"] = $f_object["master.isys_obj__id"];
        $format_Object["related_title"] = $f_object["master.isys_obj__title"];
        $format_Object["related_type_title"] = $language->get($f_object["master_type.isys_obj_type__title"]);
        $format_Object["related_type"] = $f_object["master_type.isys_obj_type__id"];
        $format_Object["related_cmdb_status_id"] = "3"; //$f_object["master.isys_obj__id"];
        $format_Object["related_cmdb_status_title"] = "in operation"; //$f_object["master.isys_obj__id"];
        return $format_Object;
    }

    private function getBaseformatObject($f_object)
    {
        $language = isys_application::instance()->container->get('language');
        $format_Object = [];
        $format_Object["isys_obj__id"] = $f_object["relation.isys_obj__id"];
        $format_Object["isys_obj__title"] = $f_object["relation.isys_obj__title"];
        $format_Object["isys_obj_type__id"] = $f_object["master_type.isys_obj_type__id"];
        $format_Object["isys_obj_type__title"] = $language->get($f_object["master_type.isys_obj_type__title"]);
        $format_Object["isys_catg_relation_list__isys_obj__id__master"] = $f_object["master.isys_obj__id"];
        $format_Object["isys_catg_relation_list__isys_obj__id__slave"] = $f_object["slave.isys_obj__id"];
        $format_Object["isys_catg_relation_list__isys_obj__id"] = $f_object["relation.isys_obj__id"];
        return $format_Object;
    }

    private function formatObject($f_object)
    {
        $language = isys_application::instance()->container->get('language');
        $f_object["relation_type.isys_relation_type__title"] = $language->get($f_object["relation_type.isys_relation_type__title"]);
        $f_object["master_type.isys_obj_type__title"] = $language->get($f_object["master_type.isys_obj_type__title"]);
        $f_object["slave_type.isys_obj_type__title"] = $language->get($f_object["slave_type.isys_obj_type__title"]);
        return $f_object;
    }
}
