<?php
/**
 * i-doit - Documentation and CMDB solution for IT environments
 *
 * This file is part of the i-doit framework. Modify at your own risk.
 *
 * Please visit http://www.i-doit.com/license for a full copyright and license information.
 *
 * @version     1.8.3
 * @package     i-doit
 * @author      synetics GmbH
 * @copyright   synetics GmbH
 * @url         http://www.i-doit.com
 * @license     http://www.i-doit.com/license
 */

/**
 * i-doit
 *
 * Objecttype DAO
 *
 * @package    i-doit
 * @subpackage CMDB_Low-Level_API
 * @author     Dennis Stuecken <dstuecken@synetics.de>
 * @version    1.4
 * @copyright  synetics GmbH
 * @license    http://www.i-doit.com/license
 */
class smview_cmdb_dao_object_type extends isys_cmdb_dao_object_type
{

    /**
     *
     * @return array
     * @throws isys_exception_database
     */
    public function get_insertionobject()
    {
        $l_sql = 'SELECT * FROM isys_obj_type WHERE isys_obj_type.isys_obj_type__show_in_rack = 1';
        $l_res = $this->retrieve($l_sql);
        $l_object_type = array();
        while ($l_data = $l_res->get_row()) {
            $l_object_type[$l_data["isys_obj_type__id"]] = $l_data;
        }
        return $l_object_type;
    }

    /**
     *
     * @return array
     * @throws isys_exception_database
     */
    public function get_containerobjects()
    {
        $l_sql = 'SELECT * FROM isys_obj_type WHERE isys_obj_type.isys_obj_type__container = 1';
        $l_res = $this->retrieve($l_sql);
        $l_object_type = array();
        while ($l_data = $l_res->get_row()) {
            $l_object_type[$l_data["isys_obj_type__id"]] = $l_data;
        }
        return $l_object_type;
    }

    /**
     *
     * @return array
     * @throws isys_exception_database
     */
    public function get_connectableobjects()
    {
        $l_sql = "SELECT isys_obj_type.*  FROM `isys_obj_type_2_isysgui_catg` 
        join isys_obj_type ON isys_obj_type.isys_obj_type__id = isys_obj_type_2_isysgui_catg.isys_obj_type_2_isysgui_catg__isys_obj_type__id
        join isysgui_catg ON isysgui_catg.isysgui_catg__id = isys_obj_type_2_isysgui_catg.isys_obj_type_2_isysgui_catg__isysgui_catg__id
        where `isysgui_catg`.`isysgui_catg__const` = 'C__CATG__SMVIEW_CABLE_PIN'";

        $l_res = $this->retrieve($l_sql);
        $l_object_type = array();
        while ($l_data = $l_res->get_row()) {
            $l_object_type[$l_data["isys_obj_type__id"]] = $l_data;
        }
        return $l_object_type;
    }

    /**
     * Count object-types.
     *
     * @param  int|null $p_objtype
     * @param  int $p_status
     * @return array
     * @throws isys_exception_database
     */
    public function get_object_id_by_type($p_objtype = null, $p_status = C__RECORD_STATUS__NORMAL)
    {
        $l_sql = "SELECT isys_obj__id AS objects
			FROM isys_obj_type
			INNER JOIN isys_obj ON isys_obj__isys_obj_type__id = isys_obj_type__id
			WHERE isys_obj__status = " . $this->convert_sql_int($p_status) . " ";

        if ($p_objtype !== null) {
            $l_sql .= "AND isys_obj_type__id = " . $this->convert_sql_id($p_objtype) . " ";
        }

        $l_data = [];
        $l_res = $this->retrieve($l_sql);
        while ($l_row = $l_res->get_row()) {
            $l_data[] = $l_row;
        }
        return $l_data;
    }

    public function get_object_types_by_params($p_params)
    {
        try {
            assert('is_array($p_params)');
            if (array_key_exists('filter', $p_params)) {
                assert('is_array($p_params["filter"])');
            } else {
                $p_params['filter'] = [];
            }

            // Force limit to record status 'normal':
            $p_params['filter']['status'] = C__RECORD_STATUS__NORMAL;

            // Raw mode:
            $l_raw = false;
            if (array_key_exists('raw', $p_params)) {
                $l_raw = filter_var($p_params['raw'], FILTER_VALIDATE_BOOLEAN);
            }

            // Order by:
            $l_order_by = array_key_exists('order_by', $p_params) ? $p_params['order_by'] : null;

            // Sort:
            $l_sort = array_key_exists('sort', $p_params) ? $p_params['sort'] : null;

            // Limitation:
            $l_limit = array_key_exists('limit', $p_params) ? $p_params['limit'] : null;

            // Show object count
            $l_count = isset($p_params['countobjects']) && $p_params['countobjects'] ? true : false;

            // Data retrieval:
            $_dao = new isys_cmdb_dao(isys_application::instance()->database, null);
            $l_data = $_dao->get_object_types_by_properties(
                $p_params['filter'],
                $l_order_by,
                $l_sort,
                $l_limit,
                $l_count
            );

            // Data formatting:
            $l_return = [];
            $isys_api_model_shd_userright = new isys_api_model_shd_userright();
            while ($l_row = $l_data->get_row()) {
                if ($l_row['isys_obj_type__id'] != C__OBJTYPE__LOCATION_GENERIC && $l_row['isys_obj_type__id'] != C__OBJTYPE__MIGRATION_OBJECT) {
                    $hasObjTypeRight = $isys_api_model_shd_userright->hasUserObjTypeRight($l_row['isys_obj_type__id']) == true;
                    $l_new = $l_row;
                    if (! $l_raw) {
                        if ($l_count) {
                            $l_objectIds = smview_cmdb_dao_object_type::instance(isys_application::instance()->database)->get_object_id_by_type($l_row['isys_obj_type__id']);
                            $l_objectCount = 0;
                            foreach ($l_objectIds as $f_objectId) {
                                if ($hasObjTypeRight || $isys_api_model_shd_userright->hasUserObjRight($f_objectId["objects"])) {
                                    $l_objectCount++;
                                }
                            }
                            $l_new['objectcount'] = $l_objectCount;
                        }
                    }

                    if ($hasObjTypeRight || $l_objectCount != 0) {
                        $l_return[] = $l_new;
                    }
                }
            }

            // Order by translated titles:
            if ($l_order_by === 'title' && $l_raw === false) {
                usort($l_return, [$this, 'sort_by_title']);
            }
            return $l_return;
        } catch (Exception $ex) {
            dump($ex->getMessage());
            return "";
        }
    }
}
