<?php
/**
 * i-doit - Documentation and CMDB solution for IT environments
 *
 * This file is part of the i-doit framework. Modify at your own risk.
 *
 * Please visit http://www.i-doit.com/license for a full copyright and license information.
 *
 * @version     1.8.3
 * @package     i-doit
 * @author      synetics GmbH
 * @copyright   synetics GmbH
 * @url         http://www.i-doit.com
 * @license     http://www.i-doit.com/license
 */


class smview_cmdb_dao_query extends isys_module_dao
{
    protected static $m_instance = null;

    protected $m_db;

    protected $m_properties;

    protected $m_object_type;

    protected $m_table;

    /**
     * Logger.
     *
     * @var isys_log
     */
    protected $logger = null;
    
    /**
     * Database component.
     *
     * @var isys_component_database
     */
    public function __construct(isys_component_database $p_db)
    {
        global $g_absdir;
        $this->logger = isys_log::get_instance('shd-smview')
            ->set_log_file($g_absdir . '/log/shd-smview_' . date('Y-m-d') . '.log');
        
        $properties = smview_cmdb_dao_properties::instance(isys_application::instance()->database)->getProperties(false);
        //change the property-const to id
        $t_properties = [];
        foreach ($properties as $property) {
            $t_properties[$property['const'] . '-' . $property['key']] = $property;
        }

        $this->m_properties = $t_properties;
        parent::__construct($p_db);
    }

    /**
     * @param array $p_params
     * @return array
     */
    public function read($p_params)
    {
        if ($p_params["debug"] == true) {
            $l_start_time        = microtime(true);
        }

        //check Userright
        $isys_api_model_shd_userright = new isys_api_model_shd_userright();
        $hasRight = $isys_api_model_shd_userright->hasUserObjTypeRight($p_params["obj_type_id"]);

        $l_filter["shd_filter__display_rows"] = (isset($p_params["display_rows"])) ? $p_params["display_rows"] : 0;
        $l_filter["shd_filter__page"] = (isset($p_params["page"])) ? $p_params["page"] : 0;
        $l_filter["status"] = (isset($p_params["status"])) ? $p_params["status"] : C__RECORD_STATUS__NORMAL;

        $l_filter["shd_filter__display_relations"] = "0";
        $l_filter["shd_filter__empty_values"] = "1";
        $l_filter["shd_filter__obj_type_id"] = $p_params["obj_type_id"];
        $this->m_object_type = $p_params["obj_type_id"];
        $l_filter["shd_filter__columns"] = $p_params["columns"];
        $vanilla_columns = $p_params["columns"];
        $extra_columns = [];

        $l_filter["shd_filter__condition"] = $p_params["condition"];
        $shd_condition = [];
        $shd_column = [];

        $sort_column = [];

        //Query for not mounted objects
        if (isset($p_params["notMounted"]) && $p_params["notMounted"] == true) {
            $l_filter["shd_filter__columns"][] =  ["id"=>"708", "width" =>"50", "sort" => ""];
            $l_filter["shd_filter__condition"][] =
                ["category" => "C__CATG__LOCATION", "property" => "C__CATG__LOCATION-pos", "comparison" =>"IS NULL", "value" => ""];
        }

        //sort of columns-Key
        $t_filter_column = [];
        foreach ($l_filter['shd_filter__columns'] as $filterKey => $shd_filter__column) {
            if ($shd_filter__column["sort"] != "") {
                $sort_column['sorting_default'] = $shd_filter__column['id'];
                $sort_column['sorting_direction'] = $shd_filter__column['sort'];
            }

            //entferne SHD_Status werden später wieder hinzugefügt sind nur Platzhalter
            if (strpos($shd_filter__column['id'], "C__SHD") !== false) {
                //check shdcolumn is sort
                if ($shd_filter__column["sort"] != "") {
                    $shd_column = $shd_filter__column;
                }

                unset($l_filter['shd_filter__columns'][$filterKey]);
            } else {
                //check special columns
                if ($this->isSpecialColumn($shd_filter__column['id']) == true) {
                    $extra_columns[] = $l_filter['shd_filter__columns'][$filterKey];
                    unset($l_filter['shd_filter__columns'][$filterKey]);
                } else {
                    $t_filter_column[$shd_filter__column['id']] = $shd_filter__column;
                }
            }
        }
        ksort($t_filter_column);

        $l_filter['shd_filter__columns'] = $t_filter_column;

        //sort of columns
        foreach ($l_filter['shd_filter__columns'] as $filterKey => $shd_filter__column) {
            $l_filter['shd_filter__columns'][$filterKey]['id'] = $this->getProperty($shd_filter__column['id'])['id'];
            if ($shd_filter__column['sort'] != "") {
                $l_filter['sorting_default'] = $this->getProperty($shd_filter__column['id'])['id'];
                $l_filter['sorting_direction'] = $shd_filter__column['sort'];

                $sort_column['sorting_default'] = $shd_filter__column['id'];
                $sort_column['sorting_direction'] = $shd_filter__column['sort'];
            }
        }

        //delete SHD-Conditions
        foreach ($l_filter['shd_filter__condition'] as $filterKey => $shd_filter__condition) {
            if (strpos($shd_filter__condition["category"], "SHD") != false) {
                $shd_condition[] = $shd_filter__condition;
                unset($l_filter['shd_filter__condition'][$filterKey]);
            } elseif ($shd_filter__condition['property'] == "C__CATG__IP-fqdn") {
                $shd_condition[] = $shd_filter__condition;
                unset($l_filter['shd_filter__condition'][$filterKey]);
            }
        }

        //add property-id to vanila_column
        foreach ($vanilla_columns as $filterKey => $shd_filter__column) {
            $vanilla_columns[$filterKey]['id_property'] = $this->getProperty($shd_filter__column['id'])['id'];
            $vanilla_columns[$filterKey]['i18n_title'] = $this->getProperty($shd_filter__column['id'])["title"];
        }
        //Abfrage nach den Ids und anhand der das Paging anwenden
        $l_filter_ids = $l_filter;

        $smview_cmdb_dao_filter_sql = new smview_cmdb_dao_filter_sql(isys_application::instance()->database);
        $query_filter_ids = $smview_cmdb_dao_filter_sql->create_property_query_for_filter($l_filter_ids, false, true);

        $filter_ids = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->query_sql($query_filter_ids);

        //when SHD-condition isset then we get the Status
        if (count($shd_condition) > 0 || count($shd_column) > 0) {
            $a_status = [];
            $status_dao = smview_cmdb_dao_status::instance($this->get_database_component());
            foreach ($filter_ids as $f_id) {
                $livestatus = $status_dao->get_livestatus($f_id["__id__"]);
                $status = isset($livestatus) ? $livestatus : $status_dao->getGreyStatus();
                
                $status["__id__"] = $f_id["__id__"];
                $sort_status[$f_id["__id__"]] = $status;
                if ($this->ConditionSHD($shd_condition, $status) == true) {
                    $a_status[] = $f_id;
                }
            }

            if (count($shd_column) > 0) {
                $sort_status = $this->ColumnsSHD($shd_column, $sort_status);
                $filter_ids = $sort_status;
            }

            if (count($shd_condition) > 0) {
                $filter_ids = $a_status;
            }
        }

        if ($p_params["count_objects"]) {
            return count($filter_ids);
        }

        if ($hasRight === true) {
            //has right now calc the elements
            for ($i = (int)$l_filter['shd_filter__page'] * (int)$l_filter['shd_filter__display_rows']; $i < (int)$l_filter['shd_filter__page'] * (int)$l_filter['shd_filter__display_rows'] + $l_filter["shd_filter__display_rows"]; $i++) {
                $l_filter["shd_filter__condition"][] = ["category" => "C__CATG__GLOBAL",
                    "property" => "C__CATG__GLOBAL-id", "comparison" =>"=",
                    "value" => $filter_ids[$i]["__id__"]];
            }
        } else {
            //no rights check for ids and Page them
            foreach ($filter_ids as $f_filter_id) {
                if ($isys_api_model_shd_userright->hasUserObjRight($f_filter_id["__id__"]) === true) {
                    $hasRight = true;
                    $l_filter_ids_with_rights[] =  $f_filter_id;
                }
            }
            $filter_ids = $l_filter_ids_with_rights;
        }

        for ($i = (int)$l_filter['shd_filter__page'] * (int)$l_filter['shd_filter__display_rows']; $i < (int)$l_filter['shd_filter__page'] * (int)$l_filter['shd_filter__display_rows'] + $l_filter["shd_filter__display_rows"]; $i++) {
            $l_filter["shd_filter__condition"][] = ["category" => "C__CATG__GLOBAL", "property" => "C__CATG__GLOBAL-id", "comparison" =>"=", "value" => $filter_ids[$i]["__id__"]];
        }

        $l_filter['shd_filter__display_rows'] = null;

        $smview_cmdb_dao_filter_sql = new smview_cmdb_dao_filter_sql(isys_application::instance()->database);
        $query_filter = $smview_cmdb_dao_filter_sql->create_property_query_for_filter($l_filter);

        $filter = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->query($query_filter);

        //Check if User has right for Object
        if ($hasRight === false) {
            $filter["num"] = "0";
            $filter["content"] = [];
        }

        $tempTable = array();
        $sort_count = 0;
        //change Table Content

        $deleteHeaders = [];
        $multi_array = [];

        foreach ($filter["content"] as $filter_content) {
            foreach ($filter_content as $filter_key => $filter_cell) {
                //jump over ID
                if ($filter_key != "__id__") {
                    if ($isys_api_model_shd_userright->hasUserCategoryRight($this->getProperty($filter_key)["const"]) == false) {
                        if ($filter_cell != "") {
                            $filter_cell = "*****";
                            $filter_content[$filter_key] = "*****";
                        }
                    }
                }

                //catch primary_key
                if ($filter_key == "__id__") {
                    $primary_key = $filter_cell;
                }

                //Wenn die Property eine Multi ist dann soll es zusammen gepackt werden
                if ($this->getProperty($filter_key)["multi"] == "1") {
                    if ($filter_cell !== "") {
                        $multiValue = $this->getMultiValue($this->getProperty($filter_key)["const"], $this->getProperty($filter_key)["key"], $primary_key);

                        $obj_id = $filter_content[$filter_key. "___id"];
                        if ($multiValue != null) {
                            $multi_array[$filter_key][$primary_key][$filter_cell] = $multiValue;
                        } else {
                            if ($obj_id == null) {
                                $multi_array[$filter_key][$primary_key][] = array("__id__" => 0, "value" => $filter_cell);
                            } else {
                                $multi_array[$filter_key][$primary_key][$obj_id] = array("__id__" => $obj_id, "value" => $filter_cell);
                            }
                        }
                    }
                } elseif (isset($filter_content[$filter_key. "___id"])) {
                    $id = $filter_content[$filter_key. "___id"];
                    $value = $filter_content[$filter_key];
                    $multi_array[$filter_key][$primary_key][$id] = array("__id__" => $id, "value" => $value);
                }

                if (strpos($filter_key, '___id') !== false) {
                    $deleteHeaders[$filter_key] = $filter_key;
                    unset($filter_content[$filter_key]);
                }

                //hinzufügen der SHD-Columns
                foreach ($p_params["columns"] as $column) {
                    if (strpos($column["id"], "SHD") != false) {
                        $translation = $this->getProperty($column["id"])["id"];

                        $filter_content[$translation] = "";
                    }
                }
            }

            $filter_content["__index__"] = $sort_count;
            $sort_count++;

            //delete Multivalue id table
            foreach ($deleteHeaders as $f_deleteHeader) {
                unset($filter_content[$f_deleteHeader]);
            }

            //Wenn Parent noch nicht gesetzt ist dann setzen.
            if (isset($tempTable[$primary_key]) == false) {
                $tempTable[$primary_key] = $filter_content;
            }
        }

        foreach ($multi_array as $f_propert_key => $multi_element) {
            foreach ($multi_element as $f_primary_key => $property_element) {
                $tempTable[$f_primary_key][$f_propert_key] = $property_element;
            }
        }
        $this->getTable();

        //get Spezial Columns
        foreach ($tempTable as $f_row_key => $f_row) {
            foreach ($extra_columns as $extra_column) {
                $key = $this->getProperty($extra_column["id"]);
                $multiValue = $this->getMultiValue($key["const"], $key["key"], $f_row_key);
                if ($multiValue != null) {
                    $tempTable[$f_row_key][$key["id"]] = $multiValue;
                } else {
                    $tempTable[$f_row_key][$key["id"]][] = array("__id__" => 0, "value" => "");
                }
            }
        }

        foreach ($filter["headers"] as $headerKey => $header) {
            foreach ($deleteHeaders as $f_deleteHeader) {
                if ($header["const"] === $f_deleteHeader) {
                    unset($filter["headers"][$headerKey]);
                }
            }
        }

        $array_header = [];
        $count =0 ;
        foreach ($vanilla_columns as $f_column) {
            $array_header[$count] = ["const" => $f_column["id_property"], "title" => _L($f_column["i18n_title"])];
            $count++;
        }

        $filter["headers"] = $array_header;
        $filter["count"] = count($filter_ids);

        $i = 0;
        foreach ($sort_status as $status) {
            if (isset($tempTable[$status["__id__"]])) {
                $tempTable[$status["__id__"]]["__index__"] = $i;
                $i++;
            }
        }

        $tempTable = $this->sortSHDColumns($tempTable, $sort_column);

        $filter["content"] = $tempTable;
        $filter["page"] = $l_filter["shd_filter__page"];
        $filter["display_rows"] =  $p_params["display_rows"];

        if ($p_params["debug"] == true) {
            $filter["time"] = (microtime(true) - $l_start_time);
        }
        return $filter;
    }

    public function readfast($p_params)
    {
        $l_filter["shd_filter__display_rows"] = (isset($p_params["display_rows"])) ? $p_params["display_rows"] : 0;
        $l_filter["shd_filter__page"] = (isset($p_params["page"])) ? $p_params["page"] : 0;
        $l_filter["status"] = (isset($p_params["status"])) ? $p_params["status"] : C__RECORD_STATUS__NORMAL;
        $l_filter["shd_filter__display_relations"] = "0";
        $l_filter["shd_filter__empty_values"] = "1";
        $l_filter["shd_filter__obj_type_id"] = $p_params["obj_type_id"];
        $this->m_object_type = $p_params["obj_type_id"];
        $l_filter["shd_filter__columns"] = $p_params["columns"];
        $vanilla_columns = $p_params["columns"];
        $extra_columns = [];

        $l_filter["shd_filter__condition"] = $p_params["condition"];
        $shd_condition = [];
        $sort_column = [];

        //sort of columns-Key
        $t_filter_column = [];
        foreach ($l_filter['shd_filter__columns'] as $filterKey => $shd_filter__column) {
            if ($shd_filter__column["sort"] != "") {
                $sort_column['sorting_default'] = $shd_filter__column['id'];
                $sort_column['sorting_direction'] = $shd_filter__column['sort'];
            }

            //check special columns
            if ($this->isSpecialColumn($shd_filter__column['id']) == true) {
                $extra_columns[] = $l_filter['shd_filter__columns'][$filterKey];
                unset($l_filter['shd_filter__columns'][$filterKey]);
            } else {
                $t_filter_column[$shd_filter__column['id']] = $shd_filter__column;
            }
        }
        ksort($t_filter_column);

        $l_filter['shd_filter__columns'] = $t_filter_column;

        //sort of columns
        foreach ($l_filter['shd_filter__columns'] as $filterKey => $shd_filter__column) {
            $l_filter['shd_filter__columns'][$filterKey]['id'] = $this->getProperty($shd_filter__column['id'])['id'];
            if ($shd_filter__column['sort'] != "") {
                $l_filter['sorting_default'] = $this->getProperty($shd_filter__column['id'])['id'];
                $l_filter['sorting_direction'] = $shd_filter__column['sort'];

                $sort_column['sorting_default'] = $shd_filter__column['id'];
                $sort_column['sorting_direction'] = $shd_filter__column['sort'];
            }
        }

        //delete SHD-Conditions
        foreach ($l_filter['shd_filter__condition'] as $filterKey => $shd_filter__condition) {
            if (strpos($shd_filter__condition["category"], "SHD") != false) {
                $shd_condition[] = $shd_filter__condition;
                unset($l_filter['shd_filter__condition'][$filterKey]);
            } elseif ($shd_filter__condition['property'] == "C__CATG__IP-fqdn") {
                $shd_condition[] = $shd_filter__condition;
                unset($l_filter['shd_filter__condition'][$filterKey]);
            }
        }

        //add property-id to vanila_column
        foreach ($vanilla_columns as $filterKey => $shd_filter__column) {
            $vanilla_columns[$filterKey]['id_property'] = $this->getProperty($shd_filter__column['id'])['id'];
            $vanilla_columns[$filterKey]['i18n_title'] = $this->getProperty($shd_filter__column['id'])["title"];
        }
        //Abfrage nach den Ids und anhand der das Paging anwenden
        $l_filter_ids = $l_filter;

        $smview_cmdb_dao_filter_sql = new smview_cmdb_dao_filter_sql(isys_application::instance()->database);
        $query_filter_ids = $smview_cmdb_dao_filter_sql->create_property_query_for_filter($l_filter_ids, false, true);

        $filter_ids = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->query_sql($query_filter_ids);
        return $filter_ids;
    }

    public function relation($p_params)
    {
        if ($p_params["debug"] == true) {
            $l_start_time        = microtime(true);
        }

        //check Userright
        $isys_api_model_shd_userright = new isys_api_model_shd_userright();
        $hasRight = $isys_api_model_shd_userright->hasUserObjTypeRight($p_params["obj_type_id"]);

        $l_filter["shd_filter__display_rows"] = (isset($p_params["display_rows"])) ? $p_params["display_rows"] : 0;
        $l_filter["shd_filter__page"] = (isset($p_params["page"])) ? $p_params["page"] : 0;
        $l_filter["status"] = (isset($p_params["status"])) ? $p_params["status"] : C__RECORD_STATUS__NORMAL;

        $l_filter["shd_filter__display_relations"] = "0";
        $l_filter["shd_filter__empty_values"] = "1";
        $l_filter["shd_filter__columns"] = $p_params["columns"];
        $vanilla_columns = $p_params["columns"];
        $extra_columns = [];

        $l_filter["shd_filter__condition"] = $p_params["condition"];
        $shd_condition = [];
        $shd_column = [];

        $sort_column = [];

        //Query for not mounted objects
        if (isset($p_params["notMounted"]) && $p_params["notMounted"] == true) {
            $l_filter["shd_filter__columns"][] =  ["id"=>"708", "width" =>"50", "sort" => ""];
            $l_filter["shd_filter__condition"][] =
                ["category" => "C__CATG__LOCATION", "property" => "C__CATG__LOCATION-pos", "comparison" =>"IS NULL", "value" => ""];
        }

        if (isset($p_params["obj_id"]) == true) {
            $l_filter["shd_filter__columns"][] =  ["id"=>"C__CATG__LOCATION-_location_path", "width" =>"50", "sort" => ""];
            $l_filter["shd_filter__condition"][] =
                ["category" => "C__CATG__LOCATION", "property" => "C__CATG__LOCATION-location_path", "comparison" =>"IS NOT NULL"];
        }

        //sort of columns-Key
        $t_filter_column = [];
        foreach ($l_filter['shd_filter__columns'] as $filterKey => $shd_filter__column) {
            if ($shd_filter__column["sort"] != "") {
                $sort_column['sorting_default'] = $shd_filter__column['id'];
                $sort_column['sorting_direction'] = $shd_filter__column['sort'];
            }

            //entferne SHD_Status werden später wieder hinzugefügt sind nur Platzhalter
            if (strpos($shd_filter__column['id'], "C__SHD") !== false) {
                //check shdcolumn is sort
                if ($shd_filter__column["sort"] != "") {
                    $shd_column = $shd_filter__column;
                }

                unset($l_filter['shd_filter__columns'][$filterKey]);
            } else {
                //check special columns
                if ($this->isSpecialColumn($shd_filter__column['id']) == true) {
                    $extra_columns[] = $l_filter['shd_filter__columns'][$filterKey];
                    unset($l_filter['shd_filter__columns'][$filterKey]);
                } else {
                    $t_filter_column[$shd_filter__column['id']] = $shd_filter__column;
                }
            }
        }
        ksort($t_filter_column);

        $l_filter['shd_filter__columns'] = $t_filter_column;

        //sort of columns
        foreach ($l_filter['shd_filter__columns'] as $filterKey => $shd_filter__column) {
            $l_filter['shd_filter__columns'][$filterKey]['id'] = $this->getProperty($shd_filter__column['id'])['id'];
            if ($shd_filter__column['sort'] != "") {
                $l_filter['sorting_default'] = $this->getProperty($shd_filter__column['id'])['id'];
                $l_filter['sorting_direction'] = $shd_filter__column['sort'];

                $sort_column['sorting_default'] = $shd_filter__column['id'];
                $sort_column['sorting_direction'] = $shd_filter__column['sort'];
            }
        }

        //delete SHD-Conditions
        foreach ($l_filter['shd_filter__condition'] as $filterKey => $shd_filter__condition) {
            if (strpos($shd_filter__condition["category"], "SHD") != false) {
                $shd_condition[] = $shd_filter__condition;
                unset($l_filter['shd_filter__condition'][$filterKey]);
            } elseif ($shd_filter__condition['property'] == "C__CATG__IP-fqdn") {
                $shd_condition[] = $shd_filter__condition;
                unset($l_filter['shd_filter__condition'][$filterKey]);
            }
        }

        //add property-id to vanila_column
        foreach ($vanilla_columns as $filterKey => $shd_filter__column) {
            $vanilla_columns[$filterKey]['id_property'] = $this->getProperty($shd_filter__column['id'])['id'];
            $vanilla_columns[$filterKey]['i18n_title'] = $this->getProperty($shd_filter__column['id'])["title"];
        }
        //Abfrage nach den Ids und anhand der das Paging anwenden
        $l_filter_ids = $l_filter;

        unset($l_filter_ids["shd_filter__display_rows"]);

        $smview_cmdb_dao_filter_sql = new smview_cmdb_dao_filter_sql(isys_application::instance()->database);
        $query_filter_ids = $smview_cmdb_dao_filter_sql->create_property_query_for_filter($l_filter_ids, false, false);

        $filter_ids = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->query_sql($query_filter_ids);

        //when SHD-condition isset then we get the Status
        if (count($shd_condition) > 0 || count($shd_column) > 0) {
            $a_status = [];
            $status_dao = smview_cmdb_dao_status::instance($this->get_database_component());
            foreach ($filter_ids as $f_id) {
                $livestatus = $status_dao->get_livestatus($f_id["__id__"]);
                $status = isset($livestatus) ? $livestatus : $status_dao->getGreyStatus();

                $status["__id__"] = $f_id["__id__"];
                $sort_status[$f_id["__id__"]] = $status;
                if ($this->ConditionSHD($shd_condition, $status) == true) {
                    $a_status[] = $f_id;
                }
            }

            if (count($shd_column) > 0) {
                $sort_status = $this->ColumnsSHD($shd_column, $sort_status);

                $filter_ids = $sort_status;
            }

            if (count($shd_condition) > 0) {
                $filter_ids = $a_status;
            }
        }

        if ($p_params["count_objects"]) {
            return count($filter_ids);
        }

        $tempTable = array();
        $sort_count = 0;
        //change Table Content

        $deleteHeaders = [];
        $multi_array = [];

        foreach ($filter_ids as $filter_content) {
            foreach ($filter_content as $filter_key => $filter_cell) {
                //jump over main obj_id
                if ($filter_cell == $p_params["obj_id"]) {
                    continue;
                }

                //jump over __id__
                if ($filter_key != "__id__") {
                    if ($isys_api_model_shd_userright->hasUserCategoryRight($this->getProperty($filter_key)["const"]) == false) {
                        if ($filter_cell != "") {
                            $filter_cell = "*****";
                            $filter_content[$filter_key] = "*****";
                        }
                    }
                }

                //catch primary_key
                if ($filter_key == "__id__") {
                    $primary_key = $filter_cell;
                }

                //escape Html
                if (strpos($filter_cell, "<")!==false) {
                    $filter_cell = strip_tags($filter_cell);
                }

                //Wenn die Property eine Multi ist dann soll es zusammen gepackt werden
                if ($this->getProperty($filter_key)["multi"] == "1") {
                    if ($filter_cell !== "") {
                        $multiValue = $this->getMultiValue($this->getProperty($filter_key)["const"], $this->getProperty($filter_key)["key"], $primary_key);

                        $obj_id = $filter_content[$filter_key. "___id"];
                        if ($multiValue != null) {
                            $multi_array[$filter_key][$primary_key][$filter_cell] = $multiValue;
                        } else {
                            if ($obj_id == null) {
                                $multi_array[$filter_key][$primary_key][] = array("__id__" => 0, "value" => $filter_cell);
                            } else {
                                $multi_array[$filter_key][$primary_key][$obj_id] = array("__id__" => $obj_id, "value" => $filter_cell);
                            }
                        }
                    }
                } elseif (isset($filter_content[$filter_key. "___id"])) {
                    $id = $filter_content[$filter_key. "___id"];
                    $value = $filter_content[$filter_key];
                    $multi_array[$filter_key][$primary_key][$id] = array("__id__" => $id, "value" => $value);
                }

                if (strpos($filter_key, '___id') !== false) {
                    $deleteHeaders[$filter_key] = $filter_key;
                    unset($filter_content[$filter_key]);
                }

                //hinzufügen der SHD-Columns
                foreach ($p_params["columns"] as $column) {
                    if (strpos($column["id"], "SHD") != false) {
                        $translation = $this->getProperty($column["id"])["id"];
                        $filter_content[$translation] = "";
                    }
                }

                //Translate the Table Content
                $filter_content[$filter_key] = _L($filter_cell);
            }

            $filter_content["__index__"] = $sort_count;
            $sort_count++;

            //delete Multivalue id table
            foreach ($deleteHeaders as $f_deleteHeader) {
                unset($filter_content[$f_deleteHeader]);
            }

            //Wenn Parent noch nicht gesetzt ist dann setzen.
            if (isset($tempTable[$primary_key]) == false) {
                $tempTable[$primary_key] = $filter_content;
            }
        }

        foreach ($multi_array as $f_propert_key => $multi_element) {
            foreach ($multi_element as $f_primary_key => $property_element) {
                $tempTable[$f_primary_key][$f_propert_key] = $property_element;
            }
        }

        $this->getTableRelation();
        foreach ($tempTable as $f_row_key => $f_row) {
            foreach ($extra_columns as $extra_column) {
                $key = $this->getProperty($extra_column["id"]);
                $multiValue = $this->getMultiValue($key["const"], $key["key"], $f_row_key);

                //condition for relation
                if ($extra_column["id"] == "C__CATG__LOCATION-_location_path") {
                    $addValue = false;
                    foreach ($multiValue as $f_value) {
                        if ($f_value["__id__"] == $p_params["obj_id"]) {
                            $addValue = true;
                        }
                    }
                } else {
                    $addValue = true;
                }

                if ($addValue === false) {
                    //delete Element
                    unset($tempTable[$f_row_key]);
                } else {
                    if ($multiValue != null) {
                        $tempTable[$f_row_key][$key["id"]] = $multiValue;
                    } else {
                        $tempTable[$f_row_key][$key["id"]][] = array("__id__" => 0, "value" => "");
                    }
                }
            }
        }

        $t_Table_Content = [];
        $i = 0;
        $tempCount = count($tempTable);

        if ($hasRight === true) {
            //has right now calc the elements
            foreach ($tempTable as $f_item) {
                if ($i > (int)$l_filter['shd_filter__page'] * (int)$l_filter['shd_filter__display_rows']  &&
                    $i <= (int)$l_filter['shd_filter__page'] * (int)$l_filter['shd_filter__display_rows'] + $l_filter["shd_filter__display_rows"]) {
                    $t_Table_Content[$f_item["__id__"]] = $f_item;
                }
                $i++;
            }
        } else {
            //no rights check for ids and Page them
            foreach ($filter_ids as $f_filter_id) {
                if ($isys_api_model_shd_userright->hasUserObjRight($f_filter_id["__id__"]) === true) {
                    $hasRight = true;
                    $l_filter_ids_with_rights[] =  $f_filter_id;
                }
            }
            $filter_ids = $l_filter_ids_with_rights;
        }

        $tempTable = $t_Table_Content;

        $l_filter['shd_filter__display_rows'] = null;

        $smview_cmdb_dao_filter_sql = new smview_cmdb_dao_filter_sql(isys_application::instance()->database);
        $query_filter = $smview_cmdb_dao_filter_sql->create_property_query_for_filter($l_filter);
        $filter = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->query($query_filter);

        //Check if User has right for Object
        if ($hasRight === false) {
            $filter["num"] = "0";
            $filter["content"] = [];
        }

        foreach ($filter["headers"] as $headerKey => $header) {
            foreach ($deleteHeaders as $f_deleteHeader) {
                if ($header["const"] === $f_deleteHeader) {
                    unset($filter["headers"][$headerKey]);
                }
            }
        }

        $array_header = [];
        $count =0 ;
        foreach ($vanilla_columns as $headerKey => $f_column) {
            $array_header[$count] = ["const" => $f_column["id_property"], "title" => _L($f_column["i18n_title"])];
            $count++;
        }

        $filter["headers"] = $array_header;
        $filter["num"] = count($filter_ids);
        $filter["count"] = $tempCount;

        $i = 0;
        foreach ($sort_status as $status) {
            if (isset($tempTable[$status["__id__"]])) {
                $tempTable[$status["__id__"]]["__index__"] = $i;
                $i++;
            }
        }

        $tempTable = $this->sortSHDColumns($tempTable, $sort_column);

        $filter["content"] = $tempTable;
        $filter["page"] = $l_filter["shd_filter__page"];
        $filter["display_rows"] =  $p_params["display_rows"];

        if ($p_params["debug"] == true) {
            $filter["time"] = (microtime(true) - $l_start_time);
        }
        return $filter;
    }

    private function getMultiValue($const, $value)
    {
        $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao(isys_application::instance()->database));
        switch ($const) {
            case "C__CATG__IT_SERVICE-connected_object":
                if (is_numeric($value)) {
                    $p_param = [
                        'objID' => $value,
                        'category' => "C__CATG__IT_SERVICE"
                    ];

                    $l_categoryDatas = $l_api_model_category->read($p_param);
                    $l_multiValue = [];
                    foreach ($l_categoryDatas as $f_categoryData) {
                        $l_multiValue[] = ["__id__" => $f_categoryData["connected_object"]["id"], "value" => $f_categoryData["connected_object"]["title"]];
                    }

                    if ($l_multiValue !== []) {
                        return $l_multiValue;
                    } else {
                        return null;
                    }
                }
                return null;

            case "C__CATG__IP-fqdn":
                if (is_numeric($value)) {
                    $l_return = $this->getValue("category_g_ip", $value);
                    $l_multiValue = [];
                    foreach ($l_return as $f_categoryData) {
                        if ($f_categoryData["isys_catg_ip_list__hostname"] != "" || $f_categoryData["isys_catg_ip_list__domain"] != "") {
                            $l_multiValue[] = ["__id__" => $f_categoryData["isys_catg_ip_list__id"], "value" => $f_categoryData["isys_catg_ip_list__hostname"] . "." . $f_categoryData["isys_catg_ip_list__domain"]];
                        }
                    }

                    if ($l_multiValue !== []) {
                        return $l_multiValue;
                    } else {
                        return null;
                    }
                }
                return null;

            case "C__CATG__LOCATION-_location_path":
                $smview_cmdb_dao_location    = new smview_cmdb_dao_location(isys_application::instance()->database);
                $l_str_location = $smview_cmdb_dao_location->get_shd_path($value);
                return $l_str_location;

            case "C__CATG__LOCATION-location_path":
                $smview_cmdb_dao_location    = new smview_cmdb_dao_location(isys_application::instance()->database);
                $l_str_location = $smview_cmdb_dao_location->get_shd_path_to_string($value);
                return $l_str_location;

            case "C__CATG__VIRTUAL_MACHINE-primary":
                if (is_numeric($value)) {
                    $p_param = [
                        'objID' => $value,
                        'category' => "C__CATG__VIRTUAL_MACHINE"
                    ];

                    $l_categoryDatas = $l_api_model_category->read($p_param);
                    $l_multiValue = [];
                    foreach ($l_categoryDatas as $f_categoryData) {
                        if ($f_categoryData["primary"]["title"] != "") {
                            $l_multiValue[] = ["__id__" => $f_categoryData["primary"]["id"], "value" => $f_categoryData["primary"]["title"]];
                        }
                    }

                    if ($l_multiValue !== []) {
                        return $l_multiValue;
                    } else {
                        return null;
                    }
                }
                return null;

            case "C__CATG__SMVIEW_IMPORT_ELID-device_elid":
                $zit_cmdb_dao_elid = new zit_cmdb_dao_elid(isys_application::instance()->database);
                $zit_cmdb_dao_elid->getElid($value);
                return true;
        }
        return null;
    }

    private function getTable()
    {
        if ($this->m_table["category_g_ip"] == null) {
            $l_api_model_category = new isys_cmdb_dao_category_g_ip(isys_application::instance()->database);

            $this->m_table["category_g_ip"] = $l_api_model_category->get_data_as_array(null, null, "AND mainObject.isys_obj__isys_obj_type__id = ". $this->m_object_type, null, null);
        }

        if ($this->m_table["category_g_virtual_machine"] == null) {
            $l_api_model_category = new isys_cmdb_dao_category_g_virtual_machine(isys_application::instance()->database);

            $this->m_table["category_g_virtual_machine"] = $l_api_model_category->get_data_as_array(null, null, "AND mainObject.isys_obj__isys_obj_type__id = ". $this->m_object_type, null, null);
        }
        return null;
    }

    private function getTableRelation()
    {
        if ($this->m_table["category_g_ip"] == null) {
            $l_api_model_category = new isys_cmdb_dao_category_g_ip(isys_application::instance()->database);

            $this->m_table["category_g_ip"] = $l_api_model_category->get_data_as_array(null, null, null, null, null);
        }

        if ($this->m_table["category_g_virtual_machine"] == null) {
            $l_api_model_category = new isys_cmdb_dao_category_g_virtual_machine(isys_application::instance()->database);

            $this->m_table["category_g_virtual_machine"] = $l_api_model_category->get_data_as_array(null, null, null, null, null);
        }
        return null;
    }

    private function getValue($const, $value)
    {
        $this->getTable();
        $return  = [];
        foreach ($this->m_table[$const] as $l_categoryData) {
            if ($l_categoryData["isys_obj__id"] == $value) {
                $return[] = $l_categoryData;
            }
        }
        return $return;
    }

    private function ConditionSHD($condition, $status)
    {
        $a_columns = [];
        foreach ($condition as $f_condition) {
            switch ($f_condition["property"]) {
                case "C__SHD__LIVESTATUS-output":
                    switch ($f_condition["comparison"]) {
                        case "LIKE %...%":
                            if (strpos(strtolower($status["output"]), strtolower($f_condition["value"])) !== false) {
                                $a_columns["output"][] = true;
                            } else {
                                $a_columns["output"][] = false;
                            }
                            break;

                        case "=":
                            if ($status["output"] == $f_condition["value"]) {
                                $a_columns["output"][] = true;
                            } else {
                                $a_columns["output"][] = false;
                            }
                            break;
                    }
                    break;

                case "C__SHD__LIVESTATUS-status":
                    if (isset($status["status"]) == false) {
                        $status["status"] = "-1";
                    }

                    switch ($f_condition["comparison"]) {
                        case "LIKE %...%":
                            if (strpos(strtolower($status["status"]), strtolower($f_condition["value"])) !== false) {
                                $a_columns["status"][] = true;
                            } else {
                                $a_columns["status"][] = false;
                            }
                            break;

                        case "=":
                            if ($status["status"] == $f_condition["value"]) {
                                $a_columns["status"][] = true;
                            } else {
                                $a_columns["status"][] = false;
                            }
                            break;
                    }
                    break;

                case "C__SHD__LIVESTATUS-service_status":
                    break;
            }
        }

        $return = true;
        foreach ($a_columns as $f_item) {
            $condition = false;
            foreach ($f_item as $bool) {
                $condition = $condition || $bool;
            }
            $return = $return && $condition;
        }
        return $return;
    }

    /**
     * Sort the SHD Columns
     * @param $column
     * @param $sort_status
     * @return mixed
     */
    private function ColumnsSHD($column, $sort_status)
    {
        switch ($column["id"]) {
            case "C__SHD__LIVESTATUS-service_status":
                if ($column["sort"] == "DESC") {
                    usort($sort_status, function ($a, $b) {

                        if (isset($a["service_status"]) == false) {
                            $a["service_status"] = "999|999|999|999";
                        }
                        if (isset($b["service_status"]) == false) {
                            $b["service_status"] = "999|999|999|999";
                        }

                        $split_a = explode("|", $a["service_status"]);
                        $split_b = explode("|", $b["service_status"]);

                        foreach (range(0, 3) as $i) {
                            if ($split_a[$i] < $split_b[$i]) {
                                return -1;
                            } elseif ($split_a[$i] > $split_b[$i]) {
                                return 1;
                            }
                        }
                        return -1;
                    });
                } elseif ($column["sort"] == "ASC") {
                    usort($sort_status, function ($a, $b) {

                        if (isset($a["service_status"]) == false) {
                            $a["service_status"] = "0|0|0|0";
                        }
                        if (isset($b["service_status"]) == false) {
                            $b["service_status"] = "0|0|0|0";
                        }

                        $split_a = explode("|", $a["service_status"]);
                        $split_b = explode("|", $b["service_status"]);

                        foreach (range(0, 3) as $i) {
                            if ($split_a[$i] > $split_b[$i]) {
                                return -1;
                            } elseif ($split_a[$i] < $split_b[$i]) {
                                return 1;
                            }
                        }
                        return -1;
                    });
                }
                return $sort_status;

            case "C__SHD__LIVESTATUS-status":
                if ($column["sort"] == "ASC") {
                    usort($sort_status, function ($a, $b) {

                        //Set -1 when status not set
                        if (isset($a["status"]) == false) {
                            $a["status"] = -1;
                        }
                        if (isset($b["status"]) == false) {
                            $b["status"] = -1;
                        }

                        if ($a["status"] == $b["status"]) {
                            return 0;
                        }
                        return ($a["status"] > $b["status"]) ? -1 : 1;
                    });
                } elseif ($column["sort"] == "DESC") {
                    usort($sort_status, function ($a, $b) {

                        //Set -1 when status not set
                        if (isset($a["status"]) == false) {
                            $a["status"] = 99;
                        }
                        if (isset($b["status"]) == false) {
                            $b["status"] = 99;
                        }

                        if ($a["status"] == $b["status"]) {
                            return 0;
                        }
                        return ($a["status"] < $b["status"]) ? -1 : 1;
                    });
                }
                return $sort_status;

            case "C__SHD__LIVESTATUS-output":
                if ($column["sort"] == "ASC") {
                    usort($sort_status, function ($a, $b) {
                        return strcasecmp(strval($a['output']), strval($b['output']));
                    });
                } elseif ($column["sort"] == "DESC") {
                    usort($sort_status, function ($a, $b) {
                        return strcasecmp(strval($b['output']), strval($a['output']));
                    });
                }
                return $sort_status;
        }
        return $sort_status;
    }

    private function getProperty($key)
    {
        if (is_numeric($key)) {
            //foreach and if for id
            foreach ($this->m_properties as $m_property) {
                if ($m_property["id"] == $key) {
                    return $m_property;
                }
            }
        } else {
            return $this->m_properties[$key];
        }
        return null;
    }

    private function sortSHDColumns($content, $sort_column)
    {
        switch ($sort_column["sorting_default"]) {
            case "C__CATG__IP-fqdn":
                $column_id = $this->getProperty($sort_column['sorting_default'])['id'];

                $array = [];
                foreach ($content as $item) {
                    $array[$item["__id__"]] = $item[$column_id];
                }

                if ($sort_column["sorting_direction"] == "ASC") {
                    uasort($array, function ($a, $b) {

                        //Set -1 when status not set
                        if (isset($a[0]["value"]) == false) {
                            $a[0]["value"] = "";
                        }
                        if (isset($b[0]["value"]) == false) {
                            $a[0]["value"] = "";
                        }

                        if ($a[0]["value"] == $b[0]["value"]) {
                            return 0;
                        }
                        return ($a[0]["value"] > $b[0]["value"]) ? -1 : 1;
                    });
                } else {
                    uasort($array, function ($a, $b) {

                        //Set -1 when status not set
                        if (isset($a[0]["value"]) == false) {
                            $a[0]["value"] = "";
                        }
                        if (isset($b[0]["value"]) == false) {
                            $a[0]["value"] = "";
                        }

                        if ($a[0]["value"] == $b[0]["value"]) {
                            return 0;
                        }
                        return ($a[0]["value"] < $b[0]["value"]) ? -1 : 1;
                    });
                }
                break;
        }

        //set Index from sort
        $f_counter = 0;
        foreach ($array as $key => $item) {
            $content[$key]["__index__"] = $f_counter;
            $f_counter++;
        }
        return $content;
    }

    private function isSpecialColumn($const)
    {
        switch ($const) {
            case "C__SHD__LIVESTATUS-output":
            case "C__CATG__IT_SERVICE-connected_object":
            case "C__CATG__IP-fqdn":
            case "C__CATG__LOCATION-_location_path":
            case "C__CATG__LOCATION-location_path":
            case "C__CATG__VIRTUAL_MACHINE-primary":
            case "C__CATG__SMVIEW_IMPORT_ELID-device_elid":
                return true;
        }
        return false;
    }

    /**
     * get_data always retrieves the data of the main table of this module
     */
    public function get_data()
    {
        // TODO: Implement get_data() method.
    }
}
