<?php

use idoit\Module\Api\Exception\ValidationException;
use idoit\Module\Cable\CableTracing;
use idoit\Module\Cable\SectionTracing;

/**
 * i-doit - Documentation and CMDB solution for IT environments
 *
 * This file is part of the i-doit framework. Modify at your own risk.
 *
 * Please visit http://www.i-doit.com/license for a full copyright and license information.
 *
 * @version     1.8.3
 * @package     i-doit
 * @author      synetics GmbH
 * @copyright   synetics GmbH
 * @url         http://www.i-doit.com
 * @license     http://www.i-doit.com/license
 */


class smview_cmdb_dao_query_category extends isys_module_dao
{
    protected static $m_instance = null;

    protected $m_db;

    protected $m_properties;

    protected $m_obj_ids;

    protected $m_sortColumn;

    protected $m_global_column;

    protected $m_column;

    protected $m_specialcolumn;

    protected $m_category;

    protected $m_object_type;

    protected $m_table;

    protected $m_display_rows;

    protected $m_cable_name;

    /**
     * Cable object cache.
     *
     * @var array
     */
    private $_cables = [];

    /**
     * Logger.
     *
     * @var isys_log
     */
    protected $logger = null;

    /**
     * Database component.
     *
     * @param  isys_component_database
     * @throws Exception
     */
    public function __construct(isys_component_database $p_db)
    {
        global $g_absdir;
        $this->logger = isys_log::get_instance('shd-smview')
            ->set_log_file($g_absdir . '/log/shd-smview_' . date('Y-m-d') . '.log');
        
        $properties = smview_cmdb_dao_properties::instance(isys_application::instance()->database)->getProperties(false);
        //change the property-const to id
        $t_properties = [];
        foreach ($properties as $property) {
            $t_properties[$property['const'] . '-' . $property['key']] = $property;
        }

        $this->m_properties = $t_properties;
        parent::__construct($p_db);
    }

    /**
     * Read method.
     *
     * @param  array $p_params
     * @return array
     */
    public function read($p_params)
    {
        $l_start_time = microtime(true);
        if(! isset($p_params['display_rows'])){
            $p_params['display_rows'] = 99999999999;
        }

        $this->m_display_rows = $p_params['display_rows'];
        foreach ($p_params["condition"] as $condition) {
            if ($condition["property"] == "C__CATG__GLOBAL-id") {
                $this->m_obj_ids[] = $condition["value"];
            } elseif ($condition["property"] == "map_id") {
                $map_objects = smview_cmdb_dao_map::instance($this->get_database_component())->getMapObjectAndMapLinkByMap($condition["value"]);
                foreach ($map_objects as $f_object) {
                    if($f_object["isys_obj__id"] != "0") {
                        $this->m_obj_ids[] = $f_object["isys_obj__id"];
                    }
                }
            } else {
                foreach ($this->m_properties as $f_property) {
                    if ($f_property["const"] . "-" . $f_property["key"] == $condition["property"]) {
                        $this->m_Condition[] = $condition;
                    }
                }
            }
        }

        foreach ($p_params["columns"] as $column) {
            $split = explode("-", $column["id"]);
            foreach ($this->m_properties as $f_property) {
                if ($f_property["const"] . "-" . $f_property["key"] == $column["id"]) {
                    $column["property_id"] = $f_property["id"];
                    if ($split[0] == "C__CATG__GLOBAL") {
                        $this->m_global_column[] = $column;
                    } else {
                        $this->m_category = $split[0];
                        if ($this->isSpecialColumn($column["id"]) == true) {
                            $this->m_specialcolumn[] = $column;
                        } else {
                            $this->m_column[] = $column;
                        }
                    }
                }
            }
        }

        $return = [];
        $return["headers"] = $this->getHeader($p_params);

        $content = [];
        if ($this->m_obj_ids != null) {
            foreach ($this->m_obj_ids as $obj_id) {
                $content = array_replace($this->getColumns($obj_id), $content);
            }
        } else {
            $content = $this->getColumns(null);
        }

        foreach ($content as $key => $row) {
            if ($this->isConditionTrue($row) === false) {
                unset($content[$key]);
            }
        }

        $content = $this->sortColumns($content);
        $return["count"] = count($content);
        $return["total"] = count($content);

        //pageable
        if ($p_params['display_rows'] < count($content)) {
            $i = 0;
            $start = ((int)$p_params['page']) * (int)$p_params['display_rows'];
            $end = ((int)$p_params['page'] + 1) * (int)$p_params['display_rows'];
            $paged_contend = [];
            foreach ($content as $obj_id_key => $f_row) {
                if ($i >= $start && $i < $end) {
                    $paged_contend[$obj_id_key] = $f_row;
                }
                $i++;
            }
            $content = $paged_contend;
            //paging irelavant
        }

        $temp_content = [];
        foreach ($content as $content_key => $row) {
            foreach ($row as $row_key => $row_value) {
                if (is_numeric($row_key)) {
                    $property = $this->getProperty($row_key);
                    $temp_content[$content_key][$property["const"] . "-" . $property["key"]] = $row_value;
                } else {
                    $temp_content[$content_key][$row_key] = $row_value;
                }
            }
        }

        if($p_params['display_rows'] == 99999999999){
            $p_params['display_rows'] = "0";
        }

        $return["content"] = $temp_content;
        $return["page"] = $p_params['page'];
        $return["display_rows"] = isset($p_params["display_rows"]) ? $p_params["display_rows"] : "0";
        $return["time"] = (microtime(true) - $l_start_time);
        return $return;
    }

    private function getHeader($p_params)
    {
        $count = 0;
        $array_header = [];
        foreach ($p_params["columns"] as $f_column) {
            foreach ($this->m_properties as $f_property) {
                if ($f_property["const"] . "-" . $f_property["key"] ==  $f_column["id"]) {
                    if ($f_column["sort"] != "") {
                        $sortColumn = $f_column;
                        $sortColumn["property_id"] = $f_property["id"];
                        $this->m_SortColumn = $sortColumn;
                    }

                    $array_header[$count] = [
                        "const" => $f_property["const"] . "-" . $f_property["key"],
                        "title" => $f_property["title"]
                    ];
                    $count++;
                }
            }
        }
        return $array_header;
    }

    private function getColumns($p_obj_id)
    {
        $language = isys_application::instance()->container->get('language');
        $return = [];
        $global_return = [];
        if ($p_obj_id != null) {
            $global = $this->getCategoryValue($p_obj_id, "C__CATG__GLOBAL");
            foreach ($this->m_global_column as $global_column) {
                $split = explode("-", $global_column["id"]);
                if (isset($global[0][$split[1]]["title"])) {
                    $global_return[$global_column["id"]] = $global[0][$split[1]]["title"];
                } else {
                    $global_return[$global_column["id"]] = $global[0][$split[1]];
                }
            }

            if ($this->m_category == "C__CATG__LIVESTATUS") {
                $m_status = smview_cmdb_dao_status::instance($this->get_database_component())->get_all_services_by_hostname($p_obj_id);
                $t_content = $m_status;
            } elseif ($this->m_category != null) {
                //Performance problem Data not needed
                $non_performant_categories = [
                    "C__CATG__SMVIEW_CABLE_PIN",
                    "C__CATG__SMVIEW_FEED_JIRA",
                    "C__CATG__SMVIEW_FEED_VIRTUAL_DEVICE",
                ];
                if (! in_array($this->m_category, $non_performant_categories)) {
                    $t_content = $this->getCategoryValue($p_obj_id, $this->m_category);
                }
            }
        }

        if ($this->m_category == "C__CATG__BVG_RENTING" || $this->m_category == "C__CATG__BVG_ASSIGNED_RENTING") {
            if ($t_content != null) {
                $newContent = [];
                // add location information
                $route_dao = new isys_cmdb_dao_category_g_smview_cable_route(isys_application::instance()->database);
                $renting_dao = new isys_cmdb_dao_category_g_bvg_renting(isys_application::instance()->database);
                foreach ($t_content as $content) {
                    $route = $route_dao->get_data(null, $content['objID'])->get_row();
                    $start_obj_id = $route['isys_catg_smview_cable_route_list__start_obj_id'];
                    $end_obj_id = $route['isys_catg_smview_cable_route_list__end_obj_id'];
                    $content['location_in'] = $renting_dao->get_location_path($start_obj_id);
                    $content['location_out'] = $renting_dao->get_location_path($end_obj_id);
                    $content['damping'] = $renting_dao->get_damping($content['objID']);
                    $newContent[] = $content;
                }
                $t_content = $newContent;
            }
        }

        $t_specialContent = $this->getSpecialValue($p_obj_id);
        if ($this->m_category == "C__CATG__SMVIEW_CABLE_PIN") {
//          Der specialKey is die Pin-In Id um beim speichern mit dieser wieder die richtigen werte zuhaben.
            foreach ($t_specialContent as $specialContent) {
                $content_return = [];

                $specialKey = $specialContent["obj_in"]["id"];
                foreach ($this->m_specialcolumn as $specialcolumn) {
                    switch ($specialcolumn["id"]) {
                        case "C__CATG__SMVIEW_CABLE_PIN-in_cable":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_in"]["cable_obj_id"]] = [
                                "__id__" => $specialContent["obj_in"]["cable_obj_id"],
                                "value"  => (string)$specialContent["obj_in"]["cable_title"]
                            ];
                            break;
                        case "C__CATG__SMVIEW_CABLE_PIN-in_cable_wire":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_in"]["cable_obj_id"]] = [
                                "__id__" => $specialContent["obj_in"]["wire_id"],
                                "value"  => (string)$specialContent["obj_in"]["wire_title"]
                            ];
                            break;
                        case "C__CATG__SMVIEW_CABLE_PIN-in_connected_with":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_in"]["id"]] =
                                [
                                    "__id__" => $specialContent["obj_in"]["id"],
                                    "value"  => (string)$specialContent["obj_in"]["connected_object_title"]
                                ];
                            break;
                        case "C__CATG__SMVIEW_CABLE_PIN-in_connector_title":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_in"]["id"]] =
                                [
                                    "__id__" => $specialContent["obj_in"]["id"],
                                    "value"  => (string)$specialContent["obj_in"]["connector_title"]
                                ];
                            break;
                        case "C__CATG__SMVIEW_CABLE_PIN-in_description":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_in"]["id"]] =
                                [
                                    "__id__" => $specialContent["obj_in"]["id"],
                                    "value"  => (string)$specialContent["obj_in"]["description"]
                                ];
                            break;
                        case "C__CATG__SMVIEW_CABLE_PIN-in_scope":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_in"]["id"]] =
                                [
                                    "__id__" => $specialContent["obj_in"]["id"],
                                    "value"  => (string)$specialContent["obj_in"]["classification"]
                                ];
                            break;
                        case "C__CATG__SMVIEW_CABLE_PIN-out_cable":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_out"]["id"]] =
                                [
                                    "__id__" => $specialContent["obj_out"]["id"],
                                    "value"  => (string)$specialContent["obj_out"]["cable_title"]
                                ];
                            break;
                        case "C__CATG__SMVIEW_CABLE_PIN-out_cable_wire":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_out"]["cable_obj_id"]] = [
                                "__id__" => $specialContent["obj_out"]["wire_id"],
                                "value"  => (string)$specialContent["obj_out"]["wire_title"]
                            ];
                            break;
                        case "C__CATG__SMVIEW_CABLE_PIN-out_connected_with":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_out"]["id"]] =
                                [
                                    "__id__" => $specialContent["obj_out"]["id"],
                                    "value"  => (string)$specialContent["obj_out"]["connected_object_title"]
                                ];
                            break;
                        case "C__CATG__SMVIEW_CABLE_PIN-out_connector_title":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_out"]["id"]] =
                                [
                                    "__id__" => $specialContent["obj_out"]["id"],
                                    "value"  => (string)$specialContent["obj_out"]["connector_title"]
                                ];
                            break;
                        case "C__CATG__SMVIEW_CABLE_PIN-out_description":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_out"]["id"]] =
                                [
                                    "__id__" => $specialContent["obj_out"]["id"],
                                    "value"  => (string)$specialContent["obj_out"]["description"]
                                ];
                            break;
                        case "C__CATG__SMVIEW_CABLE_PIN-out_scope":
                            $content_return[$specialcolumn["id"]][$specialContent["obj_out"]["id"]] =
                                [
                                    "__id__" => $specialContent["obj_out"]["id"],
                                    "value"  => (string)$specialContent["obj_out"]["classification"]
                                ];
                            break;
                    }
                }

                $t_array = array_replace($global_return, $content_return);
                $t_array["__id__"] = $specialKey;
                $t_array["__index__"] = "0";
                $return[$specialKey] = $t_array;
            }
        } elseif ($this->m_category == "C__CATG__SMVIEW_CABLE_SECTION") {
            foreach ($this->m_specialcolumn as $specialcolumn) {
                $value = $t_specialContent[$specialcolumn["id"]];
                switch ($specialcolumn["id"]) {
                    case "C__CATG__SMVIEW_CABLE_SECTION-user_state_id":
                        $dao = new isys_cmdb_dao_smview_cable_user_state($this->m_db);
                        $group = $dao->get_object("id", $value);
                        $name = ! empty($group) ? $group["name"] . ': ' . $group["description"] : '';
                        $content_return[$specialcolumn["id"]] = [["__id__" => $value, "value" => $name]];
                        break;
                    case "C__CATG__SMVIEW_CABLE_SECTION-start_object":
                        $value = $t_specialContent['C__CATG__SMVIEW_CABLE_SECTION-start_obj_id'];
                        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
                        $obj = $dao->get_object($value)->get_row();
                        $content_return[$specialcolumn["id"]] = [["__id__" => $value, "value" => $obj['isys_obj__title']]];
                        break;
                    case "C__CATG__SMVIEW_CABLE_SECTION-end_object":
                        $value = $t_specialContent['C__CATG__SMVIEW_CABLE_SECTION-end_obj_id'];
                        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
                        $obj = $dao->get_object($value)->get_row();
                        $content_return[$specialcolumn["id"]] = [["__id__" => $value, "value" => $obj['isys_obj__title']]];
                        break;
                    case "C__CATG__SMVIEW_CABLE_SECTION-route_id":
                        $section_id = $t_specialContent['C__CATG__SMVIEW_CABLE_SECTION-id'];

                        $route_element_dao = new isys_cmdb_dao_smview_cable_route_element(\isys_application::instance()->database);
                        $route_element = $route_element_dao->get_object('section_id', $section_id);
                        if (! empty($route_element)) {
                            $route_dao = new isys_cmdb_dao_smview_cable_route(\isys_application::instance()->database);
                            $element = $route_dao->get_object('id', $route_element['route_id']);
                            $global_dao = new isys_cmdb_dao_category_g_global(\isys_application::instance()->database);
                            $obj = $global_dao->get_object_by_id($element['obj_id'])->get_row();
                            $content_return[$specialcolumn["id"]] = [["__id__" => $obj['isys_obj__id'], "value" => $obj['isys_obj__title']]];
                        } else {
                            $content_return[$specialcolumn["id"]] = "";
                        }
                        break;
                    default:
                        $content_return[$specialcolumn["id"]] = $value;
                        break;
                }
            }
            $t_array = array_replace($global_return, $content_return);
            $t_array["__id__"] = $t_specialContent["id"];
            $t_array["__index__"] = "0";
            $return[$t_specialContent["id"]] = $t_array;
        } elseif ($this->m_category == "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT") {
            foreach ($t_specialContent as $specialKey => $specialContent) {
                $content_return = $this->_getSectionElementColumn($specialContent);
                $t_array = $content_return;
                $t_array["__id__"] = $specialKey;
                $t_array["__index__"] = $content_return["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-index"];
                $return[$specialKey] = $t_array;
            }
            return $return;
        } elseif ($this->m_category == "C__CATG__SMVIEW_CABLE") {
            $content_return = $this->_getCableColumn($t_specialContent, $p_obj_id);
            $t_array = array_replace($global_return, $content_return);
            $t_array["__id__"] = $t_specialContent["id"];
            $t_array["__index__"] = "0";
            $return[$t_specialContent["id"]] = $t_array;
        } elseif ($this->m_category == "C__CATG__SMVIEW_CABLE_WIRE") {
            $this->_cables = [];
            foreach ($t_specialContent as $specialKey => $specialContent) {
                $content_return = $this->_getWireColumn($specialContent);
                $t_array = array_replace($global_return, $content_return);
                $t_array["__id__"] = $specialContent["id"];
                $t_array["__index__"] = "0";
                $return["10" . $specialKey] = $t_array;
            }
        } elseif ($this->m_category == "C__CATG__SMVIEW_CABLE_ROUTE_ELEMENT" || $this->m_category == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE_ELEMENT") {
            foreach ($t_specialContent as $specialKey => $specialContent) {
                $content_return = $this->_getRouteElementColumn($specialContent, $this->m_category);
                $t_array = array_replace($global_return, $content_return);
                $t_array["__id__"] = $specialKey;
                $t_array["__index__"] = "0";
                $return[$specialKey] = $t_array;
            }
        } elseif ($this->m_category == "C__CATG__SMVIEW_CABLE_ROUTE") {
            foreach ($this->m_specialcolumn as $specialcolumn) {
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_ROUTE-route_usage_id") {
                    $dao = new isys_cmdb_dao_smview_cable_route_usage($this->m_db);
                    $value = $t_specialContent["C__CATG__SMVIEW_CABLE_ROUTE-route_usage_id"];
                    $group = $dao->get_object("id", $value);
                    $title = $group["name"];
                    if (! empty($group["technology"])) {
                        $title .= ' (' . $group["technology"] . ')';
                    }
                    $content_return[$specialcolumn["id"]] = [["__id__" => $value, "value" => $title]];
                    continue;
                }
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_ROUTE-start_object") {
                    $value = $t_specialContent['C__CATG__SMVIEW_CABLE_ROUTE-start_obj_id'];
                    if ($value) {
                        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
                        $obj = $dao->get_object($value)->get_row();
                        $content_return[$specialcolumn["id"]] = [["__id__" => $value, "value" => $obj['isys_obj__title']]];
                    }
                    continue;
                }
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_ROUTE-end_object") {
                    $value = $t_specialContent['C__CATG__SMVIEW_CABLE_ROUTE-end_obj_id'];
                    if ($value) {
                        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
                        $obj = $dao->get_object($value)->get_row();
                        $content_return[$specialcolumn["id"]] = [["__id__" => $value, "value" => $obj['isys_obj__title']]];
                    }
                    continue;
                }
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_ROUTE-route_elements") {
                    $dao = new isys_cmdb_dao_category_g_smview_cable_route_element($this->m_db);
                    // todo: this method call is a performance killer!!!!
                    $value = $t_specialContent["C__CATG__SMVIEW_CABLE_ROUTE-obj_id"];
                    $elements = $dao->get_elements_by_obj_id($value, true);
                    $items = [];
                    foreach ($elements as $e) {
                        if (! empty($e['section_id'])) {
                            $title = $e['section_title'];
                        } else {
                            $title = $e['cable_title'] . ' - ' . $e['wire_scope'] . ' - ' . $e['wire_title'];
                        }
                        $items[] = ["__id__" => $e['id'], "value" => $title];
                    }
                    $content_return[$specialcolumn["id"]] = $items;
                    continue;
                }
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_ROUTE-is_external") {
                    $is_external = $t_specialContent["C__CATG__SMVIEW_CABLE_ROUTE-is_external"];
                    $id = -1;
                    $value = null;
                    if ($is_external != "") {
                        $id = $is_external;
                        if ($is_external == 0) {
                            $value = $language->get("LC__UNIVERSAL__NO");
                        } elseif ($is_external == 1) {
                            $value = $language->get("LC__UNIVERSAL__YES");
                        }
                    }
                    $content_return[$specialcolumn["id"]] = [[
                        "__id__" => $id,
                        "value"  => $value
                    ]];
                    continue;
                }
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_ROUTE-is_rented") {
                    $id = -1;
                    $value = null;
                    if (class_exists('isys_cmdb_dao_category_g_bvg_renting')) {
                        $renting_dao = new isys_cmdb_dao_category_g_bvg_renting(isys_application::instance()->database);
                        $sql = ' AND isys_catg_bvg_renting_list__obj_id = ' . $renting_dao->convert_sql_id($p_obj_id);
                        $sql .= ' AND isys_catg_bvg_renting_list__status = ' . defined_or_default('C__RECORD_STATUS__NORMAL');
                        $rentingCount = $renting_dao->get_data(null, null, $sql)->count();
                        $id = $rentingCount > 0 ? 1 : 0;
                        if ($id == 0) {
                            $value = $language->get("LC__UNIVERSAL__NO");
                        } else {
                            $value = $language->get("LC__UNIVERSAL__YES");
                        }
                    }
                    $content_return[$specialcolumn["id"]] = [["__id__" => $id, "value" => $value]];
                    continue;
                }
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_ROUTE-redundant_routes") {
                    $dao = new isys_cmdb_dao_category_g_smview_cable_route($this->m_db);
                    $value = $t_specialContent["C__CATG__SMVIEW_CABLE_ROUTE-obj_id"];
                    $res = $dao->get_assigned_redundant_routes($value);
                    $items = [];
                    while ($row = $res->get_row()) {
                        $title = $row['isys_obj__title'];
                        if ((! empty($row['associated_obj_id']))) {
                            $title .= ' [' . $row['associated_obj_title'] . ']';
                        }
                        $items[] = ["__id__" => $row['isys_obj__id'], "value" => $title];
                    }
                    $content_return[$specialcolumn["id"]] = $items;
                    continue;
                }
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_ROUTE-associated_route") {
                    $dao = new isys_cmdb_dao_category_g_smview_cable_route($this->m_db);
                    $value = $t_specialContent["C__CATG__SMVIEW_CABLE_ROUTE-associated_route"];
                    $items = [];
                    if (! empty($value)) {
                        $row = $dao->get_data(null, $value)->get_row();
                        $items[] = ["__id__" => $row['isys_obj__id'], "value" => $row['isys_obj__title']];
                    }
                    $content_return[$specialcolumn["id"]] = $items;
                    continue;
                }

                foreach ($t_specialContent as $specialKey => $specialContent) {
                    if ($specialcolumn["id"] == $specialKey) {
                        $content_return[$specialcolumn["id"]] = $specialContent;
                        continue 2;
                    }
                }
            }
            $t_array = array_replace($global_return, $content_return);
            $t_array["__id__"] = $t_specialContent["id"];
            $t_array["__index__"] = "0";
            $return[$t_specialContent["id"]] = $t_array;
        } elseif ($this->m_category == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE") {
            foreach ($this->m_specialcolumn as $specialcolumn) {
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-route_usage_id") {
                    $dao = new isys_cmdb_dao_smview_cable_route_usage($this->m_db);
                    $value = $t_specialContent["C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-route_usage_id"];
                    $group = $dao->get_object("id", $value);
                    $title = $group["name"];
                    if (! empty($group["technology"])) {
                        $title .= ' (' . $group["technology"] . ')';
                    }
                    $content_return[$specialcolumn["id"]] = [["__id__" => $value, "value" => $title]];
                    continue;
                }
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-start_object") {
                    $value = $t_specialContent['C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-start_obj_id'];
                    if ($value) {
                        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
                        $obj = $dao->get_object($value)->get_row();
                        $content_return[$specialcolumn["id"]] = [["__id__" => $value, "value" => $obj['isys_obj__title']]];
                    }
                    continue;
                }
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-end_object") {
                    $value = $t_specialContent['C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-end_obj_id'];
                    if ($value) {
                        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
                        $obj = $dao->get_object($value)->get_row();
                        $content_return[$specialcolumn["id"]] = [["__id__" => $value, "value" => $obj['isys_obj__title']]];
                    }
                    continue;
                }
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-route_elements") {
                    $dao = new isys_cmdb_dao_category_g_smview_cable_virtual_route_element($this->m_db);
                    // todo: this method call is a performance killer!!!!
                    $value = $t_specialContent["C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-obj_id"];
                    $elements = $dao->get_elements_by_obj_id($value, true);
                    $items = [];
                    foreach ($elements as $e) {
                        if (! empty($e['section_id'])) {
                            $title = $e['section_title'];
                        } else {
                            $title = $e['cable_title'] . ' - ' . $e['wire_scope'] . ' - ' . $e['wire_title'];
                        }
                        $items[] = ["__id__" => $e['id'], "value" => $title];
                    }
                    $content_return[$specialcolumn["id"]] = $items;
                    continue;
                }
                if ($specialcolumn["id"] == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-is_external") {
                    $is_external = $t_specialContent["C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-is_external"];
                    $id = -1;
                    $value = null;
                    if ($is_external != "") {
                        $id = $is_external;
                        if ($is_external == 0) {
                            $value = $language->get("LC__UNIVERSAL__NO");
                        } elseif ($is_external == 1) {
                            $value = $language->get("LC__UNIVERSAL__YES");
                        }
                    }
                    $content_return[$specialcolumn["id"]] = [[
                        "__id__" => $id,
                        "value"  => $value
                    ]];
                    continue;
                }

                foreach ($t_specialContent as $specialKey => $specialContent) {
                    if ($specialcolumn["id"] == $specialKey) {
                        $content_return[$specialcolumn["id"]] = $specialContent;
                        continue 2;
                    }
                }
            }
            $t_array = array_replace($global_return, $content_return);
            $t_array["__id__"] = $t_specialContent["id"];
            $t_array["__index__"] = "0";
            $return[$t_specialContent["id"]] = $t_array;
        } elseif ($this->m_category == "C__CATG__SMVIEW_FEED_JIRA") {
            // important: use full qualifier here to support a non-installed feed add-on
            $proxy_api = new \idoit\Module\Feed\ProxyApi\Jira();
            $return = $this->_prepareVirtualFields($return, $proxy_api, $t_specialContent, $global_return, 'key');
        } elseif ($this->m_category == "C__CATG__SMVIEW_FEED_VIRTUAL_DEVICE") {
            // important: use full qualifier here to support a non-installed feed add-on
            $proxy_api = new \idoit\Module\Feed\ProxyApi\VirtualDevice();
            $return = $this->_prepareVirtualFields($return, $proxy_api, $t_specialContent, $global_return, 'mac_address');
        } elseif ($this->m_category == "C__CATG__LIVESTATUS") {
            $f = 0;
            //schleife über jeden Service
            foreach ($t_content as $t_row) {
                $f++;
                //Schleife über jede Column die mit der Anfrage kommt
                foreach ($this->m_specialcolumn as $specialcolumn) {
                    //wir brauchen die property
                    $split = explode("-", $specialcolumn["id"]);
                    //wir brauchen den Wert der im Service liegt
                    $t_value = $t_row[$split[1]];
                    $val = $split[1] == "last_check" ? gmdate("Y-m-d H:i:s", $t_value) : $t_value;
                    $content_return[$specialcolumn["id"]] = [["__id__" => $p_obj_id, "value" => $val]];
                }

                $t_array = array_replace($global_return, $content_return);
                $t_array["__id__"] = $p_obj_id;
                $t_array["__index__"] = "0";
                $return[$f . $p_obj_id] = $t_array;
            }
        } else {
            if ($this->m_column == null) {
                //only Category Global
                $t_array = array_replace($global_return, []);
                $t_array["__id__"] = $p_obj_id;
                $t_array["__index__"] = "0";
                $return[$p_obj_id] = $t_array;
                return $return;
            }

            if ($t_content != null) {
                foreach ($t_content as $content) {
                    $content_return = [];
                    foreach ($this->m_column as $column) {
                        $split = explode("-", $column["id"]);
                        $property_key = $split[1];

                        //abfrage nach Title
                        if (isset($content[$property_key]["title"])) {
                            if (isset($content[$property_key]["ref_title"])) {
                                //Wenn es eine ID gibt soll ein Object drauß gemacht werden
                                if (isset($content[$property_key]["id"])) {
                                    $content_return[$column["id"]][$content[$property_key]["id"]] = [
                                        "__id__" => $content[$property_key]["id"],
                                        "value"  => (string)$content[$property_key]["ref_title"]
                                    ];
                                } else {
                                    $content_return[$column["id"]] = (string)$content[$property_key]["ref_title"];
                                }
                            } else {
                                if (isset($content[$property_key]["id"])) {
                                    $content_return[$column["id"]][$content[$property_key]["id"]] = [
                                        "__id__" => $content[$property_key]["id"],
                                        "value"  => (string)$content[$property_key]["title"]
                                    ];
                                } else {
                                    $content_return[$column["id"]] = (string)$content[$property_key]["title"];
                                }
                            }
                        } else {
                            if (isset($content[$property_key]["value"])) {
                                //Array has value_key
                                $content_return[$column["id"]] = (string)$content[$property_key]["value"];
                            } else {
                                if (is_array($content[$property_key])) {
                                    foreach ($content[$property_key] as $item) {
                                        if (isset($item["ref_title"])) {
                                            $content_return[$column["id"]][$item["id"]] = ["__id__" => $item["id"], "value" => (string)$item["ref_title"]];
                                        } else {
                                            $content_return[$column["id"]][$item["id"]] = ["__id__" => $item["id"], "value" => (string)$item["title"]];
                                        }
                                    }
                                } else {
                                    if (isset($content[$property_key]["id"])) {
                                        $content_return[$column["id"]][$content[$property_key]["id"]] = [
                                            "__id__" => $content[$property_key]["id"],
                                            "value"  => (string)$content[$property_key]
                                        ];
                                    } else {
                                        $content_return[$column["id"]] = (string)$content[$property_key];
                                    }
                                }
                            }
                        }
                    }

                    foreach ($t_specialContent as $specialKey => $specialContent) {
                        $property = $this->getProperty($specialKey);
                        $content_return[$property["const"] . "-" . $property["key"]] = $specialContent[$content["id"]];
                    }

                    $t_array = array_replace($global_return, $content_return);
                    $t_array["__id__"] = $content["id"];
                    $t_array["__index__"] = "0";
                    $return[$content["id"]] = $t_array;
                }
//            } else {
                //only Category Global
                //bug auskommentiert da ansonsten ein neuer Eintrag in Categories.update angelegt wird und dieser nicht gespeichert werden kann.
//                $t_array = array_replace($global_return, []);
//
//                $t_array["__id__"] = $p_obj_id;
//
//                $t_array["__index__"] = "0";
//                $return[$p_obj_id] = $t_array;
            }
        }
        return $return;
    }

    /**
     * Get values of a category.
     *
     * @param  int $p_obj_id
     * @param  int $p_category
     * @return array
     * @throws ValidationException
     * @throws isys_exception_api
     * @throws isys_exception_auth
     * @throws isys_exception_database
     */
    private function getCategoryValue($p_obj_id, $p_category)
    {
        if ($p_category == "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT") {
            //Error in Function
            return [];
        }

        $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao($this->m_db));
        $p_param = [
            'objID'    => $p_obj_id,
            'category' => $p_category,
            'status'   => C__RECORD_STATUS__NORMAL

        ];
        return $l_api_model_category->read($p_param);
    }

    /**
     * Get cable column.
     *
     * @param  array $t_specialContent
     * @param  int $p_obj_id
     * @return array
     * @throws isys_exception_database
     */
    private function _getCableColumn($t_specialContent, $p_obj_id)
    {
        $content_return = [];
        foreach ($this->m_specialcolumn as $specialColumn) {
            $split = explode("-", $specialColumn["id"]);
            $value = $t_specialContent[$split[1]];
            switch ($specialColumn["id"]) {
                case "C__CATG__SMVIEW_CABLE-state_id":
                    $dao = new isys_cmdb_dao_smview_cable_user_state($this->m_db);
                    $group = $dao->get_object("id", $value);
                    $name = $group["name"] . ': ' . $group["description"];
                    $content_return[$specialColumn["id"]] = [["__id__" => $value, "value" => $name]];
                    break;
                case "C__CATG__SMVIEW_CABLE-group_id":
                    $dao = new isys_cmdb_dao_smview_cable_group($this->m_db);
                    $group = $dao->get_object("id", $value);
                    $content_return[$specialColumn["id"]] = [["__id__" => $value, "value" => $group["description"]]];
                    break;
                case "C__CATG__SMVIEW_CABLE-admin_unit":
                    $dao = new isys_cmdb_dao_smview_cable_admin_unit($this->m_db);
                    $value = $t_specialContent["admin_unit_id"];
                    $group = $dao->get_object("id", $value);
                    $content_return[$specialColumn["id"]] = [["__id__" => $value, "value" => $group["description"]]];
                    break;
                case "C__CATG__SMVIEW_CABLE-description_sort":
                    $content_return[$specialColumn["id"]] = $this->getCableName($p_obj_id);
                    break;
                case "C__CATG__SMVIEW_CABLE-type_id":
                    $dao = new isys_cmdb_dao_smview_cable_type($this->m_db);
                    $group = $dao->get_object("id", $value);
                    $content_return[$specialColumn["id"]] = [["__id__" => $value, "value" => $group["description"]]];
                    break;
                case "C__CATG__SMVIEW_CABLE-is_section_cable":
                    $title = null;
                    if ($value != null) {
                        $language = isys_application::instance()->container->get('language');
                        $title = $value == 1 ? $language->get("LC__UNIVERSAL__YES"): $language->get("LC__UNIVERSAL__NO");
                    }
                    $content_return[$specialColumn["id"]] = [["__id__" => $value, "value" => $title]];
                    break;
                case "C__CATG__SMVIEW_CABLE-location_out":
                    $smview_cmdb_dao_location = new smview_cmdb_dao_location($this->m_db);
                    $value = $t_specialContent["location_out_id"];
                    $l_str_location = $smview_cmdb_dao_location->get_shd_path($value);
                    $content_return[$specialColumn["id"]] = $l_str_location[0]["value"];
                    break;
                case "C__CATG__SMVIEW_CABLE-location_out_path":
                    $smview_cmdb_dao_location = new smview_cmdb_dao_location($this->m_db);
                    $value = $t_specialContent["location_out_id"];
                    $l_str_location = $smview_cmdb_dao_location->get_shd_path_to_string($value, true);
                    $content_return[$specialColumn["id"]] = $l_str_location;
                    break;
                case "C__CATG__SMVIEW_CABLE-location_in":
                    $smview_cmdb_dao_location = new smview_cmdb_dao_location($this->m_db);
                    $value = $t_specialContent["location_in_id"];
                    $l_str_location = $smview_cmdb_dao_location->get_shd_path($value);
                    $content_return[$specialColumn["id"]] = $l_str_location[0]["value"];
                    break;
                case "C__CATG__SMVIEW_CABLE-location_in_path":
                    $smview_cmdb_dao_location = new smview_cmdb_dao_location($this->m_db);
                    $value = $t_specialContent["location_in_id"];
                    $l_str_location = $smview_cmdb_dao_location->get_shd_path_to_string($value, true);
                    $content_return[$specialColumn["id"]] = $l_str_location;
                    break;
                case "C__CATG__SMVIEW_CABLE-cable_patched_wires":
                    $content_return[$specialColumn["id"]] = (string)$t_specialContent["patched_wires"];
                    break;
                case "C__CATG__SMVIEW_CABLE-cable_free_wires":
                    $content_return[$specialColumn["id"]] = (string)$t_specialContent["free_wires"];
                    break;
                case "C__CATG__SMVIEW_CABLE-cable_capacity":
                    $cable_free_wires = $t_specialContent["free_wires"];
                    $cable_patched_wires = $t_specialContent["patched_wires"];
                    $content_return[$specialColumn["id"]] = $this->_calculateCableCapacity($cable_free_wires, $cable_patched_wires);
                    break;
                default:
                    $content_return[$specialColumn["id"]] = $value;
                    break;
            }
        }
        return $content_return;
    }

    /**
     * Calculate cable capacity in percent.
     *
     * @param  int $cable_free_wires
     * @param  int $cable_patched_wires
     * @return string
     */
    private function _calculateCableCapacity($cable_free_wires, $cable_patched_wires)
    {
        $capacity = 0;
        if ($cable_free_wires + $cable_patched_wires > 0) {
            $capacity = ($cable_patched_wires * 100) / ($cable_free_wires + $cable_patched_wires);
        }
        return (string)round($capacity);
    }

    /**
     * Get wire column.
     *
     * @param  array $specialContent
     * @return array
     * @throws isys_exception_database
     */
    private function _getWireColumn($specialContent)
    {
        $content_return = [];
        $language = isys_application::instance()->container->get('language');
        $cableObjId = $specialContent["obj_id"];
        if (! isset($this->_cables[$cableObjId])) {
            $isys_cmdb_dao_smview_cable = isys_cmdb_dao_smview_cable::instance($this->m_db);
            $this->_cables[$cableObjId] = $isys_cmdb_dao_smview_cable->get_data($cableObjId);
        }
        $cable_data = $this->_cables[$cableObjId];
        foreach ($this->m_specialcolumn as $specialColumn) {
            $id = $specialColumn["id"];
            switch ($id) {
                case "C__CATG__SMVIEW_CABLE_WIRE-description":
                    $content_return[$id] = $specialContent["description"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-color":
                    $content_return[$id] = $specialContent["color"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-scope":
                    $content_return[$id] = $specialContent["scope"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-scope_index":
                    $content_return[$id] = $specialContent["scope_index"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-damping":
                    $content_return[$id] = $specialContent["damping"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-notice":
                    $content_return[$id] = $specialContent["notice"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-length":
                    $content_return[$id] = $cable_data[0]["length"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-capacity":
                    $content_return[$id] = $cable_data[0]["capacity"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-cable_name":
                    $content_return[$id] = $this->getCableName($specialContent["obj_id"]);
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-cable_id":
                    $content_return[$id] = $cableObjId;
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-cable_patched_wires":
                    $content_return[$id] = (string)$cable_data[0]["patched_wires"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-cable_free_wires":
                    $content_return[$id] = (string)$cable_data[0]["free_wires"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-cable_capacity":
                    $cable_free_wires = $cable_data[0]["free_wires"];
                    $cable_patched_wires = $cable_data[0]["patched_wires"];
                    $content_return[$id] = $this->_calculateCableCapacity($cable_free_wires, $cable_patched_wires);
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-reserved":
                    if ($specialContent["reserved"] != "") {
                        if ($specialContent["reserved"] == 0) {
                            $content_return[$id] = [
                                [
                                    "__id__" => 0,
                                    "value"  => $language->get("LC__UNIVERSAL__NO")
                                ]
                            ];
                        } elseif ($specialContent["reserved"] == 1) {
                            $content_return[$id] = [
                                [
                                    "__id__" => 1,
                                    "value"  => $language->get("LC__UNIVERSAL__YES")
                                ]
                            ];
                        }
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-user_state_id":
                    if ($specialContent["user_state_id"] != "") {
                        $dao = new isys_cmdb_dao_smview_cable_user_state($this->m_db);
                        $group = $dao->get_object("id", $specialContent["user_state_id"]);
                        $content_return[$id] = [[
                            "__id__" => $group["id"],
                            "value" => $group["name"]]];
                        break;
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-user_state":
                    $content_return[$id] = (string)$specialContent["user_state"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-connection_type_id":
                    if ($specialContent["connection_type_id"] != "") {
                        $dao = new isys_cmdb_dao_smview_cable_connection_type($this->m_db);
                        $group = $dao->get_object("id", $specialContent["connection_type_id"]);
                        $content_return[$id] = [[
                            "__id__" => $group["id"],
                            "value" => $group["name"]]];
                        break;
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-description_incoming":
                    if ($specialContent["description_incoming"] != "") {
                        $content_return[$id] = $specialContent["description_incoming"];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-description_outgoing":
                    if ($specialContent["description_outgoing"] != "") {
                        $content_return[$id] = $specialContent["description_outgoing"];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-pin_out":
                    if ($specialContent["pin_out_description"] != "") {
                        $content_return[$id] = $specialContent["pin_out_description"];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-pin_in":
                    if ($specialContent["pin_in_description"] != "") {
                        $content_return[$id] = $specialContent["pin_in_description"];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-scope_in":
                    if ($specialContent["pin_in_scope"] != "") {
                        $content_return[$id] = $specialContent["pin_in_scope"];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-scope_out":
                    if ($specialContent["pin_out_scope"] != "") {
                        $content_return[$id] = $specialContent["pin_out_scope"];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-object_out":
                    if ($specialContent["pin_out_object"] != "") {
                        $content_return[$id] =  [[
                            "__id__" => $specialContent["pin_out_object_id"],
                            "value" => $specialContent["pin_out_object"]]];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-object_in":
                    if ($specialContent["pin_in_object"] != "") {
                        $content_return[$id] =  [[
                            "__id__" => $specialContent["pin_in_object_id"],
                            "value" => $specialContent["pin_in_object"]
                        ]];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-purpose_out":
                    if ($specialContent["pin_out_purpose"] != "") {
                        $content_return[$id] = $specialContent["pin_out_purpose"];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-purpose_in":
                    if ($specialContent["pin_in_purpose"] != "") {
                        $content_return[$id] = $specialContent["pin_in_purpose"];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-state_in":
                    if ($specialContent["state_in"] != "") {
                        $states = isys_cmdb_dao_smview_cable_wire_pin::get_wire_states();
                        $status_id = array_search($specialContent["state_in"], $states);
                        $content_return[$id] = [[
                            "__id__" => $status_id,
                            "value"  => $specialContent["state_in"]]];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-state_out":
                    if ($specialContent["state_out"] != "") {
                        $states = isys_cmdb_dao_smview_cable_wire_pin::get_wire_states();
                        $status_id = array_search($specialContent["state_out"], $states);
                        $content_return[$id] = [[
                            "__id__" => $status_id,
                            "value"  => $specialContent["state_out"]]];
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-in_use":
                    if ($specialContent["in_use"] != "") {
                        if ($specialContent["in_use"] == $language->get("LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IS_DISPOSED")) {
                            $content_return[$id] = [
                                [
                                    "__id__" => 1,
                                    "value"  => $specialContent["in_use"]
                                ]
                            ];
                        } elseif ($specialContent["in_use"] == $language->get("LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IS_WIRED")) {
                            $content_return[$id] = [
                                [
                                    "__id__" => 0,
                                    "value"  => $specialContent["in_use"],
                                ]
                            ];
                        }
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-section_id":
                    if ($specialContent["section_id"] != "") {
                        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
                        $group = $dao->get_object_by_id($specialContent["section_id"])->get_row();
                        $content_return[$id] = [[
                            "__id__" => $group["isys_obj__id"],
                            "value"  => $group["isys_obj__title"]]];
                        break;
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-route_id":
                    if ($specialContent["route_id"] != "") {
                        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
                        $group = $dao->get_object_by_id($specialContent["route_id"])->get_row();
                        $content_return[$id] = [[
                            "__id__" => $group["isys_obj__id"],
                            "value"  => $group["isys_obj__title"]]];
                        break;
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-virtual_route_id":
                    if ($specialContent["virtual_route_id"] != "") {
                        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
                        $group = $dao->get_object_by_id($specialContent["virtual_route_id"])->get_row();
                        $content_return[$id] = [[
                            "__id__" => $group["isys_obj__id"],
                            "value"  => $group["isys_obj__title"]]];
                        break;
                    } else {
                        $content_return[$id] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-extended_location":
                    $content_return[$id] = $specialContent["extended_location"];
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-connection_count":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'connection_count');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-current_connection_count":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'current_connection_count');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-permission":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'permission');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-location":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'location');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-room":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'room');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-device_type":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'device_type');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-device_number":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'device_number');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-phone_number":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'phone_number');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-phone_subscriber":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'phone_subscriber');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-hint":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'hint');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-route_length":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'length');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-measured_length":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'measured_length');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-measure_date":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'measure_date');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-tka_participant":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'tka_participant');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-damping_left1":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'damping_left1');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-damping_left2":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'damping_left2');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-damping_right1":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'damping_right1');
                    break;
                case "C__CATG__SMVIEW_CABLE_WIRE-damping_right2":
                    $content_return[$id] = $this->_getRouteValue($specialContent, 'damping_right2');
                    break;
                default:
                    $content_return[$id] = "";
                    break;
            }
        }
        return $content_return;
    }

    /**
     * Get cable name by object ID.
     *
     * @param  int $objId
     * @return mixed|string
     * @throws isys_exception_database
     */
    private function getCableName($objId)
    {
        if (isset($this->m_cable_name[$objId])) {
            return $this->m_cable_name[$objId];
        }

        $sql = "SELECT isys_obj.isys_obj__title FROM `isys_obj` WHERE isys_obj.isys_obj__id = " . $objId;
        $isys_component_dao = new isys_component_dao($this->m_db);
        $l_row = $isys_component_dao->retrieve($sql)->get_row();
        if (! empty($l_row)) {
            $this->m_cable_name[$objId] = $l_row["isys_obj__title"];
            return $l_row["isys_obj__title"];
        }
        return "";
    }

    /**
     * Get section element column.
     *
     * @param  array $specialContent
     * @return array
     */
    private function _getSectionElementColumn($specialContent)
    {
        $content_return = [];
        foreach ($this->m_specialcolumn as $specialColumn) {
            switch ($specialColumn["id"]) {
                case "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-obj_in_title":
                    $content_return[$specialColumn["id"]][$specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-obj_in_id"]] = [
                        "__id__" => $specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-obj_in_id"],
                        "value"  => (string)$specialContent[$specialColumn["id"]]
                    ];
                    break;
                case "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-pin_in_title":
                    $content_return[$specialColumn["id"]][$specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-pin_in_id"]] = [
                        "__id__" => $specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-pin_in_id"],
                        "value"  => (string)$specialContent[$specialColumn["id"]]
                    ];
                    break;
                case "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-obj_out_title":
                    $content_return[$specialColumn["id"]][$specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-obj_out_id"]] = [
                        "__id__" => $specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-obj_out_id"],
                        "value"  => (string)$specialContent[$specialColumn["id"]]
                    ];
                    break;
                case "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-pin_out_title":
                    $content_return[$specialColumn["id"]][$specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-pin_out_id"]] = [
                        "__id__" => $specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-pin_out_id"],
                        "value"  => (string)$specialContent[$specialColumn["id"]]
                    ];
                    break;
                case "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-cable_title":
                    $content_return[$specialColumn["id"]][$specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-cable_id"]] = [
                        "__id__" => $specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-cable_id"],
                        "value"  => (string)$specialContent[$specialColumn["id"]]
                    ];
                    break;
                case "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-cable_capacity":
                    $cable_free_wires = $specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-cable_free_wires"];
                    $cable_patched_wires = $specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-cable_patched_wires"];
                    $content_return[$specialColumn["id"]][$specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-cable_id"]] = [
                        "__id__" => $specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-cable_id"],
                        "value"  => $this->_calculateCableCapacity($cable_free_wires, $cable_patched_wires)
                    ];
                    break;
                case "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-pin_in_state":
                    $const_value = $specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-wire_pin_in_state"];
                    $dao = new isys_cmdb_dao_smview_cable_wire_pin_status($this->m_db);
                    $group = $dao->get_object("id", $const_value);
                    $name = ! empty($group) ? $group["const"] : '';
                    if ($const_value != "") {
                        $content_return[$specialColumn["id"]][$specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-wire_pin_in_state"]] = [
                            "__id__" => $const_value,
                            "value"  => $name
                        ];
                    } else {
                        $content_return[$specialColumn["id"]] = "";
                    }
                    break;
                case "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-pin_out_state":
                    $const_value = $specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-wire_pin_out_state"];
                    $dao = new isys_cmdb_dao_smview_cable_wire_pin_status($this->m_db);
                    $group = $dao->get_object("id", $const_value);
                    $name = ! empty($group) ? $group["const"] : '';
                    if ($const_value != "") {
                        $content_return[$specialColumn["id"]][$specialContent["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT-wire_pin_out_state"]] = [
                            "__id__" => $const_value,
                            "value"  => $name
                        ];
                    } else {
                        $content_return[$specialColumn["id"]] = "";
                    }
                    break;
                default:
                    $content_return[$specialColumn["id"]] = (string)$specialContent[$specialColumn["id"]];
                    break;
            }
        }
        return $content_return;
    }

    /**
     * Get route element column.
     *
     * @param  array $specialContent
     * @param  string $prefix
     * @return array
     * @throws isys_exception_database
     */
    private function _getRouteElementColumn($specialContent, $prefix = 'C__CATG__SMVIEW_CABLE_ROUTE_ELEMENT')
    {
        $content_return = [];
        foreach ($this->m_specialcolumn as $specialColumn) {
            $split = explode("-", $specialColumn["id"]);
            switch ($split[1]) {
                case "obj_in_title":
                    $content_return[$specialColumn["id"]][$specialContent["$prefix-obj_in_id"]] = [
                        "__id__" => $specialContent["$prefix-obj_in_id"],
                        "value"  => (string)$specialContent[$specialColumn["id"]]
                    ];
                    break;
                case "pin_in_title":
                    $content_return[$specialColumn["id"]][$specialContent["$prefix-pin_in_id"]] = [
                        "__id__" => $specialContent["$prefix-pin_in_id"],
                        "value"  => (string)$specialContent[$specialColumn["id"]]
                    ];
                    break;
                case "obj_out_title":
                    $content_return[$specialColumn["id"]][$specialContent["$prefix-obj_out_id"]] = [
                        "__id__" => $specialContent["$prefix-obj_out_id"],
                        "value"  => (string)$specialContent[$specialColumn["id"]]
                    ];
                    break;
                case "pin_out_title":
                    $content_return[$specialColumn["id"]][$specialContent["$prefix-pin_out_id"]] = [
                        "__id__" => $specialContent["$prefix-pin_out_id"],
                        "value"  => (string)$specialContent[$specialColumn["id"]]
                    ];
                    break;
                case "cable_title":
                    $content_return[$specialColumn["id"]][$specialContent["$prefix-cable_id"]] = [
                        "__id__" => $specialContent["$prefix-cable_id"],
                        "value"  => (string)$specialContent[$specialColumn["id"]]
                    ];
                    break;
                case "section_title":
                    if ($specialContent["$prefix-section_id"] != null) {
                        $content_return[$specialColumn["id"]][$specialContent["$prefix-section_id"]] = [
                            "__id__" => $specialContent["$prefix-section_id"],
                            "value"  => (string)$specialContent[$specialColumn["id"]]
                        ];
                    } else {
                        $content_return[$specialColumn["id"]] = "";
                    }
                    break;
                case "cable_capacity":
                    $cable_free_wires = $specialContent["$prefix-cable_free_wires"];
                    $cable_patched_wires = $specialContent["$prefix-cable_patched_wires"];
                    $content_return[$specialColumn["id"]][$specialContent["$prefix-cable_id"]] = [
                        "__id__" => $specialContent["$prefix-cable_id"],
                        "value"  => $this->_calculateCableCapacity($cable_free_wires, $cable_patched_wires)
                    ];
                    break;
                case "pin_in_state":
                    if ($specialContent["$prefix-section_id"] != null) {
                        $sectionId = $specialContent["$prefix-section_id"];
                        $endpoints = SectionTracing::get_section_endpoints($sectionId);
                        $const_value = $endpoints['wire_pin_in_state'];
                    } else {
                        $const_value = $specialContent["$prefix-wire_pin_in_state"];
                    }

                    $dao = new isys_cmdb_dao_smview_cable_wire_pin_status($this->m_db);
                    $group = $dao->get_object("id", $const_value);
                    $name = ! empty($group) ? $group["const"] : '';
                    if ($const_value != "") {
                        $content_return[$specialColumn["id"]][$specialContent["$prefix-wire_pin_in_state"]] = [
                            "__id__" => $const_value,
                            "value"  => $name
                        ];
                    } else {
                        $content_return[$specialColumn["id"]] = "";
                    }
                    break;
                case "pin_out_state":
                    if ($specialContent["$prefix-section_id"] != null) {
                        $sectionId = $specialContent["$prefix-section_id"];
                        $endpoints = SectionTracing::get_section_endpoints($sectionId);
                        $const_value = $endpoints['wire_pin_out_state'];
                    } else {
                        $const_value = $specialContent["$prefix-wire_pin_out_state"];
                    }

                    $dao = new isys_cmdb_dao_smview_cable_wire_pin_status($this->m_db);
                    $group = $dao->get_object("id", $const_value);
                    $name = ! empty($group) ? $group["const"] : '';
                    if ($const_value != "") {
                        $content_return[$specialColumn["id"]][$specialContent["$prefix-wire_pin_out_state"]] = [
                            "__id__" => $const_value,
                            "value"  => $name
                        ];
                    } else {
                        $content_return[$specialColumn["id"]] = "";
                    }
                    break;
                default:
                    $content_return[$specialColumn["id"]] = (string)$specialContent[$specialColumn["id"]];
                    break;
            }
        }
        return $content_return;
    }

    /**
     * Get value of a route property.
     *
     * @param  array $specialContent
     * @param  string $propertyName
     * @return mixed|string
     * @throws isys_exception_database
     */
    private function _getRouteValue($specialContent, $propertyName)
    {
        if ($specialContent["route_id"] != "") {
            $dao = new isys_cmdb_dao_smview_cable_route($this->m_db);
            $route = $dao->get_object('obj_id', $specialContent["route_id"]);
            return $route[$propertyName];
        }
        return "";
    }

    /**
     * Prepare virtual fields.
     *
     * @param  array $return
     * @param  idoit\Module\Feed\ProxyApi\AbstractApi $api
     * @param  array $t_specialContent
     * @param  array $global_return
     * @param  string $key
     * @return array
     */
    private function _prepareVirtualFields($return, $api, $t_specialContent, $global_return, $key)
    {
        try {
            $fields = $api->get_all_fields();
        } catch (isys_exception_database | isys_exception_api $e) {
            $fields = [];
        }

        foreach ($t_specialContent as $jira) {
            $content_return = [];
            foreach ($fields as $f) {
                // prefix with category name
                $k = $this->m_category . "-" . $f['id'];
                $content_return[$k] = $jira[$k];
            }

            $content = array_replace($global_return, $content_return);
            $issueKey = $jira[$this->m_category . '-' . $key];
            $content["__id__"] = $issueKey;
            $content["__index__"] = "0";
            $return[$issueKey] = $content;
        }
        return $return;
    }

    /**
     * Get property by key.
     *
     * @param  int|string $key
     * @return mixed|null
     */
    private function getProperty($key)
    {
        if (is_numeric($key)) {
            foreach ($this->m_properties as $m_property) {
                if ($m_property["id"] == $key) {
                    return $m_property;
                }
            }
        } else {
            return $this->m_properties[$key];
        }
        return null;
    }

    /**
     * Get special value.
     *
     * @param  int $p_obj_id
     * @return array|mixed
     * @throws isys_exception_database
     * @throws Exception
     */
    private function getSpecialValue($p_obj_id)
    {
        $return = [];
        if (class_exists("isys_cmdb_dao_smview_cable_group") == true) {
            if ($this->m_category == "C__CATG__SMVIEW_CABLE") {
                $isys_cmdb_dao_smview_cable = isys_cmdb_dao_smview_cable::instance($this->m_db);
                $cable_data = $isys_cmdb_dao_smview_cable->get_data($p_obj_id);
                return isset($cable_data[0]) ? $cable_data[0] : [];
            }

            if ($this->m_category == "C__CATG__SMVIEW_CABLE_PIN") {
                $isys_cmdb_dao_smview_cable_pin = isys_cmdb_dao_smview_cable_pin::instance($this->m_db);
                return $isys_cmdb_dao_smview_cable_pin->get_listing_by_obj_id($p_obj_id);
            }

            if ($this->m_category == "C__CATG__SMVIEW_CABLE_SECTION") {
                $dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
                $section = $dao->get_object("obj_id", $p_obj_id);

                $t_array = [];
                foreach ($section as $key => $item) {
                    $t_array["C__CATG__SMVIEW_CABLE_SECTION-" . $key] = $item;
                }
                return $t_array;
            }

            if ($this->m_category == "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT") {
                $dao = new isys_cmdb_dao_category_g_smview_cable_section_element($this->m_db);
                // @todo: adding capacity information may decrease performance!!!!
                $section_elements = $dao->get_elements_by_obj_id($p_obj_id, false, false, true);
                return $this->_prefix_element_keys($section_elements);
            }

            if ($this->m_category == "C__CATG__SMVIEW_CABLE_ROUTE_ELEMENT") {
                $dao = new isys_cmdb_dao_category_g_smview_cable_route_element($this->m_db);
                $route_elements = $dao->get_elements_by_obj_id($p_obj_id, false, false);
                return $this->_prefix_element_keys($route_elements);
            }

            if ($this->m_category == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE_ELEMENT") {
                $dao = new isys_cmdb_dao_category_g_smview_cable_virtual_route_element($this->m_db);
                $route_elements = $dao->get_elements_by_obj_id($p_obj_id, false, false);
                return $this->_prefix_element_keys($route_elements);
            }

            if ($this->m_category == "C__CATG__SMVIEW_CABLE_ROUTE") {
                $dao = new isys_cmdb_dao_smview_cable_route($this->m_db);
                $route = $dao->get_object("obj_id", $p_obj_id);

                $t_array = [];
                foreach ($route as $key => $item) {
                    $t_array["C__CATG__SMVIEW_CABLE_ROUTE-" . $key] = $item;
                }
                return $t_array;
            }

            if ($this->m_category == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE") {
                $dao = new isys_cmdb_dao_smview_cable_virtual_route($this->m_db);
                $route = $dao->get_object("obj_id", $p_obj_id);

                $t_array = [];
                foreach ($route as $key => $item) {
                    $t_array["C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-" . $key] = $item;
                }
                return $t_array;
            }

            $cableWireDao = new isys_cmdb_dao_category_g_smview_cable_wire($this->m_db);
            $isys_cmdb_dao_smview_cable_wire = isys_cmdb_dao_smview_cable_wire::instance($this->m_db);
            if ($this->m_category == "C__CATG__SMVIEW_CABLE_WIRE") {
                if ($p_obj_id != null) {
                    // Auflistung aller verfuegbaren Adern eines Kabels (fuer Uebersicht der Netzverbindungen)
                    $wires = $isys_cmdb_dao_smview_cable_wire->get_data($p_obj_id);
                    $cableInfo = CableTracing::get_capacity_by_cable_id($p_obj_id);

                    $array = [];
                    foreach ($wires as $wire) {
                        $pin = $cableWireDao->get_additional_wire_information_by_id($wire["id"]);
                        $wire["cable_free_wires"] = $cableInfo["free_wires"];
                        $wire["cable_patched_wires"] = $cableInfo["patched_wires"];
                        $array[] = array_replace($wire, $pin);
                    }
                    return $array;
                } else {
                    // Auflistung aller Adern von (bzw. bis) zu einem gewaehlten Objekt,
                    // je nach Filter werden alle oder nur nicht-gepatchte Adern angezeigt
                    $object_out = null;
                    $object_in = null;
                    $unused_wires = true;
                    $array = [];
                    foreach ($this->m_Condition as $condition_key => $condition) {
                        if ($condition["property"] == "C__CATG__SMVIEW_CABLE_WIRE-object_out" &&
                            $condition["comparison"] == "id") {
                            //get the Out-Object
                            $object_out = $condition["value"];
                            unset($this->m_Condition[$condition_key]);
                        }

                        if ($condition["property"] == "C__CATG__SMVIEW_CABLE_WIRE-object_in" &&
                            $condition["comparison"] == "id") {
                            //get the Out-Object
                            $object_in = $condition["value"];
                            unset($this->m_Condition[$condition_key]);
                        }

                        if ($condition["property"] == "C__CATG__SMVIEW_CABLE_WIRE-in_use" &&
                            ($condition["value"] == "0" || $condition["value"] == "beschaltet" || $condition["value"] == "wired")) {
                            $unused_wires = false;
                        }
                    }

                    if ($object_out != null && $object_in != null) {
                        $CableTracing = new CableTracing($object_in);
                        if ($unused_wires) {
                            $_objects = $CableTracing->get_unpatched_connections();
                            $objectWires = $_objects["objects"];
                            $objOutKey = "obj_out_id";
                            $objInKey = "obj_in_id";
                        } else {
                            $objectWires = $CableTracing->get_connections();
                            $objOutKey = "obj_id_out";
                            $objInKey = "obj_id_in";
                        }

                        foreach ($objectWires as $f_wire) {
                            if (($f_wire[$objOutKey] == $object_out && $f_wire[$objInKey] == $object_in) ||
                                ($f_wire[$objOutKey] == $object_in && $f_wire[$objInKey] == $object_out)) {
                                $w = $this->_filter_wire_objects($cableWireDao, $f_wire);
                                if (! empty($w)) {
                                    $array[] = $w;
                                }
                            }
                        }
                        // attention: do not reduce elements by using $this->m_display_rows (this will be done later in code)
                        return $array;
                    }

                    if ($object_out == null && $object_in == null) {
                        // get all wires for route-view
                        $wires = $isys_cmdb_dao_smview_cable_wire->get_data();

                        $i = 0;
                        foreach ($wires as $wire) {
                            $w = $this->_filter_wire_objects($cableWireDao, $wire);
                            if (! empty($w)) {
                                $array[] = $w;
                            }
                            if ($i > $this->m_display_rows) {
                                break;
                            }
                            $i++;
                        }
                        return $array;
                    }

                    $wires = [];
                    if ($object_out != null) {
                        $CableTracing = new CableTracing($object_out);
                        if ($unused_wires) {
                            $_objects = $CableTracing->get_unpatched_connections();
                            $wires = $_objects["objects"];
                        } else {
                            $wires = $CableTracing->get_connections();
                        }
                    }

                    if ($object_in != null) {
                        $CableTracing = new CableTracing($object_in);
                        if ($unused_wires) {
                            $_objects = $CableTracing->get_unpatched_connections();
                            $wires = $_objects["objects"];
                        } else {
                            $wires = $CableTracing->get_connections();
                        }
                    }

                    foreach ($wires as $wire) {
                        $w = $this->_filter_wire_objects($cableWireDao, $wire);
                        if (! empty($w)) {
                            $array[] = $w;
                        }
                    }
                    return $array;
                }
            }
        }

        if (class_exists("isys_cmdb_dao_category_g_smview_feed_jira")) {
            if ($this->m_category == "C__CATG__SMVIEW_FEED_JIRA") {
                $feed_dao = isys_cmdb_dao_category_g_smview_feed_jira::instance($this->m_db);
                return $this->_prefix_virtual_element_keys($feed_dao, $p_obj_id, 'isys_key');
            }
        }

        if (class_exists("isys_cmdb_dao_category_g_smview_feed_virtual_device")) {
            if ($this->m_category == "C__CATG__SMVIEW_FEED_VIRTUAL_DEVICE") {
                $feed_dao = isys_cmdb_dao_category_g_smview_feed_virtual_device::instance($this->m_db);
                return $this->_prefix_virtual_element_keys($feed_dao, $p_obj_id, 'isys_mac_address');
            }
        }
        return $return;
    }

    /**
     * Filter wire objects.
     *
     * Heute mal in deutsch:
     * Diese Methode filtert alle Ader-Objekte vor um ausgeschlossene Objekte zu entfernen.
     * Dies ist noetig, da die allgemeine Filter-Funktion alle Werte anzeigt, die auf mindestens ein Filter-Kriterium zutreffen.
     * Damit werden ggf. ausgeschlossene Objekte dann doch wieder angezeigt.
     *
     * @param isys_cmdb_dao_category_g_smview_cable_wire $cableWireDao
     * @param  array                                     $wire
     *
     * @return array|null
     * @throws Exception
     */
    private function _filter_wire_objects($cableWireDao, $wire)
    {
        $pin = $cableWireDao->get_additional_wire_information_by_id($wire["wire_id"]);
        $w = array_replace($wire, $pin);

        foreach ($this->m_Condition as $condition) {
            if ($condition["property"] == "C__CATG__SMVIEW_CABLE_WIRE-object_in" &&
                $condition["comparison"] == "!=") {
                if ($w['obj_in_title'] == $condition["value"]) {
                    return null;
                }
            }
            if ($condition["property"] == "C__CATG__SMVIEW_CABLE_WIRE-object_out" &&
                $condition["comparison"] == "!=") {
                if ($w['obj_out_title'] == $condition["value"]) {
                    return null;
                }
            }
            if ($condition["property"] == "C__CATG__SMVIEW_CABLE_WIRE-cable_name" &&
                $condition["comparison"] == "!=") {
                if ($w['cable_title'] == $condition["value"]) {
                    return null;
                }
            }
            if ($condition["property"] == "C__CATG__SMVIEW_CABLE_WIRE-virtual_route_id" &&
                $condition["comparison"] == "=" &&
                $condition["value"] == "") {
                // do not display wires used in virtual routes
                if (! empty($w['virtual_route_id'])) {
                    return null;
                }
            }
        }
        return $w;
    }
    
    /**
     * Prefix element keys with category ID.
     *
     * @param array $elements
     * @return array
     */
    private function _prefix_element_keys($elements)
    {
        $return = [];
        foreach ($elements as $element) {
            $prefixed_elements = [];
            foreach ($element as $key => $value) {
                $id = $this->m_category . '-' . $key;
                $prefixed_elements[$id] = $value;
            }
            $return[$element["id"]] = $prefixed_elements;
        }
        return $return;
    }

    /**
     * Prefix virtual elements with category ID.
     *
     * @param  isys_cmdb_dao_category $dao
     * @param  int $obj_id
     * @param  string $key
     * @return array
     */
    private function _prefix_virtual_element_keys($dao, $obj_id, $key)
    {
        try {
            $rawElements = $dao->get_data_as_array(null, $obj_id);
        } catch (Exception $e) {
            $rawElements = [];
        }

        $elements = [];
        foreach ($rawElements as $props) {
            $prefixed_props = [];
            foreach ($props as $k => $v) {
                // replace isys_ by category prefix
                $k = str_replace('isys_', $this->m_category . "-", $k);
                $prefixed_props[$k] = $v;
            }
            $issueKey = $props[$key];
            $elements[$issueKey] = $prefixed_props;
        }
        return $elements;
    }

    /**
     * Specify special columns by its name.
     *
     * @param  string $column
     * @return boolean
     */
    private function isSpecialColumn($column)
    {
        $parts = explode('-', $column);
        $special_catg = [
            "C__CATG__SMVIEW_CABLE",
            "C__CATG__SMVIEW_CABLE_PIN",
            "C__CATG__SMVIEW_CABLE_WIRE",
            "C__CATG__SMVIEW_CABLE_SECTION",
            "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT",
            "C__CATG__SMVIEW_CABLE_ROUTE",
            "C__CATG__SMVIEW_CABLE_ROUTE_ELEMENT",
            "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE",
            "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE_ELEMENT",
            "C__CATG__LIVESTATUS"
        ];

        if (in_array($parts[0], $special_catg)) {
            return true;
        }

        $special_columns = [
            "C__SHD__LIVESTATUS-output",
            "C__CATG__IT_SERVICE-connected_object",
            "C__CATG__IP-fqdn",
            "C__CATG__IP-primary_hostaddress",
            "C__CATG__LOCATION-_location_path",
            "C__CATG__LOCATION-location_path",
            "C__CATG__VIRTUAL_MACHINE-primary",
        ];
        return in_array($column, $special_columns);
    }

    /**
     * Check if a condition matches.
     *
     * @param  array $row
     * @return boolean
     */
    private function isConditionTrue($row)
    {
        $result = [];
        if ($this->m_Condition === null || $this->m_Condition === []) {
            //no Conditions so everything is true
            return true;
        }

        foreach ($this->m_Condition as $f_condition) {
            $condition_lower_case = isset($f_condition["value"]) ? mb_strtolower($f_condition["value"], 'UTF-8') : null;
            $property = $f_condition["property"];
            $split = explode("-", $property);
            if (is_string($row[$property])) {
                $value = $row[$property];
            } elseif ($split[1] == "id") {
                $value = $row["__id__"];
            } else {
                $value = $row[$property];
            }

            switch ($f_condition["comparison"]) {
                case "LIKE":
                    //kein wert also raus
                    if ($value === null || isset($value[""]) ) {
                        $result[$property][] = false;
                        break;
                    }
                    //replace % to .*
                    $preg_match = "/(^" . str_replace("%", ".*", $condition_lower_case) . ")/";
                    preg_match($preg_match, mb_strtolower($value, 'UTF-8'), $matches);

                    //check for more then 1 Object.
                    if (count($matches) == 0) {
                        $result[$property][] = false;
                    } else {
                        //alle anderen Werte löschen und true zurück liefern
                        $result[$property] = [];
                        $result[$property][] = true;
                        break;
                    }
                    break;

                case "LIKE %...%":
                    //kein wert also raus
                    if ($value === null) {
                        $result[$property][] = false;
                        break;
                    }

                    if (isset($value["__id__"]) && isset($value["value"])) {
                        //object value
                        $val = mb_strtolower($value, 'UTF-8');
                        if (strpos($val, $condition_lower_case) === false) {
                            $result[$property][] = false;
                        } else {
                            $result[$property][] = true;
                        }
                    } elseif (is_array($value)) {
                        foreach ($value as $item) {
                            $val = mb_strtolower($item["value"], 'UTF-8');
                            if (strpos($val, $condition_lower_case) === false) {
                                $result[$property][] = false;
                            } else {
                                $result[$property] = [];
                                $result[$property][] = true;
                                break;
                            }
                        }
                    } elseif (strpos(mb_strtolower($value, 'UTF-8'), $condition_lower_case) === false) {
                        //simple Value prüfen ob der Wert im Value vorhanden ist.
                        $result[$property][] = false;
                    } else {
                        //alle anderen Werte löschen und true zurück liefern
                        $result[$property] = [];
                        $result[$property][] = true;
                        break;
                    }
                    break;
                case "=":
                    //abfrage nach der ID
                    $propertyValue = $this->m_properties[$property];
                    if ($propertyValue["multi"] == "1" || isset($propertyValue["dialog"]) || is_array($value)) {
                        //if empty get out
                        if (($value === null || $value === "" ) && $f_condition["value"] == "-1") {
                            $result[$property][] = true;
                            break;
                        }

                        if (isset($value["__id__"]) && isset($value["value"])) {
                            $result[$property][] = $value["__id__"] == $f_condition["value"];
                        } elseif (is_array($value)) {
                            $result[$property][] = false;
                            foreach ($value as $item) {
                                //check with Integer for the __id__
                                if (is_numeric($f_condition["value"])) {
                                    if ($item["__id__"] == $f_condition["value"]) {
                                        $result[$property] = [];
                                        $result[$property][] = true;
                                    }
                                } else {
                                    $val = mb_strtolower($item["value"], 'UTF-8');
                                    if ($val == $condition_lower_case) {
                                        $result[$property] = [];
                                        $result[$property][] = true;
                                    }
                                }
                            }
                        } else {
                            //normal string value
                            $val = mb_strtolower($value, 'UTF-8');
                            if ($val == $condition_lower_case) {
                                $result[$property][] = true;
                            } else {
                                $result[$property] = [];
                                $result[$property][] = false;
                                break;
                            }
                        }
                    } else {
                        if ($value === null) {
                            $result[$property][] = false;
                            break;
                        }

                        $val = mb_strtolower($value, 'UTF-8');
                        if ($val == $condition_lower_case) {
                            $result[$property][] = true;
                        } else {
                            $result[$property] = [];
                            $result[$property][] = false;
                            break;
                        }
                    }
                    break;

                case "!=":
                    //abfrage nach der ID
                    $propertyValue = $this->m_properties[$property];
                    if ($propertyValue["multi"] == "1" || isset($propertyValue["dialog"]) || is_array($value)) {
                        //if empty get out
                        if (($value === null || $value === "" ) && $f_condition["value"] == "-1") {
                            $result[$property][] = true;
                            break;
                        }

                        if (isset($value["__id__"]) && isset($value["value"])) {
                            $result[$property][] = $value["__id__"] == $f_condition["value"];
                        } elseif (is_array($value)) {
                            foreach ($value as $item) {
                                //check with Integer for the __id__
                                if (is_numeric($f_condition["value"])) {
                                    if ($item["__id__"] == $f_condition["value"]) {
                                        $result[$property][] = false;
                                    } else {
                                        $result[$property] = [];
                                        $result[$property][] = true;
                                        break;
                                    }
                                } else {
                                    $val = mb_strtolower($item["value"], 'UTF-8');
                                    if ($val == $condition_lower_case) {
                                        $result[$property][] = false;
                                    } else {
                                        $result[$property] = [];
                                        $result[$property][] = true;
                                        break;
                                    }
                                }
                            }
                        } else {
                            //normal string value
                            $val = mb_strtolower($value, 'UTF-8');
                            if ($val == $condition_lower_case) {
                                $result[$property][] = false;
                            } else {
                                $result[$property] = [];
                                $result[$property][] = true;
                                break;
                            }
                        }
                    } else {
                        if ($value === null) {
                            $result[$property][] = false;
                            break;
                        }
                        $val = mb_strtolower($value, 'UTF-8');
                        if ($val == $condition_lower_case) {
                            $result[$property] = [];
                            $result[$property][] = false;
                            break;
                        } else {
                            $result[$property][] = true;
                        }
                    }
                    break;

                case "NOT LIKE":
                    //added from SM-view filter
                    $condition_value  = str_replace("%", "", $f_condition["value"]);
                    $condition_value  = str_replace("_", "", $condition_value);

                    //prüfen ob der Wert im Value vorhanden ist.
                    if (strpos(mb_strtolower($value, 'UTF-8'), mb_strtolower($condition_value, 'UTF-8')) === false) {
                        $result[$property][] = true;
                    } else {
                        //alle anderen Werte löschen und true zurück liefern
                        $result[$property] = [];
                        $result[$property][] = false;
                    }
                    break;

                case "IS NULL":
                    $result[$property][] = $value === null || $value === "";
                    break;

                case "IS NOT NULL":
                    if (is_array($value)) {
                        foreach ($value as $item) {
                            $result[$property][] = ! ($item["value"] === null || $item["value"] === "");
                        }                        
                    } else {
                        $result[$property][] = ! ($value === null || $value === "");
                    }
                    break;

                case ">":
                    $result[$property][] = count($value) > $f_condition["value"];
                    break;

                case "<":
                    $result[$property][] = count($value) < $f_condition["value"];
                    break;

                case "id":
                    //kein wert also raus
                    if ($value === null) {
                        $result[$property][] = false;
                        break;
                    }

                    if (isset($value["__id__"]) && isset($value["value"])) {
                        //object value
                        $result[$property][] = $value["__id__"] == $f_condition["value"];
                    } elseif (is_array($value)) {
                        foreach ($value as $item) {
                            if ($item["__id__"] == $f_condition["value"]) {
                                $result[$property] = [];
                                $result[$property][] = true;
                            } else {
                                $result[$property][] = false;
                                break;
                            }
                        }
                    } else {
                        if ($value == $f_condition["value"]) {
                            $result[$property] = [];
                            $result[$property][] = true;
                        } else {
                            $result[$property][] = false;
                        }
                        break;
                    }
                    break;

                case "!id":
                    //kein wert also raus
                    if ($value === null) {
                        $result[$property][] = false;
                        break;
                    }
                    if (isset($value["__id__"]) && isset($value["value"])) {
                        //object value
                        $result[$property][] = $value["__id__"] != $f_condition["value"];
                    } elseif (is_array($value)) {
                        foreach ($value as $item) {
                            if ($item["__id__"] != $f_condition["value"]) {
                                $check_other_conditions = true;
                                foreach ($result as $a_column) {
                                    if ($a_column[0] == false) {
                                        $check_other_conditions = false;
                                    }
                                }
                                $result[$property][] = $check_other_conditions == true;
                            } else {
                                $result[$property] = [];
                                $result[$property][] = false;
                            }
                        }
                    } else {
                        if ($value != $f_condition["value"]) {
                            $check_other_conditions = true;
                            foreach ($result as $a_column) {
                                if ($a_column[0] == false) {
                                    $check_other_conditions = false;
                                }
                            }
                            $result[$property][] = $check_other_conditions == true;
                        } else {
                            $result[$property] = [];
                            $result[$property][] = false;
                        }
                    }
                    break;
            }
        }

        $return = $result != [] ? true : null;
        foreach ($result as $f_item) {
            $condition = false;
            foreach ($f_item as $bool) {
                $condition = $condition || $bool;
            }
            $return = $return && $condition;
        }
        return $return == null ? false : $return;
    }

    /**
     * Sort columns.
     *
     * @param  array $content
     * @return array
     */
    private function sortColumns($content)
    {
        $sortColumnId = $this->m_SortColumn && $this->m_SortColumn["id"] ? $this->m_SortColumn["id"] : null;
        if (! $sortColumnId) {
            // return unsorted
            return $content;
        }

        $temp_content = [];
        // separate sortable column
        foreach ($content as $item_key => $item) {
            if (isset($item[$sortColumnId])) {
                $element = $item[$sortColumnId];
                $temp_content[$item_key] = $element;
            }
        }

        if ($this->m_SortColumn["sort"] == "ASC") {
            uasort($temp_content, function ($a, $b) {
                $a_item = null;
                $b_item = null;

                if (is_string($a)) {
                    $a_item = $a == "" ? "zzzzzzzzz" : $a;
                } else {
                    //is Array and get it from value
                    foreach ($a as $item) {
                        $a_item = $item["value"];
                    }
                    if ($a_item == null) {
                        $a_item = "zzzzzzzzz";
                    }
                }

                if (is_string($b)) {
                    $b_item = $b == "" ? "zzzzzzzzz" : $b;
                } else {
                    //is Array and get it from value
                    foreach ($b as $item) {
                        $b_item = $item["value"];
                    }
                    if ($b_item == null) {
                        $b_item = "zzzzzzzzz";
                    }
                }
                return strnatcasecmp($a_item, $b_item);
            });
        } else {
            uasort($temp_content, function ($a, $b) {
                $a_item = null;
                $b_item = null;

                if (is_string($a)) {
                    $a_item = $a == "" ? "000000000" : $a;
                } else {
                    //is Array and get it from value
                    foreach ($a as $item) {
                        $a_item = $item["value"];
                    }
                    if ($a_item == null) {
                        $a_item = "000000000";
                    }
                }

                if (is_string($b)) {
                    $b_item = $b == "" ? "000000000" : $b;
                } else {
                    //is Array and get it from value
                    foreach ($b as $item) {
                        $b_item = $item["value"];
                    }
                    if ($b_item == null) {
                        $b_item = "000000000";
                    }
                }
                return strnatcasecmp($b_item, $a_item);
            });
        }

        $count = 0;
        $sortedContent = [];
        foreach (array_keys($temp_content) as $temp_key) {
            if (isset($content[$temp_key])) {
                // important: add items in same order as in sorted array!!!!
                $sortedContent[$temp_key] = $content[$temp_key];
                $sortedContent[$temp_key]["__index__"] = $count++;
            }
        }
        return $sortedContent;
    }

    /**
     * get_data always retrieves the data of the main table of this module
     */
    public function get_data()
    {
        // TODO: Implement get_data() method.
    }
}
