<?php

use idoit\Module\Api\Exception\ValidationException;

/**
 * i-doit - Documentation and CMDB solution for IT environments
 *
 * This file is part of the i-doit framework. Modify at your own risk.
 *
 * Please visit http://www.i-doit.com/license for a full copyright and license information.
 *
 * @version     1.8.3
 * @package     i-doit
 * @author      synetics GmbH
 * @copyright   synetics GmbH
 * @url         http://www.i-doit.com
 * @license     http://www.i-doit.com/license
 */

class smview_cmdb_dao_queryfast extends isys_module_dao
{
    protected static $m_instance = null;

    protected $m_db;

    protected $m_sortColumn = [];

    protected $m_specialSortColumn = [];

    protected $m_Column = [];

    protected $m_Condition = [];

    protected $m_sqlColumn = [];

    protected $m_sqlCondition = [];

    protected $m_specialColumn = [];

    protected $m_specialCondition = [];

    protected $m_object_type_id;

    protected $m_obj_title;

    protected $m_table = [];

    /**
     * The caching instance for monitoring rows.
     *
     * @var  isys_cache_memcached
     */
    protected static $m_cache = null;

    /**
     * Logger.
     *
     * @var isys_log
     */
    protected $logger = null;
    
    /**
     * Database component.
     *
     * @var isys_component_database
     */
    public function __construct(isys_component_database $p_db)
    {
        global $g_absdir;
        $this->logger = isys_log::get_instance('shd-smview')
            ->set_log_file($g_absdir . '/log/shd-smview_' . date('Y-m-d') . '.log');
        
        $properties = smview_cmdb_dao_properties::instance(isys_application::instance()->database)->getProperties(false);
        //change the property-const to id
        $tproperties = [];
        foreach ($properties as $property) {
            $tproperties[$property['const'] . '-' . $property['key']] = $property;
        }

        $this->m_properties = $tproperties;
        parent::__construct($p_db);
    }

    /**
     * @param array $p_params
     * @return array|int
     */
    public function read($p_params)
    {
        $l_start_time = microtime(true);
        $isys_api_model_shd_userright = new isys_api_model_shd_userright();
        $language = isys_application::instance()->container->get('language');

        $page = isset($p_params["page"]) ? (int) $p_params["page"] : 0;
        $rows_per_page = isset($p_params["display_rows"]) ? (int) $p_params["display_rows"] : null;

        // consider status as integer or constant
        $status = C__RECORD_STATUS__NORMAL;
        if (isset($p_params["status"])) {
            if (is_int($p_params["status"])) {
                $status = $p_params["status"];
            } else {
                $status = defined_or_default($p_params["status"], C__RECORD_STATUS__NORMAL);
            }
        }

        $l_filter = [
            "shd_filter__status"            => $status,
            "shd_filter__display_rows"      => $rows_per_page,
            "shd_filter__page"              => $page,
            "shd_filter__display_relations" => "0",
            "shd_filter__empty_values"      => "1",
            "shd_filter__columns"           => $p_params["columns"],
        ];

        $this->m_Column = $p_params["columns"];

        $this->m_Condition = $p_params["condition"];

        $this->m_object_type_id = $p_params["obj_type_id"];

        if (isset($p_params["obj_type_id"]) == true) {
            $this->m_Condition[] = ["category" => "C__CATG__GLOBAL", "property" => "C__CATG__GLOBAL-type", "comparison" =>"=", "value" => $p_params["obj_type_id"]];
        }

        //Query for not mounted objects
        if (isset($p_params["notMounted"]) && $p_params["notMounted"] == 1) {
            $this->m_Column[] =  ["id"=>"C__CATG__LOCATION-pos", "width" =>"50", "sort" => ""];
            $this->m_Condition[] = ["category" => "C__CATG__LOCATION", "property" => "C__CATG__LOCATION-pos", "comparison" =>"=", "value" => ""];
        }

        //sort of columns-Key
        $t_filter_column = [];
        foreach ($this->m_Column as $filterKey => $shd_filter__column) {
            $this->m_Column[$filterKey]['const'] = $this->m_Column[$filterKey]['id'];

            $prop = $this->getProperty($shd_filter__column['id']);
            $this->m_Column[$filterKey]['id'] = $prop['id'];
            $this->m_Column[$filterKey]['i18n_title'] = $prop["title"];

            if ($this->isSpecialColumn($shd_filter__column['id']) == true) {
                $this->m_specialColumn[] = $this->m_Column[$filterKey];
                if ($shd_filter__column["sort"] != "") {
                    $this->m_specialSortColumn['sorting_default'] = $prop['id'];
                    $this->m_specialSortColumn['const'] = $shd_filter__column['id'];
                    $this->m_specialSortColumn['sorting_direction'] = $shd_filter__column['sort'];
                }
            } else {
                $this->m_sqlColumn[] = $this->m_Column[$filterKey];
                if ($shd_filter__column["sort"] != "") {
                    $this->m_sortColumn['sorting_default'] = $prop['id'];
                    $this->m_sortColumn['const'] = $shd_filter__column['id'];
                    $this->m_sortColumn['sorting_direction'] = $shd_filter__column['sort'];
                }
            }
        }
        ksort($t_filter_column);
        $l_filter['shd_filter__columns'] = $t_filter_column;

        //delete SHD-Conditions
        foreach ($this->m_Condition as $shd_filter__condition) {

                //find -3M +1Y
                preg_match_all('/[-|+]\d+[a-z|A-Z]/', $shd_filter__condition["value"], $matches, PREG_UNMATCHED_AS_NULL);

                if($shd_filter__condition["property"] == "C__CATG__GLOBAL-changed" || $shd_filter__condition["property"] == "C__CATG__GLOBAL-created") {

                    if (count($matches[0]) > 0) {
                        $dt = new DateTime();
                        foreach ($matches[0] as $match) {

                            if (strpos($match, '+') !== false) {
                                $time = str_replace("+", "", $match);
                                $dt->add(new DateInterval('P' . $time));
                            }

                            if (strpos($match, '-') !== false) {
                                $time = str_replace("-", "", $match);
                                $dt->sub(new DateInterval('P' . $time));
                            }

                        }
                        $shd_filter__condition["value"] = $dt->format('Y-m-d H:i:s');
                    }
                }
            if ($this->isSpecialColumn($shd_filter__condition['property']) == true) {
                $this->m_specialCondition[] = $shd_filter__condition;
            } else {
                $this->m_sqlCondition[] = $shd_filter__condition;
            }
        }

        $l_filter_ids = $l_filter;
        $l_filter_ids["shd_filter__columns"] =  $this->m_sqlColumn;
        $l_filter_ids["shd_filter__condition"] = $this->prepare_normal_condition($this->m_sqlCondition);
        if (isset($this->m_sortColumn) == true) {
            $l_filter_ids["sorting_default"] = $this->m_sortColumn['sorting_default'];
            $l_filter_ids["sorting_direction"] = $this->m_sortColumn['sorting_direction'];
        }

        //no limits
        $l_filter_ids['shd_filter__display_rows'] = null;

        $smview_cmdb_dao_filter_sql = new smview_cmdb_dao_query_sql($this->m_db);
        $query_filter_ids = $smview_cmdb_dao_filter_sql->create_property_query_for_filter($l_filter_ids, false, false);
        $view_filter = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->query_sql($query_filter_ids);

        //when SHD-condition isset then we get the Status
        if ($this->m_specialCondition != []) {
            // increase memory limit depending on filter object count
            if (count($view_filter) > 15000) {
                ini_set('memory_limit', '1024M');
            } elseif (count($view_filter) > 30000) {
                ini_set('memory_limit', '1500M');
            }

            foreach ($view_filter as $f_key => $f_id) {

                if (! $this->isConditionTrue($f_id["__id__"])) {
                    unset($view_filter[$f_key]);
                }
            }
        }

        //Spezial-Sortierung
        if ($this->m_specialSortColumn !== []) {
            $sort_array = [];
            $view_filter_hash = [];
            foreach ($view_filter as $f_id) {
                $obj_id = $f_id["__id__"];
                $sort_array[$obj_id] = $this->getSpecialValue($this->m_specialSortColumn["const"], $obj_id);
                $view_filter_hash[$obj_id] = $f_id;
            }

            //get new Sorting
            $sort_array = $this->sortSpecialColumns($sort_array, $this->m_specialSortColumn);

            $t_view_filter = [];
            foreach ($sort_array as $f_key => $sort_item) {
                $view_item = $view_filter_hash[$f_key];
                $view_item["__index__"] = $sort_item["__index__"];
                if (isset($sort_item["value"])) {
                    //Single-Value Element
                    $sort_item = $sort_item["value"];
                } else {
                    //Multi-Value Element
                    //löschen des Indexes
                    if (isset($sort_item["__index__"])) {
                        unset($sort_item["__index__"]);
                    }
                    if ($sort_item["value"] == null) {
                        unset($sort_item["value"]);
                    }
                }
                //um die restlichen Elemente einzufügen
                $view_item[$this->m_specialSortColumn["sorting_default"]] = $sort_item;
                $t_view_filter[] = $view_item;
            }
            $view_filter = $t_view_filter;
        }

        //new condition
        $deleteHeaders = [];
        $multi_array = [];
        $translations = [];
        $multiByFilterKey = [];
        $permissionByFilterKey = [];
        foreach ($view_filter as $filter_content) {
            foreach ($filter_content as $filter_key => $filter_cell) {
                //jump over __id__
                if ($filter_key != "__id__") {
                    if (!isset($permissionByFilterKey[$filter_key])) {
                        $property = $this->getProperty($filter_key);
                        $permissionByFilterKey[$filter_key] = $isys_api_model_shd_userright->hasUserCategoryRight($property["const"]);
                    }

                    if ($permissionByFilterKey[$filter_key] == false) {
                        if ($filter_cell != "") {
                            $filter_cell = "*****";
                            $filter_content[$filter_key] = "*****";
                        }
                    }
                } else {
                    //catch primary_key
                    $f_obj_id = $filter_cell;
                }

                if (isset($this->m_specialSortColumn["sorting_default"]) && $filter_key == $this->m_specialSortColumn["sorting_default"]) {
                    $multi_array[$filter_key][$f_obj_id] = $filter_cell;
                    continue;
                }

                if (!empty($filter_cell)) {
                    if (!isset($translations[$filter_cell])) {
                        $translations[$filter_cell] = $language->get($filter_cell);
                    }
                    $translated_cell = $translations[$filter_cell];
                } else {
                    $translated_cell = $filter_cell;
                }

                if (!isset($multiByFilterKey[$filter_key])) {
                    $property = $this->getProperty($filter_key);
                    $multiByFilterKey[$filter_key] = $property["multi"] == "1";
                }

                //Wenn die Property eine Multi ist dann soll es zusammen gepackt werden
                if ($multiByFilterKey[$filter_key]) {
                    $multi_obj_id = $filter_content[$filter_key . "___id"];
                    if ($multi_obj_id == null) {
                        if ($translated_cell !== null) {
                            $multi_array[$filter_key][$f_obj_id][$translated_cell] = array("__id__" => 0, "value" => $translated_cell);
                        }
                    } else {
                        $multi_array[$filter_key][$f_obj_id][$multi_obj_id] = array("__id__" => $multi_obj_id, "value" => $translated_cell);
                    }
                }

                if (isset($filter_content[$filter_key . "___id"])) {
                    $id = $filter_content[$filter_key . "___id"];
                    $value = $filter_content[$filter_key];
                    $multi_array[$filter_key][$f_obj_id][$id] = array("__id__" => $id, "value" => $value);
                }

                if (strpos($filter_key, '___id') !== false) {
                    $deleteHeaders[$filter_key] = $filter_key;
                    unset($filter_content[$filter_key]);
                }

                //Translate the Table Content
                $filter_content[$filter_key] = $translated_cell;
            }

            //delete Multivalue id table
            foreach ($deleteHeaders as $f_deleteHeader) {
                unset($filter_content[$f_deleteHeader]);
            }

            //Wenn Parent noch nicht gesetzt ist dann setzen.
            if (isset($tempView[$f_obj_id]) == false) {
                $tempView[$f_obj_id] = $filter_content;
            }
        }

        foreach ($multi_array as $f_propert_key => $multi_element) {
            foreach ($multi_element as $f_primary_obj_id => $property_element) {
                if (is_array($property_element)) {
                    $index = 1;
                    $new_property_element = [];
                    foreach ($property_element as $k => $v) {
                        if (! is_int($k)) {
                            // replace non-int keys with an index
                            $k = "$index";
                            $index++;
                        }
                        $new_property_element[$k] = $v;
                    }
                    $property_element = $new_property_element;
                }
                $tempView[$f_primary_obj_id][$f_propert_key] = $property_element;
            }
        }

        // get count of filtered objects by status
        $requested_status = isset($l_filter_ids['shd_filter__status']) ? $l_filter_ids['shd_filter__status'] : C__RECORD_STATUS__NORMAL;
        $count_normal = $this->_getFilteredObjectCountByStatus($requested_status, 'C__RECORD_STATUS__NORMAL', $query_filter_ids);
        $count_archived = $this->_getFilteredObjectCountByStatus($requested_status, 'C__RECORD_STATUS__ARCHIVED', $query_filter_ids);
        $count_deleted = $this->_getFilteredObjectCountByStatus($requested_status, 'C__RECORD_STATUS__DELETED', $query_filter_ids);

        //#########Filter count Object
        if ($p_params["count_objects"]) {
            return [
                "count"          => count($tempView),
                "count_normal"   => $count_normal,
                "count_archived" => $count_archived,
                "count_deleted"  => $count_deleted,
            ];
        }

        //set Index
        $tempTable = [];
        $i = 0;
        foreach ($tempView as $item) {
            $item["__index__"] = "$i";
            $i++;
            $tempTable[$i] = $item;
        }

        $tempView = $tempTable;
        if ($rows_per_page === null || ($page * $rows_per_page) > count($tempView)) {
            // return all elements
            foreach ($tempView as $item) {
                $view_elements[$item["__id__"]] = $item;
            }
        } else {
            for ($i = $page * $rows_per_page + 1; $i <= $page * $rows_per_page + $rows_per_page; $i++) {
                if (isset($tempTable[$i]) == true) {
                    $view_elements[$tempTable[$i]["__id__"]] = $tempTable[$i];
                }
            }
        }

        //rewrite the id to const
        $temp_view_elements = [];
        foreach ($view_elements as $f_row_key => $f_row) {
            $obj_id = $f_row["__id__"];
            foreach ($f_row as $row_key => $value) {
                if ($this->getProperty($row_key) != null) {
                    $t_propery =$this->getProperty($row_key);
                    $temp_view_elements[$obj_id][$t_propery["const"] . "-" . $t_propery["key"]] = $value;
                } else {
                    //basecolumns like __index__, __id__
                    $temp_view_elements[$obj_id][$row_key] = $value;
                }
            }

            foreach ($this->m_specialColumn as $extra_column) {
                //überspringe die extra Column diese wird oben schon berechnet und gesetzt
                if ($extra_column["id"] != $this->m_specialSortColumn["sorting_default"]) {
                    $SpecialValue = $this->getSpecialValue($extra_column["const"], $obj_id, true);
                    if (isset($SpecialValue["value"])) {
                        //Single-Value Element
                        $SpecialValue = $SpecialValue["value"];
                    }
                    $temp_view_elements[$obj_id][$extra_column["const"]] = $SpecialValue;
                } else {
                    //reset of return Value for C__SHD__LIVESTATUS
                    if (strpos($this->m_specialSortColumn["const"], "C__SHD__LIVESTATUS") !== false) {
                        $temp_view_elements[$obj_id][$extra_column["const"]] = null;
                    }
                }
            }
        }

        $temp_view_elements = $this->check_post_data($temp_view_elements);

        $array_header = [];
        $count = 0;
        foreach ($this->m_Column as $f_column) {
            $array_header[$count] = [
                "const" => $f_column["const"],
                "title" => $language->get($f_column["i18n_title"])
            ];
            $count++;
        }

        return [
            "headers"        => $array_header,
            "num"            => count($tempView),
            "count"          => count($tempTable),
            "count_normal"   => $count_normal,
            "count_archived" => $count_archived,
            "count_deleted"  => $count_deleted,
            "content"        => $temp_view_elements,
            "page"           => $page,
            "display_rows"   => $p_params["display_rows"],
            "time"           => (microtime(true) - $l_start_time),
        ];
    }

    /**
     * Get count of filtered objects by specified status.
     *
     * @param int $requested_status
     * @param string $new_status
     * @param string $query_filter_ids
     * @return int
     */
    private function _getFilteredObjectCountByStatus($requested_status, $new_status, $query_filter_ids)
    {
        // check if status is in query
        $new_status = defined_or_default($new_status, -1);
        $status_query = 'obj_main.isys_obj__status =';
        if (strpos($query_filter_ids, $status_query) === false) {
            // add status condition
            $search = "WHERE TRUE";
            $replace = "WHERE TRUE AND $status_query '$new_status'";
        } else {
            // replace existing condition
            $search = "$status_query '$requested_status'";
            $replace = "$status_query '$new_status'";
        }

        $query_ids = str_replace($search, $replace, $query_filter_ids);
        $view_filter = smview_cmdb_dao_filter::instance(isys_application::instance()->database)->query_sql($query_ids);
        return count($view_filter);
    }

    /**
     * @param array $p_params
     */
    public function inspector($p_params)
    {
    }

    public function relation($p_params)
    {
        //todo du relation in new style
        return 0;
    }

    /**
     * @param  $const
     * @param  $value
     * @param  bool $isReturnValue
     * @return array|bool|false|float|mixed|string|null
     * @throws ValidationException
     * @throws isys_exception_api
     * @throws isys_exception_auth
     * @throws isys_exception_cache
     * @throws isys_exception_database
     */
    private function getSpecialValue($const, $value, $isReturnValue = false)
    {
        $split_a = explode("-", $const);
        $catConstant = $split_a[0];
        $propertyName = $split_a[1];
        switch ($const) {
            case "C__SHD__LIVESTATUS-status":
            case "C__SHD__LIVESTATUS-output":
            case "C__SHD__LIVESTATUS-service_status":
            case "C__SHD__SERVICESTATUS-output":
                if ($isReturnValue == true) {
                    return null;
                }

                $status_dao = smview_cmdb_dao_status::instance($this->get_database_component());
                $livestatus = $status_dao->get_livestatus($value);
                return isset($livestatus) ? $livestatus : $status_dao->getGreyStatus();

            case "C__CATG__CPU-service_status":
            case "C__CATG__CPU-host_name":
            case "C__CATG__CPU-output":
            case "C__CATG__CPU-description":


                $m_status = smview_cmdb_dao_status::instance($this->get_database_component())->getStatus($value, true);

                return $m_status;

            case "C__SHD__SERVICESTATUS-status":
                $bsm_api_model_config = new bsm_api_model_config();
                $param = ["service_id" => $value];
                $l_Service_status = $bsm_api_model_config->status($param);

                if ($l_Service_status["status"] != "-1") {
                    if ($l_Service_status["status"] == 0) {
                        $l_Service_status["color"] = C__SHD_COLOR__GREEN;
                    } elseif ($l_Service_status["status"] == 1) {
                        $l_Service_status["color"] = C__SHD_COLOR__YELLOW;
                    } elseif ($l_Service_status["status"] == 2) {
                        $l_Service_status["color"] = C__SHD_COLOR__RED;
                    }
                }
                return $l_Service_status;

            case "C__CATG__GLOBAL-type_group_title":

                $m_dao = new isys_api_model_shd_object_types(isys_application::instance()->database);
                $m_object_types = $m_dao->read([]);


                $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao($this->m_db));
                $p_param = [
                    'objID'    => $value,
                    'category' => "C__CATG__GLOBAL"
                ];

                $l_categoryDatas = $l_api_model_category->read($p_param);

                foreach ($m_object_types as $m_object_type) {

                    if($m_object_type["isys_obj_type__id"] == $l_categoryDatas[0]["type"]["id"]) {
                        return $m_object_type["isys_obj_type_group__title"];
                    }
                }

                return "";
            case "C__CATG__GLOBAL-group_image":

                $m_dao = new isys_api_model_shd_object_types(isys_application::instance()->database);

                $m_object_types = $m_dao->read([]);

                $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao($this->m_db));
                $p_param = [
                    'objID'    => $value,
                    'category' => "C__CATG__GLOBAL"
                ];

                $l_categoryDatas = $l_api_model_category->read($p_param);

                foreach ($m_object_types as $m_object_type) {

                    if($m_object_type["isys_obj_type__id"] == $l_categoryDatas[0]["type"]["id"]) {
                        return $m_object_type["isys_obj_type__obj_img_name"];
                    }
                }

                return "";

            case "C__CATG__GLOBAL-tag":

                $l_multiValue = [];

                $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao($this->m_db));
                $p_param = [
                    'objID'    => $value,
                    'category' => "C__CATG__GLOBAL"
                ];

                $l_categoryDatas = $l_api_model_category->read($p_param);


                foreach ($l_categoryDatas[0]["tag"] as $f_tag) {

                    $l_multiValue[] = ["__id__" => $f_tag["id"], "value" => $f_tag["title"]];
                }
                return $l_multiValue !== [] ? $l_multiValue : null;

            case "C__CATG__IT_SERVICE-connected_object":
                if (is_numeric($value)) {
                    $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao($this->m_db));
                    $p_param = [
                        'objID'    => $value,
                        'category' => "C__CATG__IT_SERVICE"
                    ];

                    $l_categoryDatas = $l_api_model_category->read($p_param);

                    $l_multiValue = [];
                    foreach ($l_categoryDatas as $f_categoryData) {
                        $l_multiValue[] = ["__id__" => $f_categoryData["connected_object"]["id"], "value" => $f_categoryData["connected_object"]["title"]];
                    }
                    return $l_multiValue !== [] ? $l_multiValue : null;
                }
                return null;

            case "C__CATG__IP-fqdn":
                if (is_numeric($value)) {
                    $l_multiValue = [];
                    $l_categoryDatas = [];

                    foreach ($this->m_table["category_g_ip"] as $item) {
                        if ($item["isys_obj__id"] == $value) {
                            $l_categoryDatas[] = $item;
                        }
                    }

                    foreach ($l_categoryDatas as $f_categoryData) {
                        if ($f_categoryData["isys_catg_ip_list__hostname"] != "" || $f_categoryData["isys_catg_ip_list__domain"] != "") {
                            $l_multiValue[] = [
                                "__id__" => $f_categoryData["isys_catg_ip_list__id"],
                                "value"  => $f_categoryData["isys_catg_ip_list__hostname"] . "." . $f_categoryData["isys_catg_ip_list__domain"]
                            ];
                        }
                    }
                    return $l_multiValue !== [] ? $l_multiValue : null;
                }
                return null;

            case "C__CATG__IP-primary_hostaddress":
                foreach ($this->m_table["category_g_ip"] as $item) {
                    if ($item["isys_obj__id"] == $value) {
                        if ($item["isys_catg_ip_list__primary"] == "1" && $item["isys_cats_net_ip_addresses_list__title"] != "") {
                            return $item["isys_cats_net_ip_addresses_list__title"];
                        }
                    }
                }
                return null;

            case "C__CATG__GUEST_SYSTEMS-connected_object":
                if (is_numeric($value)) {
                    $l_multiValue = [];
                    $l_categoryDatas = [];

                    foreach ($this->m_table["catg_guest_systems"] as $item) {
                        if ($item["isys_connection__isys_obj__id"] == $value) {
                            $l_categoryDatas[] = $item;
                        }
                    }

                    foreach ($l_categoryDatas as $f_categoryData) {
                        $l_multiValue[] = [
                            "__id__" => intval($f_categoryData["isys_obj__id"]),
                            "value"  => $f_categoryData["isys_obj__title"]
                        ];
                    }
                    return $l_multiValue !== [] ? $l_multiValue : null;
                }
                return null;

            case "C__CATG__LOCATION-_location_path":
                $smview_cmdb_dao_location = new smview_cmdb_dao_location($this->m_db);
                return $smview_cmdb_dao_location->get_shd_path($value);

            case "C__CATG__LOCATION-longitude":
                $l_row = isys_cmdb_dao_category_g_location::instance($this->m_db)
                    ->get_data(null, $value)
                    ->get_row();
                return $l_row["longitude"];

            case "C__CATG__LOCATION-latitude":
                $l_row = isys_cmdb_dao_category_g_location::instance($this->m_db)
                    ->get_data(null, $value)
                    ->get_row();
                return $l_row["latitude"];

            case "C__CATG__LOCATION-location_path":
                $smview_cmdb_dao_location = new smview_cmdb_dao_location($this->m_db);
                return $smview_cmdb_dao_location->get_shd_path_to_string($value);

            case "C__CATG__LOCATION-slot":
                //get Parent Chassie
                $l_row = isys_cmdb_dao_category_g_location::instance($this->m_db)
                    ->get_data(null, $value)
                    ->get_row();

                $l_device_dao = isys_cmdb_dao_category_s_chassis::instance($this->m_db);
                $l_slot_dao = isys_cmdb_dao_category_s_chassis_slot::instance($this->m_db);

                $l_slot_res = $l_slot_dao->get_data(null, $l_row["isys_catg_location_list__parentid"], '', null, C__RECORD_STATUS__NORMAL);
                if (count($l_slot_res)) {
                    while ($l_slot_row = $l_slot_res->get_row()) {
                        $l_items = $l_slot_dao->get_assigned_chassis_items_by_cat_id($l_slot_row['isys_cats_chassis_slot_list__id']);

                        foreach ($l_items as $l_key => $l_item) {
                            $l_item_object = $l_device_dao->get_assigned_device_title_by_cat_id($l_item['isys_cats_chassis_list__id'], 'raw');

                            if ($l_item_object['isys_obj__id'] == $value) {
                                return $l_slot_row['isys_cats_chassis_slot_list__title'];
                            }

                            $l_slot_row['items'][$l_key] = $l_item_object;
                        }
                    }
                }
                return "";

            case "C__CATG__LOCATION-slot_nr":
                return "";

            case "C__CATG__LOCATION-pos":
                $l_row = isys_cmdb_dao_category_g_location::instance($this->m_db)
                    ->get_data(null, $value)
                    ->get_row();

                if ($l_row["isys_catg_location_list__pos"] == "" || $l_row["isys_catg_location_list__pos"] == "0") {
                    return "";
                }
                $l_rack = isys_cmdb_dao_category_s_enclosure::instance($this->get_database_component())
                    ->get_data(null, $l_row["isys_catg_location_list__parentid"])
                    ->get_row();

                if ($l_rack["isys_cats_enclosure_list__slot_sorting"] == "desc") {
                    $l_formfactor = isys_cmdb_dao_category_g_formfactor::instance($this->get_database_component())
                        ->get_data(null, $l_row["isys_catg_location_list__parentid"])
                        ->get_row();

                    if ($l_formfactor["isys_catg_formfactor_list__rackunits"] == "" || $l_formfactor["isys_catg_formfactor_list__rackunits"] == "0") {
                        return "";
                    }

                    $l_new_pos = $l_formfactor["isys_catg_formfactor_list__rackunits"] - $l_row["isys_catg_location_list__pos"] + 1;
                    return (string)$l_new_pos;
                } else {
                    return (string)$l_row["isys_catg_location_list__pos"];
                }

            case "C__CATG__LOGICAL_UNIT-parent":
                $l_row = isys_cmdb_dao_category_g_logical_unit::instance($this->m_db)->get_data(null, $value)->get_row();
                if ($l_row) {
                    $dao = new isys_cmdb_dao($this->m_db);
                    $obj = $dao->get_object($l_row['isys_catg_logical_unit_list__isys_obj__id__parent'])->get_row();
                    $l_multiValue[] = [
                        "__id__" => intval($obj["isys_obj__id"]),
                        "value"  => $obj["isys_obj__title"]
                    ];
                    return $l_multiValue;
                }
                return null;

            case "C__CATG__VIRTUAL_MACHINE-primary":
                if (is_numeric($value)) {
                    $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao($this->m_db));
                    $p_param = [
                        'objID'    => $value,
                        'category' => "C__CATG__VIRTUAL_MACHINE"
                    ];

                    $l_categoryDatas = $l_api_model_category->read($p_param);
                    $l_multiValue = [];
                    foreach ($l_categoryDatas as $f_categoryData) {
                        if ($f_categoryData["primary"]["title"] != "") {
                            $l_multiValue[] = ["__id__" => $f_categoryData["primary"]["id"], "value" => $f_categoryData["primary"]["title"]];
                        }
                    }
                    return $l_multiValue !== [] ? $l_multiValue : null;
                }
                return null;

            case "C__CATG__CONTACT-primary":
            case "C__CATG__CONTACT-primary_contact":
                if (is_numeric($value)) {
                    foreach ($this->m_table["category_g_contact"] as $f_categoryData) {
                        if ($f_categoryData["objID"] == $value) {
                            if ($f_categoryData["primary"]["value"] == "1") {
                                return [
                                    $f_categoryData["contact_object"]["id"] => [
                                        "__id__" => $f_categoryData["contact_object"]["id"],
                                        "value"  => $f_categoryData["contact_object"]["title"]
                                    ]
                                ];
                            }
                        }
                    }
                }
                return null;

            case "C__CATG__CONTACT-role":
                if (is_numeric($value)) {
                    $l_multiValue = [];
                    $l_categoryDatas = [];

                    foreach ($this->m_table["category_g_contact"] as $item) {
                        if ($item["objID"] == $value) {
                            $l_categoryDatas[] = $item;
                        }
                    }

                    foreach ($l_categoryDatas as $f_categoryData) {
                        $l_multiValue[] = [
                            "__id__" => $f_categoryData["role"]["id"],
                            "value"  => $f_categoryData["role"]["title"],
                            "parent" => $f_categoryData["id"]
                        ];
                    }
                }
                return $l_multiValue;

            case "C__CATG__CONTACT-contact_object":
            case "C__CATG__CONTACT-contact_list":
            case "C__CATG__CONTACT-contact":
                if (is_numeric($value)) {
                    $l_multiValue = [];
                    $l_categoryDatas = [];

                    foreach ($this->m_table["category_g_contact"] as $item) {
                        if ($item["objID"] == $value) {
                            $l_categoryDatas[] = $item;
                        }
                    }

                    foreach ($l_categoryDatas as $f_categoryData) {
                        $l_multiValue[] = [
                            "__id__" => $f_categoryData["contact_object"]["id"],
                            "value"  => $f_categoryData["contact_object"]["title"],
                            "parent" => $f_categoryData["id"]
                        ];
                    }
                }
                return $l_multiValue;

            case "C__CATS__PERSON_CONTACT_ASSIGNMENT-object_type":
                $l_multiValue = [];
                $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao($this->m_db));
                $p_param = [
                    'objID'    => $value,
                    'category' => "C__CATS__ORGANIZATION_CONTACT_ASSIGNMENT"
                ];

                $l_categoryDatas = $l_api_model_category->read($p_param);
                foreach ($l_categoryDatas as $f_categoryData) {
                    $l_multiValue[] = [
                        "__id__" => $f_categoryData["object"]["id"],
                        "value"  => $f_categoryData["object"]["title"],
                        "parent" => $f_categoryData["id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATS__PERSON_CONTACT_ASSIGNMENT-role":
                $l_multiValue = [];
                $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao($this->m_db));
                $p_param = [
                    'objID'    => $value,
                    'category' => "C__CATS__ORGANIZATION_CONTACT_ASSIGNMENT"
                ];

                $l_categoryDatas = $l_api_model_category->read($p_param);
                foreach ($l_categoryDatas as $f_categoryData) {
                    $l_multiValue[] = [
                        "__id__" => $f_categoryData["role"]["id"],
                        "value"  => $f_categoryData["role"]["title"],
                        "parent" => $f_categoryData["id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATG__APPLICATION-application":
                foreach ($this->m_table["category_g_application"] as $item) {
                    if ($item["isys_catg_application_list__isys_obj__id"] == $value) {
                        $l_categoryDatas[] = $item;
                    }
                }
                foreach ($l_categoryDatas as $f_categoryData) {
                    $l_multiValue[] = [
                        "__id__" => $f_categoryData["isys_obj__id"],
                        "value"  => $f_categoryData["isys_obj__title"],
                        "parent" => $f_categoryData["isys_catg_application_list__id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATG__APPLICATION-application_type":
                foreach ($this->m_table["category_g_application"] as $item) {
                    if ($item["isys_catg_application_list__isys_obj__id"] == $value) {
                        $l_categoryDatas[] = $item;
                    }
                }
                foreach ($l_categoryDatas as $f_categoryData) {
                    $l_multiValue[] = [
                        "__id__" => $f_categoryData["isys_obj_type__id"],
                        "value"  => _L($f_categoryData["isys_obj_type__title"]),
                        "parent" => $f_categoryData["isys_catg_application_list__id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATG__APPLICATION-application_priority":
                foreach ($this->m_table["category_g_application"] as $item) {
                    if ($item["objID"] == $value) {
                        $l_categoryDatas[] = $item;
                    }
                }
                foreach ($l_categoryDatas as $f_categoryData) {
                    $l_multiValue[] = [
                        "__id__" => $f_categoryData["application_priority"]["id"],
                        "value"  => $f_categoryData["application_priority"]["title"],
                        "parent" => $f_categoryData["id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATG__APPLICATION-description":
                foreach ($this->m_table["category_g_application"] as $item) {
                    if ($item["objID"] == $value) {
                        $l_categoryDatas[] = $item;
                    }
                }
                foreach ($l_categoryDatas as $f_categoryData) {
                    $l_multiValue[] = ["__id__" => $f_categoryData["id"], "value" => $f_categoryData["description"]];
                }
                return $l_multiValue;

            case "C__CATG__APPLICATION-bequest_nagios_services":
                foreach ($this->m_table["category_g_application"] as $item) {
                    if ($item["objID"] == $value) {
                        $l_categoryDatas[] = $item;
                    }
                }
                foreach ($l_categoryDatas as $f_categoryData) {
                    $l_multiValue[] = ["__id__" => $f_categoryData["id"], "value" => $f_categoryData["bequest_nagios_services"]["title"]];
                }
                return $l_multiValue;

            case "C__CATS__APPLICATION-install_count":
                $l_daoApp_s = isys_cmdb_dao_category_s_application::instance($this->m_db);
                return (string)$l_daoApp_s->dynamic_property_callback_installation_count(["isys_cats_application_list__isys_obj__id" => $value]);

            case "C__CATS__PERSON_ASSIGNED_GROUPS-connected_object":
                $l_multiValue = [];
                $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao($this->m_db));
                $p_param = [
                    'objID'    => $value,
                    'category' => "C__CATS__PERSON_ASSIGNED_GROUPS"
                ];

                $l_categoryDatas = $l_api_model_category->read($p_param);
                foreach ($l_categoryDatas as $f_categoryData) {
                    $l_multiValue[] = [
                        "__id__" => $f_categoryData["connected_object"]["id"],
                        "value"  => $f_categoryData["connected_object"]["title"],
                        "parent" => $f_categoryData["id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATS__PERSON_GROUP_MEMBERS-connected_object":
                $l_multiValue = [];
                $l_api_model_category = new isys_api_model_cmdb_category(new isys_cmdb_dao($this->m_db));
                $p_param = [
                    'objID'    => $value,
                    'category' => "C__CATS__PERSON_GROUP_MEMBERS"
                ];

                $l_categoryDatas = $l_api_model_category->read($p_param);
                foreach ($l_categoryDatas as $f_categoryData) {
                    $l_multiValue[] = [
                        "__id__" => $f_categoryData["connected_object"]["id"],
                        "value"  => $f_categoryData["connected_object"]["title"],
                        "parent" => $f_categoryData["id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATG__MEMORY-total_capacity":
                $capacity = 0;
                foreach ($this->m_table["category_g_memory"] as $item) {
                    if ($item["isys_obj__id"] == $value) {
                        $l_categoryDatas[] = $item;
                    }
                }

                $isys_memory_unit__const = "";
                foreach ($l_categoryDatas as $f_categoryData) {
                    $capacity += $f_categoryData["isys_catg_memory_list__capacity"];
                    $l_multiValue[] = ["__id__" => $f_categoryData["isys_catg_memory_list__capacity"], "value" => $f_categoryData["bequest_nagios_services"]["title"]];
                    $isys_memory_unit__const = $f_categoryData["isys_memory_unit__const"];
                }

                if ($capacity !== 0) {
                    $return = isys_convert::memory($capacity, $isys_memory_unit__const, C__CONVERT_DIRECTION__BACKWARD);
                    return $return . " GB" . " (" . count($l_multiValue) .")";
                }
                return "";

            case "C__CATG__SMVIEW_CABLE-description_sort":
                $items = $this->m_table["category_cable_description_sort"];
                $item = isset($items[$value]) ? $items[$value] : null;
                $l_multiValue = [];
                if ($item) {
                    $l_multiValue[] = ["__id__" => $item["id"], "value" => $item["title"]];
                }
                return $l_multiValue;

            case "C__CATG__SMVIEW_REPORTING_REPORT-query":
            case "C__CATG__SMVIEW_REPORTING_REPORT-description":
            case "C__CATG__SMVIEW_REPORTING_REPORT-default_template":
                $l_api_model_category = new reporting_api_model_report();

                $l_categoryDatas = $l_api_model_category->read([]);
                foreach ($l_categoryDatas as $f_categoryData) {
                    if ($f_categoryData["obj_id"] == $value) {
                        switch ($propertyName) {
                            case "query":
                                return $f_categoryData["query"];

                            case "description":
                                return $f_categoryData["description"];

                            case "default_template":
                                return $f_categoryData["default_template"];
                        }
                    }
                }
                return "";

            case "C__CATG__SMVIEW_REPORTING_REPORT-object_types":
                $l_multiValue = [];
                $l_api_model_category = new reporting_api_model_report();

                $l_categoryDatas = $l_api_model_category->read([]);
                foreach ($l_categoryDatas as $f_categoryData) {
                    if ($f_categoryData["obj_id"] == $value) {
                        foreach ($f_categoryData["object_types"] as $key_object_types => $f_object_types) {
                            $l_multiValue[] = [
                                "__id__" => $f_object_types,
                                "value"  => $f_object_types,
                                "parent" => $value
                            ];
                        }
                    }
                }
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE-net_type":
            case "C__CATG__SMVIEW_CABLE-user_state":
            case "C__CATG__SMVIEW_CABLE-capacity":
            case "C__CATG__SMVIEW_CABLE-state":
            case "C__CATG__SMVIEW_CABLE-length":
                $cable_items = $this->m_table["category_g_cable"];
                if (isset($cable_items[$value])) {
                    $cable_item = $cable_items[$value];
                    $l_multiValue[] = [
                        "__id__" => $cable_item["id"],
                        "value"  => $cable_item[$propertyName],
                        "parent" => $cable_item["id"]
                    ];

                }
                return $l_multiValue;

            case "C__CATG__CABLE-length":
                $dao = new isys_cmdb_dao_category_g_cable($this->m_db);
                $row = $dao->get_data(null, $value)->get_row();
                $length = (int) $row['isys_catg_cable_list__length'];
                if ($row['isys_depth_unit__factor']) {
                    $factor = (int) $row['isys_depth_unit__factor'];
                    $length = $factor > 0 ? $length / $factor : $length;
                }
                $l_multiValue[] = [
                    "__id__" => $row['isys_catg_cable_list__id'],
                    "value"  => $length,
                    "parent" => $value
                ];
                return $l_multiValue;

            case "C__CATG__CABLE-connection":
                return $this->_getCableConnections($value);


            case "C__CATG__SMVIEW_CABLE-cable_free_wires":
                $cable_items = $this->m_table["category_g_cable"];
                if (isset($cable_items[$value])) {
                    $cable_item = $cable_items[$value];
                    return (string) $cable_item["free_wires"];
                }
                return "";

            case "C__CATG__SMVIEW_CABLE-cable_patched_wires":
                $cable_items = $this->m_table["category_g_cable"];
                if (isset($cable_items[$value])) {
                    $cable_item = $cable_items[$value];
                    return (string) $cable_item["patched_wires"];
                }
                return "";

            case "C__CATG__SMVIEW_CABLE-cable_capacity":
                $cable_items = $this->m_table["category_g_cable"];
                if (isset($cable_items[$value])) {
                    $item = $cable_items[$value];
                    $cable_free_wires = $item["free_wires"];
                    $cable_patched_wires = $item["patched_wires"];

                    $capacity = $cable_free_wires + $cable_patched_wires > 0 ? ($cable_patched_wires * 100) / ($cable_free_wires + $cable_patched_wires) : 0;
                    return (string)round($capacity);
                    }
                break;

            case "C__CATG__SMVIEW_CABLE-state_id":
                return $this->getDependingCableValue(
                    $value,
                    "category_g_cable_user_state",
                      "state_id",
                    "name",
                    "description");

            case "C__CATG__SMVIEW_CABLE-group_id":
                return $this->getDependingCableValue(
                    $value,
                    "category_g_cable_group",
                    "group_id",
                    "description");

            case "C__CATG__SMVIEW_CABLE-connection_type_id":
                return $this->getDependingCableValue(
                    $value,
                    "category_g_cable_connection_type",
                    "connection_type_id",
                    "name");

            case "C__CATG__SMVIEW_CABLE-admin_unit":
                return $this->getDependingCableValue(
                    $value,
                    "category_g_cable_admin_unit",
                    "admin_unit_id",
                    "description");

            case "C__CATG__SMVIEW_CABLE-type_id":
                return $this->getDependingCableValue(
                    $value,
                    "category_g_cable_type",
                    "type_id",
                    "description");

            case "C__CATG__SMVIEW_CABLE-is_section_cable":
                $l_multiValue = null;
                $cable_items = $this->m_table["category_g_cable"];
                if (isset($cable_items[$value])) {
                    $cable_item = $cable_items[$value];
                    $isSectionCable = $cable_item['is_section_cable'];

                    $id = -1;
                    $title = null;
                    if ($isSectionCable != null) {
                        $language = isys_application::instance()->container->get('language');
                        $id = $isSectionCable == 1 ? 1 : 0;
                        $title = $isSectionCable == 1 ? $language->get("LC__UNIVERSAL__YES"): $language->get("LC__UNIVERSAL__NO");
                    }
                    $l_multiValue[] = [
                        "__id__" => $id,
                        "value"  => $title,
                        "parent" => $value,
                    ];
                }
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE-location_in_path":
            case "C__CATG__SMVIEW_CABLE-location_out_path":
                $cable_items = $this->m_table["category_g_cable"];
                if (isset($cable_items[$value])) {
                    $cable_item = $cable_items[$value];
                    $smview_cmdb_dao_location = new smview_cmdb_dao_location($this->m_db);

                    $locationKey = $const == "C__CATG__SMVIEW_CABLE-location_in_path" ? "location_in_id" : "location_out_id";
                    $l_str_location = $smview_cmdb_dao_location->get_shd_path_to_string($cable_item[$locationKey], true);

                    $l_multiValue[] = [
                        "__id__" => $cable_item[$locationKey],
                        "value"  => $l_str_location,
                        "parent" => $cable_item["id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE-location_in":
            case "C__CATG__SMVIEW_CABLE-location_out":
                $cable_items = $this->m_table["category_g_cable"];
                if (isset($cable_items[$value])) {
                    $cable_item = $cable_items[$value];
                    $smview_cmdb_dao_location = new smview_cmdb_dao_location($this->m_db);

                    $locationKey = $const == "C__CATG__SMVIEW_CABLE-location_in" ? "location_in_id" : "location_out_id";
                    $f_item = $smview_cmdb_dao_location->get_parent_object_by_object($cable_item[$locationKey]);
                    $l_multiValue[] = [
                        "__id__" => $cable_item[$locationKey],
                        "value"  => $f_item["value"],
                        "parent" => $cable_item["id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE_WIRE-color":
                $l_multiValue = [];
                $l_api_model_category = isys_cmdb_dao_smview_cable_wire::instance($this->m_db);

                $l_categoryDatas = $l_api_model_category->get_data($value);
                foreach ($l_categoryDatas as $f_categoryData) {
                    if ($f_categoryData["color"]) {
                        $l_multiValue[] = [
                            "__id__" => $f_categoryData["id"],
                            "value"  => $f_categoryData["color"],
                            "parent" => $f_categoryData["id"]
                        ];
                    }
                }
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE_WIRE-state_in":
                return $this->_getWirePinStatus($value, 'wire_pin_state_in');

            case "C__CATG__SMVIEW_CABLE_WIRE-object_in":
                return $this->_getWirePin($value, 'obj_id_in', 'obj_title_in');

            case "C__CATG__SMVIEW_CABLE_WIRE-scope_in":
                return $this->_getWirePin($value, 'pin_id_in', 'pin_scope_in');

            case "C__CATG__SMVIEW_CABLE_WIRE-pin_in":
                return $this->_getWirePin($value, 'pin_id_in', 'pin_title_in');

            case "C__CATG__SMVIEW_CABLE_WIRE-description_incoming":
                return $this->_getWirePin($value, 'wire_id', 'wire_pin_title_in');

            case "C__CATG__SMVIEW_CABLE_WIRE-state_out":
                return $this->_getWirePinStatus($value, 'wire_pin_state_out');

            case "C__CATG__SMVIEW_CABLE_WIRE-object_out":
                return $this->_getWirePin($value, 'obj_id_out', 'obj_title_out');

            case "C__CATG__SMVIEW_CABLE_WIRE-scope_out":
                return $this->_getWirePin($value, 'pin_id_out', 'pin_scope_out');

            case "C__CATG__SMVIEW_CABLE_WIRE-pin_out":
                return $this->_getWirePin($value, 'pin_id_out', 'pin_title_out');

            case "C__CATG__SMVIEW_CABLE_WIRE-extended_location":
                $extendedLocationDao = new isys_cmdb_dao_category_g_smview_cable_extended_location($this->m_db);
                $location = $extendedLocationDao->get_data(null, $value)->get_row();
                $extendedLocationId = $location['isys_catg_smview_cable_extended_location_list__location_id'];
                if ($extendedLocationId) {
                    $sql = "SELECT * FROM isys_shd_extended_location WHERE isys_shd_extended_location__id = $extendedLocationId";
                    $l = $extendedLocationDao->retrieve($sql)->get_row();
                    return $l['isys_shd_extended_location__title'];
                }
                return '';

            case "C__CATG__BVG_RENTING-obj_id":
                $renting_dao = new isys_cmdb_dao_category_g_bvg_renting($this->m_db);
                $rentedRoutes = $renting_dao->get_data_as_array(null, $value);
                return $this->_getRentedRoutes($rentedRoutes, $value);

            case "C__CATG__BVG_ASSIGNED_RENTING-obj_id":
                $renting_dao = new isys_cmdb_dao_category_g_bvg_renting($this->m_db);
                $condition = $value ? " AND isys_catg_bvg_renting_list__obj_id = $value " : null;
                $rentedRoutes = $renting_dao->get_data_as_array(null, null, $condition);
                return $this->_getRentedRoutes($rentedRoutes, $value);

            case "C__CATG__BVG_RENTING-location_in":
                $renting_dao = new isys_cmdb_dao_category_g_bvg_renting($this->m_db);
                $route_dao = new isys_cmdb_dao_category_g_smview_cable_route(isys_application::instance()->database);

                $rentedRoutes = $renting_dao->get_data_as_array(null, $value);
                $l_multiValue = [];
                foreach ($rentedRoutes as $renting) {
                    $route_id = $renting['isys_catg_bvg_renting_list__obj_id'];
                    $route = $route_dao->get_data(null, $route_id)->get_row();
                    $start_obj_id = $route['isys_catg_smview_cable_route_list__start_obj_id'];
                    $l_multiValue[] = [
                        "__id__" => $value,
                        "value"  => $renting_dao->get_location_path($start_obj_id),
                        "parent" => $value
                    ];
                }
                return $l_multiValue;

            case "C__CATG__BVG_RENTING-location_out":
                $renting_dao = new isys_cmdb_dao_category_g_bvg_renting($this->m_db);
                $route_dao = new isys_cmdb_dao_category_g_smview_cable_route(isys_application::instance()->database);

                $rentedRoutes = $renting_dao->get_data_as_array(null, $value);
                $l_multiValue = [];
                foreach ($rentedRoutes as $renting) {
                    $route_id = $renting['isys_catg_bvg_renting_list__obj_id'];
                    $route = $route_dao->get_data(null, $route_id)->get_row();
                    $end_obj_id = $route['isys_catg_smview_cable_route_list__end_obj_id'];
                    $l_multiValue[] = [
                        "__id__" => $value,
                        "value" => $renting_dao->get_location_path($end_obj_id),
                        "parent" => $value
                    ];
                }
                return $l_multiValue;

            case "C__CATG__BVG_RENTING-damping":
                $renting_dao = new isys_cmdb_dao_category_g_bvg_renting($this->m_db);
                $rentedRoutes = $renting_dao->get_data_as_array(null, $value);
                $l_multiValue = [];
                foreach ($rentedRoutes as $renting) {
                    $route_id = $renting['isys_catg_bvg_renting_list__obj_id'];
                    $l_multiValue[] = [
                        "__id__" => $value,
                        "value" => $renting_dao->get_damping($route_id),
                        "parent" => $value
                    ];
                }
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE_WIRE-description_outgoing":
                return $this->_getWirePin($value, 'wire_id', 'wire_pin_title_out');

            case "C__CATG__SMVIEW_CABLE_WIRE-classification":
                $l_multiValue = [];
                $l_api_model_category = isys_cmdb_dao_smview_cable_wire::instance($this->m_db);

                $l_categoryDatas = $l_api_model_category->get_data($value);
                foreach ($l_categoryDatas as $f_categoryData) {
                    $l_multiValue[] = [
                        "__id__" => $f_categoryData["id"],
                        "value"  => $f_categoryData["classification"],
                        "parent" => $f_categoryData["id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE_WIRE-description":
            case "C__CATG__SMVIEW_CABLE_WIRE-description_outgoing":
            case "C__CATG__SMVIEW_CABLE_WIRE-description_incoming":
                $l_multiValue = [];
                $l_api_model_category = isys_cmdb_dao_smview_cable_wire::instance($this->m_db);

                $l_categoryDatas = $l_api_model_category->get_data($value);
                foreach ($l_categoryDatas as $f_categoryData) {
                    $l_multiValue[] = [
                        "__id__" => $f_categoryData["id"],
                        "value"  => $f_categoryData["description"],
                        "parent" => $f_categoryData["id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATG__SMVIEW_IMPORT_ELID-device_elid":
                $zit_cmdb_dao_elid = new zit_cmdb_dao_elid($this->m_db);
                $column_value = $zit_cmdb_dao_elid->getElid($value);
                return $column_value !== false ? $column_value : "";

            case "C__CATG__SMVIEW_CABLE_SECTION-user_state_id":
                // get section
                $sections = $this->m_table[$catConstant];
                $section = $sections[$value];

                // get usage
                $usage_dao = new isys_cmdb_dao_smview_cable_user_state($this->m_db);
                $usage = $usage_dao->get_object('id', $section["user_state_id"]);
                $value = ! empty($usage) ? $usage["name"] . ': ' . $usage["description"] : null;

                $l_multiValue[] = [
                    "__id__" => $section["user_state_id"],
                    "value"  => $value,
                    "parent" => $section["id"]
                ];
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE_SECTION-route_id":
                // get section
                $sections = $this->m_table[$catConstant];
                $section = $sections[$value];

                $route_element_dao = new isys_cmdb_dao_smview_cable_route_element(\isys_application::instance()->database);
                $route_element = $route_element_dao->get_object('section_id', $section['id']);
                $val = null;
                if (! empty($route_element)) {
                    $route_dao = new isys_cmdb_dao_smview_cable_route(\isys_application::instance()->database);
                    $element = $route_dao->get_object('id', $route_element['route_id']);
                    $global_dao = new isys_cmdb_dao_category_g_global(\isys_application::instance()->database);
                    $obj = $global_dao->get_object_by_id($element['obj_id'])->get_row();
                    $val = $obj['isys_obj__title'];
                }

                $l_multiValue[] = [
                    "__id__" => $value,
                    "value"  => $val,
                    "parent" => $value
                ];
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE_ROUTE-route_usage_id":
            case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-route_usage_id":
                // get route
                $routes = $this->m_table[$catConstant];
                $route = $routes[$value];

                // get usage
                $usage_dao = new isys_cmdb_dao_smview_cable_route_usage($this->m_db);
                $usage = $usage_dao->get_object('id', $route["route_usage_id"]);

                $title = $usage["name"];
                if (! empty($usage["technology"])) {
                    $title .= ' (' . $usage["technology"] . ')';
                }
                $l_multiValue[] = [
                    "__id__" => $route["route_usage_id"],
                    "value"  => $title,
                    "parent" => $route["id"]
                ];
                return $l_multiValue;
            case "C__CATG__SMVIEW_CABLE_ROUTE-is_external":
            case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-is_external":
                $routes = $this->m_table[$catConstant];
                $route = $routes[$value];

                $id = -1;
                $name = null;
                if ($route["is_external"] != "") {
                    $language = isys_application::instance()->container->get('language');
                    $id = $route["is_external"];
                    $name = $route["is_external"] == 1 ? $language->get("LC__UNIVERSAL__YES") : $language->get("LC__UNIVERSAL__NO");
                }

                $l_multiValue[] = [
                    "__id__" => $id,
                    "value"  => $name,
                    "parent" => $route["id"]
                ];
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE_ROUTE-is_rented":
                $routes = $this->m_table[$catConstant];
                $route = $routes[$value];

                $id = -1;
                if (class_exists('isys_cmdb_dao_category_g_bvg_renting')) {
                    $language = isys_application::instance()->container->get('language');
                    $renting_dao = new isys_cmdb_dao_category_g_bvg_renting(isys_application::instance()->database);
                    $sql = ' AND isys_catg_bvg_renting_list__obj_id = ' . $renting_dao->convert_sql_id($value);
                    $sql .= ' AND isys_catg_bvg_renting_list__status = ' . defined_or_default('C__RECORD_STATUS__NORMAL');
                    $rentingCount = $renting_dao->get_data(null, null, $sql)->count();
                    $id = $rentingCount > 0 ? 1 : 0;
                    if ($id == 0) {
                        $value = $language->get("LC__UNIVERSAL__NO");
                    } else {
                        $value = $language->get("LC__UNIVERSAL__YES");
                    }
                }
                $l_multiValue[] = [
                    "__id__" => $id,
                    "value"  => $value,
                    "parent" => $route["id"]
                ];
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE_ROUTE-route_elements":
            case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-route_elements":
                $dao = new isys_cmdb_dao_category_g_smview_cable_route_element($this->m_db);
                if ($catConstant == 'C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE') {
                    $dao = new isys_cmdb_dao_category_g_smview_cable_virtual_route_element($this->m_db);
                }
                // todo: this method call is a performance killer!!!!
                $elements = $dao->get_elements_by_obj_id($value, true);
                $l_multiValue = [];
                foreach ($elements as $e) {
                    if (! empty($e['section_id'])) {
                        $title = $e['section_title'];
                    } else {
                        $title = $e['cable_title'] . ' - ' . $e['wire_scope'] . ' - ' . $e['wire_title'];
                    }
                    $l_multiValue[] = [
                        "__id__" => $e['id'],
                        "value" => $title,
                        "parent" => $value
                    ];
                }
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE_ROUTE-redundant_routes":
                $routes = $this->m_table[$catConstant];
                $route = $routes[$value];

                $dao = new isys_cmdb_dao_category_g_smview_cable_route($this->m_db);
                $res = $dao->get_assigned_redundant_routes($value);
                while ($row = $res->get_row()) {
                    $title = $row['isys_obj__title'];
                    if ((! empty($row['associated_obj_id']))) {
                        $title .= ' [' . $row['associated_obj_title'] . ']';
                    }

                    $l_multiValue[] = [
                        "__id__" => $row['isys_obj__id'],
                        "value"  => $title,
                        "parent" => $route["id"]
                    ];
                }
                return $l_multiValue;

            case "C__CATG__SMVIEW_CABLE_ROUTE-associated_route":
                $routes = $this->m_table[$catConstant];
                $route = $routes[$value];
                if (! empty($route['associated_route'])) {
                    $dao = new isys_cmdb_dao_category_g_smview_cable_route($this->m_db);
                    $row = $dao->get_data(null, $route['associated_route'])->get_row();
                    $l_multiValue[] = [
                        "__id__" => $row['isys_obj__id'],
                        "value"  => $row['isys_obj__title'],
                        "parent" => $route["id"]
                    ];
                }
                return $l_multiValue;
        }

        switch ($catConstant) {
            case "C__CATG__SMVIEW_CABLE_SECTION":
                if (is_numeric($value)) {
                    $sections = $this->m_table[$catConstant];
                    $f_categoryData = $sections[$value];
                    //search Objekt-Title
                    switch ($propertyName) {
                        case "start_object":
                            return [
                                $f_categoryData["id"] => [
                                    "__id__" => $f_categoryData["start_obj_id"],
                                    "value" => $this->getObj_Title($f_categoryData["start_obj_id"])
                                ]
                            ];
                        case "end_object":
                            return [
                                $f_categoryData["id"] => [
                                    "__id__" => $f_categoryData["end_obj_id"],
                                    "value" => $this->getObj_Title($f_categoryData["end_obj_id"])
                                ]
                            ];
                        case "route_id":
                            return [
                                $f_categoryData["id"] => [
                                    "__id__" => $f_categoryData["route_id"],
                                    "value" => $this->getObj_Title($f_categoryData["route_id"])
                                ]
                            ];
                        default:
                            return [
                                $f_categoryData["id"] => [
                                    "__id__" => $f_categoryData["id"],
                                    "value" => $f_categoryData[$propertyName]
                                ]
                            ];
                    }
                }
                return null;

            case "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT":
                $return = [];
                if (is_numeric($value)) {
                    foreach ($this->m_table[$catConstant] as $f_key => $f_categoryData) {
                        if ($f_key == $value) {
                            foreach ($f_categoryData as $f_categoryData_columns) {
                                $return[$f_categoryData_columns["id"]] = [
                                        "__id__" => $f_categoryData_columns["id"],
                                        "value" => $f_categoryData_columns[$propertyName]
                                    ];
                            }
                        }
                    }
                }
                return $return;

            case "C__CATG__SMVIEW_CABLE_ROUTE":
            case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE":
                if (is_numeric($value)) {
                    $routes = $this->m_table[$catConstant];
                    $f_categoryData = $routes[$value];
                    switch ($propertyName) {
                        case "start_object":
                            return [
                                $f_categoryData["id"] => [
                                    "__id__" => $f_categoryData["start_obj_id"],
                                    "value" => $this->getObj_Title($f_categoryData["start_obj_id"])
                                ]
                            ];
                        case "end_object":
                            return [
                                $f_categoryData["id"] => [
                                    "__id__" => $f_categoryData["end_obj_id"],
                                    "value" => $this->getObj_Title($f_categoryData["end_obj_id"])
                                ]
                            ];
                        default:
                            return [
                                $f_categoryData["id"] => [
                                    "__id__" => $f_categoryData["id"],
                                    "value" => $f_categoryData[$propertyName]
                                ]
                            ];
                    }
                }
                return null;

            case "C__CATG__SMVIEW_CABLE_ROUTE_ELEMENT":
            case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE_ELEMENT":
                $return = [];
                if (is_numeric($value)) {
                    foreach ($this->m_table[$catConstant] as $f_key => $f_categoryData) {
                        if ($f_key == $value) {
                            foreach ($f_categoryData as $f_categoryData_columns) {
                                $value = $f_categoryData_columns[$propertyName];
                                if ($propertyName == 'pin_out_state' || $propertyName == 'pin_in_state') {
                                    $fieldKey = $propertyName == 'pin_out_state' ? 'wire_pin_out_state' : 'wire_pin_in_state';
                                    $id = $f_categoryData_columns[$fieldKey];
                                    $dao = new isys_cmdb_dao_smview_cable_wire_pin_status($this->m_db);
                                    $group = $dao->get_object("id", $id);
                                    $value = ! empty($group) ? $group["const"] : '';
                                }
                                $return[$f_categoryData_columns["id"]] = [
                                    "__id__" => $f_categoryData_columns["id"],
                                    "value" => $value
                                ];
                            }
                        }
                    }
                }
                return $return;
            case "C__CATG__SMVIEW_FEED_JIRA":
                if (class_exists("isys_cmdb_dao_category_g_smview_feed_jira")) {
                    $feed_dao = isys_cmdb_dao_category_g_smview_feed_jira::instance($this->m_db);
                    return $this->_prepareVirtualProperties($feed_dao, $value, 'isys_key', $propertyName);
                }
                break;
            case "C__CATG__SMVIEW_FEED_VIRTUAL_DEVICE":
                if (class_exists("isys_cmdb_dao_category_g_smview_feed_virtual_device")) {
                    $virtual_device_dao = isys_cmdb_dao_category_g_smview_feed_virtual_device::instance($this->m_db);
                    return $this->_prepareVirtualProperties($virtual_device_dao, $value, 'isys_mac_address', $propertyName);
                }
                break;
            case "C__CATG__LIVESTATUS":
                $return = [];
                try {
                    $m_services = smview_cmdb_dao_status::instance($this->get_database_component())->get_all_services_by_hostname($value);
                    $f = 100;
                    foreach ($m_services as $service) {
                        $f++;
                        foreach ($service as $service_key => $service_value) {
                            if ($service_key == $propertyName) {
                                $return[$f] = [
                                        "__id__" => $f,
                                        "value"  => $service_value
                                ];
                            }
                        }
                    }
                    return $return;
                } catch (Exception $e) {}
        }
        return null;
    }

    /**
     * Get dependent object of a cable property.
     *
     * @param  int $obj_id Object-ID
     * @param  string $depTable Name of dependent table
     * @param  string $depKey Foreign key
     * @param  string $depName Property-Name of dependent object
     * @param  string|null $depName2 Second Property-Name of dependent object
     * @return array|null
     */
    private function getDependingCableValue($obj_id, $depTable, $depKey, $depName, $depName2 = null)
    {
        $l_multiValue = null;
        $cable_items = $this->m_table["category_g_cable"];
        if (isset($cable_items[$obj_id])) {
            $cable_item = $cable_items[$obj_id];
            $dep_object = null;
            $depId = $cable_item[$depKey];
            if ($depId !== null) {
                $dep_items = $this->m_table[$depTable];
                $dep_object = isset($dep_items[$depId]) ? $dep_items[$depId] : null;
            }

            $value = $dep_object ? $dep_object[$depName] : null;
            if ($dep_object && ! empty($depName2)) {
                $value .= isset($dep_object[$depName2]) ? ': ' . $dep_object[$depName2] : '';
            }
            $l_multiValue[] = [
                "__id__" => $cable_item[$depKey],
                "value"  => $value,
                "parent" => $cable_item["id"]
            ];
        }
        return $l_multiValue;
    }

    /**
     * Get wire PIN status.
     *
     * @param  int $objId
     * @param  string $idField
     * @return array
     */
    private function _getWirePinStatus($objId, $idField)
    {
        $l_multiValue = [];
        $states = isys_cmdb_dao_smview_cable_wire_pin::get_wire_states();
        $dao_view = new \isys_cmdb_dao_smview_cable_connection_view(\isys_application::instance()->database);
        $objs = $dao_view->filter_objects(['cable_id' => $objId]);
        foreach ($objs as $c) {
            // verify, if "left_wire_pin" is really the left one (its ID must be lower than the right one)
            $c = $this->_fixWirePinInAndOutIfNeeded($c);
            if (! empty($states[$c[$idField]])) {
                $l_multiValue[] = [
                    "__id__" => $c[$idField],
                    "value"  => $states[$c[$idField]],
                    "parent" => $objId
                ];
            }
        }
        return $l_multiValue;
    }

    /**
     * Get wire PIN values.
     *
     * @param  int $objId
     * @param  string $idField
     * @param  string $valueField
     * @return array
     */
    private function _getWirePin($objId, $idField, $valueField)
    {
        $l_multiValue = [];
        $dao_view = new \isys_cmdb_dao_smview_cable_connection_view(\isys_application::instance()->database);
        $objs = $dao_view->filter_objects(['cable_id' => $objId]);
        foreach ($objs as $c) {
            // verify, if "left_wire_pin" is really the left one (its ID must be lower than the right one)
            $c = $this->_fixWirePinInAndOutIfNeeded($c);
            $l_multiValue[] = [
                "__id__" => $c[$idField],
                "value"  => $c[$valueField],
                "parent" => $objId,
            ];
        }
        return $l_multiValue;
    }

    /**
     * Flip incoming and outgoing wire pin if needed (ID of left pin must be lower than the right one).
     *
     * @param  array $connection
     * @return array
     */
    private function _fixWirePinInAndOutIfNeeded($connection)
    {
        // verify, if "left_wire_pin" is really the left one (its ID must be lower than the right one)
        if ($connection['id_in'] < $connection['id_out']) {
            // everything is fine
            return $connection;
        }

        $row = [];
        foreach ($connection as $key => $value) {
            $k = $key;
            if ($this->_endsWith($k, '_in')) {
                $k = $this->_replaceAtEnd($k, '_in', '_out');
            } elseif ($this->_endsWith($k, '_out')) {
                $k = $this->_replaceAtEnd($k, '_out', '_in');
            }
            $row[$k] = $value;
        }
        return $row;
    }

    /**
     * Prepare virtual properties by adding 'isys_' prefix.
     *
     * @param  isys_cmdb_dao_category $dao
     * @param  int $obj_id
     * @param  string $idProperty
     * @param  string $property
     * @return array
     */
    private function _prepareVirtualProperties($dao, $obj_id, $idProperty, $property)
    {
        try {
            $elements = $dao->get_data_as_array(null, $obj_id);
        } catch (Exception $e) {
            $elements = [];
        }

        $return = [];
        foreach ($elements as $props) {
            $key = $props[$idProperty];
            $id = 'isys_' . $property;
            if ($props[$id]) {
                $val = $props[$id];
                $return[$key] = [
                    "__id__" => $key,
                    "value"  => $val,
                ];
            }
        }
        return $return;
    }

    /**
     * Get all cable connections.
     *
     * @param  int $l_object_id
     * @return array
     * @throws isys_exception_database
     */
    private function _getCableConnections($l_object_id)
    {
        // This statement will retrieve all connections of the current cable object.
        // copied from isys_cmdb_ui_category_g_cable_connection.class.php -> process
        $dao = new isys_cmdb_dao_category_g_cable_connection($this->m_db);
        $l_sql = "SELECT
                isys_catg_connector_list__id,
                isys_catg_connector_list__title,
                isys_cable_connection__id,
                isys_catg_connector_list__assigned_category,
                isys_obj__id,
                isys_obj__title
            FROM isys_catg_connector_list
            LEFT JOIN isys_cable_connection ON isys_catg_connector_list__isys_cable_connection__id = isys_cable_connection__id
            INNER JOIN isys_obj ON isys_obj__id = isys_catg_connector_list__isys_obj__id
            WHERE isys_cable_connection__isys_obj__id = " . $dao->convert_sql_id($l_object_id) . ";";

        $l_res = $dao->retrieve($l_sql);

        $l_connections = [];
        if (is_countable($l_res) && count($l_res)) {
            while ($l_row = $l_res->get_row()) {
                if (! isset($l_connections[$l_row['isys_cable_connection__id']])) {
                    $l_connections[$l_row['isys_cable_connection__id']] = [];
                }
                $l_connections[$l_row['isys_cable_connection__id']][] = $l_row['isys_obj__title'] . '/' . $l_row["isys_catg_connector_list__title"];
            }
        }

        $l_multiValue = [];
        foreach ($l_connections as $id => $connections) {
            $l_multiValue[] = [
                "__id__" => $id,
                "value"  => $connections[0] . ' <---> ' . $connections[1],
                "parent" => $l_object_id
            ];
        }
        return $l_multiValue;
    }

    /**
     * Check if string ends with specified end.
     *
     * @param  string $string
     * @param  string $end
     * @return boolean
     */
    private function _endsWith($string, $end)
    {
        $str = substr($string, strlen($string) - strlen($end), strlen($string));
        return $str == $end;
    }

    /**
     * Replace end of string.
     *
     * @param  string $string
     * @param  string $old
     * @param  string $new
     * @return string
     */
    private function _replaceAtEnd($string, $old, $new)
    {
        $str = substr($string, 0, strlen($string) - strlen($old));
        $str .= $new;
        return $str;
    }

    /**
     * Get rented objects.
     *
     * @param  array $rentedWires
     * @param  int $value
     * @return array
     * @throws isys_exception_database
     */
    private function _getRentedRoutes($rentedWires, $value)
    {
        $l_multiValue = [];
        $renting_dao = new isys_cmdb_dao_category_g_bvg_renting($this->m_db);
        foreach ($rentedWires as $r) {
            if ($r['isys_catg_bvg_renting_list__obj_id']) {
                $obj = $renting_dao->get_object($r['isys_catg_bvg_renting_list__obj_id'])->get_row();
                $l_multiValue[] = [
                    "__id__" => $obj["isys_obj__id"],
                    "value"  => $obj["isys_obj__title"],
                    "parent" => $value
                ];
            }
        }
        return $l_multiValue;
    }

    /**
     * Initialize all values of the specified category table.
     * These tables acts like a cache.
     *
     * @param  string $category_object
     * @return null
     * @throws isys_exception_api
     * @throws isys_exception_auth
     * @throws isys_exception_database
     */
    private function getTable($category_object)
    {
        if ($this->m_table["category_g_ip"] == null && $category_object == "category_g_ip") {
            $l_api_model_category = new isys_cmdb_dao_category_g_ip($this->m_db);
            $this->m_table["category_g_ip"] = $l_api_model_category->get_data_as_array(null, null, null, null, null);
        }

        if ($this->m_table["catg_guest_systems"] == null && $category_object == "catg_guest_systems") {
            $l_api_model_category = new isys_cmdb_dao_list_catg_guest_systems($this->m_db);
            $l_res = $l_api_model_category->get_result(null, null, null);
            while ($l_data = $l_res->get_row()) {
                $this->m_table["catg_guest_systems"][] = $l_data;
            }
        }

        if ($this->m_table["category_g_virtual_machine"] == null && $category_object == "category_g_virtual_machine") {
            $l_api_model_category = new isys_cmdb_dao_category_g_virtual_machine($this->m_db);
            $this->m_table["category_g_virtual_machine"] = $l_api_model_category->get_data_as_array(null, null, null, null, null);
        }

        if ($this->m_table["category_g_contact"] == null && $category_object == "category_g_contact") {
            $l_api_model_category = new smview_cmdb_dao_category(new isys_cmdb_dao($this->m_db));
            $p_param = ['category' => "C__CATG__CONTACT", 'condition' => " AND isys_obj.isys_obj__isys_obj_type__id = " . $this->m_object_type_id];
            $this->m_table["category_g_contact"] = $l_api_model_category->read_all($p_param);
        }

        if ($this->m_table["category_g_application"] == null && $category_object == "category_g_application") {
            $l_api_model_category = new isys_cmdb_dao_category_g_application($this->m_db);
            $this->m_table["category_g_application"] = $l_api_model_category->get_data_as_array(null, null, null, null, null);
        }

        if ($this->m_table["category_g_memory"] == null && $category_object == "category_g_memory") {
            $l_api_model_category = new isys_cmdb_dao_category_g_memory($this->m_db);
            $this->m_table["category_g_memory"] = $l_api_model_category->get_data_as_array(null, null, null, null, null);
        }

        if ($this->m_table["category_cable_description_sort"] == null && $category_object == "category_cable_description_sort") {
            //get Cable title for sorting
            $sql = "SELECT * FROM `isys_obj` Join isys_obj_type ON isys_obj.isys_obj__isys_obj_type__id = isys_obj_type.isys_obj_type__id WHERE isys_obj_type.isys_obj_type__const = 'C__SHD__TELECOMMUNICATION_CABLE' ORDER BY CAST(`isys_obj`.`isys_obj__title` AS int)";

            $isys_component_dao = new isys_component_dao($this->m_db);
            $l_res =  $isys_component_dao->retrieve($sql);

            while ($l_data = $l_res->get_row()) {
                $this->m_table["category_cable_description_sort"][$l_data["isys_obj__id"]] = [ "id" => $l_data["isys_obj__id"], "title" => $l_data["isys_obj__title"]];
            }
        }

        if ($this->m_table["category_g_cable"] == null && $category_object == "category_g_cable") {
            $patchedWires = $this->_getPatchedWires();

            $dao = new isys_cmdb_dao_smview_cable($this->m_db);
            $sql = 'SELECT * FROM isys_catg_smview_cable_capacity_view;';
            $res = $dao->retrieve($sql);
            $objects = [];
            while ($row = $res->get_row()) {
                $patched = $patchedWires[$row['obj_id']] ? count($patchedWires[$row['obj_id']]) : 0;
                $row['patched_wires'] = $patched;
                $row['free_wires'] = $row['wire_count'] - $patched;
                $objects[$row['obj_id']] = $row;
            }
            $this->m_table["category_g_cable"] = $objects;
        }

        if ($this->m_table["category_g_cable_user_state"] == null && $category_object == "category_g_cable_user_state") {
            $dao = new isys_cmdb_dao_smview_cable_user_state($this->m_db);
            $this->m_table["category_g_cable_user_state"] = $this->getIndexedObjectList($dao, 'id');
        }

        if ($this->m_table["category_g_cable_group"] == null && $category_object == "category_g_cable_group") {
            $dao = new isys_cmdb_dao_smview_cable_group($this->m_db);
            $this->m_table["category_g_cable_group"] = $this->getIndexedObjectList($dao, 'id');
        }

        if ($this->m_table["category_g_cable_admin_unit"] == null && $category_object == "category_g_cable_admin_unit") {
            $dao = new isys_cmdb_dao_smview_cable_admin_unit($this->m_db);
            $this->m_table["category_g_cable_admin_unit"] = $this->getIndexedObjectList($dao, 'id');
        }

        if ($this->m_table["category_g_cable_connection_type"] == null && $category_object == "category_g_cable_connection_type") {
            $dao = new isys_cmdb_dao_smview_cable_connection_type($this->m_db);
            $this->m_table["category_g_cable_connection_type"] = $this->getIndexedObjectList($dao, 'id');
        }

        if ($this->m_table["category_g_cable_type"] == null && $category_object == "category_g_cable_type") {
            $dao = new isys_cmdb_dao_smview_cable_type($this->m_db);
            $this->m_table["category_g_cable_type"] = $this->getIndexedObjectList($dao, 'id');
        }

        if ($this->m_table["C__CATG__SMVIEW_CABLE_SECTION"] == null && $category_object == "C__CATG__SMVIEW_CABLE_SECTION") {
            $dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
            $this->m_table["C__CATG__SMVIEW_CABLE_SECTION"] = $this->getIndexedObjectList($dao, 'obj_id');
        }

        if ($this->m_table["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT"] == null && $category_object == "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT") {
            $dao = new isys_cmdb_dao_category_g_smview_cable_section_element($this->m_db);
            $section_elements = $dao->get_all_elements();
            $this->m_table["C__CATG__SMVIEW_CABLE_SECTION_ELEMENT"] = $section_elements;
        }

        if ($this->m_table["C__CATG__SMVIEW_CABLE_ROUTE"] == null && $category_object == "C__CATG__SMVIEW_CABLE_ROUTE") {
            $dao = new isys_cmdb_dao_smview_cable_route($this->m_db);
            $this->m_table["C__CATG__SMVIEW_CABLE_ROUTE"] = $this->getIndexedObjectList($dao, 'obj_id');
        }

        if ($this->m_table["C__CATG__SMVIEW_CABLE_ROUTE_ELEMENT"] == null && $category_object == "C__CATG__SMVIEW_CABLE_ROUTE_ELEMENT") {
            $dao = new isys_cmdb_dao_category_g_smview_cable_route_element($this->m_db);
            $section_elements = $dao->get_all_elements();
            $this->m_table["C__CATG__SMVIEW_CABLE_ROUTE_ELEMENT"] = $section_elements;
        }

        if ($this->m_table["C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE"] == null && $category_object == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE") {
            $dao = new isys_cmdb_dao_smview_cable_virtual_route($this->m_db);
            $this->m_table["C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE"] = $this->getIndexedObjectList($dao, 'obj_id');
        }

        if ($this->m_table["C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE_ELEMENT"] == null && $category_object == "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE_ELEMENT") {
            $dao = new isys_cmdb_dao_category_g_smview_cable_virtual_route_element($this->m_db);
            $section_elements = $dao->get_all_elements();
            $this->m_table["C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE_ELEMENT"] = $section_elements;
        }
        return null;
    }

    /**
     * Get all patched wires.
     *
     * @return array
     * @throws isys_exception_database
     */
    private function _getPatchedWires()
    {
        $identifier = 'patched_wires';
        $m_cache = new isys_cache_memcached();
        $m_cache->ns('smview_cable.patched_wires');
        if (isys_cache_memcached::available()) {
            $patchedWires = $m_cache->get($identifier);
            if (! empty($patchedWires)) {
                return $patchedWires;
            }
        }

        // handle each patch-case in a separate view to increase performance
        $patchViews = [
            'isys_catg_smview_cable_capacity_patched_route_wire_view',
            'isys_catg_smview_cable_capacity_patched_section_wire_view',
            'isys_catg_smview_cable_capacity_patched_user_state_wire_view',
        ];
        $patchedWires = [];
        $dao = new isys_cmdb_dao_smview_cable($this->m_db);
        foreach ($patchViews as $patchView) {
            $sql = "SELECT * FROM $patchView;";
            try {
                $res = $dao->retrieve($sql);
            } catch (Exception $e) {
                continue;
            }
            while ($row = $res->get_row()) {
                $patchedWires[$row['obj_id']][$row['wire_id']] = 1;
            }
            $res->free_result();
        }

        if (isys_cache_memcached::available()) {
            $m_cache->set($identifier, $patchedWires, 300);
        }
        return $patchedWires;
    }

    /**
     * Get indexed object list.
     *
     * @param  isys_cmdb_dao_smview_cable_abstract $dao DAO object
     * @param  string $index Name of index property
     * @return array
     */
    private function getIndexedObjectList($dao, $index)
    {
        $raw_objects = $dao->get_objects();
        $objects = [];
        foreach ($raw_objects as $row) {
            $objects[$row[$index]] = $row;
        }
        return $objects;
    }

    /**
     * @param $obj_id
     * @return bool
     */
    private function isConditionTrue($obj_id)
    {
        $result = [];
        foreach ($this->m_specialCondition as $f_condition) {
            $condition_lower_case = isset($f_condition["value"]) ? mb_strtolower($f_condition["value"], 'UTF-8') : null;

            //get Value
            switch ($f_condition["property"]) {
                case "C__SHD__LIVESTATUS-output":
                    $status = $this->getSpecialValue("C__SHD__LIVESTATUS-status", $obj_id);
                    $value = $status["output"];
                    break;
                case "C__SHD__LIVESTATUS-status":
                    $status = $this->getSpecialValue("C__SHD__LIVESTATUS-status", $obj_id);
                    $value = $status["status"];

                    //host has no monitoring
                    if ($value === null) {
                        $value = "-1";
                    }
                    break;
                default:
                    $value = $this->getSpecialValue($f_condition["property"], $obj_id);
                    break;
            }

            $property = $f_condition["property"];
            switch ($property) {
                //String Column
                case "C__SHD__LIVESTATUS-output":
                case "C__SHD__LIVESTATUS-status":
                case "C__CATG__LOCATION-location_path":
                case "C__CATG__SMVIEW_IMPORT_ELID-device_elid":
                case "C__CATG__IP-primary_hostaddress":
                case "C__CATG__SMVIEW_CABLE_WIRE-extended_location":
                    switch ($f_condition["comparison"]) {
                        case "LIKE":
                            //kein wert also raus
                            if ($value === null) {
                                $result[$property][] = false;
                                break;
                            }
                            foreach ($value as $f_item) {
                                //replace % to .* And _ to .
                                $regex = str_replace("%", ".*", $condition_lower_case);
                                $regex = str_replace("_", ".", $regex);

                                $preg_match = "/(^" . $regex . ")/" ;
                                $val = mb_strtolower($f_item["value"], 'UTF-8');
                                preg_match($preg_match, $val, $matches);

                                //check for more then 1 Object.
                                if (count($matches) == 0) {
                                    $result[$property][] = false;
                                } else {
                                    $val = mb_strtolower($f_item["value"], 'UTF-8');
                                    $result[$property][] = $matches[0] == $val;
                                }
                            }
                            break;

                        case "LIKE %...%":
                            $val = mb_strtolower($value, 'UTF-8');
                            $result[$property][] = strpos($val, $condition_lower_case) !== false;
                            break;
                        case "=":
                            $result[$property][] = $value == $f_condition["value"];
                            break;
                        case "!=":
                            $result[$property][] = $value != $f_condition["value"];
                            break;
                        case "IS NULL":
                            $result[$property][] = $value == "";
                            break;
                    }
                    break;

                //Integer Column
                case "C__CATS__APPLICATION-install_count":
                case "C__CATG__SMVIEW_CABLE-cable_capacity":
                case "C__CATG__SMVIEW_CABLE-cable_free_wires":
                case "C__CATG__SMVIEW_CABLE-cable_patched_wires":
                    $res = $this->compareInt($f_condition["comparison"], $f_condition["value"], $value);
                    if ($res !== null) {
                        $result[$property][] = $res;
                    }
                    break;

                //Multi_Value Column with String Value
                case "C__CATS__PERSON_GROUP_MEMBERS-connected_object":
                case "C__CATS__PERSON_ASSIGNED_GROUPS-connected_object":
                case "C__CATS__PERSON_CONTACT_ASSIGNMENT-object_type":
                case "C__CATG__CABLE-connection":
                case "C__CATG__CONTACT-primary":
                case "C__CATG__CONTACT-primary_contact":
                case "C__CATG__CONTACT-contact_object":
                case "C__CATG__CONTACT-contact_list":
                case "C__CATG__CONTACT-contact":
                case "C__CATG__SMVIEW_CABLE-description_sort":
                case "C__CATG__SMVIEW_CABLE-net_type":
                case "C__CATG__SMVIEW_CABLE-state":
                case "C__CATG__SMVIEW_CABLE-user_state":
                case "C__CATG__SMVIEW_CABLE-location_out_path":
                case "C__CATG__SMVIEW_CABLE-location_out":
                case "C__CATG__SMVIEW_CABLE-location_in_path":
                case "C__CATG__SMVIEW_CABLE-location_in":
                case "C__CATG__SMVIEW_CABLE_ROUTE-description":
                case "C__CATG__SMVIEW_CABLE_ROUTE-permission":
                case "C__CATG__SMVIEW_CABLE_ROUTE-location":
                case "C__CATG__SMVIEW_CABLE_ROUTE-room":
                case "C__CATG__SMVIEW_CABLE_ROUTE-device_type":
                case "C__CATG__SMVIEW_CABLE_ROUTE-device_number":
                case "C__CATG__SMVIEW_CABLE_ROUTE-phone_number":
                case "C__CATG__SMVIEW_CABLE_ROUTE-phone_subscriber":
                case "C__CATG__SMVIEW_CABLE_ROUTE-hint":
                case "C__CATG__SMVIEW_CABLE_ROUTE-measure_date":
                case "C__CATG__SMVIEW_CABLE_ROUTE-damping_left1":
                case "C__CATG__SMVIEW_CABLE_ROUTE-damping_left2":
                case "C__CATG__SMVIEW_CABLE_ROUTE-damping_right1":
                case "C__CATG__SMVIEW_CABLE_ROUTE-damping_right2":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-description":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-permission":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-location":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-room":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-device_type":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-device_number":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-phone_number":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-phone_subscriber":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-hint":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-measure_date":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-damping_left1":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-damping_left2":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-damping_right1":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-damping_right2":
                case "C__CATS__PERSON_CONTACT_ASSIGNMENT-role":
                case "C__CATG__CONTACT-role":
                case "C__CATG__IT_SERVICE-connected_object":
                case "C__CATG__LOCATION-_location_path":
                case "C__CATG__IP-fqdn":
                default:
                    switch ($f_condition["comparison"]) {
                        case "BETWEEN":

                            $split = explode("..", $f_condition["value"]);

                            $low_value = $split[0];
                            $int_low = (int) filter_var($low_value, FILTER_SANITIZE_NUMBER_INT);
                            $str_low =  str_replace($int_low, "", $low_value);

                            $high_value = $split[1];
                            $int_high = (int) filter_var($high_value, FILTER_SANITIZE_NUMBER_INT);
                            $str_high =  str_replace($int_high, "", $high_value);

                            //kein wert also raus
                            if ($value === null) {
                                $result[$property][] = false;
                                break;
                            }
                            foreach ($value as $f_item) {

                                $int_value = (int) filter_var($f_item["value"], FILTER_SANITIZE_NUMBER_INT);

                                $str_value =  str_replace($int_value, "", $f_item["value"]);

                                if ($int_low <= $int_value && $int_high >= $int_value && $str_low <= $str_value && $str_high >= $str_value) {

//                                            if ($str_low <= $str_value && $str_high > $str_value) {
//                                    dump("low",$str_low,";;", $str_high,";;", $str_value);
//                                            }

                                    //                                    if ($int_low <= $int_value && $int_high > $int_value && $str_low <= $str_value && $str_high > $str_value) {
//                                    dump("low",$low_value,  $f_item["value"]);
                                    $result[$property][] = true;
                                } else {
                                    $result[$property][] = false;
                                }
                            }
                            break;

                        case "LIKE":
                            //kein wert also raus
                            if ($value === null) {
                                $result[$property][] = false;
                                break;
                            }
                            foreach ($value as $f_item) {
                                //replace % to .* And _ to .
                                $regex = str_replace("%", ".*", $condition_lower_case);
                                $regex = str_replace("_", ".", $regex);

                                $preg_match = "/(^" . $regex . ")/" ;
                                $val = mb_strtolower($f_item["value"], 'UTF-8');
                                preg_match($preg_match, $val, $matches);

                                //check for more then 1 Object.
                                if (count($matches) == 0) {
                                    $result[$property][] = false;
                                } else {
                                    $val = mb_strtolower($f_item["value"], 'UTF-8');
                                    $result[$property][] = $matches[0] == $val;
                                }
                            }
                            break;
                        case "LIKE %...%":
                            //kein wert also raus
                            if ($value === null) {
                                $result[$property][] = false;
                                break;
                            }
                            foreach ($value as $f_item) {
                                //prüfen ob der Wert im Value vorhanden ist.
                                $val = mb_strtolower($f_item["value"], 'UTF-8');
                                if (strpos($val, $condition_lower_case) === false) {
                                    $result[$property][] = false;
                                } else {
                                    $result[$property][] = true;
                                }
                            }
                            break;
                        case "=":
                            if ($value === null) {
                                $result[$property][] = false;
                            }
                            if (is_array($value)) {
                                foreach ($value as $f_item) {
                                    $val = mb_strtolower($f_item["value"], 'UTF-8');
                                    $result[$property][] = $val == $condition_lower_case;
                                }
                            } else {
                                $val = mb_strtolower($value, 'UTF-8');
                                $result[$property][] = $val == $condition_lower_case;
                            }
                            break;
                        case "!=":
                            if ($value === null) {
                                $result[$property][] = false;
                                break;
                            }
                            foreach ($value as $f_item) {
                                $val = mb_strtolower($f_item["value"], 'UTF-8');
                                if ($val == $condition_lower_case) {
                                    $result[$property] = [];
                                } else {
                                    $result[$property][] = true;
                                }
                            }
                            break;
                        case "NOT LIKE":
                            //kein wert also raus
                            if ($value === null) {
                                $result[$property][] = true;
                                break;
                            }
                            foreach ($value as $f_item) {
                                //replace % to .*
                                $preg_match = "/(^" . str_replace("%", ".*", $condition_lower_case) . ")/" ;
                                $val = mb_strtolower($f_item["value"], 'UTF-8');
                                preg_match($preg_match, $val, $matches);

                                //check for more then 1 Object.
                                $result[$property][] = count($matches) == 0;
                            }
                            break;
                        case "IS NULL":
                            $column = true;
                            foreach ($value as $f_item) {
                                $column = false;
                            }
                            $result[$property][] = $column;
                            break;
                        case "IS NOT NULL":
                            $column = false;
                            foreach ($value as $f_item) {
                                $column = true;
                            }
                            $result[$property][] = $column;
                            break;
                        case ">":
                            foreach ($value as $f_item) {
                                $result[$property][] = strnatcasecmp($f_item["value"], $f_condition["value"]) > 0;
                                break;
                            }
                            $result[$property][] = false;
                            break;
                        case "<":
                            foreach ($value as $f_item) {
                                $result[$property][] = strnatcasecmp($f_item["value"], $f_condition["value"]) < 0;
                                break;
                            }
                            $result[$property][] = false;
                            break;
                        case ">=":
                            foreach ($value as $f_item) {
                                $result[$property][] = strnatcasecmp($f_item["value"], $f_condition["value"]) >= 0;
                                break;
                            }
                            $result[$property][] = false;
                            break;
                        case "<=":
                            foreach ($value as $f_item) {
                                $result[$property][] = strnatcasecmp($f_item["value"], $f_condition["value"]) <= 0;
                                break;
                            }
                            $result[$property][] = false;
                            break;
                        case "id":
                            //kein wert also raus
                            if ($value === null) {
                                $result[$property][] = false;
                                break;
                            }

                            if (isset($value["__id__"]) && isset($value["value"])) {
                                //object value
                                $result[$property][] = $value["__id__"] == $f_condition["value"];
                            } elseif (is_array($value)) {
                                //[["__id__"]=>  string(1) "3", ["value"]=>  string(4) "Koax", ["__id__"]=>  string(1) "2", ["value"]=>  string(4) "Cu"]
                                foreach ($value as $item) {
                                    if ($item["__id__"] == $f_condition["value"]) {
                                        $result[$property] = [];
                                        $result[$property][] = true;
                                    } else {
                                        $result[$property][] = false;
                                        break;
                                    }
                                }
                            } else {
                                if ($value == $f_condition["value"]) {
                                    $result[$property] = [];
                                    $result[$property][] = true;
                                } else {
                                    $result[$property][] = false;
                                }
                                break;
                            }
                            break;

                        case "!id":
                            //kein wert also raus
                            if ($value === null) {
                                $result[$property][] = false;
                                break;
                            }
                            if (isset($value["__id__"]) && isset($value["value"])) {
                                //object value
                                $result[$property][] = $value["__id__"] != $f_condition["value"];
                            } elseif (is_array($value)) {
                                //[["__id__"]=>  string(1) "3", ["value"]=>  string(4) "Koax", ["__id__"]=>  string(1) "2", ["value"]=>  string(4) "Cu"]
                                foreach ($value as $item) {
                                    if ($item["__id__"] != $f_condition["value"]) {
                                        $check_other_conditions = true;
                                        foreach ($result as $a_column) {
                                            if ($a_column[0] == false) {
                                                $check_other_conditions = false;
                                            }
                                        }
                                        $result[$property][] = $check_other_conditions == true;
                                    } else {
                                        $result[$property] = [];
                                        $result[$property][] = false;
                                    }
                                }
                            } else {
                                if ($value != $f_condition["value"]) {
                                    $check_other_conditions = true;
                                    foreach ($result as $a_column) {
                                        if ($a_column[0] == false) {
                                            $check_other_conditions = false;
                                        }
                                    }
                                    $result[$property][] = $check_other_conditions == true;
                                } else {
                                    $result[$property] = [];
                                    $result[$property][] = false;
                                }
                            }
                            break;
                    }
                    break;

                //Dialog-columns
                case "C__CATG__SMVIEW_CABLE-state_id":
                case "C__CATG__SMVIEW_CABLE-group_id":
                case "C__CATG__SMVIEW_CABLE-type_id":
                case "C__CATG__SMVIEW_CABLE-admin_unit":
                case "C__CATG__SMVIEW_CABLE-is_section_cable":
                case "C__CATG__SMVIEW_CABLE-connection_type_id":
                case "C__CATG__SMVIEW_CABLE_SECTION-user_state_id":
                case "C__CATG__SMVIEW_CABLE_ROUTE-route_usage_id":
                case "C__CATG__SMVIEW_CABLE_ROUTE-is_external":
                case "C__CATG__SMVIEW_CABLE_ROUTE-is_rented":
                case "C__CATG__SMVIEW_CABLE_ROUTE-redundant_routes":
                case "C__CATG__SMVIEW_CABLE_ROUTE-associated_route":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-route_usage_id":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-is_external":
                case "C__CATG__SMVIEW_CABLE_WIRE-state_in":
                case "C__CATG__SMVIEW_CABLE_WIRE-state_out":
                case "C__CATG__SMVIEW_CABLE_WIRE-user_state_id":
                case "C__CATG__SMVIEW_CABLE_WIRE-connection_type_id":
                case "C__CATG__GLOBAL-tag":

                    switch ($f_condition["comparison"]) {
                        case "=":
                            if ($value === null) {
                                $result[$property][] = false;
                            }
                            $conditionValue = $f_condition["value"] == '-1' ? null : $f_condition["value"];
                            foreach ($value as $f_item) {
                                $result[$property][] = strtolower($f_item["__id__"]) == strtolower($conditionValue);
                            }
                            break;
                        case "LIKE %...%":               
                            if ($value === null) {
                                $result[$property][] = false;
                            }
                            $conditionValue = $f_condition["value"] == '-1' ? null : $f_condition["value"];
                            if ($conditionValue != null) {
                                $conditionValue = str_replace("\\\\", "\\", $conditionValue);
                                foreach ($value as $f_item) {
                                    $result[$property][] = strpos(strtolower($f_item["value"]), strtolower($conditionValue)) !== false;
                                }
                            }
                            break;
                            
                    }
                    break;

                //Multi_Value Column with Integer Value
                case "C__CATG__SMVIEW_CABLE-length":
                case "C__CATG__CABLE-length":
                case "C__CATG__SMVIEW_CABLE-capacity":
                case "C__CATG__SMVIEW_CABLE_ROUTE-connection_count":
                case "C__CATG__SMVIEW_CABLE_ROUTE-current_connection_count":
                case "C__CATG__SMVIEW_CABLE_ROUTE-length":
                case "C__CATG__SMVIEW_CABLE_ROUTE-measured_length":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-connection_count":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-current_connection_count":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-length":
                case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-measured_length":
                case "C__CATG__SMVIEW_CABLE_SECTION-length":
                case "C__CATG__SMVIEW_CABLE_SECTION-channel_count":
                case "C__CATG__SMVIEW_CABLE_SECTION-current_channel_count":
                    $v = array_values($value)[0];
                    $res = $this->compareInt($f_condition["comparison"], $f_condition["value"], $v["value"]);
                    if ($res !== null) {
                        $result[$property][] = $res;
                    }
                    break;
            }
        }

        $return = $result != [] ? true : null;
        foreach ($result as $f_item) {
            $condition = false;
            foreach ($f_item as $bool) {
                $condition = $condition || $bool;
            }
            $return = $return && $condition;
        }
        return $return == null ? false : $return;
    }

    /**
     * Compare integer values.
     *
     * @param  string $comparison Name of comparison method
     * @param  int $conditionValue Matching value
     * @param  int $currentValue Current value to be checked
     * @return boolean|null Return null if no condition matches
     */
    private function compareInt($comparison, $conditionValue, $currentValue)
    {
        switch ($comparison) {
            case "LIKE":
            case "LIKE %...%":
                if ($currentValue == "") {
                    return false;
                }
                return strpos($currentValue, $conditionValue) !== false;
            case "=":
                return $currentValue == $conditionValue;
            case "IS NULL":
                return $currentValue == "";
            case "IS NOT NULL":
                return $currentValue != "";
            case "!=":
            case "NOT LIKE":
                if ($currentValue == "") {
                    return false;
                }

                $compare_value = str_replace("%", "", $conditionValue);
                $compare_value = str_replace("_", "", $compare_value);
                return strpos($currentValue, $compare_value) === false;
            case ">":
                if ($currentValue == "") {
                    return false;
                }
                return $currentValue > $conditionValue;
            case ">=":
                if ($currentValue == "") {
                    return false;
                }
                return $currentValue >= $conditionValue;
            case "<":
                if ($currentValue == "") {
                    return false;
                }
                return $currentValue < $conditionValue;
            case "<=":
                if ($currentValue == "") {
                    return false;
                }
                return $currentValue <= $conditionValue;
            case "BETWEEN":
            case "NOT BETWEEN":
                if ($currentValue == "") {
                    return false;
                }

                $parts = explode('..', $conditionValue);
                $min = (int)$parts[0];
                $max = (int)$parts[1];
                $currentValue = (int)$currentValue;

                if ($comparison == "BETWEEN") {
                    return $currentValue > $min && $currentValue < $max;
                } else {
                    return $currentValue < $min || $currentValue > $max;
                }
        }
        return null;
    }

    /**
     * get the translated Property from the ID
     * @param $key
     * @return mixed|null
     */
    private function getProperty($key)
    {
        if (is_numeric($key)) {
            //foreach and if for id
            foreach ($this->m_properties as $m_property) {
                if ($m_property["id"] == $key) {
                    return $m_property;
                }
            }
        } else {
            return isset($this->m_properties[$key]) ? $this->m_properties[$key] : null;
        }
        return null;
    }

    /**
     * sort the extra-Columns
     * @param $content
     * @param $sort_column
     * @return mixed
     */
    private function sortSpecialColumns($content, $sort_column)
    {
        switch ($sort_column["const"]) {
            case "C__CATG__IP-fqdn":
                if ($sort_column["sorting_direction"] == "ASC") {
                    //presort of multivalue
                    foreach ($content as $f_key => $f_item) {
                        usort($f_item, function ($a, $b) {
                            return strnatcasecmp($a["value"], $b["value"]);
                        });
                        $content[$f_key] = $f_item;
                    }
                    uasort($content, function ($a, $b) {
                        //Set -1 when status not set
                        $val_a = isset($a[0]["value"]) ? $a[0]["value"] : "zzzzzzzzz";
                        $val_b = isset($b[0]["value"]) ? $b[0]["value"] : "zzzzzzzzz";
                        return strnatcasecmp($val_a, $val_b);
                    });
                } else {
                    //presort of multivalue
                    foreach ($content as $f_key => $f_item) {
                        usort($f_item, function ($a, $b) {
                            return strnatcasecmp($b["value"], $a["value"]);
                        });
                        $content[$f_key] = $f_item;
                    }

                    uasort($content, function ($a, $b) {
                        //Set -1 when status not set
                        $val_a = isset($a[0]["value"]) ? $a[0]["value"] : "0000000000000";
                        $val_b = isset($b[0]["value"]) ? $b[0]["value"] : "0000000000000";
                        return strnatcasecmp($val_b, $val_a);
                    });
                }
                break;

            case "C__SHD__LIVESTATUS-service_status":
                if ($sort_column["sorting_direction"] == "ASC") {
                    uasort($content, function ($a, $b) {
                        if (! isset($a["service_status"])) {
                            $a["service_status"] = "0|0|0|0";
                        }
                        if (! isset($b["service_status"])) {
                            $b["service_status"] = "0|0|0|0";
                        }

                        $split_a = explode("|", $a["service_status"]);
                        $split_b = explode("|", $b["service_status"]);

                        foreach (range(0, 3) as $i) {
                            if ($split_a[$i] > $split_b[$i]) {
                                return -1;
                            } elseif ($split_a[$i] < $split_b[$i]) {
                                return 1;
                            }
                        }
                        return -1;
                    });
                } else {
                    uasort($content, function ($a, $b) {
                        if (! isset($a["service_status"]) || $a["service_status"] == "0|0|0|0") {
                            $a["service_status"] = "999|999|999|999";
                        }
                        if (! isset($b["service_status"]) || $b["service_status"] == "0|0|0|0") {
                            $b["service_status"] = "999|999|999|999";
                        }

                        $split_a = explode("|", $a["service_status"]);
                        $split_b = explode("|", $b["service_status"]);

                        foreach (range(0, 3) as $i) {
                            if ($split_a[$i] < $split_b[$i]) {
                                return -1;
                            } elseif ($split_a[$i] > $split_b[$i]) {
                                return 1;
                            }
                        }
                        return -1;
                    });
                }
                break;

            case "C__SHD__LIVESTATUS-status":
                if ($sort_column["sorting_direction"] == "ASC") {
                    uasort($content, function ($a, $b) {
                        //Set -1 when status not set
                        if (! isset($a["status"])) {
                            $a["status"] = -1;
                        }
                        if (! isset($b["status"])) {
                            $b["status"] = -1;
                        }

                        if ($a["status"] == $b["status"]) {
                            return 0;
                        }
                        return ((int)$a["status"] > (int)$b["status"]) ? -1 : 1;
                    });
                } else {
                    uasort($content, function ($a, $b) {
                        //Set -1 when status not set
                        if (! isset($a["status"]) || $a["status"] == "-1") {
                            $a["status"] = 99;
                        }
                        if (! isset($b["status"]) || $b["status"] == "-1") {
                            $b["status"] = 99;
                        }

                        if ($a["status"] == $b["status"]) {
                            return 0;
                        }
                        return ((int)$a["status"] < (int)$b["status"]) ? -1 : 1;
                    });
                }
                break;

            case "C__SHD__LIVESTATUS-output":
                if ($sort_column["sorting_direction"] == "ASC") {
                    uasort($content, function ($a, $b) {
                        if (! isset($a['output']) || $a['output'] == "") {
                            $a['output'] = "zzzzzzzzz";
                        }
                        if (! isset($b['output']) || $b['output'] == "") {
                            $b['output'] = "zzzzzzzzz";
                        }
                        return strcasecmp(strval($a['output']), strval($b['output']));
                    });
                } else {
                    uasort($content, function ($a, $b) {
                        if (! isset($a['output']) || $a['output'] == "") {
                            $a['output'] = "aaaaaaaaaaaa";
                        }
                        if (! isset($b['output']) || $b['output'] == "") {
                            $b['output'] = "aaaaaaaaaaaa";
                        }
                        return strcasecmp(strval($b['output']), strval($a['output']));
                    });
                }
                break;

            case "C__CATG__SMVIEW_CABLE-user_state":
            case "C__CATG__SMVIEW_CABLE-connection_type_id":
            case "C__CATG__SMVIEW_CABLE-state_id":
            case "C__CATG__SMVIEW_CABLE-group_id":
            case "C__CATG__SMVIEW_CABLE-admin_unit":
            case "C__CATG__SMVIEW_CABLE-type_id":
            case "C__CATG__SMVIEW_CABLE-is_section_cable":
            case "C__CATG__SMVIEW_CABLE-location_in":
            case "C__CATG__SMVIEW_CABLE-location_in_path":
            case "C__CATG__SMVIEW_CABLE-location_out":
            case "C__CATG__SMVIEW_CABLE-location_out_path":
            case "C__CATG__SMVIEW_CABLE-net_type":
            case "C__CATG__SMVIEW_CABLE-capacity":
            case "C__CATG__SMVIEW_CABLE-state":
            case "C__CATG__SMVIEW_CABLE-length":
            case "C__CATG__CABLE-length":
            case "C__CATG__CABLE-connection":
            case "C__CATG__SMVIEW_CABLE_SECTION-user_state_id":
            case "C__CATG__SMVIEW_CABLE_ROUTE-route_usage_id":
            case "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE-route_usage_id":
            case "C__CATS__PERSON_GROUP_MEMBERS-connected_object":
            case "C__CATS__PERSON_ASSIGNED_GROUPS-connected_object":
            case "C__CATS__PERSON_CONTACT_ASSIGNMENT-object_type":
            case "C__CATS__PERSON_CONTACT_ASSIGNMENT-role":
            case "C__CATG__CONTACT-role":
            case "C__CATG__CONTACT-contact_object":
            case "C__CATG__CONTACT-contact_list":
            case "C__CATG__IT_SERVICE-connected_object":
                if ($sort_column["sorting_direction"] == "ASC") {
                    //presort of multivalue
                    foreach ($content as $f_key => $f_item) {
                        usort($f_item, function ($a, $b) {
                            return strnatcasecmp($a["value"], $b["value"]);
                        });
                        $content[$f_key] = $f_item;
                    }
                    uasort($content, function ($a, $b) {
                        //Set -1 when status not set
                        if (! isset($a[0]["value"])) {
                            $a[0]["value"] = "zzzzzzzzz";
                        }
                        if (! isset($b[0]["value"])) {
                            $b[0]["value"] = "zzzzzzzzz";
                        }
                        return strnatcasecmp($a[0]["value"], $b[0]["value"]);
                    });
                } else {
                    //presort of multivalue
                    foreach ($content as $f_key => $f_item) {
                        usort($f_item, function ($a, $b) {
                            return strnatcasecmp($b["value"], $a["value"]);
                        });
                        $content[$f_key] = $f_item;
                    }

                    uasort($content, function ($a, $b) {
                        //Set -1 when status not set
                        if (! isset($a[0]["value"])) {
                            $a[0]["value"] = "0000000000000";
                        }
                        if (! isset($b[0]["value"])) {
                            $b[0]["value"] = "0000000000000";
                        }
                        return strnatcasecmp($b[0]["value"], $a[0]["value"]);
                    });
                }
                break;

                //sort with no Array only plain values
            case "C__CATG__SMVIEW_CABLE-cable_capacity":
            case "C__CATG__SMVIEW_CABLE-cable_patched_wires":
            case "C__CATG__SMVIEW_CABLE-cable_free_wires":
                if ($sort_column["sorting_direction"] == "ASC") {
                    uasort($content, function ($a, $b) {
                        //Set -1 when status not set
                        $a_item = !isset($a) ? "zzzzzzzzz" : $a;
                        $b_item = !isset($b) ? "zzzzzzzzz" : $b;
                        return strnatcasecmp($a_item, $b_item);
                    });
                } else {
                    uasort($content, function ($a, $b) {
                        $a_item = !isset($a) ? "zzzzzzzzz" : $a;
                        $b_item = !isset($b) ? "zzzzzzzzz" : $b;
                        return strnatcasecmp($b_item, $a_item);
                    });
                }
                break;

            case "C__CATG__CONTACT-primary":
            case "C__CATG__CONTACT-primary_contact":
                if ($sort_column["sorting_direction"] == "ASC") {
                    uasort($content, function ($a, $b) {
                        //Set -1 when status not set
                        $a_item = "zzzzzzzzz";
                        $b_item = "zzzzzzzzz";

                        foreach ($a as $a_key => $f_aItem) {
                            $a_item = $f_aItem["value"];
                        }

                        foreach ($b as $b_key => $f_bItem) {
                            $b_item = $f_bItem["value"];
                        }
                        return strnatcasecmp($a_item, $b_item);
                    });
                } else {
                    uasort($content, function ($a, $b) {
                        $a_item = "0000000000000";
                        $b_item = "0000000000000";

                        foreach ($a as $a_key => $f_aItem) {
                            $a_item = $f_aItem["value"];
                        }

                        foreach ($b as $b_key => $f_bItem) {
                            $b_item = $f_bItem["value"];
                        }
                        return strnatcasecmp($b_item, $a_item);
                    });
                }
                break;

                //simple string sort
            case "C__CATG__SMVIEW_CABLE-description_sort":
                if ($sort_column["sorting_direction"] == "ASC") {
                    uasort($content, function ($a, $b) {
                        return strnatcmp($a[0]["value"], $b[0]["value"]);
                    });
                } else {
                    uasort($content, function ($a, $b) {
                        return strnatcasecmp($b[0]["value"], $a[0]["value"]);
                    });
                }
                break;

                //simple integer sort
            case "C__CATS__APPLICATION-install_count":
                if ($sort_column["sorting_direction"] == "ASC") {
                    uasort($content, function ($a, $b) {
                        //Set -1 when status not set
                        return ((int)$a > (int)$b) ? -1 : 1;
                    });
                } else {
                    uasort($content, function ($a, $b) {
                        return ((int)$a < (int)$b) ? -1 : 1;
                    });
                }
                break;

            default:
                if ($sort_column["sorting_direction"] == "ASC") {
                    uasort($content, function ($a, $b) {
                        $a_item = null;
                        $b_item = null;

                        if (is_string($a["value"])) {
                            $a_item = $a["value"] == "" ? "zzzzzzzzz" : $a["value"];
                        } elseif (is_array($a)) {
                            //sort of Array
                            $a = $this->sortStringArray($a, "ASC");
                            //is Array and get it from value
                            foreach ($a as $item) {
                                $a_item = $item["value"];
                                break;
                            }
                            if ($a_item == null) {
                                $a_item = "zzzzzzzzz";
                            }
                        } else {
                            $a_item = $a == null ? "zzzzzzzzz" : $a;
                        }

                        if (is_string($b["value"])) {
                            $b_item = $b["value"] == "" ? "zzzzzzzzz" : $b["value"];
                        } elseif (is_array($b)) {
                            //sort of Array
                            $b = $this->sortStringArray($b, "DESC");
                            //is Array and get it from value
                            foreach ($b as $item) {
                                $b_item = $item["value"];
                                break;
                            }
                            if ($b_item == null) {
                                $b_item = "zzzzzzzzz";
                            }
                        } else {
                            $b_item = $b == null ? "zzzzzzzzz" : $b;
                        }
                        return strnatcasecmp($a_item, $b_item);
                    });
                } else {
                    uasort($content, function ($a, $b) {
                        if (is_string($a["value"])) {
                            $a_item = $a["value"] == "" ? "000000000" : $a["value"];
                        } elseif (is_array($a)) {
                            //is Array and get it from value
                            foreach ($a as $item) {
                                $a_item = $item["value"];
                            }
                            if ($a_item == null) {
                                $a_item = "000000000";
                            }
                        } else {
                            $a_item = $a == null ? "zzzzzzzzz" : $a;
                        }

                        if (is_string($b["value"])) {
                            $b_item = $b["value"] == "" ? "000000000" : $b["value"];
                        } elseif (is_array($b)) {
                            //is Array and get it from value
                            foreach ($b as $item) {
                                $b_item = $item["value"];
                            }
                            if ($b_item == null) {
                                $b_item = "000000000";
                            }
                        } else {
                            $b_item = $b == null ? "zzzzzzzzz" : $b;
                        }
                        return strnatcasecmp($b_item, $a_item);
                    });
                }
                break;
        }
        //set Index from sort
        $f_counter = 0;
        $t_content = [];
        foreach ($content as $key => $item) {
            $t_content[$key]["__index__"] = $f_counter;
            $t_content[$key]["value"] = $item;
            $f_counter++;
        }
        return $t_content;
    }

    /**
     * @param $const
     * @return bool
     * @throws isys_exception_api
     * @throws isys_exception_auth
     * @throws isys_exception_database
     */
    private function isSpecialColumn($const)
    {
        if (strpos($const, "C__CATG__CONTACT") !== false) {
            $this->getTable("category_g_contact");
            return true;
        } elseif (strpos($const, "C__CATG__GUEST_SYSTEMS") !== false) {
            $this->getTable("catg_guest_systems");
            return true;
        } elseif (strpos($const, "C__CATS__PERSON_CONTACT_ASSIGNMENT") !== false) {
            return true;
        } elseif (strpos($const, "C__SHD__SERVICESTATUS") !== false) {
            return true;
        } elseif (strpos($const, "C__SHD__LIVESTATUS") !== false) {
            return true;
        } elseif (strpos($const, "C__CATG__LIVESTATUS") !== false) {
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_CABLE_SECTION_ELEMENT") !== false) {
            $this->getTable("C__CATG__SMVIEW_CABLE_SECTION_ELEMENT");
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_CABLE_SECTION") !== false) {
            if ($const == "C__CATG__SMVIEW_CABLE_SECTION-user_state_id") {
                $this->getTable("category_g_cable_user_state");
            }
            $this->getTable("C__CATG__SMVIEW_CABLE_SECTION");
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_CABLE_ROUTE_ELEMENT") !== false) {
            $this->getTable("C__CATG__SMVIEW_CABLE_ROUTE_ELEMENT");
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_CABLE_ROUTE") !== false) {
            $this->getTable("C__CATG__SMVIEW_CABLE_ROUTE");
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE_ELEMENT") !== false) {
            $this->getTable("C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE_ELEMENT");
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE") !== false) {
            $this->getTable("C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE");
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_CABLE") !== false) {
            if (strpos($const, "C__CATG__SMVIEW_CABLE_EXTENDED_LOCATION") !== false) {
                // extended location category is not special
                return false;
            }
            if ($const == "C__CATG__SMVIEW_CABLE-state_id") {
                $this->getTable("category_g_cable_user_state");
            } elseif ($const == "C__CATG__SMVIEW_CABLE-group_id") {
                $this->getTable("category_g_cable_group");
            } elseif ($const == "C__CATG__SMVIEW_CABLE-admin_unit") {
                $this->getTable("category_g_cable_admin_unit");
            } elseif ($const == "C__CATG__SMVIEW_CABLE-type_id") {
                $this->getTable("category_g_cable_type");
            } elseif ($const == "C__CATG__SMVIEW_CABLE-connection_type_id") {
                $this->getTable("category_g_cable_connection_type");
            } elseif ($const == "C__CATG__SMVIEW_CABLE-description_sort") {
                $this->getTable("category_cable_description_sort");
            }
            $this->getTable("category_g_cable");
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_CABLE_WIRE") !== false) {
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_CABLE_PIN") !== false) {
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_REPORTING_REPORT") !== false) {
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_FEED_JIRA") !== false) {
            return true;
        } elseif (strpos($const, "C__CATG__SMVIEW_FEED_VIRTUAL_DEVICE") !== false) {
            return true;
        }

        switch ($const) {
            case "C__CATG__GLOBAL-type_group_title":
            case "C__CATG__GLOBAL-group_image":
            case "C__CATG__IT_SERVICE-connected_object":
            case "C__CATG__LOCATION-_location_path":
            case "C__CATG__LOCATION-location_path":
            case "C__CATG__LOCATION-latitude":
            case "C__CATG__LOCATION-longitude":
            case "C__CATG__LOCATION-pos":
            case "C__CATS__APPLICATION-install_count":
            case "C__CATS__PERSON_ASSIGNED_GROUPS-connected_object":
            case "C__CATS__PERSON_GROUP_MEMBERS-connected_object":
            case "C__CATG__RAID-full_capacity":
            case "C__CATG__LOCATION-slot_nr":
            case "C__CATG__LOCATION-slot":
            case "C__CATG__LOGICAL_UNIT-parent":
            case "C__CATG__GLOBAL-tag":
            case "C__CATG__BVG_RENTING-obj_id":
            case "C__CATG__BVG_RENTING-location_in":
            case "C__CATG__BVG_RENTING-location_out":
            case "C__CATG__BVG_RENTING-damping":
            case "C__CATG__BVG_ASSIGNED_RENTING-obj_id":
            case "C__CATG__CABLE-length":
            case "C__CATG__CABLE-connection":
                return true;
            case "C__CATG__VIRTUAL_MACHINE-primary":
                $this->getTable("category_g_virtual_machine");
                return true;
            case "C__CATG__IP-fqdn":
            case "C__CATG__IP-primary_hostaddress":
                $this->getTable("category_g_ip");
                return true;
            case "C__CATG__APPLICATION-application_type":
            case "C__CATG__APPLICATION-application":
                $this->getTable("category_g_application");
                return true;
            case "C__CATG__MEMORY-total_capacity":
                $this->getTable("category_g_memory");
                return true;
        }
        return false;
    }

    public function sortStringArray($array, $sort)
    {
        if ($sort == "ASC") {
            uasort($array, function ($a, $b) {
                $a_item = null;
                $b_item = null;

                if (is_string($a["value"])) {
                    $a_item = $a["value"] == "" ? "zzzzzzzzz" : $a["value"];
                }

                if (is_string($b["value"])) {
                    $b_item = $b["value"] == "" ? "zzzzzzzzz" : $b["value"];
                }
                return strnatcasecmp($a_item, $b_item);
            });
        } else {
            uasort($array, function ($a, $b) {
                $a_item = null;
                $b_item = null;

                if (is_string($a["value"])) {
                    $a_item = $a["value"] == "" ? "zzzzzzzzz" : $a["value"];
                }

                if (is_string($b["value"])) {
                    $b_item = $b["value"] == "" ? "zzzzzzzzz" : $b["value"];
                }
                return strnatcasecmp($b_item, $a_item);
            });
        }
        return $array;
    }

    /**
     * prepare conditions value for the normal sql-querys
     * @param $p_condition
     *
     * @return mixed
     */
    public function prepare_normal_condition($p_condition)
    {
        foreach ($p_condition as $f_key => $f_condition) {
            switch ($f_condition["property"]) {
                case "C__CATG__GLOBAL-created":
                case "C__CATG__GLOBAL-changed":
                    if ($f_condition["comparison"] == ">" || $f_condition["comparison"] == "<=") {
                        if (strpos($f_condition["value"], '-') !== false) {
                            $split = explode("-", $f_condition["value"]);

                            if (ctype_digit(strval($split[0]))) {
                                $year = $split[0];
                            }
                            if (ctype_digit(strval($split[1]))) {
                                $month = $split[1];
                            }
                            if (ctype_digit(strval($split[2]))) {
                                $day = $split[2];
                            } else {
                                //06 17, 06 17:15:00
                                if (strpos($split[2], ' ') !== false) {
                                    $split_day = explode(" ", $split[2]);
                                    $day = $split_day[0];

                                    if (strpos($split_day[1], ':') !== false) {
                                        $split_clock = explode(":", $split_day[1]);
                                        if (ctype_digit(strval($split_clock[0]))) {
                                            $hour = $split_clock[0];
                                        }

                                        if (ctype_digit(strval($split_clock[1]))) {
                                            $minute = $split_clock[1];
                                        }

                                        if (ctype_digit(strval($split_clock[2]))) {
                                            $second = $split_clock[2];
                                        }
                                    }
                                }
                            }
                        } else {
                            //2017
                            if (ctype_digit(strval($f_condition["value"]))) {
                                $year = $f_condition["value"];
                            }
                        }

                        if (isset($year) == false) {
                            break;
                        }
                        if (isset($month) == false) {
                            $month = "12";
                        }
                        if (isset($day) == false) {
                            $day = "31";
                        }
                        if (isset($hour) == false) {
                            $hour = "24";
                        }
                        if (isset($minute) == false) {
                            $minute = "59";
                        }
                        if (isset($second) == false) {
                            $second = "59";
                        }

                        $format = '%d-%d-%d %d:%d:%d';
                        $value = sprintf($format, $year, $month, $day, $hour, $minute, $second);

                        $p_condition[$f_key]["value"] = $value;
                    } elseif ($f_condition["comparison"] == "<" || $f_condition["comparison"] == ">=") {
                        if (strpos($f_condition["value"], '-') !== false) {
                            $split = explode("-", $f_condition["value"]);

                            if (ctype_digit(strval($split[0]))) {
                                $year = $split[0];
                            }
                            if (ctype_digit(strval($split[1]))) {
                                $month = $split[1];
                            }
                            if (ctype_digit(strval($split[2]))) {
                                $day = $split[2];
                            } else {
                                //06 17, 06 17:15:00
                                if (strpos($split[2], ' ') !== false) {
                                    $split_day = explode(" ", $split[2]);
                                    $day = $split_day[0];

                                    if (strpos($split_day[1], ':') !== false) {
                                        $split_clock = explode(":", $split_day[1]);
                                        if (ctype_digit(strval($split_clock[0]))) {
                                            $hour = $split_clock[0];
                                        }

                                        if (ctype_digit(strval($split_clock[1]))) {
                                            $minute = $split_clock[1];
                                        }

                                        if (ctype_digit(strval($split_clock[2]))) {
                                            $second = $split_clock[2];
                                        }
                                    }
                                }
                            }
                        } else {
                            //2017
                            if (ctype_digit(strval($f_condition["value"]))) {
                                $year = $f_condition["value"];
                            }
                        }

                        if (isset($year) == false) {
                            break;
                        }
                        if (isset($month) == false) {
                            $month = "01";
                        }
                        if (isset($day) == false) {
                            $day = "01";
                        }
                        if (isset($hour) == false) {
                            $hour = "00";
                        }
                        if (isset($minute) == false) {
                            $minute = "00";
                        }
                        if (isset($second) == false) {
                            $second = "00";
                        }

                        $format = '%d-%d-%d %d:%d:%d';
                        $value = sprintf($format, $year, $month, $day, $hour, $minute, $second);

                        $p_condition[$f_key]["value"] = $value;
                    }
                    break;
            }
        }

        //sort Condition for Not Like and !=
        $t_array = [];
        foreach ($p_condition as $f_condition) {
            if ($t_array[$f_condition["property"]] == []) {
                $t_array[$f_condition["property"]][] = $f_condition;
            } else {
                if ($f_condition["comparison"] == "!=" || $f_condition["comparison"] == "NOT LIKE") {
                    $t_array[$f_condition["property"]][] = $f_condition;
                } else {
                    array_unshift($t_array[$f_condition["property"]], $f_condition);
                }
            }
        }

        $t_condition = [];
        foreach ($t_array as $f_condition_group) {
            foreach ($f_condition_group as $f_condition) {
                $t_condition[] = $f_condition;
            }
        }
        return $t_condition;
    }

    public function check_post_data($view_elements)
    {
        foreach ($view_elements as $element_key => $item) {
            foreach ($item as $category => $value) {
                switch ($category) {
                    case "C__CATG__IP-ipv6_address":
                        foreach ($value as $value_key => $element) {
                            if (strpos($element["value"], '.') !== false) {
                                unset($view_elements[$element_key][$category][$value_key]);
                            }
                        }
                        break;

                    case "C__CATG__IP-ipv4_address":
                        foreach ($value as $value_key => $element) {
                            if (strpos($element["value"], ':') !== false) {
                                unset($view_elements[$element_key][$category][$value_key]);
                            }
                        }
                        break;
                }
            }
        }
        return $view_elements;
    }

    /**
     * get_data always retrieves the data of the main table of this module
     */
    public function get_data()
    {
        // TODO: Implement get_data() method.
    }

    /**
     * @param  int $objid
     * @return string
     * @throws isys_exception_database
     */
    private function getObj_Title($objid)
    {
        if (!is_numeric($objid)) {
            return "";
        }

        if (isset($this->m_obj_title[$objid])) {
            return $this->m_obj_title[$objid];
        }

        $sql = "SELECT isys_obj.isys_obj__title FROM `isys_obj` WHERE isys_obj.isys_obj__id = " . $objid;

        $isys_component_dao = new isys_component_dao($this->m_db);
        $l_res =  $isys_component_dao->retrieve($sql);
        while ($l_data = $l_res->get_row()) {
            $this->m_obj_title[$objid] = $l_data["isys_obj__title"];
            return $l_data["isys_obj__title"];
        }
        return "";
    }
}
