<?php

use idoit\Module\Smview\ModuleUpdater;

/**
 * SM-VIEW Updater AJAX handler
 *
 * @package    i-doit
 * @subpackage smview
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_ajax_handler_smview_updater extends isys_ajax_handler
{

    /**
     * Init method, which gets called from the framework.
     */
    public function init()
    {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            $updater = new ModuleUpdater();
            $module = ! empty($this->m_get['module']) ? $this->m_get['module'] : $this->m_post['module'];
            switch ($this->m_get['func']) {
                case 'get-available-version':
                    $installedVersion = $updater->getCurrentModuleVersion($module);
                    $availableVersion = $updater->getAvailableModuleVersion($module);
                    $cmp = version_compare($installedVersion, $availableVersion);
                    $l_return['data'] = [
                        'available_version' => $availableVersion,
                        'is_updatable'      => $cmp < 0,
                    ];
                    break;
                case 'update':
                    $availableVersion = $updater->getAvailableModuleVersion($module);
                    $updater->update([$module]);
                    $l_return['data'] = [
                        'installed_version' => $availableVersion
                    ];
                    break;
            }
        }
        catch (isys_exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);

        $this->_die();
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return boolean
     */
    public static function needs_hypergate()
    {
        return true;
    }
}
