<?php

/**
 * Test data generator controller.
 *
 * @package    i-doit
 * @subpackage smview
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2020 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_handler_smview_test_data_generator extends isys_handler
{

    const TEST_API_KEY = 'IAmInsecure';

    /**
     * Initialization method.
     *
     * You can call this controller via CLI from i-doit root:
     * sudo -u www-data ./controller -v -d -u admin -p admin -i 1 -m smview_test_data_generator
     *
     * @return void
     */
    public function init()
    {
        verbose("start\n");

        // setup test API key
        $db = \isys_application::instance()->database_system;
        $mandator__id = 1;
        $apikey = self::TEST_API_KEY;
        $l_update = 'UPDATE isys_mandator SET isys_mandator__apikey = "' . $apikey . '" '
            . 'WHERE isys_mandator__id = "' . $mandator__id . '";';
        if ($db->query($l_update) && $db->commit()) {
            verbose('set api key');
        } else {
            error('could not find mandator with ID 1');
        }

        // rebuild properties
        $db = \isys_application::instance()->database;
        $l_dao = new \isys_cmdb_dao_category_property($db);
        $l_dao->rebuild_properties();
        verbose('rebuild properties');

        // prepare a virtual host
        $model_dao = new isys_cmdb_dao_category_g_model($db);
        $sql = "SELECT isys_catg_cpu_manufacturer__id from isys_catg_cpu_manufacturer WHERE isys_catg_cpu_manufacturer__title = 'AMD'";
        $manufacturerRow = $model_dao->retrieve($sql)->get_row();

        $sql = "SELECT isys_catg_cpu_type__id from isys_catg_cpu_type WHERE isys_catg_cpu_type__title = 'Athlon'";
        $cpuTypeRow1 = $model_dao->retrieve($sql)->get_row();

        $dao = new \isys_cmdb_dao_category_g_cpu($db);
        $vHostId = $dao->create_object('VHost01', defined_or_default('C__OBJTYPE__VIRTUAL_HOST'));
        $dao->create($vHostId, C__RECORD_STATUS__NORMAL, 'CPU01', $manufacturerRow['isys_model_manufacturer__id'], 3, $cpuTypeRow1['isys_catg_cpu_type__id'], '', 2);
        $dao->create($vHostId, C__RECORD_STATUS__NORMAL, 'CPU02', $manufacturerRow['isys_model_manufacturer__id'], 4, $cpuTypeRow1['isys_catg_cpu_type__id'], '', 4);
        $dao->create($vHostId, C__RECORD_STATUS__ARCHIVED, 'CPU03', $manufacturerRow['isys_model_manufacturer__id'], 2, $cpuTypeRow1['isys_catg_cpu_type__id'], '', 1);

        $vHostId2 = $dao->create_object('VHost02', defined_or_default('C__OBJTYPE__VIRTUAL_HOST'), C__RECORD_STATUS__ARCHIVED);
        $dao->create($vHostId2, C__RECORD_STATUS__NORMAL, 'CPUA1', $manufacturerRow['isys_model_manufacturer__id'], 5, $cpuTypeRow1['isys_catg_cpu_type__id'], '', 2);
        $dao->create($vHostId2, C__RECORD_STATUS__ARCHIVED, 'CPUA2', $manufacturerRow['isys_model_manufacturer__id'], 6, $cpuTypeRow1['isys_catg_cpu_type__id'], '', 4);

        // enable quick purge
        \isys_settings::set('cmdb.quickpurge', true);

        verbose("finished\n");
    }
}
