<?php

use idoit\Module\Smview\ModuleUpdater;

/**
 * i-doit
 *
 * @package    i-doit
 * @subpackage smview client
 * @author     Maik Rhode <maik.rhode@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */

class isys_module_smview extends isys_module //implements isys_module_authable
{
    // Define, if this module shall be displayed in the named menus.
    const DISPLAY_IN_MAIN_MENU   = true;
    const DISPLAY_IN_SYSTEM_MENU = true;
    const LICENSE_FLAG = 'SHD_GENERATE_LICENSE';

    private $m_userrequest;

    /**
     * Represents the name, of the corresponding auth class - "false" means, there is none.
     * @var  string
     */
    const AUTH_CLASS_NAME = 'isys_auth_module_smview';
        
    /**
     * Get related auth class for module
     *
     * @author Selcuk Kekec <skekec@i-doit.com>
     * @return isys_auth
     */
    public static function get_auth() {
        return isys_auth_smview::instance();
    }

    /**
     * Method for retrieving the path to the module directory (needed for includes).
     *
     * @static
     * @return  string
     */
    public static function get_module_dir() {
        return __DIR__ . DS;
    }

    /**
     * Method for retrieving the www-path to the assets directory (needed for the frontend).
     *
     * @static
     * @return  string
     */
    public static function get_assets_www_dir() {
        return isys_application::instance()->www_path . 'src/classes/modules/smview/assets/';
    }

    /**
     * Initializes the module.
     * @param isys_module_request $p_req
     * @return bool
     */
    public function init(isys_module_request &$p_req) {
        if (is_object($p_req)) {
            $this->m_userrequest = & $p_req;
            return true;
        }
        return false;
    }

    /**
     * @desc Starts module process
     */
    public function start() {
        $l_template = $this->m_userrequest->get_template();
        $l_tree = $this->m_userrequest->get_menutree();
        $lang = isys_application::instance()->container->get('language');

        $l_tree->add_node(0, -1, $lang->get('LC__MODULE__SMVIEW__TREE_TITLE'));
        $l_tree->add_node(1, 0, $lang->get('LC__MODULE__SMVIEW__DOWNLOAD_SMVIEW'),
            isys_helper_link::create_url([
                C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW'),
                C__GET__TREE_NODE => 1,
                'request'         => 'download_smview',
            ])
        );
        $l_tree->add_node(2, 0, $lang->get('LC__MODULE__SMVIEW__ACTIVATE_API'),
            isys_helper_link::create_url([
                C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW'),
                C__GET__TREE_NODE => 2,
                'request'         => 'activate_api',
            ])
        );
        $l_tree->add_node(3, 0, $lang->get('LC__MODULE__SMVIEW__LICENSE_ACTIVATE'),
            isys_helper_link::create_url([
                C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW'),
                C__GET__TREE_NODE => 3,
                'request'         => 'license_activate',
            ])
        );
        if (isys_auth_auth::instance()->is_allowed_to(isys_auth::SUPERVISOR, "MODULE/C__MODULE__SMVIEW") && $this->_canGenerateLicenses()) {
            $l_tree->add_node(4, 3, $lang->get('LC__MODULE__SMVIEW__LICENSE_GENERATE'),
                isys_helper_link::create_url([
                    C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW'),
                    C__GET__TREE_NODE => 4,
                    'request'         => 'license_generate',
                ])
            );
        }
        if (isys_auth_auth::instance()->is_allowed_to(isys_auth::SUPERVISOR, "MODULE/C__MODULE__SMVIEW")) {
            $l_tree->add_node(5, 0, $lang->get('LC__MODULE__SMVIEW__UPDATES'),
                isys_helper_link::create_url([
                    C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW'),
                    C__GET__TREE_NODE => 5,
                    'request'         => 'updates',
                ])
            );
        }

        // Assign tree.
        $l_template->assign("menu_tree", $l_tree->process($_GET[C__GET__TREE_NODE]));

        if (isset($_GET['request'])) {
            $this->route($_GET['request']);
        }
        else $this->route('/');
    }

    /**
     * Check if licenses can be created.
     *
     * @return boolean
     */
    private function _canGenerateLicenses()
    {
        if (file_exists(__DIR__ . "/init_index.php")) {
            include __DIR__ . "/init_index.php";
            $flag = defined_or_default(self::LICENSE_FLAG, 0);
            return $flag == '1';
        }
        return false;
    }

    /**
     * Routing example
     *
     * @param  string $p_request
     * @throws isys_exception_api
     */
    public function route($p_request) {

        $l_template = $this->m_userrequest->get_template();
        $l_path_url = parse_url($_SERVER['REQUEST_URI'])["path"];

        switch ($p_request) {
            case 'download_smview':
                $l_template
                    ->include_template('contentbottomcontent', __DIR__ . '/templates/download_smview.tpl')
                    ->assign('img_url', isys_application::instance()->www_path . 'src/classes/modules/smview/theme/img/');
                break;
            case 'activate_api':
                $isys_api_model_shd_apikey = new isys_api_model_shd_apikey();
                $method = $_SERVER['REQUEST_METHOD'];
                if ($method == 'POST') {
                    $key = $this->m_userrequest->get_post("api_enabled", null);
                    if (isset($key) && $key == "on") {
                        $isys_api_model_shd_apikey->enableKey();
                    } else {
                        $isys_api_model_shd_apikey->disableKey();
                    }
                }

                $l_template
                    ->include_template('contentbottomcontent', __DIR__ . '/templates/activate_api.tpl')
                    ->assign('api_enabled', $isys_api_model_shd_apikey->isActive());
                break;
            case 'license_activate':
                $isys_api_model_shd_licences = new isys_api_model_shd_licences();
                $method = $_SERVER['REQUEST_METHOD'];
                if ($method == 'POST') {
                    $a_license_key = ["licence_key" => $this->m_userrequest->get_post("licensekey", null)];
                    $isys_api_model_shd_licences->activation($a_license_key);
                }

                $license_request = $isys_api_model_shd_licences->licencesrequest([]);
                $license_key = $isys_api_model_shd_licences->getLicencesKey([]);
                $licenseStatus = $isys_api_model_shd_licences->getLicenceStatus([]);

                $l_template
                    ->include_template('contentbottomcontent', __DIR__ . '/templates/license_activate.tpl')
                    ->assign('license_request', $license_request)
                    ->assign('license_key', $license_key)
                    ->assign('license_status', $licenseStatus);
                break;
            case 'license_generate':
                $licenseKey = '';
                $month = 12;
                $year = date('Y', strtotime('+2 year'));
                $availableModules = [
                    smview_cmdb_dao_lic::MODULE_INSIGHT,
                    smview_cmdb_dao_lic::MODULE_MAPS,
                    smview_cmdb_dao_lic::MODULE_BSM,
                    smview_cmdb_dao_lic::MODULE_CABLE,
                    smview_cmdb_dao_lic::MODULE_PATCH,
                    smview_cmdb_dao_lic::MODULE_TV,
                ];
                $selectedModules = [];

                $method = $_SERVER['REQUEST_METHOD'];
                if ($method == 'POST') {
                    $licenseRequest = $_POST['license_request'];
                    $selectedModules = $_POST['modules'];
                    $month = $_POST['month'];
                    $year = $_POST['year'];
                    $license = new smview_cmdb_dao_lic(isys_application::instance()->database);
                    $licenseKey = $license->generateLicense($licenseRequest, $month, $year, $selectedModules);
                } else {
                    $licenceApi = new isys_api_model_shd_licences();
                    $licenseRequest = $licenceApi->licencesrequest();
                }

                $l_template
                    ->include_template('contentbottomcontent', __DIR__ . '/templates/license_generate.tpl')
                    ->assign('modules', $availableModules)
                    ->assign('selected_modules', $selectedModules)
                    ->assign('license_request', $licenseRequest)
                    ->assign('month', $month)
                    ->assign('year', $year)
                    ->assign('object_count', 50000)
                    ->assign('license_key', $licenseKey);
                break;
            case 'updates':
                $updater = new ModuleUpdater();
                $available_modules = $updater->getAvailableModules();
                $modules = [];
                foreach ($available_modules as $name) {
                    $currentVersion = $updater->getCurrentModuleVersion($name);
                    if (empty($currentVersion)) {
                        // only list installed add-ons
                        continue;
                    }

                    $modules[] = [
                        'name'              => $name,
                        'title'             => $name,
                        'installed_version' => $currentVersion,
                    ];
                }

                // create JS configuration
                $js_config = [
                    'ajax_url'       => isys_helper_link::create_url([C__GET__AJAX => 1, C__GET__AJAX_CALL => 'smview_updater']),
                    'session'        => $_COOKIE['PHPSESSID'],
                    'language'       => $this->_get_js_translations(),
                    'spinner_url'    => isys_helper_link::get_base() . '/images/ajax-loading.gif',
                    'is_update_mode' => true,
                    'modules'        => $modules,
                ];

                $l_template
                    ->include_template('contentbottomcontent', __DIR__ . '/templates/updates.tpl')
                    ->assign('assets_dir', isys_module_smview::get_assets_www_dir())
                    ->assign('js_config', json_encode($js_config));
                break;
            default:
                $l_template
                    ->include_template('contentbottomcontent', __DIR__ . '/templates/index.tpl')
                    ->assign('modul_path', $l_path_url .'?moduleID=' . defined_or_default('C__MODULE__SMVIEW'))
                    ->assign('moduleId', defined_or_default('C__MODULE__SMVIEW'));
        }
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    private function _get_js_translations()
    {
        // language templates
        $language_templates = [
            'LC__CMDB__CATG__SMVIEW__ADD_ON_NAME',
            'LC__CMDB__CATG__SMVIEW__CURRENT_VERSION',
            'LC__CMDB__CATG__SMVIEW__AVAILABLE_VERSION',
            'LC__CMDB__CATG__SMVIEW__UPDATE',
        ];

        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }
}
