<?php

namespace idoit\Module\Smview\Console\Command;

use Exception;
use idoit\Console\Command\AbstractCommand;
use isys_application;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Command to activate SM-VIEW API.
 *
 * @package    i-doit
 * @subpackage smview
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class ActivateApiCommand extends AbstractCommand
{

    /**
     * Get name for command
     *
     * @return string
     */
    public function getCommandName()
    {
        return 'smview_activate_api';
    }

    /**
     * Get description for command
     *
     * @return string
     */
    public function getCommandDescription()
    {
        return 'Command to activate SM-VIEW API.';
    }

    /**
     * Retrieve Command InputDefinition
     *
     * @return InputDefinition
     */
    public function getCommandDefinition()
    {
        return new InputDefinition();
    }

    /**
     * Checks if a command can have a config file via --config
     *
     * @return bool
     */
    public function isConfigurable()
    {
        return false;
    }

    /**
     * Execute command.
     *
     * @param  InputInterface  $input
     * @param  OutputInterface $output
     * @return int
     * @throws Exception
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        global $g_absdir, $g_mandator;
        try {
            $output->writeln("start\n");

            $keyFile = $g_absdir . '/images/shd-smview.fdg';
            $mandatorId = ! empty($g_mandator) ? $g_mandator : 1;

            $apiKeyData = isys_application::instance()->database_system->fetch_row_assoc(
                isys_application::instance()->database_system->query(
                    'SELECT isys_mandator__apikey, isys_mandator__title FROM isys_mandator 
                  WHERE isys_mandator__id = "' . $mandatorId . '";'
                )
            );

            $apiTitle = $apiKeyData["isys_mandator__title"];
            $apiKey = $apiKeyData["isys_mandator__apikey"];

            if (empty($apiKey)) {
                $apiKey = $this->_generateApiKey();
                $db = isys_application::instance()->database_system;
                $l_update = 'UPDATE isys_mandator SET isys_mandator__apikey = "' . $apiKey . '" '
                    . 'WHERE isys_mandator__id = "' . $mandatorId . '";';
                if ($db->query($l_update) && $db->commit()) {
                    $output->writeln('created new api key');
                }
            }

            $apiArray = array();
            if (file_exists($keyFile)) {
                // get current content
                $apikeyString = file_get_contents($keyFile);
                $apiArray = json_decode(base64_decode($apikeyString),true);
            }

            // set mandator data
            $apiArray[$mandatorId]["title"] = $apiTitle;
            $apiArray[$mandatorId]["apikey"] = $apiKey;

            // save to file
            $ifp = fopen($keyFile, "wb" );
            fwrite( $ifp, base64_encode( json_encode( $apiArray ) ) );
            fclose( $ifp );

            $output->writeln("finished\n");
        } catch (Exception $e) {
            $output->writeln("");
            $output->writeln($e);
            $output->writeln("");
        }
        return 0;
    }

    /**
     * Generate a random API key.
     *
     * @return string
     */
    private function _generateApiKey(): string
    {
        $characters = 'abcdefghijklmnopqrstuvwxyz0123456789';
        $string = '';
        $max = strlen($characters) - 1;
        for ($i = 0; $i < 10; $i++) {
            $string .= $characters[mt_rand(0, $max)];
        }
        return $string;
    }

    /**
     * Returns an array of command usages
     *
     * @return array
     */
    public function getCommandUsages(): array
    {
        return [
            'smview_activate_api -u admin -p admin -i 1',
        ];
    }
}
