<?php

namespace idoit\Module\Smview\Console\Command;

use Exception;
use idoit\Console\Command\AbstractCommand;
use isys_application;
use isys_cmdb_dao_category_g_cpu;
use isys_cmdb_dao_category_g_model;
use isys_cmdb_dao_category_property;
use isys_settings;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Command to generate some test data.
 *
 * @package    i-doit
 * @subpackage smview
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class TestDataGeneratorCommand extends AbstractCommand
{

    const TEST_API_KEY = 'IAmInsecure';

    /**
     * Get name for command
     *
     * @return string
     */
    public function getCommandName()
    {
        return 'smview_test_data_generator';
    }

    /**
     * Get description for command
     *
     * @return string
     */
    public function getCommandDescription()
    {
        return 'Command to generate some SM-VIEW test data.';
    }

    /**
     * Retrieve Command InputDefinition
     *
     * @return InputDefinition
     */
    public function getCommandDefinition()
    {
        return new InputDefinition();
    }

    /**
     * Checks if a command can have a config file via --config
     *
     * @return bool
     */
    public function isConfigurable()
    {
        return false;
    }

    /**
     * Execute command.
     *
     * @param  InputInterface  $input
     * @param  OutputInterface $output
     * @return int
     * @throws Exception
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        global $g_mandator;
        try {
            $output->writeln("start\n");

            // setup test API key
            $db = isys_application::instance()->database_system;
            $mandatorId = ! empty($g_mandator) ? $g_mandator : 1;
            $apikey = self::TEST_API_KEY;
            $l_update = 'UPDATE isys_mandator SET isys_mandator__apikey = "' . $apikey . '" '
                . 'WHERE isys_mandator__id = "' . $mandatorId . '";';
            if ($db->query($l_update) && $db->commit()) {
                $output->writeln('set api key');
            } else {
                $output->writeln('could not find mandator with ID 1');
            }

            // rebuild properties
            $db = isys_application::instance()->database;
            $l_dao = new isys_cmdb_dao_category_property($db);
            $l_dao->rebuild_properties();
            $output->writeln('rebuild properties');

            // prepare a virtual host
            $model_dao = new isys_cmdb_dao_category_g_model($db);
            $sql = "SELECT isys_catg_cpu_manufacturer__id from isys_catg_cpu_manufacturer WHERE isys_catg_cpu_manufacturer__title = 'AMD'";
            $manufacturerRow = $model_dao->retrieve($sql)->get_row();

            $sql = "SELECT isys_catg_cpu_type__id from isys_catg_cpu_type WHERE isys_catg_cpu_type__title = 'Athlon'";
            $cpuTypeRow1 = $model_dao->retrieve($sql)->get_row();

            $dao = new isys_cmdb_dao_category_g_cpu($db);
            $vHostId = $dao->create_object('VHost01', defined_or_default('C__OBJTYPE__VIRTUAL_HOST'));
            $dao->create($vHostId, C__RECORD_STATUS__NORMAL, 'CPU01', $manufacturerRow['isys_model_manufacturer__id'], 3, $cpuTypeRow1['isys_catg_cpu_type__id'], '', 2);
            $dao->create($vHostId, C__RECORD_STATUS__NORMAL, 'CPU02', $manufacturerRow['isys_model_manufacturer__id'], 4, $cpuTypeRow1['isys_catg_cpu_type__id'], '', 4);
            $dao->create($vHostId, C__RECORD_STATUS__ARCHIVED, 'CPU03', $manufacturerRow['isys_model_manufacturer__id'], 2, $cpuTypeRow1['isys_catg_cpu_type__id'], '', 1);

            $vHostId2 = $dao->create_object('VHost02', defined_or_default('C__OBJTYPE__VIRTUAL_HOST'), C__RECORD_STATUS__ARCHIVED);
            $dao->create($vHostId2, C__RECORD_STATUS__NORMAL, 'CPUA1', $manufacturerRow['isys_model_manufacturer__id'], 5, $cpuTypeRow1['isys_catg_cpu_type__id'], '', 2);
            $dao->create($vHostId2, C__RECORD_STATUS__ARCHIVED, 'CPUA2', $manufacturerRow['isys_model_manufacturer__id'], 6, $cpuTypeRow1['isys_catg_cpu_type__id'], '', 4);

            // enable quick purge
            isys_settings::set('cmdb.quickpurge', true);

            $output->writeln("finished\n");
        } catch (Exception $e) {
            $output->writeln("");
            $output->writeln($e);
            $output->writeln("");
        }
        return 0;
    }

    /**
     * Returns an array of command usages
     *
     * @return array
     */
    public function getCommandUsages(): array
    {
        return [
            'smview_test_data_generator -u admin -p admin -i 1',
        ];
    }
}
