<?php

namespace idoit\Module\Smview\Console\Command;

use Exception;
use idoit\Console\Command\AbstractCommand;
use idoit\Module\Smview\ModuleUpdater;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Command to update all installed add-ons.
 *
 * @package    i-doit
 * @subpackage smview
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class UpdateModulesCommand extends AbstractCommand
{

    /**
     * Get name for command
     *
     * @return string
     */
    public function getCommandName()
    {
        return 'smview_update_modules';
    }

    /**
     * Get description for command
     *
     * @return string
     */
    public function getCommandDescription()
    {
        return 'Command to update all installed add-ons.';
    }

    /**
     * Retrieve Command InputDefinition
     *
     * @return InputDefinition
     */
    public function getCommandDefinition()
    {
        return new InputDefinition();
    }

    /**
     * Checks if a command can have a config file via --config
     *
     * @return bool
     */
    public function isConfigurable()
    {
        return false;
    }

    /**
     * Execute command.
     *
     * @param  InputInterface  $input
     * @param  OutputInterface $output
     * @return int
     * @throws Exception
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        try {
            $output->writeln("start\n");

            $updater = new ModuleUpdater(true);
            $modules = $updater->getAvailableModules();
            $updater->update(array_values($modules));

            $output->writeln("finished\n");
        } catch (Exception $e) {
            $output->writeln("");
            $output->writeln($e);
            $output->writeln("");
        }
        return 0;
    }

    /**
     * Returns an array of command usages
     *
     * @return array
     */
    public function getCommandUsages(): array
    {
        return [
            'smview_update_modules -u admin -p admin -i 1',
        ];
    }
}
