<?php

/**
 * i-doit
 *
 * JSON-RPC Test case
 *
 * @author     Dennis Stücken <dstuecken@i-doit.org>
 * @author     Benjamin Heisig <bheisig@synetics.de>
 * @package    i-doit
 * @subpackage UnitTesting
 * @copyright  synetics GmbH
 * @license    http://www.i-doit.com/license
 */

/** Include JSON-RPC client. */
global $g_absdir;
//include_once '/var/www/html/i-doit/src/tests/jsonrpc.class.php';
include_once '/var/www/html/i-doit/src/classes/modules/smview/tests/jsonrpc.php';
use PHPUnit\Framework\TestCase;

/**
 * Test class for isys_api_controller_jsonrpc
 */
class test_api_objects extends TestCase
{

    private $m_client;

    private $m_session;

    private $m_apikey;

    private $m_global_id;

    public function test_create()
    {
        global $g_test_config;

        $l_params = [
            'apikey'  => $this->m_apikey,
            'objects' => [
                ['C__CATG__GLOBAL-title' => 'test_api_call','C__CATG__GLOBAL-cmdb_status' => 'In Betrieb']


            ],
            'obj_type_id' => '5'
        ];

        $l_result = $this->m_client->__call('shd.objects.create', $l_params);

        $this->assertEquals('objects.create.success', $l_result["message"]);

        $this->assertInternalType('array', $l_result);

        //there are 15 properties in Global
//        $this->assertEquals(15, $i_check_Categorys);


        foreach ($l_result["response"][0] as $item) {

            return $item;
        }

        $this->assertTrue(false);
    }

    public function test_create_all_object_type()
    {
        global $g_test_config;

        $return_ids = [];

        $l_params_object_types = [
            'apikey'  => $this->m_apikey,
        ];

        $l_result_object_types = $this->m_client->__call('shd.object_types', $l_params_object_types);

        foreach ($l_result_object_types as $l_result_object_type) {

//            print_r($l_result_object_type);

            $l_params = [
                'apikey'  => $this->m_apikey,
                'objects' => [
                    ['C__CATG__GLOBAL-title' => 'test_api_call','C__CATG__GLOBAL-cmdb_status' => 'In Betrieb']


                ],
                'obj_type_id' => $l_result_object_type['isys_obj_type__id']
            ];

            $l_result = $this->m_client->__call('shd.objects.create', $l_params);

            $this->assertEquals('objects.create.success', $l_result["message"]);

            $this->assertInternalType('array', $l_result);

            //there are 15 properties in Global
    //        $this->assertEquals(15, $i_check_Categorys);


            foreach ($l_result["response"][0] as $item) {

                $return_ids[] = [ "id" =>$item, "obj_type_id" => $l_result_object_type['isys_obj_type__id']];
            }
        }

        return $return_ids;
    }

    /**
     * @depends test_create_all_object_type
     */
    public function test_read_all_object_type()
    {
        //get ID from last Create
        $id = func_get_args();

        foreach ($id[0] as $item) {

            $l_params = [
                'session'     => $this->m_session,
                'id'          => '1',
                'apikey'      => $this->m_apikey,
                'condition'   => [["comparison" => "LIKE %...%", "property" => "C__CATG__GLOBAL-id", "category" => "C__CATG__GLOBAL", "value" => $item["id"]]],
                'columns'     => [["id" => "C__CATG__GLOBAL-title", "width" => "140", "sort" => ""]],
                'obj_type_id' => $item["obj_type_id"]
            ];

            $l_result = $this->m_client->__call('shd.query.read', $l_params);

            $this->assertEquals('test_api_call', $l_result["content"][$item["id"]]["C__CATG__GLOBAL-title"]);

            //Time check
            if ((float)$l_result["time"] < 1) {
                $this->assertTrue(true);
            } else {
                print_r("Time check false");
                $this->assertTrue(false);
            }

            $this->assertInternalType('array', $l_result);

        }

        return $id;
    }

    /**
     * @depends test_read_all_object_type
     */
    public function test_update()
    {
        //get ID from last Create
        $id = func_get_args();

        foreach ($id[0] as $item) {

            $l_params = [
                'session'     => $this->m_session,
                'id'          => '1',
                'apikey'      => '365478025',
                'with_dialog' => true
            ];

            $l_result = $this->m_client->__call('shd.properties', $l_params);

            $i_check_Categorys = 0;

            foreach ($l_result as $item) {
                if ($item["const"] == "C__CATG__GLOBAL") {
                    $i_check_Categorys += 1;
                }
            }

            $this->assertInternalType('array', $l_result);

            //there are 15 properties in Global
            $this->assertEquals(15, $i_check_Categorys);

        }

    }


    /**
     * @depends test_create_all_object_type
     */
    public function test_delete_all_object_type()
    {
        //get ID from last Create
        $id = func_get_args();

        foreach ($id[0] as $item) {

            $l_params = [
                'session' => $this->m_session,
                'id'      => $item["id"],
                'apikey' => '365478025',
                'status' => 'C__RECORD_STATUS__PURGE',
            ];

            $l_result = $this->m_client->__call('cmdb.object.delete', $l_params);

            $this->assertEquals('1', $l_result["success"]);
        }

    }


//    /**
//     * @depends test_create
//     */
//    public function test_read()
//    {
//        global $g_test_config;
//
//        //get ID from last Create
//        $id = func_get_args();
//
//        $l_params = [
//            'session' => $this->m_session,
//            'id'      => '1',
//            'apikey' => $this->m_apikey,
//            'condition' => [["comparison"=>"LIKE %...%","property"=>"C__CATG__GLOBAL-id", "category"=> "C__CATG__GLOBAL","value"=> $id[0]]],
//            'columns' => [["id"=>"C__CATG__GLOBAL-title","width"=>"140", "sort"=> ""]],
//            'obj_type_id' => '5'
//        ];
//
//        $l_result = $this->m_client->__call('shd.query.read', $l_params);
//
//        $this->assertEquals('test_api_call', $l_result["content"][$id[0]]["C__CATG__GLOBAL-title"]);
//
//        //Time check
//        if((float)$l_result["time"] < 1){
//            $this->assertTrue(true);
//        }else {
//            print_r("Time check false");
//            $this->assertTrue(false);
//        }
//
//        $this->assertInternalType('array', $l_result);
//
//    }
//

//
//    public function test_delete()
//    {
//        global $g_test_config;
//
//        $l_params = [
//            'session' => $this->m_session,
//            'id'      => '1',
//            'apikey' => '365478025',
//            'with_dialog' => true
//        ];
//
//        $l_result = $this->m_client->__call('shd.properties', $l_params);
//
//        $i_check_Categorys = 0;
//
//        foreach ($l_result as $item) {
//            if($item["const"] == "C__CATG__GLOBAL") {
//                $i_check_Categorys += 1;
//            }
//        }
//
//
//        $this->assertInternalType('array', $l_result);
//
//        //there are 15 properties in Global
//        $this->assertEquals(15, $i_check_Categorys);
//
//
//    }

//
//    /**
//     * Sets up the fixture, for example, opens a network connection.
//     * This method is called before a test is executed.
//     */
    protected function setUp(): void {
        global $g_test_config;

        // Login required:
        $this->m_session = $g_test_config['session'];

//        $l_url = $g_test_config['path']['url'] . 'https://mrh-sma.shd-online.de/i-doit/src/jsonrpc.php';

        $l_url = 'https://mrh-sma.shd-online.de/i-doit/src/jsonrpc.php';

        $l_debug = $g_test_config['debug']['var_dump'];

        $this->m_client = new isys_json_rpc($l_url, $l_debug);

        $this->m_client->setRPCAuth('admin', 'admin');

        $this->m_apikey = '365478025';

    }

}

?>
